// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../barcode.h"
#include "doc-navi.h"
#include "events.h"
#include "fonts.h"
#include "measure.h"

namespace SX
{
	namespace XHTML { interface IRichText; }

	namespace PDF
	{
		object_type obj_type_pdf_annot = 0x18;

		enum annot_type
		{
			annot_type_undefined	= 0,
			annot_type_text			= 0x00000001,
			annot_type_link			= 0x00000002,
			annot_type_freetext		= 0x00000004,
			annot_type_line			= 0x00000008,
			annot_type_square		= 0x00000010,
			annot_type_circle		= 0x00000020,
			annot_type_polygon		= 0x00000040,
			annot_type_polyline		= 0x00000080,
			annot_type_highlight	= 0x00000100,
			annot_type_underline	= 0x00000200,
			annot_type_squiggly		= 0x00000400,
			annot_type_strikeout	= 0x00000800,
			annot_type_caret		= 0x00001000,
			annot_type_stamp		= 0x00002000,
			annot_type_ink			= 0x00004000,
			annot_type_popup		= 0x00008000,
			annot_type_attachment	= 0x00010000,
			annot_type_sound		= 0x00020000,
			annot_type_movie		= 0x00040000,
			annot_type_screen		= 0x00080000,
			annot_type_widget		= 0x00100000,
			annot_type_printermark	= 0x00200000,
			annot_type_trapnet		= 0x00400000,
			annot_type_watermark	= 0x00800000,
			annot_type_3D			= 0x01000000,
			annot_type_redact		= 0x02000000,
			annot_type_projection	= 0x04000000,
			annot_type_richmedia	= 0x08000000,
			annot_type_reply_to		= 0x10000000
		};

		const sx_flags annot_comments		= annot_type_text | annot_type_freetext | annot_type_stamp | annot_type_ink;
		const sx_flags annot_ext_comments	= annot_comments | annot_type_watermark ;
		const sx_flags annot_text_markup	= annot_type_highlight | annot_type_underline | annot_type_squiggly | annot_type_strikeout | annot_type_caret;
		const sx_flags annot_shapes			= annot_type_line | annot_type_square | annot_type_circle | annot_type_polygon | annot_type_polyline;
		const sx_flags annot_markup			= annot_comments | annot_text_markup | annot_shapes | annot_type_attachment | annot_type_sound | annot_type_redact | annot_type_projection;
		const sx_flags annot_multimedia		= annot_type_sound | annot_type_movie | annot_type_screen | annot_type_richmedia | annot_type_3D;
		const sx_flags annot_all = 0x0FFFFFFF;

		enum annot_flags
		{
			annot_flags_invisible		= 0x001,
			annot_flags_hidden			= 0x002,
			annot_flags_print			= 0x004,
			annot_flags_nozoom			= 0x008,
			annot_flags_norotate		= 0x010,
			annot_flags_noview			= 0x020,
			annot_flags_readonly		= 0x040,
			annot_flags_locked			= 0x080,
			annot_flags_toggle_noview	= 0x100,
			annot_flags_locked_contents	= 0x200
		};

		interface IFormField;
		interface IPopup;
		interface IAnnotDesign;
		interface IPDFActions;
		interface ILayer;
		interface IPage;
		interface ICompositeImage;
		interface IImageObject;
		interface IRichText;
		interface IAssociatedFiles;

		interface IAnnot : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual annot_type get_Type() const = 0;
			virtual sx_bool get_Markup() const = 0;
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDescription) = 0;
			virtual IStr* get_Ident() const = 0;
			virtual void set_Ident(sx_str sIdent) = 0;
			virtual sx_flags get_Flags() const = 0;
			virtual void set_Flags(sx_flags flags) = 0;
			virtual sx_double get_Opacity() const = 0;
			virtual void set_Opacity(sx_double dOpacity) = 0;
			virtual ILayer* get_Layer() = 0;
			virtual void set_Layer(ILayer* pLayer) = 0;
			virtual sx_uint get_LSID() const = 0;
			virtual void set_LSID(sx_uint unit) = 0;
			virtual IAssociatedFiles* get_AssocFiles() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_bool GetRectArea(sx_rect& box) const = 0;
			virtual void SetRectArea(const sx_rect& box) = 0;
			virtual void TransformRectArea(const sx_matrix& m) = 0;
			virtual sx_bool GetModDate(sx_time& time) const = 0;
			virtual void SetModDate(const sx_time& time) = 0;
			virtual void SetAppearance(ICompositeImage* image, sx_bool bAdjustAnnotSize = true) = 0;
			virtual ICompositeImage* GetAppearance() = 0;
			virtual IPDFActions* GetEventActions(SX::PDF::pdf_event evn) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IMarkupAnnot : public IAnnot
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IPopup* get_Popup() = 0;
			virtual const sx_color* GetColor() const = 0;
			virtual void SetColor(const sx_color& c) = 0;
			virtual IStr* get_Author() const = 0;
			virtual void set_Author(sx_str sAuthor) = 0;
			virtual IStr* get_Subject() const = 0;
			virtual void set_Subject(sx_str sSubject) = 0;
			virtual IRichText* get_FormattedDescription() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_bool GetCreateDate(sx_time& time) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// sticky note annotations
		enum sticky_icon
		{
			sticky_icon_custom,
			sticky_icon_comment,
			sticky_icon_key,
			sticky_icon_note,
			sticky_icon_help,
			sticky_icon_new_paragraph,
			sticky_icon_paragraph,
			sticky_icon_insert,
			sticky_icon_check,
			sticky_icon_circle,
			sticky_icon_cross,
			sticky_icon_star,
			sticky_icon_rarrow,
			sticky_icon_uarrow,
			sticky_icon_ularrow,
			sticky_icon_rpointer
		};

		//////////////////////////////////////////////////////////////////////////
		interface IStickyNote : public IMarkupAnnot
		{
			virtual sticky_icon get_Icon() const = 0;
			virtual void set_Icon(sticky_icon eIcon) = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// stamp annotation
		enum stamp_icon
		{
			stamp_icon_custom,
			stamp_icon_approved,
			stamp_icon_experimental,
			stamp_icon_not_approved,
			stamp_icon_as_is,
			stamp_icon_expired,
			stamp_icon_not_for_public_release,
			stamp_icon_confidential,
			stamp_icon_final,
			stamp_icon_sold,
			stamp_icon_departmental,
			stamp_icon_for_comment,
			stamp_icon_top_secret,
			stamp_icon_draft,
			stamp_icon_for_public_release,
			stamp_icon_completed,
			stamp_icon_information_only,
			stamp_icon_preliminary_results,
			stamp_icon_void
		};

		//////////////////////////////////////////////////////////////////////////
		interface IStamp : public IMarkupAnnot
		{
			virtual stamp_icon get_Icon() const = 0;
			virtual void set_Icon(stamp_icon eIcon) = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// hyperlink annotations
		enum link_highlight
		{
			link_highlight_none,
			link_highlight_invert,
			link_highlight_outline,
			link_highlight_push
		};

		enum border_style
		{
			border_style_unknown,
			border_style_solid,
			border_style_dashed,
			border_style_beveled,
			border_style_inset,
			border_style_underline
		};

		//////////////////////////////////////////////////////////////////////////
		interface ILink : public IAnnot
		{
			//////////////////////////////////////////////////////////////////////////
			// navi
			virtual IStr* get_NamedDest() const = 0;
			virtual void set_NamedDest(sx_str sTitle) = 0;
			virtual IStr* get_FormedURI() const = 0;
			virtual void set_FormedURI(sx_str sURI) = 0;
			virtual IDest* get_Dest() const = 0;
			virtual Graphics::IRegion* get_Region() const = 0;

			virtual IDest* ResetDest() = 0;
			virtual void SetRegion(Graphics::IRegion *pReg) = 0;

			//////////////////////////////////////////////////////////////////////////
			// design
			virtual link_highlight get_Highlight() const = 0;
			virtual void set_Highlight(link_highlight eHL) = 0;
			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual const sx_color* GetBorderColor() const = 0;
			virtual void SetBorderColor(const sx_color& c) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// text markup annotations (highlight, underline, squiggly and strikeout)
		interface ITextMarkup : public IMarkupAnnot
		{
			virtual Graphics::IRegion* get_Region() const = 0;
			virtual void SetRegion(Graphics::IRegion *pReg) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// caret annotation
		enum caret_symbol
		{
			caret_symbol_none = 0,
			caret_symbol_paragraph = 1
		};

		//////////////////////////////////////////////////////////////////////////
		interface ICaret : public ITextMarkup
		{
			virtual void GetMargins(sx_double &dLeft, sx_double &dTop, sx_double &dRight, sx_double &dBottom) const = 0;
			virtual void SetMargins(const sx_double &dLeft, const sx_double &dTop, const sx_double &dRight, const sx_double &dBottom) = 0;
			virtual caret_symbol get_CaretSymbol() const = 0;

			virtual void Reset(Graphics::IRegion *pReg, caret_symbol eSymbol, sx_bool bLeftSide = false) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// line annotation
		enum line_ending
		{
			line_ending_none,
			line_ending_square,
			line_ending_circle,
			line_ending_diamond,
			line_ending_open_arrow,
			line_ending_closed_arrow,
			line_ending_butt,
			line_ending_ropen_arrow,
			line_ending_rclosed_arrow,
			line_ending_slash
		};

		enum line_caption
		{
			line_caption_none,
			line_caption_inline,
			line_caption_top,
		};

		enum border_effect
		{
			border_effect_none,
			border_effect_cloudy
		};

		//////////////////////////////////////////////////////////////////////////
		interface ILine : public IMarkupAnnot
		{
			virtual void GetStartPoint(sx_point &p) const = 0;
			virtual void SetStartPoint(sx_point &p) = 0;
			virtual line_ending get_StartStyle() const = 0;
			virtual void set_StartStyle(line_ending eStyle) = 0;
			virtual void GetEndPoint(sx_point &p) const = 0;
			virtual void SetEndPoint(sx_point &p) = 0;
			virtual line_ending get_EndStyle() const = 0;
			virtual void set_EndStyle(line_ending eStyle) = 0;
			virtual sx_double get_Width() const = 0;
			virtual void set_Width(sx_double dWidth) = 0;
			virtual border_style get_Style() const = 0;
			virtual void set_Style(border_style eBS) = 0;
			virtual const sx_color* get_FillColor() const = 0;
			virtual void set_FillColor(const sx_color& c) = 0;
			virtual sx_double get_LeaderLine() const = 0;
			virtual void set_LeaderLine(const sx_double &dLen) = 0;
			virtual sx_double get_LeaderLineExt() const = 0;
			virtual void set_LeaderLineExt(const sx_double &dLen) = 0;
			virtual sx_double get_LeaderLineOffset() const = 0;
			virtual void set_LeaderLineOffset(const sx_double &dOff) = 0;
			virtual line_caption get_CaptionStyle() const = 0;
			virtual void set_CaptionStyle(line_caption eStyle) = 0;
			virtual void GetCaptionOffset(sx_point &p) const = 0;
			virtual void SetCaptionOffset(sx_point &p) = 0;
			virtual IStr* get_Intent() const = 0;
			virtual void set_Intent(sx_str sName) = 0;

			virtual IMeasure* GetMeasure(sx_bool bCreate = false, measure_sys_type t = measure_sys_type_unknown) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// square annotation
		interface ISquare : public IMarkupAnnot
		{
			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual border_effect get_BorderEffect() const = 0;
			virtual void set_BorderEffect(border_effect eBE) = 0;
			virtual sx_double get_BorderEffectIntensity() const = 0;
			virtual void set_BorderEffectIntensity(const sx_double &dIntensity) = 0;
			virtual const sx_color* get_FillColor() const = 0;
			virtual void set_FillColor(const sx_color& c) = 0;
			virtual void GetMargins(sx_double &dLeft, sx_double &dTop, sx_double &dRight, sx_double &dBottom) const = 0;
			virtual void SetMargins(const sx_double &dLeft, const sx_double &dTop, const sx_double &dRight, const sx_double &dBottom) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// circle annotation
		interface ICircle : public IMarkupAnnot
		{
			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual border_effect get_BorderEffect() const = 0;
			virtual void set_BorderEffect(border_effect eBE) = 0;
			virtual sx_double get_BorderEffectIntensity() const = 0;
			virtual void set_BorderEffectIntensity(const sx_double &dIntensity) = 0;
			virtual const sx_color* get_FillColor() const = 0;
			virtual void set_FillColor(const sx_color& c) = 0;
			virtual void GetMargins(sx_double &dLeft, sx_double &dTop, sx_double &dRight, sx_double &dBottom) const = 0;
			virtual void SetMargins(const sx_double &dLeft, const sx_double &dTop, const sx_double &dRight, const sx_double &dBottom) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// poly annotations
		interface IPolyline : public IMarkupAnnot, public Graphics::IPointsArray
		{
			virtual line_ending get_StartStyle() const = 0;
			virtual void set_StartStyle(line_ending eStyle) = 0;
			virtual line_ending get_EndStyle() const = 0;
			virtual void set_EndStyle(line_ending eStyle) = 0;
			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual const sx_color* get_FillColor() const = 0;
			virtual void set_FillColor(const sx_color& c) = 0;
			virtual IStr* get_Intent() const = 0;
			virtual void set_Intent(sx_str sName) = 0;

			virtual IMeasure* GetMeasure(sx_bool bCreate = false, measure_sys_type t = measure_sys_type_unknown) = 0;
			virtual void ResetAppearance() = 0;
		};

		interface IPolygon : public IMarkupAnnot, public Graphics::IPointsArray
		{
			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual border_effect get_BorderEffect() const = 0;
			virtual void set_BorderEffect(border_effect eBE) = 0;
			virtual sx_double get_BorderEffectIntensity() const = 0;
			virtual void set_BorderEffectIntensity(const sx_double &dIntensity) = 0;
			virtual const sx_color* get_FillColor() const = 0;
			virtual void set_FillColor(const sx_color& c) = 0;
			virtual IStr* get_Intent() const = 0;
			virtual void set_Intent(sx_str sName) = 0;

			virtual IMeasure* GetMeasure(sx_bool bCreate = false, measure_sys_type t = measure_sys_type_unknown) = 0;
			virtual void ResetAppearance() = 0;
		};

		interface IInkLineSegment : public IRefObject, public Graphics::IPointsArray
		{
		};

		interface IInkEraser : public IRefObject, public Graphics::IPointsArray
		{
			virtual void Reset() = 0;
			virtual sx_bool Apply(sx_double Width) = 0;
		};

		interface IInk : public IMarkupAnnot
		{
			virtual sx_uint get_SegmentsCount() const = 0;
			virtual IInkLineSegment* GetSegment(sx_uint uInd) = 0;
			virtual IInkLineSegment* InsertSegment(sx_uint uBefore) = 0;
			virtual void RemoveSegment(sx_uint uInd) = 0;
			virtual IInkEraser* GetInkEraser() = 0;

			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;

			virtual void ResetAppearance() = 0;
		};


		//////////////////////////////////////////////////////////////////////////
		// IAttachment
		enum attachment_icon
		{
			attachment_icon_custom,
			attachment_icon_graph,
			attachment_icon_pushpin,
			attachment_icon_paperclip,
			attachment_icon_tag,
		};

		interface IAttachment : public IMarkupAnnot
		{
			virtual attachment_icon get_Icon() const = 0;
			virtual void set_Icon(attachment_icon eIcon) = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef *pFileRef) = 0;

			virtual void ResetAppearance() = 0;
		};


		//////////////////////////////////////////////////////////////////////////
		// IWatermark
		interface IWatermark : public IAnnot
		{
			virtual sx_bool get_RelPosition() const = 0;
			virtual void set_RelPosition(sx_bool bRel) = 0;
			virtual sx_double get_HorOffset() const = 0;
			virtual void set_HorOffset(const sx_double &dOffset) = 0;
			virtual sx_double get_VertOffset() const = 0;
			virtual void set_VertOffset(const sx_double &dOffset) = 0;

			virtual void GetMatrix(sx_matrix& m) const = 0;
			virtual void SetMatrix(const sx_matrix& m) = 0;
		};


		//////////////////////////////////////////////////////////////////////////
		// IFreeText
		enum text_alignment
		{
			text_alignment_left = 0,
			text_alignment_center = 1,
			text_alignment_right = 2
		};
		interface IRichText;

		interface IFreeText : public IMarkupAnnot
		{
			// geometry
			virtual sx_bool GetTextBox(sx_rect& box) const = 0;
			virtual void SetTextBox(const sx_rect& box) = 0;
			virtual void SetCallout(const sx_rect& annot, const sx_rect& textbox, const sx_point &start, const sx_point &knee, const sx_point &end) = 0;
			virtual void RemoveCallout() = 0;
			virtual sx_bool GetStartPoint(sx_point &p) const = 0;
			virtual void SetStartPoint(const sx_point &p) = 0;
			virtual sx_bool GetKneePoint(sx_point &pt) const = 0;
			virtual void SetKneePoint(const sx_point &pt) = 0;
			virtual void RemoveKnee() = 0;
			virtual sx_bool GetEndPoint(sx_point &p) const = 0;
			virtual void SetEndPoint(const sx_point &p) = 0;
			virtual sx_rotation_angle get_Rotation() const = 0;
			virtual void set_Rotation(sx_rotation_angle r) = 0;
			// design
			virtual IFont* get_TextFont() = 0;
			virtual void set_TextFont(IFont* pFont) = 0;
			virtual sx_uint get_TextSize() const = 0;
			virtual void set_TextSize(const sx_uint s) = 0;
			virtual const sx_color* GetTextColor() const = 0;
			virtual void SetTextColor(const sx_color& c) = 0;
			virtual text_alignment get_TextAlignment() const = 0;
			virtual void set_TextAlignment(text_alignment align) = 0;
			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual border_effect get_BorderEffect() const = 0;
			virtual void set_BorderEffect(border_effect eBE) = 0;
			virtual sx_double get_BorderEffectIntensity() const = 0;
			virtual void set_BorderEffectIntensity(const sx_double &dIntensity) = 0;
			virtual line_ending get_EndStyle() const = 0;
			virtual void set_EndStyle(line_ending eStyle) = 0;
			virtual IStr* get_Intent() const = 0;
			virtual void set_Intent(sx_str sName) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// ISound
		enum sound_icon
		{
			sound_icon_custom,
			sound_icon_speaker,
			sound_icon_mic
		};

		interface ISoundStream : public IRefObject
		{
			virtual sx_uint get_Rate() const = 0;
			virtual void set_Rate(const sx_uint r) = 0;
			virtual sx_uint get_Channels() const = 0;
			virtual void set_Channels(const sx_uint ch) = 0;
			virtual sx_uint get_Bits() const = 0;
			virtual void set_Bits(const sx_uint bits) = 0;
			virtual IStr* get_Encoding() const = 0;
			virtual void set_Encoding(sx_str sName) = 0;
			virtual IStr* get_Compression() const = 0;
			virtual void set_Compression(sx_str sName) = 0;
			virtual void GetStream(ISequentialStream *pData) const = 0;
			virtual void SetStream(ISequentialStream *pData) = 0;
		};

		interface ISound : public IMarkupAnnot
		{
			virtual ISoundStream* get_SoundStream() = 0;
			virtual sound_icon get_Icon() const = 0;
			virtual void set_Icon(sound_icon eIcon) = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
			virtual void ResetAppearance() = 0;
			virtual void ExportWav(ISequentialStream *pData) const = 0;
			virtual void ImportWav(ISequentialStream *pData) = 0;
		};


		//////////////////////////////////////////////////////////////////////////
		// IMovie
		enum activate_movie
		{
			activate_movie_none,
			activate_movie_play_once,
			activate_movie_open,
			activate_movie_repeate,
			activate_movie_palindrome
		};

		interface IMovie : public IAnnot
		{
			virtual IStr* get_Title() const = 0;
			virtual void  set_Title(sx_str sTitle) = 0;
			virtual sx_uint get_Width() const = 0;
			virtual void set_Width(sx_uint dWidth) = 0;
			virtual sx_int get_Height() const = 0;
			virtual void set_Height(sx_uint dHeight) = 0;
			virtual sx_uint get_Rotation() const = 0;
			virtual void set_Rotation(sx_uint uR) = 0;
			// media
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef *pFile) = 0;
			// poster
			virtual IImageObject* GetPoster(sx_bool& bDisplay) const = 0;
			virtual void SetPoster(IImageObject* pImgPoster, sx_bool bDisplay) = 0;
			// movie activation
			virtual activate_movie get_ActivateMode() const = 0;
			virtual void set_ActivateMode(activate_movie eMode) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// IScreen
		interface IScreen : public IAnnot
		{
			virtual IStr* get_Title() const = 0;
			virtual void  set_Title(sx_str sTitle) = 0;
			virtual IAnnotDesign* get_Design() = 0;
			virtual IMediaRendition* get_Rendition() = 0;

			virtual IActionRendition* GetRenditionAction(sx_bool bCreate = false) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// widget annotations
		interface IBarcodeImage;

		interface IWidget : public IAnnot
		{
			virtual IFormField* get_FormField() = 0;
			virtual IAnnotDesign* get_Design() = 0;
			virtual IBarcodeImage* get_BarcodeImage() = 0;
			virtual void ResetBarcodeImage(Barcode::barcode_format format) = 0;
			virtual void ResetAppearance(sx_bool bClear = false) = 0;
			virtual IPage* FindPage(sx_bool bRemoveLost = false) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// popup annotations
		interface IPopup : public IAnnot
		{
			virtual sx_bool get_Opened() const = 0;
			virtual void set_Opened(sx_bool bOpened) = 0;
			virtual IMarkupAnnot* get_Parent() = 0;
		};

		enum media_type
		{
			media_type_unknown,
			media_type_3D,
			media_type_sound,
			media_type_video,
			media_type_flash
		};

		enum media_act_case
		{
			media_act_case_PO = 1,
			media_act_case_PV,
			media_act_case_XA,
			media_act_case_PC = 11,
			media_act_case_PI,
			media_act_case_XD
		};

		enum media_anim
		{
			media_anim_unknown,
			media_anim_none,
			media_anim_linear,
			media_anim_oscil
		};

		interface IRichMediaContent;
		interface IRichMediaScene;
		interface IRichMediaFile;
		interface IRichMediaPresent;
		interface I3DView;

		//////////////////////////////////////////////////////////////////////////
		interface IRichMedia : public IAnnot
		{
			virtual media_act_case get_Activate() const = 0;
			virtual void set_Activate(media_act_case act) = 0;
			virtual media_act_case get_Deactivate() const = 0;
			virtual void set_Deactivate(media_act_case deact) = 0;
			virtual media_anim get_AnimationStyle() const = 0;
			virtual void set_AnimationStyle(media_anim anim) = 0;
			virtual sx_int get_AnimationRepeat() const = 0;
			virtual void set_AnimationRepeat(sx_int num) = 0;
			virtual sx_double get_AnimationSpeed() const = 0;
			virtual void set_AnimationSpeed(sx_double sp) = 0;
			virtual I3DView* get_StartView() = 0;
			virtual void set_StartView(I3DView* pView) = 0;
			virtual IRichMediaScene* get_StartScene() = 0;
			virtual void set_StartScene(IRichMediaScene* pScene) = 0;

			virtual IRichMediaContent* GetContent(sx_bool bCreate = false) = 0;
			virtual IRichMediaPresent* GetPresentation(sx_bool bCreate = false) = 0;
			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IRichMediaContent : public IRefObject
		{
			virtual sx_uint get_FileCount() const = 0;
			virtual IStr* GetFileID(sx_uint ind) const = 0;
			virtual IFileRef* GetFile(sx_str sID) = 0;
			virtual IFileRef* AddFile(sx_str sID) = 0;
			virtual void RemoveFile(sx_str sID = nullptr) = 0;

			virtual IRichMediaScene* GetNextScene(IRichMediaScene* pScene = nullptr) = 0;
			virtual IRichMediaScene* AddScene(media_type t) = 0;
			virtual void RemoveScene(IRichMediaScene* pScene = nullptr) = 0;

			virtual I3DView* GetNextView(I3DView* pView = nullptr) = 0;
			virtual I3DView* AddView() = 0;
			virtual void RemoveView(I3DView* pView = nullptr) = 0;
		};

		interface IRichMediaFile;
		interface IRichMediaScene : public IRefObject
		{
			virtual media_type get_Type() const = 0;
			virtual IRichMediaFile* GetNextFile(IRichMediaFile *pFile = nullptr) = 0;
			virtual IRichMediaFile* AddFile(media_type t, sx_str sID) = 0;
			virtual void RemoveFile(IRichMediaFile *pFile = nullptr) = 0;
		};

		interface IRawProps;
		interface IRichMediaFile : public IRefObject
		{
			virtual media_type get_Type() const = 0;
			virtual IFileRef* get_File() = 0;

			virtual IRawProps* GetProps(sx_bool bCreate = false) = 0;
		};

		interface IRichMediaPresent
		{
			virtual sx_bool get_ShowWindowed() const = 0;
			virtual void set_ShowWindowed(sx_bool b) = 0;
			virtual IRawProps* GetWindowProps() = 0;
			virtual sx_bool get_ShowTransparent() const = 0;
			virtual void set_ShowTransparent(sx_bool b) = 0;
			virtual sx_bool get_ShowExtUI() const = 0;
			virtual void set_ShowExtUI(sx_bool b) = 0;
			virtual sx_bool get_PassContextClick() const = 0;
			virtual void set_PassContextClick(sx_bool b) = 0;
		};

		interface I3DScene;
		interface I3DSharedScene;
		interface I3DActOptions;

		//////////////////////////////////////////////////////////////////////////
		interface I3DArtwork : public IAnnot
		{
			virtual I3DView* get_StartView() = 0;
			virtual void set_StartView(I3DView* pView) = 0;
			virtual sx_bool get_Interactive() = 0;
			virtual void set_Interactive(sx_bool act) = 0;

			virtual I3DScene* GetScene(sx_bool bCreate = false) = 0;
			virtual I3DSharedScene* GetSharedScene(sx_bool bCreate = false) = 0;
			virtual I3DActOptions* GetActOptions(sx_bool bCreate = false) = 0;
			virtual sx_bool GetViewBox(sx_rect& box) const = 0;
			virtual void SetViewBox(const sx_rect& box) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum e3d_format
		{
			e3d_format_unknown,
			e3d_format_U3D,
			e3d_format_PRC
		};

		//////////////////////////////////////////////////////////////////////////
		interface I3DScene : public IRefObject
		{
			virtual I3DView* get_StartView() = 0;
			virtual void set_StartView(I3DView* pView) = 0;
			virtual IStr* get_StartJScript() const = 0;
			virtual void  set_StartJScript(sx_str sJScript) = 0;
			virtual e3d_format get_Format() const = 0;
			virtual void  set_Format(e3d_format format) = 0;

			virtual void GetData(ISequentialStream *pData) const = 0;
			virtual void SetData(ISequentialStream *pData) = 0;
			virtual I3DView* GetNextView(I3DView* pView = nullptr) = 0;
			virtual I3DView* AddView() = 0;
			virtual void RemoveView(I3DView* pView = nullptr) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface I3DSharedScene : public IRefObject
		{
			virtual I3DScene* GetScene(sx_bool bCreate = false) = 0;
			virtual void SetScene(I3DScene* pScene) = 0;
		};

		interface I3DBackground
		{
			virtual const sx_color* GetColor() const = 0;
			virtual void SetColor(const sx_color& c) = 0;
			virtual sx_bool get_EntireAnnot() const = 0;
			virtual void set_EntireAnnot(sx_bool set) = 0;
		}; 

		enum e3d_render
		{
			e3d_render_solid,
			e3d_render_solid_wireframe,
			e3d_render_transparent,
			e3d_render_transparent_wireframe,
			e3d_render_bbox,
			e3d_render_transparent_bbox,
			e3d_render_transparent_bbox_outline,
			e3d_render_wireframe,
			e3d_render_shaded_wireframe,
			e3d_render_hidden_wireframe,
			e3d_render_vertices,
			e3d_render_shaded_vertices,
			e3d_render_illustration,
			e3d_render_solid_outline,
			e3d_render_shaded_illustration,
			e3d_render_unknown
		};

		interface I3DRendering
		{
			virtual e3d_render get_Mode() const = 0;
			virtual void set_Mode(e3d_render mode) = 0;
			virtual sx_double get_Opacity() const = 0;
			virtual void set_Opacity(sx_double dOpacity) = 0;
			virtual sx_double get_SilhouetteAngle() const = 0;
			virtual void set_SilhouetteAngle(sx_double dAng) = 0;
			virtual const sx_color* GetAuxColor() const = 0;
			virtual void SetAuxColor(const sx_color& c) = 0;
			virtual const sx_color* GetFaceColor() const = 0;
			virtual void SetFaceColor(const sx_color& c) = 0;
		};

		enum e3d_light
		{
			e3d_light_scheme_artwork,
			e3d_light_scheme_none,
			e3d_light_scheme_white,
			e3d_light_scheme_day,
			e3d_light_scheme_night,
			e3d_light_scheme_hard,
			e3d_light_scheme_primary,
			e3d_light_scheme_blue,
			e3d_light_scheme_red,
			e3d_light_scheme_cube,
			e3d_light_scheme_cad,
			e3d_light_scheme_headlamp,
			e3d_light_scheme_unknown
		};

		interface I3DLightingScheme
		{
			virtual e3d_light get_Style() const = 0;
			virtual void set_Style(e3d_light st) = 0;
		};

		enum e3d_measure
		{
			e3d_measure_ld3,
			e3d_measure_pd3,
			e3d_measure_ad3,
			e3d_measure_rd3,
			e3d_measure_3dc,
			e3d_measure_unknown
		};

		interface I3DMeasure : IRefObject
		{
			virtual e3d_measure get_Style() const = 0;
			virtual void set_Style(e3d_measure st) = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
		};

		interface I3DNode : IRefObject { };
		interface I3DCrossSection : IRefObject { };

		//////////////////////////////////////////////////////////////////////////
		interface I3DView : public IRefObject
		{
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str sName) = 0;
			virtual IStr* get_Ident() const = 0;
			virtual void set_Ident(sx_str sName) = 0;

			virtual sx_bool GetCameraMatrix(sx_3d_matrix& m) const = 0;
			virtual void SetCameraMatrix(const sx_3d_matrix& m) = 0;
			virtual sx_bool GetCameraDistance(sx_double& d) const = 0;
			virtual void SetCameraDistance(const sx_double &d, sx_bool clear = false) = 0;

			virtual ICompositeImage* GetOverlay() = 0;
			virtual void SetOverlay(ICompositeImage* image) = 0;

			virtual I3DBackground* GetBackground() const = 0;
			virtual I3DRendering* GetRendering() const = 0;
			virtual I3DLightingScheme* GetLightingScheme() const = 0;

			virtual I3DMeasure* GetNextMeasure(I3DMeasure* pMeasure = nullptr) = 0;
			virtual I3DMeasure* AddMeasure() = 0;
			virtual void RemoveMeasure(I3DMeasure* pMeasure = nullptr) = 0;

			virtual I3DCrossSection* GetNextCrossSection(I3DCrossSection* pCrossSection = nullptr) = 0;
			virtual I3DCrossSection* AddCrossSection() = 0;
			virtual void RemoveCrossSection(I3DCrossSection* pCrossSection = nullptr) = 0;

			virtual I3DNode* GetNextNode(I3DNode* pNode = nullptr) = 0;
			virtual I3DNode* AddNode() = 0;
			virtual void RemoveNode(I3DNode* pNode = nullptr) = 0;
			virtual sx_bool get_RestoreNodes() const = 0;
			virtual void set_RestoreNodes(sx_bool b) = 0;
		};

		enum e3d_act_case
		{
			e3d_act_case_PO = 1,
			e3d_act_case_PV,
			e3d_act_case_XA,
			e3d_act_case_PC = 11,
			e3d_act_case_PI,
			e3d_act_case_XD
		};

		enum e3d_act_state
		{
			e3d_act_state_I = 1,
			e3d_act_state_L,
			e3d_act_state_U
		};

		//////////////////////////////////////////////////////////////////////////
		interface I3DActOptions : public IRefObject
		{
			virtual e3d_act_case get_Activate() const = 0;
			virtual void set_Activate(e3d_act_case act) = 0;
			virtual e3d_act_case get_Deactivate() const = 0;
			virtual void set_Deactivate(e3d_act_case deact) = 0;
			virtual e3d_act_state get_ActState() const = 0;
			virtual void set_ActState(e3d_act_state st) = 0;
			virtual e3d_act_state get_DeactState() const = 0;
			virtual void set_DeactState(e3d_act_state st) = 0;
			virtual sx_bool get_ShowWindowed() const = 0;
			virtual void set_ShowWindowed(sx_bool bShow) = 0;
			virtual IRawProps* GetWindowProps() const = 0;
			virtual sx_bool get_ShowTransparent() const = 0;
			virtual void set_ShowTransparent(sx_bool bShow) = 0;
			virtual sx_bool get_ShowToolbar() const = 0;
			virtual void set_ShowToolbar(sx_bool bShow) = 0;
			virtual sx_bool get_ShowExtUI() const = 0;
			virtual void set_ShowExtUI(sx_bool bShow) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPrintersMark : public IAnnot { };

		//////////////////////////////////////////////////////////////////////////
		interface ITrapNetwork : public IAnnot { };

		//////////////////////////////////////////////////////////////////////////
		interface IRedact : public IMarkupAnnot
		{
			virtual Graphics::IRegion* get_Region() const = 0;
			virtual void SetRegion(Graphics::IRegion *pReg) = 0;

			virtual const sx_color* GetBackColor() const = 0;
			virtual void SetBackColor(const sx_color &c) = 0;
			virtual const sx_color* GetBorderColor() const = 0;
			virtual void SetBorderColor(const sx_color &c) = 0;
			virtual const sx_color* GetRedactColor() const = 0;
			virtual void SetRedactColor(const sx_color &c) = 0;

			virtual ICompositeImage* GetOverlay() const = 0;
			virtual void SetOverlay(ICompositeImage *pImage) = 0;
			virtual IStr* get_TextOverlay() const = 0;
			virtual void set_TextOverlay(sx_str sText) = 0;
			virtual sx_bool get_TextRepeat() const = 0;
			virtual void set_TextRepeat(sx_bool b) = 0;
			virtual IFont* get_TextFont() = 0;
			virtual void set_TextFont(IFont* pFont) = 0;
			virtual sx_uint get_TextSize() const = 0;
			virtual void set_TextSize(const sx_uint s) = 0;
			virtual text_alignment get_TextAlignment() const = 0;
			virtual void set_TextAlignment(text_alignment align) = 0;
			virtual const sx_color* GetTextColor() const = 0;
			virtual void SetTextColor(const sx_color& c) = 0;

			virtual void ResetAppearance() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IProjection : public IMarkupAnnot { };

		//////////////////////////////////////////////////////////////////////////
		// InReplyTo
		interface IReply : public IMarkupAnnot
		{
			virtual IAnnot* get_ReplyTo() = 0;
			virtual sx_bool get_Grouped() const = 0;
			virtual void set_Grouped(sx_bool bGr) = 0;
		};

		enum review_state
		{
			review_state_none,
			review_state_accepted,
			review_state_rejected,
			review_state_cancelled,
			review_state_completed
		};

		//////////////////////////////////////////////////////////////////////////
		// IReplies
		interface IReplies
		{
			// replies
			virtual IReply* GetReply(IAnnot* pAnnot, IReply* pPrevReply = nullptr) = 0;
			virtual IReply* AddReply(IAnnot* pAnnot) = 0;
			virtual void Remove(IReply* pReply) = 0;
			virtual void Clear(IAnnot* pAnnot) = 0;
			// marked state
			virtual sx_bool GetMarkedState(IAnnot* pAnnot, sx_str sAuthor) = 0;
			virtual void SetMarkedState(IAnnot* pAnnot, sx_str sAuthor, sx_bool bMarked) = 0;
			// review state
			virtual review_state GetReviewState(IAnnot* pAnnot, sx_str sAuthor) = 0;
			virtual void SetReviewState(IAnnot* pAnnot, sx_str sAuthor, review_state State) = 0;
			virtual IStr* GetNextAuthor(IAnnot *pAnnot, sx_str sPrevAuthor = nullptr) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// annotations
		interface IAnnots
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;
			virtual IReplies* get_Replies() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IAnnot* Item(sx_uint uIndex) = 0;
			virtual IAnnot* FindItem(const sx_point &p, sx_flags mask, IAnnot* pFromAnnot = nullptr) = 0;
			virtual IAnnot* InsertNew(annot_type Type, IAnnot* pBeforeAnnot = nullptr) = 0;
			virtual IWidget* InsertNewWidget(IFormField* pField, IAnnot* pBeforeAnnot = nullptr) = 0;
			virtual void Insert(IAnnot* pAnnot, IAnnot* pBeforeAnnot = nullptr) = 0;
			virtual void Move(IAnnot* pAnnot, IAnnot* pBeforeAnnot = nullptr) = 0;
			virtual void Remove(IAnnot* pAnnot) = 0;
			virtual void RemoveAll(sx_flags mask) = 0;
			virtual IAnnot* Copy(IAnnot* pAnnot, IAnnot* pBeforeAnnot = nullptr) = 0;
			virtual IWidget* CopyWidget(IWidget* pWidget, IFormField* pField, IAnnot* pBeforeAnnot = nullptr) = 0;			
			virtual void ResetPopup(IAnnot* pMarkupAnnot, sx_bool bClear = false) = 0;
			virtual sx_uint IndexOf(IAnnot* pAnnot) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// annotation design
		enum annot_state
		{
			annot_state_normal,
			annot_state_pressed,
			annot_state_rollover
		};

		interface IIconProperties;
		interface IAnnotDesign : public IRefObject
		{
			virtual IFont* get_TextFont() = 0;
			virtual void set_TextFont(IFont* pFont) = 0;
			virtual sx_uint get_TextSize() const = 0;
			virtual void set_TextSize(const sx_uint s) = 0;
			virtual const sx_color* GetTextColor() const = 0;
			virtual void SetTextColor(const sx_color& c) = 0;

			virtual const sx_color* GetBackColor() const = 0;
			virtual void SetBackColor(const sx_color& c) = 0;

			virtual sx_double get_BorderWidth() const = 0;
			virtual void set_BorderWidth(sx_double dWidth) = 0;
			virtual border_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(border_style eBS) = 0;
			virtual const sx_color* GetBorderColor() const = 0;
			virtual void SetBorderColor(const sx_color& c) = 0;

			virtual sx_rotation_angle get_Rotation() const = 0;
			virtual void set_Rotation(sx_rotation_angle angle) = 0;

			// buttons specific
			virtual IIconProperties* get_IconProperties() = 0;
			virtual link_highlight get_Highlight() const = 0;
			virtual void set_Highlight(link_highlight eHL) = 0;
			virtual IStr* GetLabel(annot_state st = annot_state_normal) const = 0;
			virtual void SetLabel(sx_str sLabel, annot_state st = annot_state_normal) = 0;
			virtual IImageObject* GetIcon(annot_state st = annot_state_normal) const = 0;
			virtual void SetIcon(IImageObject* pIcon, annot_state st = annot_state_normal) = 0;
		};

		enum icon_place
		{
			icon_place_none,
			icon_place_without_label,
			icon_place_above_label,
			icon_place_below_label,
			icon_place_left_of_label,
			icon_place_right_of_label,
			icon_place_under_label
		};

		enum icon_scale
		{
			icon_scale_fit,
			icon_scale_shrink,
			icon_scale_expand,
			icon_scale_never
		};

		interface IIconProperties
		{
			virtual icon_place get_Placement() const = 0;
			virtual void set_Placement(icon_place pl) = 0;
			virtual icon_scale get_Scale() const = 0;
			virtual void set_Scale(icon_scale sc) = 0;
			virtual sx_bool get_PropScale() const = 0;
			virtual void set_PropScale(sx_bool pr) = 0;
			virtual sx_double get_HorzOffset() const = 0;
			virtual void set_HorzOffset(const sx_double &dOffset) = 0;
			virtual sx_double get_VertOffset() const = 0;
			virtual void set_VertOffset(const sx_double &dOffset) = 0;
		};

		interface IBarcodeImage : public IRefObject
		{
			virtual Barcode::barcode_format get_Format() const = 0;
			virtual sx_double get_Resolution() const = 0;
			virtual void set_Resolution(sx_double dRes) = 0;
			virtual sx_double get_Width() const = 0;
			virtual void set_Width(sx_double dWidth) = 0;
			virtual sx_double get_Height() const = 0;
			virtual void set_Height(sx_double dHeight) = 0;
			virtual sx_uint get_XDistance() const = 0;
			virtual void set_XDistance(sx_uint Distance) = 0;
			virtual sx_uint get_YDistance() const = 0;
			virtual void set_YDistance(sx_uint Distance) = 0;
			virtual sx_uint get_ECC() const = 0;
			virtual void set_ECC(sx_uint uECC) = 0;
			virtual IStr* get_Caption() const = 0;
			virtual void set_Caption(sx_str sCaption) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// IRichText
		interface IRichText
		{
			virtual IStr* get_Content() const = 0;
			virtual void set_Content(sx_str sContent) = 0;
			virtual IStr* get_Style() const = 0; 
			virtual void set_Style(sx_str sStyle) = 0;
			virtual text_alignment get_Alignment() const = 0; 
			virtual void set_Alignment(text_alignment eAlign) = 0;
			virtual SX::XHTML::IRichText* GetRT() = 0;
			virtual void UpdateRT(SX::XHTML::IRichText *pRT) = 0;
		};


		interface IMediaCriteria;
		interface IMediaRendition : public IRefObject
		{
			virtual sx_bool get_Multi() const = 0;
			virtual IStr* get_Name() const = 0;
			virtual void  set_Name(sx_str sName) = 0;

			virtual IMediaCriteria* GetCriteria(sx_bool bBE, sx_bool bCreate = false) = 0;
		};

		interface IMediaClip;
		interface IMediaPlay;
		interface IMediaScreen;
		interface ISingleRendition : public IMediaRendition
		{
			virtual IMediaClip* GetClip(sx_bool bCreate = false, sx_bool bSection = false) = 0;
			virtual IMediaPlay* GetPlay(sx_bool bCreate = false) = 0;
			virtual IMediaScreen* GetScreen(sx_bool bCreate = false) = 0;
		};

		interface IMultiRendition : public IMediaRendition
		{
			virtual IMediaRendition* GetNextRendition(IMediaRendition* pRend = nullptr) = 0;
			virtual ISingleRendition* AddRendition() = 0;
			virtual void RemoveRendition(IMediaRendition* pRend = nullptr) = 0;
		};

		interface IMultiLangText : public IRefObject
		{
			virtual sx_uint get_Count() const = 0;
			virtual IStr* GetLang(sx_uint ind) const = 0;
			virtual IStr* GetText(sx_uint ind) const = 0;
			virtual void SetText(sx_str sLang, sx_str sText) = 0;
			virtual void RemoveText(sx_str sLang) = 0;
		};

		interface IMediaClip : public IRefObject
		{
			virtual sx_bool get_Section() const = 0;
			virtual IStr* get_Name() const = 0;
			virtual void  set_Name(sx_str sName) = 0;
			virtual IMultiLangText* GetMLText(sx_bool bCreate = false) = 0;
		};

		enum media_permission
		{
			media_permission_never,
			media_permission_extract,
			media_permission_access,
			media_permission_always
		};

		interface IMediaClipRange;
		interface IMediaData : public IMediaClip
		{
			virtual media_permission get_PermTempFile() const = 0;
			virtual void set_PermTempFile(media_permission perm) = 0;

			virtual IFileRef* GetDataRef() const = 0;
			virtual void SetDataRef(IFileRef *pFileRef) = 0;
			virtual IStr* GetDataMIME() const = 0;
			virtual void SetDataMIME(sx_str MIME) = 0;
			virtual ICompositeImage* GetData() = 0;
			virtual void SetData(ICompositeImage* image) = 0;
			virtual IStr* GetBaseURL(sx_bool bBE) const = 0;
			virtual void SetBaseURL(sx_bool bBE, sx_str URL) = 0;
		};

		interface IMediaSection : public IMediaClip
		{
			virtual IMediaClip* GetClip(sx_bool bCreate = false, sx_bool bSection = false) = 0;
			virtual IMediaClipRange* GetRange(sx_bool bBE, sx_bool bCreate = false) = 0;
		};

		enum rendition_mon
		{
			rendition_mon_largest_sect,
			rendition_mon_smallest_sect,
			rendition_mon_primary,
			rendition_mon_greatest_depth,
			rendition_mon_greatest_size,
			rendition_mon_greatest_height,
			rendition_mon_greatest_width,
			rendition_mon_unknown
		};

		interface IMediaCriteria : public IRefObject
		{
			virtual sx_bool get_HearAudioDesc() const = 0;
			virtual void set_HearAudioDesc(sx_bool set) = 0;
			virtual sx_bool get_SeeCaptions() const = 0;
			virtual void set_SeeCaptions(sx_bool set) = 0;
			virtual sx_bool get_HearOverdubs() const = 0;
			virtual void set_HearOverdubs(sx_bool set) = 0;
			virtual sx_bool get_SeeSubtitles() const = 0;
			virtual void set_SeeSubtitles(sx_bool set) = 0;
			virtual sx_uint get_MinBitrate() const = 0;
			virtual void set_MinBitrate(sx_uint set) = 0;
			virtual sx_uint get_MinScreenDepth() const = 0;
			virtual void set_MinScreenDepth(sx_uint set) = 0;
			virtual rendition_mon get_MinScreenDepthMon() const = 0;
			virtual void set_MinScreenDepthMon(rendition_mon set) = 0;
			virtual sx_size get_MinScreenSize() const = 0;
			virtual void set_MinScreenSize(sx_size set) = 0;
			virtual rendition_mon get_MinScreenSizeMon() const = 0;
			virtual void set_MinScreenSizeMon(rendition_mon set) = 0;
		};

		enum rendition_fit
		{
			rendition_fit_meet = 1,
			rendition_fit_slice,
			rendition_fit_fill,
			rendition_fit_scroll,
			rendition_fit_hidden
		};

		interface IMediaPlaySettings : public IRefObject
		{
			virtual sx_uint get_Volume() const = 0;
			virtual void set_Volume(sx_uint set) = 0;
			virtual sx_bool get_ShowControls() const = 0;
			virtual void set_ShowControls(sx_bool set) = 0;
			virtual rendition_fit get_FitMedia() const = 0;
			virtual void set_FitMedia(rendition_fit set) = 0;
			virtual sx_bool get_DurationInfinity() const = 0;
			virtual void set_DurationInfinity(sx_bool set) = 0;
			virtual sx_double get_DurationTimespan() const = 0; // 0.0 - intrinsic duration
			virtual void set_DurationTimespan(sx_double set) = 0;
			virtual sx_bool get_AutoPlay() const = 0;
			virtual void set_AutoPlay(sx_bool set) = 0;
			virtual sx_uint get_Repeat() const = 0;
			virtual void set_Repeat(sx_uint set) = 0;
		};

		interface IMediaPlay : public IRefObject
		{
			virtual IMediaPlaySettings* GetSettings(sx_bool bBE, sx_bool bCreate = false) = 0;
		};

		enum rendition_win
		{
			rendition_win_float,
			rendition_win_full_scr,
			rendition_win_hidden,
			rendition_win_annot_rect
		};

		interface IMediaScreenSettings;
		interface IMediaScreen : public IRefObject
		{
			virtual IMediaScreenSettings* GetSettings(sx_bool bBE, sx_bool bCreate = false) = 0;
		};

		interface IMediaWindow;
		interface IMediaScreenSettings : public IRefObject
		{
			virtual rendition_win get_Window() const = 0;
			virtual void set_Window(rendition_win set) = 0;
			virtual const sx_color* GetBackColor() const = 0; // inc opacity
			virtual void SetBackColor(const sx_color& c) = 0;
			virtual rendition_mon get_WindowMon() const = 0;
			virtual void set_WindowMon(rendition_mon set) = 0;
			virtual IMediaWindow* GetFloatingWindow(sx_bool bCreate = false) = 0;
		};

		enum rendition_winpos1
		{
			rendition_winpos1_docwin,
			rendition_winpos1_appwin,
			rendition_winpos1_desktop,
			rendition_winpos1_monitor
		};

		enum rendition_winpos2
		{
			rendition_winpos2_UL,
			rendition_winpos2_UC,
			rendition_winpos2_UR,
			rendition_winpos2_CL,
			rendition_winpos2_C,
			rendition_winpos2_CR,
			rendition_winpos2_LL,
			rendition_winpos2_LC,
			rendition_winpos2_LR
		};

		enum rendition_winpos3
		{
			rendition_winpos3_none,
			rendition_winpos3_resize,
			rendition_winpos3_non_viable
		};

		enum rendition_winopt
		{
			rendition_winopt_none,
			rendition_winopt_titlebar  = 0x001,
			rendition_winopt_title_controls = 0x002,
			rendition_winopt_prop_resize = 0x010,
			rendition_winopt_free_resize = 0x020
		};

		interface IMediaWindow : public IRefObject
		{
			virtual sx_size get_Size() const = 0;
			virtual void set_Size(sx_size set) = 0;
			virtual rendition_winpos1 get_RelTo() const = 0;
			virtual void set_RelTo(rendition_winpos1 set) = 0;
			virtual rendition_winpos2 get_RelAlign() const = 0;
			virtual void set_RelAlign(rendition_winpos2 set) = 0;
			virtual rendition_winpos3 get_OffScreen() const = 0;
			virtual void set_OffScreen(rendition_winpos3 set) = 0;
			virtual sx_flags get_Options() const = 0;
			virtual void set_Options(sx_flags set) = 0;
			virtual IMultiLangText* GetMLText(sx_bool bCreate = false) = 0;
		};

	} // namespace PDF
} // namespace SX