// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

namespace SX
{
	namespace PDF
	{
		interface IPage;
		interface IArticlePage;
		interface IArticleProperties;

		//////////////////////////////////////////////////////////////////////////
		interface IArticle : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IArticleProperties* get_Properties() = 0;
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IArticlePage* Item(sx_uint idx) = 0;
			virtual IArticlePage* Add(IPage* pPage) = 0;
			virtual IArticlePage* Insert(IPage* pPage, sx_uint uTo) = 0;
			virtual void Move(sx_uint uFrom, sx_uint uTo) = 0;
			virtual void Remove(sx_uint idx) = 0;
			virtual sx_uint IndexOf(IArticlePage* pPage) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IArticlePage : IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IPage* get_Page() const = 0;
			virtual IArticle* get_Article() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void GetCropBox(sx_rect& box) const = 0;
			virtual void SetCropBox(const sx_rect& box) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IArticleProperties
		{
			virtual sx_uint get_PropCount() const = 0;
			virtual IStr* GetPropName(sx_uint idx) const = 0;
			virtual IStr* GetProp(sx_str sName) const = 0;
			virtual void SetProp(sx_str sName, sx_str sValue) = 0;

			virtual void GetCDate(sx_time &tm) const = 0;
			virtual void GetMDate(sx_time &tm) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IArticles
		{
			virtual sx_uint get_Count() const = 0;

			virtual IArticle* Item(sx_uint idx) const = 0;
			virtual IArticle* Add(sx_str sTitle) = 0;
			virtual void Remove(sx_uint idx) = 0;
		};

	} // namespace PDF
} // namespace SX
