// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

#include "../basic.h"

namespace SX
{
	namespace PDF
	{
		interface IPage;
		interface ILogObject;

		enum DestDisplay
		{
			DestDisplay_Undefined = 0,
			DestDisplay_XYZ,
			DestDisplay_FitP,
			DestDisplay_FitH,
			DestDisplay_FitV, 
			DestDisplay_FitR,
			DestDisplay_FitB,
			DestDisplay_FitBH,
			DestDisplay_FitBV
		};

		object_type obj_type_dest = 0x19;

		enum offset_as_is
		{
			offset_as_is_none	= 0,
			offset_as_is_left	= 1,
			offset_as_is_bottom	= 2,
			offset_as_is_right	= 4,
			offset_as_is_top	= 8
		};

		//////////////////////////////////////////////////////////////////////////
		// link or outline destination 
		interface IDest : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			// local pages only, can throw IException
			virtual IPage* get_Page() = 0;
			virtual void set_Page(IPage* pPage) = 0;
			// for remote go-to only, can throw IException
			virtual sx_uint get_PageIndex() = 0;
			virtual void set_PageIndex(sx_uint Page) = 0;
			// structure destination
			virtual ILogObject* get_LogStructItem() = 0;
			virtual void set_LogStructItem(ILogObject* pItem) = 0;
			virtual IStr* get_LogStructIdent() = 0;
			virtual void set_LogStructIdent(sx_str sIdent) = 0;
			// common
			virtual DestDisplay get_DestDisplay() const = 0;
			virtual void set_DestDisplay(DestDisplay Display) = 0;
			virtual sx_double get_Zoom() = 0;
			virtual void set_Zoom(sx_double Zoom) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void GetOffsets(sx_double &left, sx_double &top, sx_double &right, sx_double &bottom, sx_flags& as_is) const = 0;
			virtual void SetOffsets(sx_double left, sx_double top = 0., sx_double right = 0., sx_double bottom = 0., sx_flags as_is = 0) = 0;
		};

		interface INamedDests
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IStr*  GetName(sx_uint uIndex) = 0; // can throw IException
			virtual IStr* GetNextName(sx_bool bStart = true) = 0;
			virtual IDest* Get(sx_str sName) = 0; // can throw IException
			virtual IDest* Add(sx_str sName) = 0; // can throw IException
			virtual void   Remove(sx_str sName) = 0; // can throw IException
			virtual IStr* FindAtPage(IPage* pPage, sx_str sPrev = nullptr) = 0;
		};
	} // namespace PDF
} // namespace SX
