// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../document.h"
#include "pdfa.h"

namespace SX
{
	namespace PDF
	{
		interface IDocProperties;
		interface IDocResources;
		interface IPages;
		interface IOutlineItem;
		interface INamedDests;
		interface IFormFields;
		interface ILayers;
		interface IEmbeddedFiles;
		interface IPortfolio;
		interface IStatusPDFA;
		interface IArticles;
		interface IOptimizer;

		typedef const sx_uint object_type;
		object_type
			obj_type_pdf =		0x0A,
			obj_type_pdf_doc =	0x0B,
			obj_type_part_document = 0x1B;

		enum linearized_part
		{
			linearized_part_def			= 0x0,
			linearized_part_length		= 0x1,
			linearized_part_file_len	= 0x2
		};

		//////////////////////////////////////////////////////////////////////////
		// PDF document
		interface IDocument : public IBaseDocument
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_str			get_FilePath() const = 0;
			virtual IDocProperties*	get_Properties() = 0;
			virtual IPages*			get_Pages() = 0;
			virtual IOutlineItem*	get_Outline() = 0;
			virtual INamedDests*	get_NamedDests() = 0;
			virtual IFormFields*	get_FormFields() = 0;
			virtual ILayers*		get_Layers() = 0;
			virtual IEmbeddedFiles*	get_Files() = 0;
			virtual IDocResources*	get_Resources() = 0;
			virtual IArticles*		get_Articles() = 0;
			virtual IOptimizer*		get_Optimizer() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void OpenStream(ISequentialStream* pStream) = 0;
			virtual void OpenFile(sx_str sFilePath, sx_uint MaxCache = 0) = 0;
			virtual void OpenFileCopy( sx_str sFilePath, sx_uint MaxCache ) = 0;
			virtual sx_qword OpenLinearizedPart( ISequentialStream* pStream, sx_flags options = linearized_part_def ) = 0;
			virtual void SelectRevision(sx_uint nRevIndex) = 0;
			virtual sx_qword SaveAsStream(ISequentialStream* pStream) = 0;
			virtual void SaveAsFile(sx_str sFilePath) = 0;
			virtual sx_bool CanSaveRevision() = 0;
			virtual sx_qword SaveRevisionAsStream(ISequentialStream* pStream) = 0;
			virtual void SaveRevisionAsFile(sx_str sFilePath) = 0;
			virtual void StartCombine(IBaseDocument* pSourceDoc, sx_flags CombineOptions = 0) = 0;
			virtual void CombinePage(sx_uint uFrom, sx_uint uTo, sx_flags CombineOptions = 0) = 0; 
			virtual void CombineAllPages(sx_uint uTo, sx_flags CombineOptions = 0, ICombineStatus* pStatus = nullptr) = 0;
			virtual void EndCombine() = 0;
			virtual void ConformToPDFA(pdfa version, sx_flags opt = 0, IStatusPDFA* pStat = nullptr) = 0;
			virtual IPortfolio* GetPortfolio(sx_bool bCreate = false) = 0;
			virtual void ClearPortfolio() = 0;
			virtual IOutlineItem* FindOutlineItem(sx_uint page) = 0;
		};
	} // namespace PDF
} // namespace SX
