// ********************************************************************
// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../document.h"
#include "../misc.h"

namespace SX
{
	namespace EInvoice
	{
		enum einv_standard
		{
			einv_standard_unknown	= 0,
			einv_standard_zugferd	= 1,
			einv_standard_zugferd20	= 2,
			einv_standard_facturx	= 3,
			einv_standard_xrechnung	= 4
		};

		enum einv_profile
		{
			einv_profile_unknown  = 0,
			einv_profile_minimum  = 1,
			einv_profile_basic_wl = 2,
			einv_profile_basic    = 3,
			einv_profile_EN16931  = 4,
			einv_profile_extended = 5
		};
	
		enum einv_options
		{
			einv_options_def 		= 0,
			einv_options_check_xml	= 0x001,
			einv_options_check_pdfa	= 0x002,
			einv_options_pdfa_3b	= 0x010,
			einv_options_zugferd20	= 0x100,
			einv_options_facturx	= 0x200
		};

		enum einv_status
		{
			einv_status_not_pdf			= 1,
			einv_status_enc_pdf			= 2,
			einv_status_not_pdfa		= 3,
			einv_status_no_meta			= 4,
			einv_status_err_meta		= 5,
			einv_status_no_emb			= 6,
			einv_status_err_emb			= 7,
			einv_status_nonconform		= 8,

			einv_status_not_xml			= 10,
			einv_status_err_xml_ns		= 11,
			einv_status_err_xml_root	= 12,
			einv_status_err_unk_prof	= 13,
			einv_status_err_xml_inv	= 14,

			einv_status_valid			= 100
		};

		const object_type obj_type_einvoice_document = 0x36;
		
		interface IInvoiceData;
		interface IPage;

		interface IInvoiceDocument : public IBaseDocument
		{
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str s) = 0;
			virtual IStr* get_Subject() const = 0;
			virtual void set_Subject(sx_str s) = 0;
			virtual IStr* get_Author() const = 0;
			virtual void set_Author(sx_str s) = 0;
			virtual IStr* get_Creator() const = 0;
			virtual void set_Creator(sx_str s) = 0;
			virtual einv_standard get_Standard() const = 0;
			virtual einv_profile get_Profile() const = 0;

			virtual einv_status Open(ISequentialStream* pStream, sx_flags opt = einv_options_def) = 0;
			virtual void Create(ISequentialStream* pInvoice, sx_flags opt = einv_options_def) = 0;
			virtual IPage* CreatePage(sx_double w, sx_double h) = 0;
			virtual void CreateFromPDF(ISequentialStream* pInvoice, ISequentialStream* pPDF, sx_flags opt = einv_options_def) = 0;
			virtual void Save(ISequentialStream* pStream) = 0;

			virtual void GetInvoice(ISequentialStream* pStream) const = 0;
			virtual void GetAttachment(ISequentialStream* pStream, sx_str pFilename) const = 0;
			virtual void SetAttachment(ISequentialStream* pStream, sx_str pFilename, sx_str pMime) = 0;

			virtual IInvoiceData* CreateInvoice() = 0;
		};

		interface ITextStyle : public IRefObject { };
		interface ILineStyle : public IRefObject { };
		interface IRasterImage : public IRefObject { };
		interface IPage : public IRefObject
		{
			virtual ITextStyle* CreateTextStyle(sx_str family, sx_double sz, sx_uint weight = 400, sx_bool italic = false) = 0;
			virtual ITextStyle* CreateTextStyle(sx_color& color, sx_str family, sx_double sz, sx_uint weight = 400, sx_bool italic = false) = 0;
			virtual ILineStyle* CreateLineStyle(sx_double w) = 0;
			virtual ILineStyle* CreateLineStyle(sx_color& color, sx_double w) = 0;
			virtual IRasterImage* LoadImage(ISequentialStream* pStream) = 0;
			virtual void PlaceText(sx_double x, sx_double y, sx_str text, ITextStyle* pStyle, sx_flags opt = 0) = 0;
			virtual void PlaceLine(sx_double x1, sx_double y1, sx_double x2, sx_double y2, ILineStyle* pStyle) = 0;
			virtual void PlaceImage(sx_double x, sx_double y, IRasterImage* pImage) = 0;
			virtual void PlaceImage(sx_double x, sx_double y, sx_double w, sx_double h, IRasterImage* pImage) = 0;
		};

		interface IInvoiceData : public IRefObject { };
	} // namespace EInvoice
} // namespace SX
