// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

namespace SX
{
	namespace PDF
	{
		enum pdf_event
		{
			pdf_event_none				= 0x000000,
			pdf_event_doc_open			= 0x000001,
			pdf_event_doc_close			= 0x000002,
			pdf_event_doc_before_save	= 0x000004,
			pdf_event_doc_after_save	= 0x000008,
			pdf_event_doc_before_print	= 0x000010, 
			pdf_event_doc_after_print	= 0x000020,
			pdf_event_page_open			= 0x000040,
			pdf_event_page_close		= 0x000080,
			pdf_event_field_keystroke	= 0x000100,
			pdf_event_field_format		= 0x000200,
			pdf_event_field_validate	= 0x000400,
			pdf_event_field_calculate	= 0x000800,
			pdf_event_annot_mouse_up	= 0x001000,
			pdf_event_annot_mouse_down	= 0x002000,
			pdf_event_annot_mouse_enter	= 0x004000,
			pdf_event_annot_mouse_exit	= 0x008000,
			pdf_event_annot_get_focus	= 0x010000,
			pdf_event_annot_lost_focus	= 0x020000,
			pdf_event_annot_open_page	= 0x040000,
			pdf_event_annot_close_page	= 0x080000,
			pdf_event_annot_show_page	= 0x100000,
			pdf_event_annot_hide_page	= 0x200000
		};

		interface IPDFEvent
		{
			virtual pdf_event get_Type() const = 0;			
		};

		enum pdf_action
		{
			pdf_action_none				= 0x000000,
			pdf_action_goto				= 0x000001,
			pdf_action_goto_remote		= 0x000002,
			pdf_action_goto_embedded	= 0x000004,
			pdf_action_goto_thread		= 0x000008,
			pdf_action_goto_URI			= 0x000010,
			pdf_action_launch			= 0x000020,
			pdf_action_play_sound		= 0x000040,
			pdf_action_play_movie		= 0x000080,
			pdf_action_hide_annot		= 0x000100,
			pdf_action_exec_named		= 0x000200,
			pdf_action_submit_form		= 0x000400,
			pdf_action_reset_form		= 0x000800,
			pdf_action_import_form		= 0x001000,
			pdf_action_exec_jscript		= 0x002000,
			pdf_action_set_layer_state	= 0x004000,
			pdf_action_rendition_comm	= 0x008000,
			pdf_action_transition		= 0x010000,
			pdf_action_goto_3D_view		= 0x020000,
			pdf_action_richmedia_comm	= 0x040000,
			pdf_action_goto_doc_part	= 0x080000
		};

		interface IPDFAction : public IRefObject
		{
			virtual pdf_action get_Type() const = 0;
		};

		interface IDest;

		interface IActionGoTo : public IPDFAction
		{
			virtual IStr*  get_NamedDest() const = 0;
			virtual void   set_NamedDest(sx_str sTitle) = 0;
			virtual IDest* get_Dest() const = 0;
			virtual IDest* ResetDest() = 0;
		};

		interface IFileRef;

		interface IActionGoToRemote : public IActionGoTo
		{
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef *pFileRef) = 0;
			virtual sx_bool get_NewWindow() const = 0;
			virtual void set_NewWindow(sx_bool bNew) = 0;
		};

		interface IEmbRelationship : public IRefObject
		{
			virtual sx_bool get_Child() const = 0;
			virtual void set_Child(sx_bool bChild) = 0;
			virtual IStr* get_EmbFileID() const = 0;
			virtual void  set_EmbFileID(sx_str sID) = 0;
			virtual sx_uint get_AttPageIndex() const = 0;
			virtual IStr* get_AttPage() const = 0;
			virtual void SetAttPage(sx_str sDestName, sx_uint idx) = 0;
			virtual sx_uint get_AttAnnotIndex() const = 0;
			virtual IStr* get_AttAnnot() const = 0;
			virtual void SetAttAnnot(sx_str sIdent, sx_uint idx) = 0;
			virtual IEmbRelationship* GetRelationship(sx_bool bCreate = false) = 0;
		};

		interface IActionGoToEmbedded : public IActionGoToRemote
		{
			virtual IEmbRelationship* GetRelationship(sx_bool bCreate = false) = 0;
		};

		interface IArticlePage;

		interface IActionGoToThread : public IPDFAction
		{
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef* pFileRef) = 0;
			virtual IArticlePage* get_Page() const = 0;
			virtual void set_Page(IArticlePage* pPage) = 0;
		};

		interface IActionGoToURI : public IPDFAction
		{
			virtual IStr* get_URI() const = 0;
			virtual void  set_URI(sx_str sURL) = 0;
			virtual sx_bool get_IsMap() const = 0;
			virtual void    set_IsMap(sx_bool bMap) = 0;
		};

		interface IActionLaunch : public IPDFAction
		{
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef *pFileRef) = 0;
			virtual sx_bool get_NewWindow() const = 0;
			virtual void set_NewWindow(sx_bool bNew) = 0;
			// Windows specific
			virtual IStr* get_WinAppPath() const = 0;
			virtual void  set_WinAppPath(sx_str sFullName) = 0;
			virtual IStr* get_WinAppDefDir() const = 0;
			virtual void  set_WinAppDefDir(sx_str sDir) = 0;
			virtual IStr* get_WinAppParams() const = 0;
			virtual void  set_WinAppParams(sx_str sParams) = 0;
			virtual sx_bool get_WinAppPrint() const = 0;
			virtual void  set_WinAppPrint(sx_bool bPrint) = 0;
		};

		interface ISoundStream;

		interface IActionPlaySound : public IPDFAction
		{
			virtual ISoundStream* get_SoundStream() = 0;
			virtual sx_double get_Volume() const = 0;
			virtual void   set_Volume(sx_double dblVol) = 0;
			virtual sx_bool get_Synchronous() const = 0;
			virtual void set_Synchronous(sx_bool bSync) = 0;
			virtual sx_bool get_Repeat() const = 0;
			virtual void set_Repeat(sx_bool bRep) = 0;
			virtual sx_bool get_Mix() const = 0;
			virtual void set_Mix(sx_bool bMix) = 0;
		};

		enum movie_operation
		{
			movie_operation_play,
			movie_operation_stop,
			movie_operation_pause,
			movie_operation_resume
		};
		interface IAnnot;

		interface IActionPlayMovie : public IPDFAction
		{
			virtual IAnnot* get_Annot() const = 0;
			virtual void    set_Annot(IAnnot* pAnnot) = 0;
			virtual IStr*   get_AnnotTitle() const = 0;
			virtual void    set_AnnotTitle(sx_str sTitle) = 0;
			virtual movie_operation get_Operation() const = 0;
			virtual void set_Operation(movie_operation eOp) = 0;
		};

		interface IActionHide : public IPDFAction
		{
			virtual sx_bool get_Hide() const = 0;
			virtual void    set_Hide(sx_bool bHide) = 0;

			virtual sx_uint get_Count() const = 0;
			virtual IStr*   GetField(sx_uint uIndex) const = 0;
			virtual IAnnot* GetAnnot(sx_uint uIndex) const = 0;
			virtual void AddField(sx_str FullName) = 0;
			virtual void AddAnnot(IAnnot* pAnnot) = 0;
			virtual void Remove(sx_uint uIndex) = 0;
		};

		interface IActionNamed : public IPDFAction
		{
			virtual IStr* get_Name() const = 0;
			virtual void  set_Name(sx_str sName) = 0;
		};

		interface IActionImportForm : public IPDFAction
		{
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef *pFileRef) = 0;
		};

		interface IFieldsList;

		interface IActionResetForm : public IPDFAction
		{
			virtual IFieldsList* get_FieldsList() = 0;
			virtual sx_bool get_ExcludeFields() const = 0;
			virtual void set_ExcludeFields(sx_bool bExc) = 0;
		};

		enum submit_format;

		interface IActionSubmitForm : public IPDFAction
		{
			virtual IFileRef* get_FileRef() const = 0;
			virtual void set_FileRef(IFileRef *pFileRef) = 0;
			virtual submit_format get_Format() const = 0;
			virtual void set_Format(submit_format eFormat) = 0;
			virtual sx_flags get_Options() const = 0;
			virtual void set_Options(sx_flags opt) = 0;
			virtual IFieldsList* get_FieldsList() = 0;
		};

		interface IActionJScript : public IPDFAction
		{
			virtual IStr* get_JScript() const = 0;
			virtual void  set_JScript(sx_str sJScript) = 0;
		};

		enum layer_state
		{
			layer_state_unknown = -1,
			layer_state_off = 0,
			layer_state_on,
			layer_state_toggle
		};
		interface ILayer;

		interface IActionLayerState : public IPDFAction
		{
			virtual sx_bool get_RadioButtonMode() const = 0;
			virtual void set_RadioButtonMode(sx_bool bRB) = 0;

			virtual sx_uint get_Count() const = 0;
			virtual ILayer* GetLayer(sx_uint uIndex) const = 0;
			virtual layer_state GetState(ILayer* pLayer) const = 0;
			virtual void SetState(ILayer* pLayer, layer_state st) = 0;
			virtual void Remove(sx_uint uIndex) = 0;
		};

		interface IPageTransition;

		interface IActionTransition : public IPDFAction
		{
			virtual IPageTransition* get_Transition() = 0;
		};

		enum e3d_view_pos
		{
			e3d_view_pos_unknown,
			e3d_view_pos_first,
			e3d_view_pos_next,
			e3d_view_pos_prev,
			e3d_view_pos_last
		};

		interface I3DArtwork;
		interface I3DView;
		interface I3DView;
		
		interface IActionGoTo3DView : public IPDFAction
		{
			virtual I3DArtwork* get_Artwork() const = 0;
			virtual void  set_Artwork(I3DArtwork* pArt) = 0;
			virtual I3DView* get_View() = 0;
			virtual void set_View(I3DView* pView) = 0;
			virtual e3d_view_pos get_ViewPos() = 0;
			virtual void set_ViewPos(e3d_view_pos pos) = 0;
		};

		enum rendition_op
		{
			rendition_op_play,
			rendition_op_stop,
			rendition_op_pause,
			rendition_op_resume,
			rendition_op_none = 32,
			rendition_op_unknown
		};

		interface IMediaRendition;
		interface IActionRendition : public IPDFAction
		{
			virtual rendition_op get_Operation() const = 0;
			virtual void set_Operation(rendition_op eOp) = 0;
			virtual IStr* get_JScript() const = 0;
			virtual void  set_JScript(sx_str sJScript) = 0;

			virtual IMediaRendition* GetRendition(sx_bool bCreate = false, sx_bool bMulti = false) = 0;
		};

		interface IActionRichMedia : public IPDFAction { };

		interface IDocPart;
		interface IActionGoToDocPart : public IPDFAction
		{
			virtual IDocPart* get_DocPart() = 0;
			virtual void set_DocPart(IDocPart *pDocPart) = 0;
		};


		interface IPDFActions : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IPDFAction* Item(sx_uint uIndex) = 0;
			virtual IPDFAction* Insert(pdf_action act, IPDFAction* pBeforeAction = nullptr) = 0;
			virtual void Remove(IPDFAction* pAction) = 0;
		};

	} // namespace PDF
} // namespace SX