// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../document.h"
#include "../misc.h"

namespace SX
{
	namespace Graphics { interface IBitmapData; }

	namespace FacturX
	{
		enum fx_profile
		{
			fx_profile_unknown  = 0,
			fx_profile_minimum  = 1,
			fx_profile_basic    = 2,
			fx_profile_extended = 3,
			fx_profile_EN16931  = 4
		};
	
		enum fx_options
		{
			fx_options_def = 0,
			fx_options_pdfa_3b  = 1
		};

		enum fx_status
		{
			fx_status_not_pdf	= 1,
			fx_status_enc_pdf	= 2,
			fx_status_no_meta	= 3,
			fx_status_err_meta	= 4,
			fx_status_not_pdfa	= 5,
			fx_status_err_emb	= 6,
			fx_status_not_fx	= 7,
			fx_status_dif_prof	= 8,
			fx_status_valid		= 10
		};

		const object_type obj_type_facturx_document = 0x36;

		interface IInvoiceDocument : public IBaseDocument
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str s) = 0;
			virtual IStr* get_Subject() const = 0;
			virtual void set_Subject(sx_str s) = 0;
			virtual IStr* get_Author() const = 0;
			virtual void set_Author(sx_str s) = 0;
			virtual IStr* get_Creator() const = 0;
			virtual void set_Creator(sx_str s) = 0;
			virtual sx_flags get_Options() const = 0;
			virtual void set_Options(sx_flags val) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual fx_status OpenStream(ISequentialStream* pStream, sx_bool ValidatePDFA = false) = 0;
			virtual fx_status OpenFile(sx_str sFilePath, sx_bool ValidatePDFA = false) = 0;
			virtual void CreateFromInvoiceAndPDF(ISequentialStream* pInvoice, ISequentialStream* pPDF) = 0;
			virtual void CreateFromInvoiceAndDoc(ISequentialStream* pInvoice, IBaseDocument* pDoc) = 0;
			virtual void AddPagesFromPDF(ISequentialStream* pStream) = 0;
			virtual void AddPagesFromDoc(IBaseDocument* pDoc) = 0;
			virtual void AddPageFromImage(Graphics::IBitmapData* pImage, sx_double pagew = 0., sx_double pageh = 0.) = 0;
			virtual void SaveAsStream(ISequentialStream* pStream) = 0;
			virtual void SaveAsFile(sx_str sFilePath) = 0;
		};
	} // namespace FacturX
} // namespace SX
