// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../document.h"
#include "events.h"

namespace SX
{
	namespace PDF
	{
		object_type obj_type_pdf_form_field		= 0x13,
			obj_type_pdf_field_button	= 0x14,
			obj_type_pdf_field_text		= 0x15,
			obj_type_pdf_field_choice	= 0x16,
			obj_type_pdf_field_signature= 0x17;

		//////////////////////////////////////////////////////////////////////////
		// form field 
		enum field_type
		{
			field_type_undefined	= 0,
			field_type_button		= 0x01,
			field_type_text			= 0x02,
			field_type_choice		= 0x04,
			field_type_signature	= 0x08
		};

		enum signature_status
		{
			signature_status_unknown,
			signature_status_unsigned,
			signature_status_invalid,
			signature_status_valid_for_revision,
			signature_status_valid,
			signature_status_certified,
			signature_status_certified_and_changed
		};

		enum timestamp_status
		{
			timestamp_status_unknown,
			timestamp_status_none,
			timestamp_status_invalid,
			timestamp_status_valid
		};

		enum signature_format
		{
			signature_format_unknown,
			signature_format_x509,
			signature_format_pkcs7,
			signature_format_pkcs7d,
			signature_format_CAdESd,
			signature_format_rfc3161
		};

		enum signature_digest
		{
			signature_digest_unknown,
			signature_digest_md5,
			signature_digest_sha1,
			signature_digest_sha256,
			signature_digest_sha384,
			signature_digest_sha512
		};

		enum button_type
		{
			button_type_pushbutton  = 1,
			button_type_radiobutton = 2,
			button_type_checkbox	= 3
		};

		enum field_activity
		{
			field_activity_widget_event,
			field_activity_change_value = 1,
			field_activity_format_value,
			field_activity_calculate
		};

		enum barcode_data_prep
		{
			barcode_data_prep_none,
			barcode_data_prep_flate
		};

		enum xfa_flags
		{
			xfa_flags_none = 0,
			xfa_flags_use = 0x1
		};

		interface IFormGroupNode;
		interface IWidget;
		interface IPDFActions;
		interface IRichText;
		interface IFonts;

		interface IFormNode : public IRefObject
		{
			virtual IStr* get_Name() const = 0;
			virtual IStr* get_FullName() const = 0;
			virtual IFormGroupNode* get_Parent() const = 0;
			virtual sx_bool IsField() const = 0;
		};

		interface IFormGroupNode : public IFormNode
		{
			virtual sx_uint get_SubNodes() const = 0;
			virtual IFormNode* GetSubNode(sx_uint ind) = 0;
			virtual sx_uint FindSubNode(sx_str sNode) = 0;
			virtual void AscendSubNode(sx_uint ind) = 0;
			virtual void DescendSubNode(sx_uint ind, sx_str sInterNode) = 0;
			virtual void MoveSubNode(sx_uint ind, IFormGroupNode* pNewParent) = 0;
		};

		interface IFormField : public IFormNode
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual field_type get_FieldType() const = 0;
			virtual IStr* get_DisplayName() const = 0;
			virtual void set_DisplayName(sx_str sName) = 0;
			virtual sx_bool get_Readonly() const = 0;
			virtual void set_Readonly(sx_bool b) = 0;
			virtual sx_bool get_Required() const = 0;
			virtual void set_Required(sx_bool b) = 0;
			virtual sx_bool get_Exportable() const = 0;
			virtual void set_Exportable(sx_bool b) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IWidget* FirstWidget() = 0;
			virtual IWidget* NextWidget(IWidget* pPrev) = 0; // can throw IException
			virtual void AttachWidget(IWidget* pW) = 0;
			virtual IPDFActions* GetEventActions(pdf_event evn, sx_bool bCreate = false) const = 0;    // can throw IException
			virtual void ResetValue() = 0;
		};

		interface IFieldButton : IFormField
		{
			virtual button_type get_ButtonType() const = 0;
			virtual void set_ButtonType(button_type eType) = 0;
			virtual sx_bool get_RadioNoToggle() const = 0;
			virtual void    set_RadioNoToggle(sx_bool bSet) = 0;
			virtual sx_bool get_RadioGroup() const = 0;
			virtual void    set_RadioGroup(sx_bool bSet) = 0;
			// radio buttons and checkboxes
			virtual sx_uint get_Buttons() const = 0;
			virtual sx_bool GetButtonState(sx_uint uBut) const = 0;
			virtual void SetButtonState(sx_uint uBut, sx_bool bCheck) = 0;
			virtual sx_bool GetButtonStateByDef(sx_uint uBut) const = 0;
			virtual void SetButtonStateByDef(sx_uint uBut, sx_bool bCheck) = 0;
			virtual IStr* GetButtonName(sx_uint uBut, sx_bool bExpName = false) const = 0;
			virtual void SetButtonName(sx_uint uBut, sx_str sName, sx_bool bExpName = false) = 0;
		};

		interface IFieldText : IFormField
		{
			virtual sx_bool get_Multiline() const = 0;
			virtual void    set_Multiline(sx_bool bSet) = 0;
			virtual sx_bool get_Password() const = 0;
			virtual void    set_Password(sx_bool bSet) = 0;
			virtual sx_bool get_FileSelect() const = 0;
			virtual void    set_FileSelect(sx_bool bSet) = 0;
			virtual sx_bool get_RichText() const = 0;
			virtual void    set_RichText(sx_bool bSet) = 0;
			virtual sx_bool get_NotSpellCheck() const = 0;
			virtual void    set_NotSpellCheck(sx_bool bSet) = 0;
			virtual sx_bool get_NonScrollable() const = 0;
			virtual void    set_NonScrollable(sx_bool bSet) = 0;
			virtual sx_bool get_SeparateChars() const = 0;
			virtual void    set_SeparateChars(sx_bool bSet) = 0;
			virtual sx_uint get_MaxLength() const = 0;
			virtual void    set_MaxLength(sx_uint uLen) = 0;
			virtual barcode_data_prep get_BarcodeDataPrep() const = 0;
			virtual void set_BarcodeDataPrep(barcode_data_prep ePrep) = 0;
			virtual IStr* get_Value() const = 0;
			virtual void set_Value(sx_str sValue) = 0;
			virtual IStr* get_ValueByDef() const = 0;
			virtual void set_ValueByDef(sx_str sValue) = 0;
			virtual IRichText* get_FormattedValue() const = 0;

			virtual sx_bool IsTimeField() const = 0;
			virtual sx_bool GetTimeValue(sx_time &t) = 0;
			virtual void SetTimeValue(const sx_time &t) = 0;
			virtual sx_bool IsNumField() const = 0;
			virtual sx_bool GetNumValue(sx_double &d) = 0;
			virtual void SetNumValue(const sx_double &d) = 0;
		};

		interface IFieldChoice : IFormField
		{
			virtual sx_bool get_ComboBox() const = 0;
			virtual void    set_ComboBox(sx_bool bSet) = 0;
			virtual sx_bool get_Editable() const = 0;
			virtual void    set_Editable(sx_bool bEdit) = 0;
			virtual sx_bool get_MultiSelect() const = 0;
			virtual void    set_MultiSelect(sx_bool bSet) = 0;
			virtual sx_bool get_Sort() const = 0;
			virtual void    set_Sort(sx_bool bSort) = 0;
			virtual sx_bool get_NoSpellCheck() const = 0;
			virtual void    set_NoSpellCheck(sx_bool bSet) = 0;
			virtual sx_bool get_CommitOnChange() const = 0;
			virtual void    set_CommitOnChange(sx_bool bSet) = 0;
			// choice items
			virtual sx_uint get_Items() const = 0;
			virtual sx_uint get_FirstVisibleItem() const = 0;
			virtual void set_FirstVisibleItem(sx_uint uItem) = 0;
			virtual sx_uint get_FirstCheckedItem() const = 0;

			virtual IStr* GetValueByDef() const = 0;
			virtual void SetValueByDef(sx_str name) = 0;
			virtual IStr* GetCustomValue() const = 0;
			virtual void SetCustomValue(sx_str name) = 0;
			virtual sx_bool GetItemState(sx_uint item) = 0;
			virtual void SetItemState(sx_uint item, sx_bool bCheck) = 0;
			virtual IStr* GetItemName(sx_uint item, sx_bool bExpName = false) const = 0;
			virtual void SetItemName(sx_uint item, sx_str name, sx_bool bExpName = false) = 0;
			virtual sx_uint AddItem(sx_str name) = 0;
			virtual void RemoveItem(sx_uint item) = 0;
			virtual void MoveItem(sx_uint from, sx_uint to) = 0;
		};

		interface ISignature;
		interface IRawProps;

		interface IFieldSignature : IFormField
		{
			virtual ISignature* GetSignature(sx_bool bForSign = false) = 0;
			virtual void Clear() = 0;
		};

		interface ISignatureReference : IRefObject
		{
			virtual IStr* get_Method() const = 0;
			virtual IStr* get_DigestMethod() const = 0;
			virtual void set_DigestMethod(sx_str method) = 0;
			virtual IRawProps* GetParams(sx_bool bCreate = false, sx_bool bClear = false) = 0;
		};

		interface ISignature : IRefObject
		{
			// signature properties
			virtual signature_status get_Status() const = 0;
			virtual IStr* get_Reason() const = 0;
			virtual void set_Reason(sx_str s) = 0;
			virtual IStr* get_Location() const = 0;
			virtual void set_Location(sx_str s) = 0;
			virtual IStr* get_ContactInfo() const = 0;
			virtual void set_ContactInfo(sx_str s) = 0;
			virtual signature_format get_Format() const = 0;
			virtual signature_digest get_Digest() const = 0;
			virtual IStr* get_Handler() const = 0;

			// ext properties
			virtual ISignatureReference* GetReference(sx_str sMethod, sx_bool bCreate = false, sx_bool bClear = false) = 0;
			virtual IRawProps* GetBuildProps(sx_bool bCreate = false, sx_bool bClear = false) = 0;

			// signer properties
			virtual IStr* get_Signer() const = 0;
			virtual ICryptoCert* get_SignerCert() const = 0;

			// sign time properties
			virtual timestamp_status get_TimestampStatus() const = 0;
			virtual ICryptoCert* get_TimestampAuthority() const = 0;
			virtual sx_bool GetSigningTime(sx_time& t) = 0;

			// prepare signing on save
			virtual void SignatureInquiry( ICryptoCert* pCert, ICryptoKey* pKey = nullptr, signature_format format = signature_format_pkcs7d, signature_digest digest = signature_digest_sha1, sx_bool bIncUpdate = true, sx_str szHandler = nullptr ) = 0;
			virtual void TimestampInquiry( sx_str sTSA_URL, sx_str sUsername = nullptr, sx_str sPassword = nullptr, sx_bool bHTTPS = false, signature_digest digest = signature_digest_sha1 ) = 0;
		};

		interface ISignatures
		{
			virtual sx_flags get_Flags() const = 0;
			virtual void set_Flags(sx_flags val) = 0;
			virtual IFormField* FirstSignature() = 0;
			virtual IFormField* NextSignature(IFormField* pPrev) = 0;
			virtual IFormField* AddSignature(sx_str sFullName) = 0;
		};

		interface ICalculationOrder
		{
			virtual IFormField* First() = 0;
			virtual IFormField* Next(IFormField* pPrev) = 0;
			virtual void Insert(IFormField* pField, IFormField* pBefore = nullptr) = 0;
			virtual void Remove(IFormField* pField) = 0;
		};

		interface IFieldsList;

		enum impdata_options
		{
			impdata_options_no_duplicates,
			impdata_options_gen_ident
		};

		enum submit_options
		{
			submit_options_exc_list 	= 0x001,
			submit_options_empty_fields	= 0x002,
			submit_options_get_method	= 0x004,
			submit_options_coordinates	= 0x008,
			submit_options_append_save	= 0x010,
			submit_options_annotations	= 0x020,
			submit_options_user_only	= 0x040,
			submit_options_no_path		= 0x080,
			submit_options_embed_form	= 0x100,
			submit_options_embed_stamp	= 0x10000,
			submit_options_embed_sound	= 0x20000
		};

		enum submit_format
		{
			submit_format_fdf = 1,
			submit_format_pdf,
			submit_format_xfdf,
			submit_format_html
		};

		interface IFormData
		{
			// import form data
			virtual void ImportData(ISequentialStream* pStream, sx_flags opt) = 0;
			// export form data
			virtual IFieldsList* GetExportFields(sx_bool bCreateTempList = false) = 0;
			virtual void SetExportFields(IFieldsList* pList = nullptr) = 0;

			virtual void ExportData(ISequentialStream* pStream, submit_format eFormat, sx_flags opt, sx_str sPDFPath = nullptr, sx_str sUsername = nullptr, sx_str sSubmitBut = nullptr) = 0;
			virtual void SubmitData(sx_str sURL, submit_format eFormat, sx_flags opt, sx_str sUsername = nullptr, sx_str sSubmitBut = nullptr) = 0;
		};

		interface IFormFields
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;
			virtual IFormGroupNode* get_Root() = 0;
			virtual sx_bool get_NeedAppearances() const = 0;
			virtual void set_NeedAppearances(sx_bool val) = 0;
			virtual sx_bool get_AutoCalculate() const = 0;
			virtual void set_AutoCalculate(sx_bool val) = 0;
			virtual sx_bool get_RaiseEvents() const = 0;
			virtual void set_RaiseEvents(sx_bool val) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IFormField* Item(sx_uint uIndex) = 0;
			virtual sx_uint Find(sx_str sFullName) = 0;
			virtual IFormField* Add(field_type Type, sx_str sFullName) = 0;
			virtual void Remove(sx_str sFullName) = 0;
			virtual void Rename(sx_str sFullName, sx_str sNewName) = 0;
			virtual IFormField* Copy(sx_str sSrcFullName, sx_str sDestFullName) = 0;
			virtual void VerifySignatures()  = 0;
			virtual ICalculationOrder* GetOrder() = 0;
			virtual void Calculate() = 0;
			virtual IFormData* GetDataProvider() = 0;
			virtual void ResetValues() = 0;
			virtual void ResetAppearances(sx_bool bClear = false) = 0;
			virtual IFonts* GetFonts() = 0;
			virtual sx_bool EmbedFullFonts(sx_bool bEmbPDFStd = false, sx_bool bEmbWinStd = false) = 0;
			// dynamic form
			virtual sx_bool GetXFA(sx_uint uPart, ISequentialStream* pStream) = 0;
			virtual void SetXFA(sx_uint uPart, ISequentialStream* pStream) = 0;
			virtual sx_flags GetXFAOptions() const = 0;
			virtual void SetXFAOptions(sx_flags opt) = 0;
		};

		interface IFieldsList : public IRefObject
		{
			virtual sx_uint get_Count() const = 0;
			virtual IFormNode* Item(sx_uint uIndex) const = 0;
			virtual void Add(IFormNode* pItem) = 0;
			virtual void Remove(sx_uint uIndex) = 0;
			virtual sx_bool IsBlank() const = 0;
			virtual void Reset(sx_bool bBlank = false) = 0;
		};

	} // namespace PDF

	enum emulate_js
	{
		emulate_js_ok,
		emulate_js_no_act,
		emulate_js_not_std,
		emulate_js_gt,
		emulate_js_gt_lt,
		emulate_js_lt,
		emulate_js_val,
		emulate_js_date
	};

	//////////////////////////////////////////////////////////////////////////
	interface IDocFieldEvent : public IDocEvent
	{
		virtual PDF::field_activity get_Activity() const = 0;
		virtual PDF::IFormField* get_Field() const = 0;
		virtual IStr* get_NewValue() const = 0;
		virtual sx_uint get_NewCheckedItem() const = 0;
		virtual sx_uint get_UncheckedItem() const = 0;
		virtual PDF::IPDFEvent* get_PDFEvent() const = 0;

		virtual void UseAsValue(sx_str value) = 0;
		virtual void Reject() = 0;
		virtual sx_flags EmulateJSAction() = 0;
		virtual IStr* GetExtInfo(sx_bool f) const = 0;
	};
} // namespace SX