// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../graphics.h"
#include "../document.h"

namespace SX
{
	namespace PDF
	{
		object_type
			obj_type_pdf_font  = 0x0F,
			obj_type_pdf_fonts = 0x10;

		enum FontType
		{
			FontTypeUndefined		= 0x0000,
			FontType1				= 0x0001,
			FontTypeTrueType		= 0x0002,
			FontType3				= 0x0004,
			FontType0_Type1			= 0x0008,
			FontType0_TrueType		= 0x0010
		};

		enum FontStatus
		{
			FontStatus_Embed		= 0x1000,
			FontStatus_NonEmbed		= 0x2000,
			FontStatus_WinStd		= 0x4000
		};

		enum FontSynthStyle
		{
			FontSynthStyleNone = 0,
			FontSynthStyleBold = 1,
			FontSynthStyleItalic = 2,
			FontSynthStyleBoldItalic = FontSynthStyleBold  | FontSynthStyleItalic 
		};

		//////////////////////////////////////////////////////////////////////////
		// font objects 
		interface IFont : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual FontType get_Type() = 0;
			virtual Graphics::FontFormat get_Format() = 0;
			virtual sx_bool get_Embedded() = 0;
			virtual sx_bool get_Subset() = 0;
			virtual sx_bool get_Symbolic() = 0;
			virtual FontSynthStyle get_SynthStyle() const = 0;
			virtual IStr* get_PostScriptName() const = 0;
			virtual IStr* get_Encoding() const = 0;
			virtual sx_bool get_EncodingExt() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_bool Embed() = 0;
			virtual sx_bool EmbedFullFont() = 0;
			virtual IStr* GetLocalFontName() const = 0;
		};

		interface IFonts : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IFont* Item(sx_uint uIndex) = 0; // can throw IException
		};
	} // namespace PDF

	//////////////////////////////////////////////////////////////////////////
	interface IDocCJKFontEvent : public IDocEvent
	{
		virtual sx_bool get_EncodingPackageAvailability() const = 0;
		virtual sx_bool get_LanguagePackAvailability() const = 0;
		virtual sx_str  get_Language() const = 0;
		virtual PDF::IFont* get_Font() const = 0;
	};
} // namespace SX
