// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../graphics.h"

namespace SX
{
	namespace Content{ interface IRichContent; interface IAutoText;}
	namespace PDF
	{
		interface IDocument;

		object_type 
			obj_type_pdf_image  = 0x0D,
			obj_type_pdf_images = 0x0E;

		enum ColorSpace
		{
			ColorSpace_Undefined = 0,
			ColorSpace_DeviceRGB,
			ColorSpace_DeviceGray,
			ColorSpace_DeviceCMYK,
			ColorSpace_CalRGB,
			ColorSpace_CalGray,
			ColorSpace_CalCMYK,
			ColorSpace_Lab,
			ColorSpace_ICCBased,
			ColorSpace_Indexed,
			ColorSpace_Separation,
			ColorSpace_DeviceN,
			ColorSpace_Pattern
		};

		enum RenderIntent
		{
			RenderIntent_Undefined = 0,
			RenderIntent_Perceptual,
			RenderIntent_RelativeColorimetric,
			RenderIntent_Saturation,
			RenderIntent_AbsoluteColorimetric
		};

		enum Filter
		{
			Filter_None = 0,
			Filter_ASCIIHex,
			Filter_ASCII85,
			Filter_LZW,
			Filter_Flate,
			Filter_RLE,
			Filter_CCITT,
			Filter_JBIG2,
			Filter_JPEG,
			Filter_JPX,
			Filter_Crypt = 0xF0,
			Filter_Undefined = 0xFF
		};


		interface ILayer;
		interface IFont;
		interface IAssociatedFiles;
		interface IRawProps;

		//////////////////////////////////////////////////////////////////////////
		// image objects 
		interface IImageObject : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_bool IsRaster() = 0;
			virtual ILayer* get_Layer() = 0;
			virtual void set_Layer(ILayer* pLayer) = 0;
			virtual sx_uint get_LSID() const = 0;
			virtual void set_LSID(sx_uint unit) = 0;
			virtual IAssociatedFiles* get_AssocFiles() = 0;
			virtual IImageObject *CopyTo(IDocument *pTargetDoc)  = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// raster image
		interface IRasterImage : public IImageObject
		{
			virtual sx_uint get_Width() const = 0;
			virtual sx_uint get_Height() const = 0;
			virtual sx_uint get_CompBits() const = 0;
			virtual ColorSpace get_CS() const = 0;
			virtual RenderIntent get_RI() const = 0;
			virtual Filter get_Compression() const = 0;
			virtual IRasterImage* get_AlphaMask() = 0;
			virtual void set_AlphaMask(IRasterImage* pMask) = 0;
			virtual sx_bool get_Interpolate() const = 0;
			virtual void set_Interpolate(sx_bool b) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual Graphics::IBitmapData* GetBitmapData() = 0;
			virtual void SetBitmapData(Graphics::IBitmapData* pBitmap, Filter Compression, sx_uint JpegQuality = 50) = 0;
			virtual void SetJpegData(ISequentialStream* pJpeg) = 0;
			virtual void Recompress(Filter Compression, Filter SecondaryCompression = Filter_None, sx_uint JpegQuality = 50) = 0;
			virtual Graphics::IBitmapData* GetBitmapWithAlpha() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// composite image 
		interface ICompositeImage : public IImageObject
		{
			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_bool GetBBox(sx_rect& box) const = 0;
			virtual void SetBBox(const sx_rect& box) = 0;
			virtual void GetMatrix(sx_matrix& m) const = 0;
			virtual void SetMatrix(const sx_matrix& m) = 0;
			virtual void CalcMatrix( sx_matrix& m, sx_double dZoom = 1.0, sx_rotation_angle angle = sx_rotation_angle_0, sx_double dx = 0., sx_double dy = 0., sx_uint uDPI = 72 ) const = 0;
			virtual sx_bool StartDraw(Graphics::IDrawingContext* ctx, sx_matrix *pTransform = nullptr,  sx_flags flags = 0) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& backColor, sx_matrix *pTransform = nullptr,  sx_flags flags = 0) = 0;
			virtual ICompositeImage* Clone(sx_flags opt = 0) = 0;
			virtual sx_bool UpdateAutoText(sx_uint PageIndex, SX::Content::IAutoText* pAutoText = nullptr) = 0;
			virtual SX::Content::IRichContent* GetRichContent(sx_flags opt = 0) = 0;
			virtual void VectorizeText(sx_flags fonts, IFont* pFont = nullptr) = 0;
			virtual IRawProps* GetPrivateProps(sx_str Name, sx_bool bCreate = false) = 0;					
			virtual void SetContainerLSID(sx_uint id) = 0;
		};

		interface IImages : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IImageObject* Item(sx_uint uIndex) = 0; // can throw IException
		};

		enum pict_find_opt{
			pict_find_opt_def = 0,			
			pict_find_opt_entire_img = 1
		};

		interface IPictures : public IImages
		{
			virtual void GetPageArea(sx_uint uIndex, sx_quad& quad) const = 0;
			virtual sx_uint FindAtPoint(const sx_point& pt, sx_uint StartFrom = 0) = 0;
			virtual sx_uint FindWithinArea(const sx_rect& rc, sx_uint StartFrom = 0, sx_flags options = pict_find_opt_def) = 0;
		};

	} // namespace PDF
} // namespace SX
