// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../misc.h"

namespace SX
{
	interface IApplication;
	namespace PDF { interface IDocument; interface IPDFEvent; interface IDocScripts; }

	namespace JS
	{
		enum js_event
		{
			js_event_wsh			= 1,
			js_event_wsh_enter,
			js_event_wsh_leave,
			js_event_wsh_error,
			js_event_invalidate,

			js_event_app			= 10,
			js_event_app_alert,
			js_event_app_beep,
			js_event_app_response,
			js_event_app_console,
			js_event_app_escript,

			js_event_doc			= 100,
			js_event_doc_save_as,
			js_event_doc_print,
			js_event_doc_mail
		};

		enum pdf_event_type
		{
			pdf_event_type_undef,
			pdf_event_type_batch,
			pdf_event_type_console,
			pdf_event_type_app,
			pdf_event_type_doc,
			pdf_event_type_page,
			pdf_event_type_external,
			pdf_event_type_bookmark,
			pdf_event_type_link,
			pdf_event_type_field,
			pdf_event_type_menu
		};

		enum pdf_event_name
		{
			pdf_event_name_undef,
			pdf_event_name_keystroke,
			pdf_event_name_validate,
			pdf_event_name_focus,
			pdf_event_name_blur,
			pdf_event_name_format,
			pdf_event_name_calculate,
			pdf_event_name_mouseup,
			pdf_event_name_mousedown,
			pdf_event_name_mouseenter,
			pdf_event_name_mouseexit,
			pdf_event_name_willprint,
			pdf_event_name_didprint,
			pdf_event_name_willsave,
			pdf_event_name_didsave,
			pdf_event_name_init,
			pdf_event_name_exec,
			pdf_event_name_open,
			pdf_event_name_close
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IJSEvent
		{
			virtual js_event get_Type() const = 0;
			virtual sx_bool get_Handled() const = 0;
			virtual void set_Handled(sx_bool bHandled = true) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSEvError : public IJSEvent
		{
			virtual sx_uint get_Line() = 0;
			virtual sx_uint get_Char() = 0;
			virtual sx_str  get_Source() = 0;

			virtual sx_uint get_Code() = 0;
			virtual sx_str  get_Desc() = 0;
		};


		//////////////////////////////////////////////////////////////////////////
		interface IJSEvAlert : public IJSEvent
		{
			virtual sx_str get_Msg() const = 0;
			virtual sx_str get_Title() const = 0;
			virtual sx_uint get_Icon() const = 0;
			virtual sx_uint get_ButtonGroup() const = 0;

			virtual sx_uint get_PressedButton() const = 0;
			virtual void set_PressedButton(sx_uint btn) = 0;
		};

		enum js_msg_beep
		{
			js_msg_beep_error		= 0,
			js_msg_beep_warning		= 1,
			js_msg_beep_question	= 2,
			js_msg_beep_status		= 3,
			js_msg_beep_default		= 4
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSEvBeep : public IJSEvent
		{
			virtual js_msg_beep get_SoundType() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSEvResponse : public IJSEvent
		{
			virtual sx_str get_Question() const = 0;
			virtual sx_str get_Title() const = 0;
			virtual sx_str get_Default() const = 0;
			virtual sx_bool get_Password() const = 0;
			virtual sx_str get_Label() const = 0;
			virtual sx_str get_Response() const = 0;
			virtual void set_Response(sx_str sResponse) = 0;
		};

		enum js_console_act
		{
			js_console_act_clear = 1,
			js_console_act_hide,
			js_console_act_msg,
			js_console_act_show
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSEvConsole : public IJSEvent
		{
			virtual  js_console_act get_Activity() const = 0;
			virtual  sx_str get_Message() const = 0;
		};

		enum js_escript_str
		{
			js_escript_str_ge = 1,
			js_escript_str_gt_lt,
			js_escript_str_le,
			js_escript_str_inv_mnt,
			js_escript_str_inv_date,
			js_escript_str_inv_date2,
			js_escript_str_inv_value,
			js_escript_str_month_info
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSEvEScript : public IJSEvent
		{
			virtual  js_escript_str get_IDS() const = 0;
			virtual void SetString(sx_str st) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSEvents
		{
			virtual sx_bool OnEvent(IJSEvent* pEvent) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSDoc
		{
			virtual sx_uint get_ID() const = 0;
			virtual sx_str get_Name() const = 0;
			virtual PDF::IDocument* GetNativeDoc() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSApp
		{
			// manage docs
			virtual IJSDoc* CreateDoc(PDF::IDocument* pDoc) = 0;
			virtual IJSDoc* GetDoc(PDF::IDocument* pDoc) = 0;
			virtual void ReleaseDoc(IJSDoc* doc) = 0;
			// execute scripts
			virtual void AttachScript(sx_str script, IJSDoc* pDoc, SX::PDF::IPDFEvent* pEvent = nullptr) = 0;
			virtual void AttachScript(PDF::IDocument* pDocScripts, IJSDoc* pDoc = nullptr) = 0;
			// process events
			virtual void SetEventHandler(IJSEvents* pEv) = 0;
			// manage persistent data (global object)
			virtual sx_str GetPersistentData() = 0;
			virtual void SetPersistentData(sx_str sData) = 0;
			// release object
			virtual void ReleaseApp() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IJSTools
		{
			virtual IJSApp* CreateApp(IApplication* pApp, LangCode2 lc2, sx_str prod, sx_str edition, sx_double vers = 3.01) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// library entry point
		typedef IJSTools* (__cdecl* JSLibraryEntryPoint)();
	} // namespace JS
} // namespace SX