// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"

namespace SX
{
	namespace PDF
	{
		object_type obj_type_pdf_layer = 0x11;

		interface ILayer : public IRefObject
		{
			virtual sx_bool get_Visible() const = 0;
			virtual sx_bool get_IsNamed() const = 0;
		};

		interface INamedLayer : public ILayer
		{
			virtual void set_Visible(sx_bool bVisible) = 0;
			virtual sx_bool get_Locked() const = 0;
			virtual void set_Locked(sx_bool bLocked) = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
			virtual IStr* get_Intent() const = 0;
			virtual void set_Intent(sx_str sName) = 0;
			virtual sx_bool get_DynVisibility() const = 0;
			virtual void set_DynVisibility(sx_bool bVis) = 0;
		};

		interface ILayersOrder
		{
			virtual INamedLayer* Find(INamedLayer* pPrev = nullptr, INamedLayer* pUnder = nullptr) = 0;
			virtual INamedLayer* FindFirst(sx_str sName, INamedLayer* pUnder = nullptr) = 0;
			virtual void Move(INamedLayer* pLayer, INamedLayer* pBefore = nullptr, INamedLayer* pUnder = nullptr) = 0;
			virtual void Remove(INamedLayer* pLayer) = 0;
			virtual void Clear(sx_bool bHideAll = false) = 0;
			virtual sx_bool IsOrdered() const = 0;
		};

		interface ILayers
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;
			virtual ILayersOrder* get_Order() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual INamedLayer* Item(sx_uint uIndex) = 0; // can throw IException
			virtual sx_uint Find(sx_str sName) = 0; // can throw IException
			virtual INamedLayer* Add(sx_str sName) = 0; // can throw IException
			virtual void Remove(sx_uint uIndex) = 0; // can throw IException
		};

	} // namespace PDF
} // namespace SX
