// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"

namespace SX
{
	namespace PDF
	{
		interface ILogObject;
		interface ILogRelation;
		interface ILogRoles;
		interface ILogClasses;
		interface ILogAttributes;
		interface IObjectFinder;

		enum raw_type;
		interface ILogicalStructure
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_bool get_MarkedContent() const = 0;
			virtual void set_MarkedContent(sx_bool b) = 0;
			virtual ILogObject* get_Root() const = 0;
			virtual ILogRoles* get_Roles() const = 0;
			virtual ILogClasses* get_Classes() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual ILogRelation* FindLogRelation(sx_uint id) = 0;
			virtual sx_uint AddLogRelation() = 0;
			virtual void Reset(sx_str sRootNode = nullptr, sx_bool bClear = false) = 0;
			virtual void Reset(sx_bool bMult, sx_uint ind = 0) = 0;
			virtual IObjectFinder* CreateObjectFinder() = 0;
		};

		enum log_struct_obj
		{
			log_struct_obj_mce = 1,
			log_struct_obj_mce_arr,
			log_struct_obj_mcr,
			log_struct_obj_pdf,
			log_struct_obj_mcid
		};

		interface ILogObject : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual log_struct_obj get_Type() const = 0;
		};

		interface ILogMCID : public ILogObject
		{
			virtual sx_uint get_MCID() const = 0;
		};

		interface ILogNode : public ILogObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IStr* get_Name() const = 0;
			virtual ILogNode* get_SuperNode() const = 0;
			virtual ILogObject* get_SubItem() const = 0;
			virtual sx_uint get_MCID() const = 0;
			virtual void set_MCID(sx_uint id) = 0;
			virtual IPage* get_Page() const = 0;
			virtual void set_Page(IPage* pPage) = 0;
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str Str) = 0;
			virtual IStr* get_Alt() const = 0;
			virtual void set_Alt(sx_str Str) = 0;
			virtual IStr* get_Lang() const = 0;
			virtual void set_Lang(sx_str Str) = 0;
			virtual ILogAttributes* get_Attributes() const = 0;
			virtual IAssociatedFiles* get_AssocFiles() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual ILogObject* SetSubNode(sx_str sNode, sx_bool bMult = false) = 0;
			virtual ILogObject* SetSubRef(sx_bool bObj = false) = 0;
			virtual void RemoveSubItem() = 0;
			virtual ILogObject* AscentSubNode(sx_uint ind) = 0;
			virtual sx_uint GetClasses() const = 0;
			virtual IStr* GetClassName(sx_uint uIndex) const = 0;
			virtual void AddClassName(sx_str sName) const = 0;
			virtual void RemoveClassName(sx_uint uIndex) const = 0;
			

		};

		interface ILogNodes : public ILogObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual ILogNode* get_SuperNode() const = 0;
			virtual ILogObject* Item(sx_uint uIndex) const = 0;
			virtual ILogObject* Add(sx_str sName) = 0;
			virtual ILogObject* Insert(sx_uint uTo, sx_str sName) = 0;
			virtual ILogObject* InsertRef(sx_uint uTo, sx_bool bObj = false) = 0;
			virtual void Move(sx_uint uFrom, sx_uint uTo) = 0;
			virtual void Remove(sx_uint uIndex) = 0;
		};

		interface ILogReference : public ILogObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IPage* get_Page() const = 0;
			virtual void set_Page(IPage* pPage) = 0;

			virtual IAnnot* get_Annot() const = 0;
			virtual void set_Annot(IAnnot* pAnn) = 0;

			virtual IImageObject* get_Image() const = 0;
			virtual void set_Image(IImageObject* pImg) = 0;

			virtual sx_uint get_MCID() const = 0;
			virtual void set_MCID(sx_uint id) = 0;
		};

		interface ILogObjReference : public ILogObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IPage* get_Page() const = 0;
			virtual void set_Page(IPage* pPage) = 0;

			virtual IAnnot* get_Annot() const = 0;
			virtual void set_Annot(IAnnot* pAnn) = 0;

			virtual IImageObject* get_Image() const = 0;
			virtual void set_Image(IImageObject* pImg) = 0;
		};

		interface ILogRelation : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_uint GetObjectsCount() const = 0;
			virtual ILogNode* Item(sx_uint uIndex) const = 0;
			virtual sx_uint Add(ILogNode* pNode) = 0;
			virtual void Set(sx_uint uIndex, ILogNode* pNode = nullptr) = 0;
			virtual ILogNode* Find(sx_uint mcid) const = 0;
		};
		
		interface ILogRoles
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IStr* Item(sx_uint uIndex) = 0;
			virtual IStr* Get(sx_str sRole) = 0;
			virtual void  Add(sx_str sRole, sx_str sStd) = 0;
			virtual void  Remove(sx_str sRole) = 0;
		};

		interface ILogClasses
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IStr* Item(sx_uint uIndex) = 0;
			virtual ILogAttributes* Get(sx_str sName) = 0;
			virtual ILogAttributes* Add(sx_str sName) = 0;
			virtual void Remove(sx_str sName) = 0;
		};

		interface ILogAttributes : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Blocks() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual raw_type GetPropType(sx_uint uBlock, sx_str name) const = 0;

			virtual sx_uint GetPropCount(sx_uint uBlock) const = 0;
			virtual IStr* GetPropName(sx_uint uBlock, sx_uint uIndex) = 0;
			virtual void  RemoveProp(sx_uint uBlock, sx_str sName) = 0;

			virtual IStr* GetString(sx_uint uBlock, sx_str name) const = 0;
			virtual void SetString(sx_uint uBlock, sx_str name, sx_str val, sx_bool bName = true) = 0;
			virtual sx_double GetNum(sx_uint uBlock, sx_str name) const = 0;
			virtual void SetNum(sx_uint uBlock, sx_str name, sx_double val) = 0;
			virtual sx_bool GetBool(sx_uint uBlock, sx_str name) const = 0;
			virtual void SetBool(sx_uint uBlock, sx_str name, sx_bool val) = 0;
			virtual IRawProps* GetArray(sx_uint uBlock, sx_str name, sx_bool bCreate = false) = 0;
		};

		interface IObjectFinder : public IRefObject
		{
			virtual void FindByName(sx_str name) = 0;
			virtual void FindByMCID(sx_uint mcid) = 0;
			virtual ILogObject* FindNext() = 0;
		};
	} // namespace PDF
} // namespace SX
