// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

namespace SX
{
	namespace PDF
	{
		enum measure_calc
		{
			measure_calc_x = 1,
			measure_calc_y,
			measure_calc_distance,
			measure_calc_perimeter,
			measure_calc_area,
			measure_calc_angle,
			measure_calc_line_slope,
			measure_calc_azimuth,
			measure_calc_latitude,
			measure_calc_longitude
		};

		interface IMeasureProvider : public Graphics::IPointsArray
		{
			virtual sx_bool IsEntity(measure_calc ent) const = 0;
			virtual sx_double GetEntity(measure_calc ent) const = 0;
			virtual void ClearPoints() = 0;
		};

		interface IRectilinearProvider : public IMeasureProvider
		{
			virtual sx_bool IsFormattedEntity(measure_calc ent) const = 0;
			virtual IStr* GetFormattedEntity(measure_calc ent, const sx_double& val) const = 0;
		};

		interface IGeospatialProvider : public IMeasureProvider
		{
			virtual sx_bool GetPointByLocation(const sx_double& lat, const sx_double& lng, sx_point &pt) const = 0;
		};

		enum measure_sys_type
		{
			measure_sys_type_unknown,
			measure_sys_type_rectilinear,
			measure_sys_type_geospatial
		};

		interface IMeasure;
		interface IMeasureViewport : public IRefObject
		{
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str sTitle) = 0;

			virtual sx_bool GetBBox(sx_rect &rc) const = 0;
			virtual void SetBBox(const sx_rect &rc) = 0;
			virtual IMeasure* GetMeasure(sx_bool bCreate = false, measure_sys_type t = measure_sys_type_rectilinear) = 0;
		};

		interface IMeasure : public IRefObject
		{
			virtual measure_sys_type get_Type() const = 0;
			virtual void CopyTo(IMeasure* pMeasure) const = 0;
		};

		interface IMeasureFormatter;
		interface IMeasureRectilinear : public IMeasure
		{
			virtual IStr* get_ScaleLabel() const = 0;
			virtual void set_ScaleLabel(sx_str label) = 0;
			virtual sx_double get_CXY() const = 0;
			virtual void set_CXY(const sx_double &cxy) = 0;
			virtual IRectilinearProvider* get_Provider() const = 0;

			virtual sx_bool GetOrigin(sx_point &pt) const = 0;
			virtual void SetOrigin(const sx_point &pt) = 0;
			virtual IMeasureFormatter* GetXFormat(sx_bool bCreate = false) const = 0;
			virtual IMeasureFormatter* GetYFormat(sx_bool bCreate = false) const = 0;
			virtual IMeasureFormatter* GetDistanceFormat(sx_bool bCreate = false) const = 0;
			virtual IMeasureFormatter* GetAreaFormat(sx_bool bCreate = false) const = 0;
			virtual IMeasureFormatter* GetAngleFormat(sx_bool bCreate = false) const = 0;
			virtual IMeasureFormatter* GetLineSlopeFormat(sx_bool bCreate = false) const = 0;
		};

		interface IMeasureUnit;
		interface IMeasureFormatter
		{
			virtual sx_uint get_Count() const = 0;

			virtual IMeasureUnit* Item(sx_uint idx) = 0;
			virtual IMeasureUnit* Add(const sx_double &ratio, sx_str sLabel) = 0;
			virtual void Remove(sx_uint idx) = 0;
		};

		enum measure_fract_value
		{
			measure_fract_value_as_decimal,
			measure_fract_value_as_fraction,
			measure_fract_value_rounded,
			measure_fract_value_truncated
		};

		interface IMeasureUnit : public IRefObject
		{
			virtual IStr* get_Label() const = 0;
			virtual void set_Label(sx_str label) = 0;
			virtual sx_double get_Ratio() const = 0;
			virtual void set_Ratio(const sx_double &val) = 0;
			virtual measure_fract_value get_DisplayFractValue() const = 0;
			virtual void set_DisplayFractValue(measure_fract_value val) = 0;
			virtual sx_uint get_Precision() const = 0;
			virtual void set_Precision(sx_uint val) = 0;
			virtual sx_bool get_TruncateZeros() const = 0;
			virtual void set_TruncateZeros(sx_bool trunc) = 0;
			virtual IStr* get_DecimalSep() const = 0;
			virtual void set_DecimalSep(sx_str sDelim) = 0;
			virtual IStr* get_ThousandsSep() const = 0;
			virtual void set_ThousandsSep(sx_str sDelim) = 0;
			virtual IStr* get_Prefix() const = 0;
			virtual void set_Prefix(sx_str str) = 0;
			virtual IStr* get_Suffix() const = 0;
			virtual void set_Suffix(sx_str str) = 0;
		};

		enum measure_linear_unit
		{
			measure_linear_unit_unknown,
			measure_linear_unit_m,		//	a metre
			measure_linear_unit_km,		//	a kilometre
			measure_linear_unit_ft,		//	an international foot
			measure_linear_unit_usft,	//	a U.S. Survey foot
			measure_linear_unit_mi,		//	an international mile
			measure_linear_unit_nm		//	an international nautical mile
		};

		enum measure_area_unit
		{
			measure_area_unit_unknown,
			measure_area_unit_sqm,	//	a square metre
			measure_area_unit_ha,	//	a hectare (10,000 square metres)
			measure_area_unit_sqkm,	//	a square kilometre
			measure_area_unit_sqft,	//	a square foot (US Survey)
			measure_area_unit_a,	//	an acre
			measure_area_unit_sqmi	//	a square mile (international)
		};

		enum measure_angular_unit
		{
			measure_angular_unit_unknown,
			measure_angular_unit_deg,	//	a degree 
			measure_angular_unit_grd	//	a grad (1/400 of a circle, or 0.9 degrees) 
		};

		interface IGeoPoints;
		interface IGeoCoordSystem;
		interface IMeasureGeospatial : public IMeasure
		{
			virtual Graphics::IPointsArray* get_Neatline() = 0;
			virtual IGeoPoints* get_GeoPoints() const = 0;
			virtual measure_linear_unit get_LinearUnit() const = 0;
			virtual void set_LinearUnit(measure_linear_unit e) = 0;
			virtual measure_area_unit get_AreaUnit() const = 0;
			virtual void set_AreaUnit(measure_area_unit e) = 0;
			virtual measure_angular_unit get_AngularUnit() const = 0;
			virtual void set_AngularUnit(measure_angular_unit e) = 0;
			virtual IGeospatialProvider* get_Provider() const = 0;

			virtual IGeoCoordSystem* GetGCS(sx_bool bCreate = false, sx_bool bProj = false) = 0;
			virtual IGeoCoordSystem* GetDCS(sx_bool bCreate = false, sx_bool bProj = false) = 0;
		};

		interface IGeoPoints : public Graphics::IPointsArray
		{
			virtual sx_double GetLatitude(sx_uint idx) const = 0;
			virtual void SetLatitude(sx_uint idx, const sx_double &v) = 0;
			virtual sx_double GetLongitude(sx_uint idx) const = 0;
			virtual void SetLongitude(sx_uint idx, const sx_double &v) = 0;
		};

		interface IGeoCoordSystem : IRefObject
		{
			virtual sx_bool get_Projected() const = 0;
			virtual void set_Projected(sx_bool val) = 0;
			virtual sx_uint get_EPSGCode() const = 0;
			virtual void set_EPSGCode(sx_uint val) = 0;
			virtual IStr* get_WKT() const = 0;
			virtual void set_WKT(sx_str val) = 0;
		};
	}	//	namespace PDF

	namespace OGC
	{
		interface IMapFrame;
		interface IMapFrames
		{
			virtual sx_uint get_Count() const = 0;

			virtual IMapFrame* Item(sx_uint idx) = 0;
			virtual IMapFrame* Add() = 0;
			virtual IMapFrame* Insert(sx_uint uBefore) = 0;
			virtual void Remove(sx_uint idx) = 0;
		};

		interface ICoordRefSystem;
		interface IGeoRegPoints;
		interface IMapFrame : public IRefObject
		{
			virtual IStr* get_Version() const = 0;
			virtual void set_Version(sx_str sVers) = 0;
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDesc) = 0;

			virtual sx_bool GetMatrix(sx_matrix& m) const = 0;
			virtual void SetMatrix(const sx_matrix& m, sx_bool bAsStr = true) = 0;
			virtual IGeoRegPoints* GetRegistration(sx_bool bCreate = false, sx_bool bAsStr = true) const = 0;
			virtual Graphics::IPointsArray* GetNeatline(sx_bool bCreate = false, sx_bool bAsStr = true) const = 0;
			virtual ICoordRefSystem* GetPCS(sx_bool bCreate = false) = 0;
			virtual ICoordRefSystem* GetDCS(sx_bool bCreate = false) = 0;
		};

		interface IGeoRegPoints : public Graphics::IPointsArray
		{
			virtual void GetMapPoint(sx_uint idx, sx_point &pt) const = 0;
			virtual void SetMapPoint(sx_uint idx, const sx_point &pt) = 0;
		};

		interface IGeodeticDatum;
		interface ICoordRefSystem : public IRefObject
		{
			virtual IStr* get_Label() const = 0;
			virtual void set_Label(sx_str sLabel) = 0;
			virtual IStr* get_Datum() const = 0;
			virtual void set_Datum(sx_str sDatum) = 0;
			virtual IGeodeticDatum* GetCustomDatum(sx_bool bReset = false) = 0;
			virtual IStr* get_ProjectionType() const = 0;
			virtual void set_ProjectionType(sx_str sType) = 0;
			virtual sx_double get_CentralMeridian() const = 0;
			virtual void set_CentralMeridian(const sx_double &lat) = 0;
			virtual sx_double get_FalseEasting() const = 0;
			virtual void set_FalseEasting(const sx_double &val) = 0;
			virtual sx_double get_FalseNording() const = 0;
			virtual void set_FalseNording(const sx_double &val) = 0;
			virtual sx_double get_Height() const = 0;
			virtual void set_Height(const sx_double &val) = 0;
			virtual IStr* get_Hemisphere() const = 0;
			virtual void set_Hemisphere(sx_str str) = 0;
			virtual sx_double get_LatitudeOne() const = 0;
			virtual void set_LatitudeOne(const sx_double &lat) = 0;
			virtual sx_double get_LatitudeTwo() const = 0;
			virtual void set_LatitudeTwo(const sx_double &lat) = 0;
			virtual sx_double get_LatitudeOfTrueScale() const = 0;
			virtual void set_LatitudeOfTrueScale(const sx_double &lat) = 0;
			virtual sx_double get_LongitudeOne() const = 0;
			virtual void set_LongitudeOne(const sx_double &lng) = 0;
			virtual sx_double get_LongitudeTwo() const = 0;
			virtual void set_LongitudeTwo(const sx_double &lng) = 0;
			virtual sx_double get_LongitudeDownFromPole() const = 0;
			virtual void set_LongitudeDownFromPole(const sx_double &lng) = 0;
			virtual sx_double get_Orientation() const = 0;
			virtual void set_Orientation(const sx_double &val) = 0;
			virtual sx_double get_OriginHeight() const = 0;
			virtual void set_OriginHeight(const sx_double &val) = 0;
			virtual sx_double get_OriginLatitude() const = 0;
			virtual void set_OriginLatitude(const sx_double &lat) = 0;
			virtual sx_double get_OriginLongitude() const = 0;
			virtual void set_OriginLongitude(const sx_double &lng) = 0;
			virtual sx_double get_ScaleFactor() const = 0;
			virtual void set_ScaleFactor(const sx_double &val) = 0;
			virtual sx_double get_StandardParallel() const = 0;
			virtual void set_StandardParallel(const sx_double &lat) = 0;
			virtual sx_double get_StandardParallelOne() const = 0;
			virtual void set_StandardParallelOne(const sx_double &lat) = 0;
			virtual sx_double get_StandardParallelTwo() const = 0;
			virtual void set_StandardParallelTwo(const sx_double &lat) = 0;
			virtual sx_uint get_Zone() const = 0;
			virtual void set_Zone(sx_uint v) = 0;
		};

		interface IEllipsoid;
		interface IWGS84Shift;
		interface IGeodeticDatum : public IRefObject
		{
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDesc) = 0;
			virtual IStr* get_Ellipsoid() const = 0;
			virtual void set_Ellipsoid(sx_str str) = 0;
			virtual IEllipsoid* GetEllipsoidDef(sx_bool bReset = false) = 0;
			virtual IStr* get_WGS84Shift() const = 0;
			virtual void set_WGS84Shift(sx_str str) = 0;
			virtual IWGS84Shift* GetWGS84Shift(sx_bool bReset = false) = 0;
		};

		interface IEllipsoid : public IRefObject
		{
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDesc) = 0;
			virtual sx_double get_SemiMajorAxis() const = 0;
			virtual void set_SemiMajorAxis(const sx_double &v) = 0;
			virtual sx_double get_SemiMinorAxis() const = 0;
			virtual void set_SemiMinorAxis(const sx_double &v) = 0;
			virtual sx_double get_InvFlattening() const = 0;
			virtual void set_InvFlattening(const sx_double &v) = 0;
		};

		interface IWGS84Shift : public IRefObject
		{
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDesc) = 0;
			virtual sx_double get_dx() const = 0;
			virtual void set_dx(const sx_double &dx) = 0;
			virtual sx_double get_dy() const = 0;
			virtual void set_dy(const sx_double &dy) = 0;
			virtual sx_double get_dz() const = 0;
			virtual void set_dz(const sx_double &dz) = 0;
			virtual sx_double get_rx() const = 0;
			virtual void set_rx(const sx_double &rx) = 0;
			virtual sx_double get_ry() const = 0;
			virtual void set_ry(const sx_double &ry) = 0;
			virtual sx_double get_rz() const = 0;
			virtual void set_rz(const sx_double &rz) = 0;
			virtual sx_double get_sf() const = 0;
			virtual void set_sf(const sx_double &sf) = 0;
		};

	}	//	namespace OGC
}	//	namespace SX