// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"

namespace SX
{
	namespace PDF
	{
		enum optimize_pages
		{
			optimize_pages_tree							= 0x0001,
			optimize_pages_remove_embedded_thumbnails	= 0x0002
		};
		
		enum optimize_dests
		{
			optimize_dests_remove_broken	= 0x0001
		};

		enum optimize_outlines
		{
			optimize_outlines_remove_broken		= 0x0001,
			optimize_outlines_replace_act_goto	= 0x0002,
			optimize_outlines_replace_dests		= 0x0004
		};

		enum optimize_links
		{
			optimize_links_remove_broken	= 0x0001,
			optimize_links_replace_act_goto	= 0x0002,
			optimize_links_replace_dests	= 0x0004
		};

		enum optimize_fields
		{
			optimize_fields_remove_broken	= 0x0001,
			optimize_fields_merge_widget	= 0x0002,
			optimize_fields_flatten			= 0x0004
		};

		enum optimize_images
		{
			optimize_images_downsample	= 0x0001,
			optimize_images_recompress	= 0x0002,
			optimize_images_change_cm	= 0x0004
		};

		enum optimize_log_struct
		{
			optimize_log_struct_clear				= 0x0001,
			optimize_log_struct_remove_lost_pages	= 0x0002
		};

		enum optimize_document
		{
			optimize_document_remove_all_js_actions				= 0x0001,
			optimize_document_remove_info_and_metadata			= 0x0002,
			optimize_document_remove_embedded_print_settings	= 0x0004,
			optimize_document_remove_private_data				= 0x0008,
			optimize_document_remove_hidden_text				= 0x0010,
			optimize_document_remove_unused_fonts				= 0x0020, 
			optimize_document_remove_unused_images				= 0x0040
		};

		enum color_mode
		{
			color_mode_color		= 0x0001,
			color_mode_grayscale	= 0x0002,
			color_mode_monochrome	= 0x0004
		};

		enum optimize_compression
		{
			optimize_compression_none		= 0x0001,
			optimize_compression_ASCIIHex	= 0x0002,
			optimize_compression_ASCII85	= 0x0004,
			optimize_compression_LZW		= 0x0008,
			optimize_compression_Flate		= 0x0010,
			optimize_compression_RLE		= 0x0020,
			optimize_compression_CCITT		= 0x0040,
			optimize_compression_JBIG2		= 0x0080,
			optimize_compression_JPEG		= 0x0100,
			optimize_compression_JPX		= 0x0200,
		};

		interface IImageOptimizer
		{
			// recompress
			virtual Filter get_Compression() const = 0;
			virtual void set_Compression(Filter flt) = 0;
			virtual sx_uint get_JpgQuality() const = 0;
			virtual void set_JpgQuality(sx_uint uQuality) = 0;
			// downsample
			virtual sx_uint get_ThresholdPixNum() const = 0;
			virtual void set_ThresholdPixNum(sx_uint val) = 0;
			virtual sx_uint get_DownsamplingPixNum() const = 0;
			virtual void set_DownsamplingPixNum(sx_uint val) = 0;
			virtual sx_double get_DownsamplingScale() const = 0;
			virtual void set_DownsamplingScale(const sx_double &val) = 0;
			virtual Graphics::ImageInterpolation get_Interpolation() const = 0;
			virtual void set_Interpolation(Graphics::ImageInterpolation eMode) = 0;
			// color mode
			virtual color_mode get_ChangedColorMode() const = 0;
			virtual void set_ChangedColorMode(color_mode cm) = 0;
		};

		interface IOptimizer
		{
			virtual void Document(sx_flags opt) = 0;
			virtual void Pages(sx_flags opt) = 0;
			virtual void NamedDests(sx_flags opt) = 0;
			virtual void Outlines(sx_flags opt) = 0;
			virtual void Links(sx_flags opt) = 0;
			virtual void Fields(sx_flags opt) = 0;
			virtual void LogicalStructure(sx_flags opt) = 0;
			virtual void Images(sx_flags opt) = 0;
			virtual void Compression(sx_flags opt) = 0;

			virtual IImageOptimizer* GetImageOptions(color_mode cm) = 0;
		};
	} // namespace PDF
} // namespace SX
