// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../graphics.h"
#include "events.h"
#include "annots.h"

namespace SX
{
	interface ITextRegion;
	namespace XPS { interface IPage; }
	namespace OCR { interface IOCREngine; }
	namespace Content { interface IRichContent; interface IAutoText; }
	namespace PDF
	{
		object_type obj_type_pdf_page = 0x0C;

		enum compact_objects
		{
			compact_objects_none =				0,
			compact_objects_content =			0x1,
			compact_objects_composite_images =	0x2,
			compact_objects_raster_images =		0x4,
			compact_objects_fonts =				0x8,
			compact_objects_resources = compact_objects_composite_images | compact_objects_raster_images | compact_objects_fonts,
			compact_objects_all = compact_objects_resources | compact_objects_content 
		};

		enum rem_cont
		{
			rem_cont_main_back_rect  = 0x01,
			rem_cont_main_back_image = 0x02,
			rem_cont_back_filled_rects = 0x04,
			rem_cont_back_images  = 0x08,
			rem_cont_all_filled_pathes = 0x10,
			rem_cont_all_stroked_pathes = 0x20,
			rem_cont_all_images   = 0x40,
			rem_cont_all_text   = 0x80,
			rem_cont_hidden_text = 0x100,
			rem_cont_unused_fonts = 0x200, 
			rem_cont_unused_xobjects = 0x400,
			rem_cont_crop   = 0x1000
		};
		
		enum flatten_annots
		{
			flatten_annots_ignore_nonviewable = 1,
			flatten_annots_ignore_nonprintable = 2,
			flatten_annots_ignore_hidden_layer = 4
		};

		interface IDocument;
		interface IPageLabels;
		interface IAnnots;
		interface IImages;
		interface IPictures;
		interface IImageObject;
		interface IRasterImage;
		interface ICompositeImage;
		interface IFonts;
		interface IPDFActions;
		interface IPageTransition;
		interface IMeasureViewport;
		interface IDocPart;
		interface IRawProps;

		//////////////////////////////////////////////////////////////////////////
		// document page 
		interface IPage : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_double get_Width() const  = 0;
			virtual sx_double get_Height() const = 0;
			virtual sx_rotation_angle get_Rotation() const = 0;
			virtual void      set_Rotation(sx_rotation_angle angle) = 0;
			virtual sx_double get_UserUnit() const = 0;
			virtual void      set_UserUnit(sx_double unit) = 0;
			virtual sx_uint   get_LSID() const = 0;
			virtual void      set_LSID(sx_uint unit) = 0;
			virtual IAnnots*  get_Annots() = 0;
			virtual IStr*     get_Label()  = 0;
			virtual IRasterImage* get_Thumbnail() = 0;
			virtual IPageTransition* get_Transition() = 0;
			virtual IAssociatedFiles* get_AssocFiles() = 0;
			virtual IDocPart* get_DocPart() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void GetMediaBox(sx_rect& box) const = 0;
			virtual void SetMediaBox(const sx_rect& box) = 0;
			virtual void GetCropBox(sx_rect& box) const = 0;
			virtual void SetCropBox(const sx_rect& box) = 0;
			virtual sx_bool RecognizeContentBox(sx_rect& box, sx_flags flags = 0 ) const = 0;
			virtual void CalcMatrix(sx_matrix& m, sx_double dZoom = 1.0, sx_rotation_angle angle = sx_rotation_angle_0, sx_double dx = 0., sx_double dy = 0., sx_uint uDPI = 72 ) const = 0;
			virtual sx_bool StartDraw(Graphics::IDrawingContext* ctx, sx_matrix *pTransform = nullptr,  sx_flags flags = 0, sx_flags annots = 0) = 0;
			virtual sx_bool StartPrint(Graphics::IDrawingContext* ctx, sx_matrix *pTransform = nullptr,  sx_flags flags = 0, sx_flags annots = 0) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(sx_matrix *pTransform = nullptr, sx_flags flags = 0, sx_flags annots = 0) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& backColor, sx_matrix *pTransform = nullptr,  sx_flags flags = 0, sx_flags drawAnnotations = 0) = 0;
			virtual ICompositeImage* CopyToCompositeImage(IDocument* pTargetDoc = nullptr) = 0;
			virtual ICompositeImage* MergeToCompositeImage(sx_flags annots, IDocument* pTargetDoc = nullptr) = 0;
			virtual void InsertImage(IImageObject* pImage, const sx_matrix& m, sx_bool bBack = false) = 0;
			virtual sx_bool RemoveImage(IImageObject* pImage) = 0;
			virtual sx_bool RemoveContent(sx_flags options) = 0;
			virtual IStr* GetPlainText(sx_flags opt = 0) = 0;
			virtual ITextRegion* GetTextRegion(sx_flags opt = 0) = 0;
			virtual IRasterImage* GetSingleImage() = 0;
			virtual void ScaleContent(sx_double sx, sx_double sy, sx_bool bAnnotsToo = false) = 0;
			virtual void RotateContent(sx_rotation_angle rot, sx_bool bAnnotsToo = false) = 0;
			virtual SX::Content::IRichContent* GetRichContent(sx_flags flags = 0) = 0;
			virtual IImages* GetImages(sx_bool bCompositeAlso = false, sx_bool bAnnots = false) = 0;
			virtual IPictures* GetPictures(sx_bool bCompositeAlso = false) = 0;
			virtual IFonts* GetFonts(sx_bool bAnnots = false) = 0;
			virtual void SetThumbnail(sx_uint maxw, sx_uint maxh) = 0;
			virtual IPDFActions* GetEventActions(SX::PDF::pdf_event evn) const = 0;
			virtual void UpdateAutoText(SX::Content::IAutoText* pAutoText = nullptr) = 0;
			virtual void RecognizeText(OCR::IOCREngine* pEngine) = 0;
			virtual void VectorizeText(sx_flags fonts, IFont* pFont = nullptr, sx_bool bCompImages = false, sx_bool bAnnots = false) = 0;
			virtual sx_bool FlattenAnnots(sx_flags annots, sx_flags options = 0, IAnnot *pAnnot = nullptr) = 0;
			virtual sx_bool ApplyLayers(sx_bool flatten_vis = true, sx_bool flatten_hidden = false, ILayer* pLayer = nullptr) = 0;
			virtual sx_bool ApplyRedactions(sx_flags options, IRedact* pRedact = nullptr) = 0;
			virtual void Compact(sx_flags compactFlags) = 0;
			virtual IStr* GetMetadata() = 0;
			virtual void SetMetadata(sx_str szMetadata) = 0;
			virtual IRawProps* GetPrivateProps(sx_str Name, sx_bool bCreate = false) = 0;
			virtual sx_flags GetLastDrawStatus() = 0;

			virtual IMeasureViewport* GetNextMeasureViewport(IMeasureViewport *pMeasureVp = nullptr) = 0;
			virtual IMeasureViewport* AddMeasureViewport() = 0;
			virtual void RemoveMeasureViewport(IMeasureViewport *pMeasureVp = nullptr) = 0;
			virtual OGC::IMapFrames* get_MapFrames() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// page collection
		interface IPages
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IPage*  Item(sx_uint uIndex) = 0; // can throw IException
			virtual IPage*  Add() = 0; // can throw IException
			virtual IPage*  Insert(sx_uint uTo) = 0; // can throw IException
			virtual IPage*  Copy(IPage* pPage, sx_uint uTo, sx_uint uReserved = 0) = 0; // can throw IException
			virtual IPage*  Copy(XPS::IPage* pPage, sx_uint uTo, sx_uint uReserved = 0) = 0; // can throw IException
			virtual void    Move(sx_uint uFrom, sx_uint uTo) = 0; // can throw IException
			virtual void    Remove(sx_uint uIndex) = 0; // can throw IException
			virtual sx_uint IndexOf(IPage* pPage) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& back, sx_uint uDPI = 72, sx_flags opt = 0, sx_flags annots = 0, sx_uint start = 0, sx_uint end = INVALID_INDEX) = 0;

			virtual IPageLabels* GetFirstLabelsRange() = 0;
			virtual IPageLabels* GetNextLabelsRange(IPageLabels* pPrevRange) = 0;
			virtual IPageLabels* GetLabelsRange(IPage* pPage) = 0;
			virtual IPageLabels* SetLabelsRange(IPage* pPage) = 0;
			virtual void         RemoveLabelsRange(IPageLabels* pRange) = 0;
			virtual sx_bool      IsHeadOfLabelsRange(IPage* pPage) = 0;

			virtual void Compact(sx_flags compactObjects) = 0;
			virtual void SetAutoCompact(sx_flags compactObjects, sx_uint uNumDeployedPages) = 0;
			virtual IImages* GetImages(sx_bool bCompositeAlso = false, sx_bool bAnnots = false) = 0;
			virtual IFonts* GetFonts(sx_bool bAnnots = false) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum label_style
		{
			label_style_none,
			label_style_decimal,
			label_style_roman_upper,
			label_style_roman_lower,
			label_style_letter_upper,
			label_style_letter_lower
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPageLabels : public IRefObject
		{
			virtual sx_uint     get_StartNum() const         = 0;
			virtual void        set_StartNum(sx_uint num)    = 0;
			virtual label_style get_Style() const            = 0;
			virtual void        set_Style(label_style style) = 0;
			virtual IStr*       get_Prefix() const           = 0;
			virtual void        set_Prefix(sx_str prefix)    = 0;
		};

		enum page_transition
		{
			page_transition_unknown,
			page_transition_split,
			page_transition_blinds,
			page_transition_box,
			page_transition_wipe,
			page_transition_dissolve,
			page_transition_glitter,
			page_transition_replace,
			page_transition_fly,
			page_transition_push,
			page_transition_cover,
			page_transition_uncover,
			page_transition_fade
		};

		enum page_trans_dir
		{
			page_trans_dir_none			= 0x00,
			page_trans_dir_hor			= 0x01,
			page_trans_dir_vert			= 0x02,
			page_trans_dir_in			= 0x04,
			page_trans_dir_out			= 0x08,
			page_trans_dir_hor_in		= 0x05,
			page_trans_dir_hor_out		= 0x09,
			page_trans_dir_vert_in		= 0x06,
			page_trans_dir_vert_out		= 0x0A,
			page_trans_dir_right		= 0x10,
			page_trans_dir_right_up		= 0x20,
			page_trans_dir_up			= 0x30,
			page_trans_dir_left_up		= 0x40,
			page_trans_dir_left			= 0x50,
			page_trans_dir_left_down	= 0x60,
			page_trans_dir_down			= 0x70,
			page_trans_dir_right_down	= 0x80
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPageTransition : public IRefObject
		{
			virtual sx_uint get_AutoFlip() const = 0;
			virtual void set_AutoFlip(sx_uint sec) = 0;
			virtual sx_uint get_Duration() const = 0;
			virtual void set_Duration(sx_uint sec) = 0;
			virtual page_transition get_Style() const = 0;
			virtual void set_Style(page_transition style) = 0;
			virtual page_trans_dir get_Direction() const = 0;
			virtual void set_Direction(page_trans_dir style) = 0;
		};

	} // namespace PDF
} // namespace SX
