// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "files.h"

namespace SX
{
	namespace NAV { interface INavigator; }
	namespace PDF
	{
		//////////////////////////////////////////////////////////////////////////
		// portfolio view mode
		enum portfolio_view
		{
			portfolio_view_unknown,
			portfolio_view_details,
			portfolio_view_tile,
			portfolio_view_hidden,
			portfolio_view_navigator
		};

		//////////////////////////////////////////////////////////////////////////
		// file info fields
		interface IFileFields
		{
			virtual IStr*     GetString(sx_str sID) const          = 0;
			virtual void      SetString(sx_str sID, sx_str sVal)   = 0;
			virtual sx_double GetNumeric(sx_str sID) const             = 0;
			virtual void      SetNumeric(sx_str sID, sx_double dblVal) = 0;
			virtual sx_time   GetDate(sx_str sID) const        = 0;
			virtual void      SetDate(sx_str sID, sx_time &tm) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// portfolio file reference
		interface IPortfolioFile : public IFileRef
		{
			virtual IFileFields* get_Fields() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// portfolio folder
		interface IPortfolioFolder : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str sName) = 0;
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDesc) = 0;
			virtual sx_uint get_ID() const = 0;
			virtual IFileFields* get_Fields() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_time GetMDate() const  = 0;
			virtual sx_time GetCDate() const  = 0;

			// folders hierarchy
			virtual IPortfolioFolder* Parent() const = 0;
			virtual IPortfolioFolder* FirstSubFolder() const = 0;
			virtual IPortfolioFolder* NextSubFolder(IPortfolioFolder* pPrev) const = 0;
			virtual IPortfolioFolder* InsertSubFolder(sx_str sName, IPortfolioFolder* pBeforeSubFolder = nullptr) = 0;
			virtual void RemoveSubFolder(IPortfolioFolder* pItem, sx_bool bRemoveNestedFiles = false) = 0;
			virtual void Move(IPortfolioFolder* pNewParent, IPortfolioFolder* pBeforeChild = nullptr) = 0; // can throw IException

			// attached files
			virtual IPortfolioFile* FirstFile() = 0;
			virtual IPortfolioFile* NextFile(IPortfolioFile* pPrev) = 0;
			virtual void AttachFile(IPortfolioFile* pFile) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// IPortfolioColors
		interface IPortfolioColors
		{
			virtual sx_color_f_rgb get_Background() const                   = 0;
			virtual void           set_Background(sx_color_f_rgb eColorRGB) = 0;
			virtual sx_color_f_rgb get_CardBackground() const                   = 0;
			virtual void           set_CardBackground(sx_color_f_rgb eColorRGB) = 0;
			virtual sx_color_f_rgb get_CardBorder() const                   = 0;
			virtual void           set_CardBorder(sx_color_f_rgb eColorRGB) = 0;
			virtual sx_color_f_rgb get_PrimaryText() const                   = 0;
			virtual void           set_PrimaryText(sx_color_f_rgb eColorRGB) = 0;
			virtual sx_color_f_rgb get_SecondaryText() const                   = 0;
			virtual void           set_SecondaryText(sx_color_f_rgb eColorRGB) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// portfolio
		interface IPortfolio
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual portfolio_view  get_InitialView() const              = 0;
			virtual void            set_InitialView(portfolio_view view) = 0;
			virtual IPortfolioFile* get_InitialDoc() const                     = 0;
			virtual void            set_InitialDoc(const IPortfolioFile* pDoc) = 0;
			virtual IPortfolioFolder*     get_RootFolder() const = 0;
			virtual IPortfolioColors*     get_Colors() const     = 0;

			virtual NAV::INavigator* GetNavigator(sx_bool bCreate = false) = 0;
		};

	} // namespace PDF
} // namespace SX
