// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "events.h"
#include "pdfa.h"

namespace SX
{
	namespace PDF
	{
		interface IViewerOptions;
		interface ISaveOptions;
		interface IDocSecurity;
		interface IDest;
		interface IPDFAction;
		interface IPDFActions;
		interface IFormData;
		interface ISignature;
		interface ISignatures;
		interface ILogicalStructure;
		interface IAssociatedFiles;
		interface IRawProps;
		interface IDocParts;
		interface IDocReqs;
		interface IDocScripts;

		enum xfa_status
		{
			xfa_status_none,
			xfa_status_static,
			xfa_status_dynamic
		};

		enum page_layout
		{
			page_layout_single_page = 1,
			page_layout_one_column,
			page_layout_two_pages,
			page_layout_two_columns,
			page_layout_two_pages_with_cover,
			page_layout_two_columns_with_cover
		};

		enum ui_options
		{
			ui_options_none				= 0x000,
			ui_options_outlines			= 0x001,
			ui_options_thumbs			= 0x002,
			ui_options_layers			= 0x004,
			ui_options_attachments		= 0x008,

			ui_options_doc_title		= 0x010,
			ui_options_menu				= 0x020,
			ui_options_toolbar			= 0x040,
			ui_options_nav_elements		= 0x080,

			ui_options_fit_window		= 0x100,
			ui_options_center_window	= 0x200,
			ui_options_full_screen		= 0x400
		};

		//////////////////////////////////////////////////////////////////////////
		// document properties
		interface IDocProperties
		{
			//////////////////////////////////////////////////////////////////////////
			// identification
			virtual IStr* get_ID1() const = 0;
			virtual void set_ID1(sx_str sID) = 0;
			virtual IStr* get_ID2() const = 0;
			virtual void set_ID2(sx_str sID) = 0;

			//////////////////////////////////////////////////////////////////////////
			// security
			virtual IDocSecurity* get_Security() = 0;

			//////////////////////////////////////////////////////////////////////////
			// PDF format
			virtual IStr* get_FormatVersion() const = 0;
			virtual void set_FormatVersion(sx_str szVersion) = 0;
			virtual sx_bool get_Linearized() const = 0;
			virtual IStr* GetExtFormatVersion(sx_str szDevName) const = 0;
			virtual sx_uint GetExtFormatLevel(sx_str szDevName) const = 0;
			virtual void SetExtFormat(sx_str szDevName, sx_str szBaseVersion, sx_uint uLevel) = 0;

			//////////////////////////////////////////////////////////////////////////
			// revisions
			virtual sx_uint get_RevisionsCount() = 0;
			virtual sx_uint get_CurrentRevision() = 0;

			//////////////////////////////////////////////////////////////////////////
			// events and actions
			virtual IDest* get_OpenDest() = 0;
			virtual IPDFAction* get_OpenAction() = 0;
			virtual IPDFAction* SetOpenAction(pdf_action act) = 0;
			virtual IPDFActions* GetEventActions(pdf_event evn) const = 0;

			//////////////////////////////////////////////////////////////////////////
			// doc description
			virtual sx_uint get_PropCount() = 0;
			virtual IStr* GetPropName(sx_uint uIndex) = 0;
			virtual IStr* GetProp(sx_str szName) = 0;
			virtual void SetProp(sx_str szName, sx_str szValue) = 0;
			virtual sx_time GetCDate() = 0;
			virtual sx_time GetMDate() = 0;
			virtual IStr* GetMetadata() = 0;
			virtual void SetMetadata(sx_str szMetadata) = 0;
			virtual IRawProps* GetPrivateProps(sx_str Name, sx_bool bCreate = false) = 0;

			//////////////////////////////////////////////////////////////////////////
			// misc
			virtual IViewerOptions* get_ViewerOptions() = 0;
			virtual ILogicalStructure* get_LogicalStructure() const = 0;
			virtual IAssociatedFiles* get_AssocFiles() = 0;
			virtual IDocReqs* get_DocReqs() = 0;
			virtual IDocScripts* get_Scripts() = 0;
			virtual IStr* get_URIBase() const = 0;
			virtual void set_URIBase(sx_str sBase) = 0;

			virtual xfa_status GetXFAStatus() const = 0;
			virtual ISignatures* GetSignatures() = 0;
			virtual ISignature* GetPermissions(sx_str sName) = 0;
			virtual ISignature* SetPermissions(sx_str sName) = 0;
			virtual void SetPermissions(sx_str sName, ISignature* pPerms) = 0;
			virtual IRawProps* GetAttestation(sx_bool bCreate = false) = 0; // Legal entry
			virtual IDocParts* GetDocParts(sx_bool bCreate = false) = 0;
			virtual IFormData* GetFormDataProvider() = 0;

			//////////////////////////////////////////////////////////////////////////
			// modification
			virtual sx_bool get_Modified() const = 0;
			virtual ISaveOptions* get_SaveOptions() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// save options
		interface ISaveOptions
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_bool get_AutoUpdateID() const = 0; // true by default
			virtual void set_AutoUpdateID(sx_bool bUpdate) = 0;
			virtual sx_bool get_AutoUpdateMDate() const = 0; // true by default
			virtual void set_AutoUpdateMDate(sx_bool bUpdate) = 0;
			virtual sx_bool get_AutoUpdateMetadata() const = 0; // false by default
			virtual void set_AutoUpdateMetadata(sx_bool bUpdate) = 0;
			virtual sx_bool get_AutoDowngradeSecurity() const = 0; // true by default
			virtual void set_AutoDowngradeSecurity(sx_bool bDowngrade) = 0;
			virtual sx_bool get_AutoResetModified() const = 0; // true by default
			virtual void set_AutoResetModified(sx_bool bReset) = 0;
			virtual pdfa get_PDFAVersionInMetadata() const = 0; 
			virtual void set_PDFAVersionInMetadata(pdfa ver) = 0;
			virtual sx_bool get_Linearized() const = 0;
			virtual void set_Linearized(sx_bool lin) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// viewer options
		interface IViewerOptions
		{
			virtual page_layout get_PageLayout() const = 0;
			virtual void set_PageLayout(page_layout lay) = 0;
			virtual sx_flags get_UIOptions() const = 0;
			virtual void set_UIOptions(sx_flags opt) = 0;
		};

		interface IDocScripts
		{
			virtual sx_uint get_Count() const = 0;

			virtual IStr* GetID(sx_uint ind) = 0;
			virtual IActionJScript* Get(sx_str sID) = 0;
			virtual IActionJScript* Add(sx_str sID) = 0;
			virtual void Remove(sx_str sID) = 0;
		};

	} // namespace PDF
} // namespace SX
