// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"

namespace SX
{
	interface ISequentialStream; interface ITextDocument; interface IImageDocument;
	namespace Graphics { interface IBitmapData; interface IInkImage; }
	namespace XHTML { interface ITextBox; }

	namespace PDF
	{
		interface IFileRef;
		interface IRasterImage;
		interface IRasterOptions;
		interface ICompositeImage;
		interface IImages;
		interface IFont;
		interface IFonts;
		
		enum Filter;
		
		interface IDocResources
		{
			//properties
			virtual IRasterOptions* get_NewImages() const = 0;
			virtual sx_bool get_ShareNewImages() const = 0;
			virtual void set_ShareNewImages(sx_bool sh) = 0;

			// raster images
			virtual IRasterImage* AddRasterImage(Graphics::IBitmapData* pBitmap, Filter Compression, sx_uint JpegQuality = 50) = 0;
			virtual IRasterImage* AddJpegImage(ISequentialStream* pJpeg) = 0;
			virtual ICompositeImage* AddCompositeImage(sx_double width, sx_double height) = 0;
			virtual ICompositeImage* AddCompositeImage(IRasterImage* pImage) = 0;
			virtual ICompositeImage* AddImagePage(IImageDocument* pDoc, sx_uint uPageIndex, Filter Compression, sx_uint JpegQuality = 50) = 0;
			virtual IImages* GetImages(sx_bool bAnnots = false) = 0;

			// text
			virtual ICompositeImage* AddTextBox(XHTML::ITextBox* pTB, sx_double w = 0., sx_double h = 0., sx_bool wrap = true) = 0;
			virtual ICompositeImage* AddTextPage(ITextDocument* pDoc, sx_uint uPageIndex) = 0;

			// vector and rich content
			virtual ICompositeImage* AddSerializedInk(ISequentialStream* pISF, sx_flags options = 0) = 0;
			virtual ICompositeImage* AddSerializedInk(Graphics::IInkImage* pInk, sx_flags options = 0) = 0;
			virtual ICompositeImage* AddGdiMetafile(ISequentialStream* pMetafile, sx_uint DPI = 0, sx_flags options = 0) = 0;
#ifdef SX_WINDESKTOP_DLL
			virtual ICompositeImage* AddGdiMetafile(HENHMETAFILE hMF, sx_uint DPI = 0, sx_flags options = 0) = 0;
#endif
			virtual ICompositeImage* AddSvgFragment(ISequentialStream* pStream, sx_uint DPI = 72, sx_double w = 595.3, sx_double h = 841.9, sx_flags options = 0, ILoadProvider* pProv = nullptr) = 0;
			virtual ICompositeImage* AddMathML(ISequentialStream* pStream, sx_flags options = 0) = 0;
			virtual ICompositeImage* AddDxfImage(ISequentialStream* pStream, sx_flags options = 0) = 0;

			// fonts
			virtual sx_bool EmbedAllFonts(sx_bool bEmbWinStd = true) = 0;
			virtual sx_bool CompactAllFonts() = 0;
			virtual IFont* AddFont(sx_str FontFamily, sx_uint weight = 400, sx_uint style = 0, sx_uint stretch = 5, sx_bool bComp = true) = 0;
			virtual IFont* AddCustomFont(ISequentialStream* pStream, sx_bool bComp = true) = 0;
			virtual IFonts* GetFonts(sx_bool bAnnots = false) = 0;

			// file refs and embedded streams
			virtual IFileRef* AddFileRef() = 0;
		};

		interface IRasterOptions
		{
			virtual sx_bool get_JpegRGB() const = 0;
			virtual void set_JpegRGB(sx_bool val) = 0;
			virtual sx_bool get_JpegCMYK() const = 0;
			virtual void set_JpegCMYK(sx_bool val) = 0;
			virtual sx_bool get_JpegGray() const = 0;
			virtual void set_JpegGray(sx_bool val) = 0;
			virtual sx_uint get_JpegMinDim() const = 0;
			virtual void set_JpegMinDim(sx_uint val) = 0;
			virtual sx_uint get_JpegQuality() const = 0;
			virtual void set_JpegQuality(sx_uint val) = 0;
			virtual sx_bool get_JpegOrFlate() const = 0;
			virtual void set_JpegOrFlate(sx_bool val) = 0;
		};

		enum raw_type
		{
			raw_type_unknown,
			raw_type_name,
			raw_type_str,
			raw_type_num,
			raw_type_bool,
			raw_type_dict,
			raw_type_arr,
			raw_type_stream
		};

		interface IRawProps : public IRefObject
		{
			// entries
			virtual sx_uint get_Count() const = 0;
			virtual IStr* GetPropName(sx_uint ind) const = 0;
			virtual raw_type GetPropType(sx_str name) const = 0;
			// entry values
			virtual IStr* GetString(sx_str name) const = 0;
			virtual void SetString(sx_str name, sx_str val, sx_bool bName = true) = 0;
			virtual sx_double GetNum(sx_str name) const = 0;
			virtual void SetNum(sx_str name, const sx_double &val) = 0;
			virtual sx_bool GetBool(sx_str name) const = 0;
			virtual void SetBool(sx_str name, sx_bool val) = 0;
			virtual IRawProps* GetSubProps(sx_str name) const = 0;
			virtual void SetSubProps(sx_str name, sx_bool bDict = true) = 0;
			virtual ISequentialStream* GetStream(sx_str name) const = 0;
			virtual void SetStream(sx_str name, ISequentialStream* pStream) = 0;
			virtual void RemoveProp(sx_str name) const = 0;
		};
	} // namespace PDF
} // namespace SX
