// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
namespace SX
{
	interface ICryptoCert;
	interface ICryptoKey;

	namespace PDF
	{
		enum doc_security
		{
			doc_security_none = 0,
			doc_security_undefined,
			doc_security_password,
			doc_security_pki,
			doc_security_custom
		};

		enum encryption_algorithm
		{
			encryption_algorithm_none = 0,
			encryption_algorithm_rc4,
			encryption_algorithm_aes
		};

		enum encryption_key_length
		{
			encryption_key_length_unknown = 0,
			encryption_key_length_40 = 40,
			encryption_key_length_128 = 128,
			encryption_key_length_256 = 256,
			encryption_key_length_384 = 384,
			encryption_key_length_512 = 512
		};

		enum doc_permissions
		{
			doc_permissions_full_control   = 0x002,
			doc_permissions_normal_print   = 0x004,
			doc_permissions_modify_content = 0x008,
			doc_permissions_copy_content   = 0x010,
			doc_permissions_modify_annots  = 0x020,
			doc_permissions_fill_form      = 0x100,
			doc_permissions_accessibility  = 0x200,
			doc_permissions_assemble       = 0x400,
			doc_permissions_quality_print  = 0x800
		};

		interface IRawProps;
		interface IStandardSecurity;
		interface ICustomSecurity;

		//////////////////////////////////////////////////////////////////////////
		// document security properties
		interface IDocSecurity
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual doc_security get_Security() const = 0;
			virtual IStandardSecurity* get_StandardSecurity() = 0;
			virtual IStr* get_HandlerName() const = 0;
			virtual IStr* get_HandlerFormat() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void ChangeSecurity(doc_security Sec) = 0;
			virtual void SetCustomSecurity(ICustomSecurity* pSecHandler) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// standard security handler
		interface IStandardSecurity
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_bool get_FullControl() const = 0;
			virtual encryption_algorithm get_EncryptionAlgorithm() const = 0;
			virtual void set_EncryptionAlgorithm(encryption_algorithm Alg) = 0;
			virtual encryption_key_length get_EncryptionKeyLength() const = 0;
			virtual void set_EncryptionKeyLength(encryption_key_length Length) = 0;
			virtual sx_flags get_Permissions() const = 0; // mask of permissions
			virtual void set_Permissions(sx_flags PermissionsMask) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// password based standard security
		interface IPasswordSecurity : public IStandardSecurity
		{
			virtual sx_bool SetOwnerPassword(sx_str sPassword) = 0;
			virtual sx_bool SetUserPassword(sx_str sPassword) = 0;
			virtual IPasswordVerifier* CreatePasswordVerifier() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// password based standard security
		interface IPKISecurity : public IStandardSecurity
		{
			virtual sx_bool AddRecipient(ICryptoCert* pCert, sx_flags PermissionsMask) = 0;
			virtual sx_bool VerifyRecipient(ICryptoCert* pCert, ICryptoKey* pPrivateKey) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// custom crypto filter
		enum crypto_event
		{
			crypto_event_encrypt_doc = 1,
			crypto_event_decrypt_doc,
			crypto_event_encrypt_stream,
			crypto_event_decrypt_stream,
			crypto_event_encrypt_string,
			crypto_event_decrypt_string
		};

		interface ICryptoEvent
		{
			virtual crypto_event get_Type() const = 0;
			virtual IRawProps* get_CryptoProps() const = 0;
			virtual ISequentialStream* get_DecData() const = 0;
			virtual ISequentialStream* get_EncData() const = 0;
			virtual sx_uint get_ObjNum() const = 0;
			virtual sx_uint get_ObjGen() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface ICustomSecurity
		{
			virtual sx_bool OnEvent(ICryptoEvent* pEvent) = 0;
		};
	} // namespace PDF
} // namespace SX
