// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../document.h"
#include "../xhtml.h"
interface IDispatch;

namespace SX
{
	interface IApplication;
	namespace PDF { interface IDocument; }

	namespace XFA
	{
		object_type obj_type_xfa_document	= 0x32;
		interface IFormNode;
		interface IFormField;

		//////////////////////////////////////////////////////////////////////////
		interface IForm
		{
			virtual sx_uint get_Count() const = 0;
			virtual IFormNode* Item(sx_uint idx) = 0;
			virtual IFormNode* Find(sx_str sSom) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDocument : public IBaseDocument
		{
			virtual IDispatch* get_xfa() = 0;
			virtual IForm* get_Form() = 0;

			virtual void Load(PDF::IDocument* pDoc) = 0;
			virtual void Upload(PDF::IDocument* pDoc) = 0;
			virtual void OpenStream(ISequentialStream* pStream) = 0;
			virtual sx_qword SaveAsStream(ISequentialStream* pStream) = 0;
			virtual void OpenPDFStream(ISequentialStream* pStream) = 0;
			virtual sx_qword SaveAsPDFStream(ISequentialStream* pStream) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum form_node
		{
			form_node_group		= 1,
			form_node_rgroup,
			form_node_field
		};

		enum field_type
		{
			field_type_undefined	= 0,
			field_type_button		= 0x01,
			field_type_check_button	= 0x02,

			field_type_text			= 0x04,
			field_type_datetime		= 0x08,
			field_type_numeric		= 0x10,
			field_type_password		= 0x20,

			field_type_choice		= 0x40,
			field_type_signature	= 0x80,
			field_type_barcode		= 0x100,
			field_type_image		= 0x200
		};

		enum check_state
		{
			check_state_off	= 0,
			check_state_on,
			check_state_neutral
		};
		
		enum attr_presence
		{
			attr_presence_visible,
			attr_presence_invisible,
			attr_presence_hidden,
		};

		interface IFormNode : public IRefObject
		{
			virtual form_node get_NodeType() const = 0;
			virtual IStr* get_Name() const = 0;
			virtual IStr* get_SomExpression() const = 0;
			virtual attr_presence get_Presence() const = 0;
			virtual void set_Presence(attr_presence ePresence) = 0;
		};

		interface IFieldsGroup : public IFormNode
		{
			virtual sx_uint get_Count() const = 0;
			virtual IFormNode* Item(sx_uint idx) = 0;

			virtual IFormNode* AddInstance(sx_str str, sx_bool bMerge) = 0;
			virtual void RemoveInstance(sx_str str, sx_uint idx) = 0;
		};

		interface IRadioGroup : public IFormNode
		{
			virtual sx_bool get_Readonly() const = 0;
			virtual sx_uint get_Count() const = 0;
			virtual IFormField* Item(sx_uint idx) = 0;
			virtual sx_uint get_Value() const = 0;
			virtual void set_Value(sx_uint val) = 0;
		};

		interface IFormField : public IFormNode
		{
			virtual field_type get_FieldType() const = 0;
			virtual sx_bool get_Readonly() const = 0;
		};

		interface ICheckField : public IFormField
		{
			virtual check_state get_State() const = 0;
			virtual void set_State(check_state eState) = 0;
			virtual check_state get_DefaultState() const = 0;
			virtual IStr* GetStateName(check_state eState) const = 0;
		};

		interface ITextField : public IFormField
		{
			virtual sx_bool get_Multiline() const = 0;
			virtual sx_bool get_AllowRichText() const = 0;
			virtual IStr* get_Value() const = 0;
			virtual void set_Value(sx_str val) = 0;
			virtual IStr* get_FormattedValue() const = 0;
			virtual XHTML::IRichText* get_RichText() const = 0;
			virtual void set_RichText(XHTML::IRichText *pRT) = 0;
		};

		interface IDateTimeField : public IFormField
		{
			virtual IStr* get_Value() const = 0;
			virtual void set_Value(sx_str val) = 0;
			virtual sx_bool GetTime(sx_time &t) const = 0;
			virtual void SetTime(const sx_time &t) = 0;
			virtual IStr* get_FormattedValue() const = 0;
		};

		interface INumericField : public IFormField
		{
			virtual IStr* get_Value() const = 0;
			virtual void set_Value(sx_str val) = 0;
			virtual sx_bool GetValue(sx_double &val) const = 0;
			virtual void SetValue(const sx_double &val) = 0;
			virtual IStr* get_FormattedValue() const = 0;
		};

		interface IChoiceField : public IFormField
		{
			virtual sx_bool get_ComboBox() const = 0;
			virtual sx_bool get_Editable() const = 0;
			virtual sx_bool get_MultiSelect() const = 0;
			virtual sx_bool get_CommitOnChange() const = 0;

			virtual sx_uint get_Items() const = 0;
			virtual sx_uint get_FirstCheckedItem() const = 0;
			virtual sx_bool GetItemState(sx_uint idx) = 0;
			virtual void SetItemState(sx_uint idx, sx_bool bCheck) = 0;
			virtual IStr* GetItemValue(sx_uint idx, sx_bool bExpValue = false) const = 0;
			virtual IStr* GetCustomValue() const = 0;
			virtual void SetCustomValue(sx_str sValue) = 0;
		};

		interface IImageField : public IFormField
		{
			virtual IStr* get_ContentType() const = 0;
			virtual void set_ContentType(sx_str sMIME) = 0;
			virtual sx_bool GetData(ISequentialStream *pData) const = 0;
			virtual void SetData(ISequentialStream *pData) const = 0;
			virtual IStr* get_DataRef() const = 0;
			virtual void set_DataRef(sx_str sRef) const = 0;
		};

		interface IBarcodeField : public IFormField	{	};
		interface ISignatureField : public IFormField	{	};

		//////////////////////////////////////////////////////////////////////////
		interface IXFATools
		{
			virtual IDocument* CreateDoc(IApplication* pApp) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// library entry point
		typedef IXFATools* (__cdecl* XFALibraryEntryPoint)();
	} // namespace XFA
} // namespace SX