// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"

namespace SX
{
	namespace XMP
	{
		enum xmp_status
		{
			xmp_status_not_xml	= 1,
			xmp_status_no_meta	= 2,
			xmp_status_not_rdf	= 3,
			xmp_status_not_xmp	= 4,
			xmp_status_xmp		= 5
		};

		enum xmp_scheme
		{
			xmp_scheme_custom		= 0,
			xmp_scheme_dublin_core	= 1,
			xmp_scheme_xmp_basic	= 2,
			xmp_scheme_xmp_mm		= 3,
			xmp_scheme_xmp_rm		= 4,
			xmp_scheme_adobe_pdf	= 10,
			xmp_scheme_photoshop	= 11,
			xmp_scheme_pdf_ext		= 12,
			xmp_scheme_pdfa_ident	= 20,
			xmp_scheme_pdfa_ext		= 21,
			xmp_scheme_zugferd		= 30,
			xmp_scheme_zugferd2		= 31,
			xmp_scheme_factur_x		= 32,
			xmp_scheme_subscheme	= 100,
			xmp_scheme_pdfa_prop	= 101,
			xmp_scheme_pdfa_schema	= 102,
			xmp_scheme_pdfa_valtype	= 103,
			xmp_scheme_pdfa_field	= 104,
			xmp_scheme_stevt		= 105
		};

		enum xmp_profile
		{
			xmp_profile_min		= 1,
			xmp_profile_pdf		= 2,
			xmp_profile_pdfa1	= 11,
			xmp_profile_pdfa3	= 13
		};

		enum rdf_cont_type
		{
			rdf_cont_type_unk	= 0,
			rdf_cont_type_bag	= 1,
			rdf_cont_type_seq	= 2,
			rdf_cont_type_alt	= 3
		};

		interface IScheme;
		interface ICustomScheme;
		interface IRdfContainer;

		const object_type
			obj_type_xmp_document =		0x33;

		interface IDocument : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual xmp_status OpenStream(ISequentialStream* pXMP, xmp_profile prof = xmp_profile_min) = 0;
			virtual xmp_status OpenFromString(sx_str sXMP, xmp_profile prof = xmp_profile_min) = 0;
			virtual sx_uint SaveAsStream(ISequentialStream* pXMP) = 0;
			virtual IStr* SaveToString() = 0;
			virtual IScheme* GetScheme(xmp_scheme sch) = 0;
			virtual ICustomScheme* GetCustomScheme(sx_str URI) = 0;
			virtual ICustomScheme* GetNextCustomScheme(ICustomScheme* pScheme = nullptr) = 0;
			virtual void Reset() = 0;
		};

		interface IScheme : public IRefObject
		{
			virtual xmp_scheme get_SchemeType() const = 0;

			virtual sx_bool IsPresent() const = 0;
			virtual void Clear() = 0;
		};

		interface IDublinCoreScheme: public IScheme
		{
			virtual sx_str get_contributor() const = 0;
			virtual void set_contributor(sx_str val) = 0;
			virtual IRdfContainer* GetContributor(sx_bool reset = false) const = 0;

			virtual sx_str get_coverage() const = 0;
			virtual void set_coverage(sx_str val) = 0;

			virtual sx_str get_creator() const = 0;
			virtual void set_creator(sx_str val) = 0;
			virtual IRdfContainer* GetCreator(sx_bool reset = false) const = 0;

			virtual sx_time get_date() const = 0;
			virtual void set_date(const sx_time &val) = 0;

			virtual sx_str get_description() const = 0;
			virtual void set_description(sx_str val) = 0;
			virtual IRdfContainer* GetDescription(sx_bool reset = false) const = 0;

			virtual sx_str get_format() const = 0;
			virtual void set_format(sx_str val) = 0;

			virtual sx_str get_identifier() const = 0;
			virtual void set_identifier(sx_str val) = 0;

			virtual sx_str get_language() const = 0;
			virtual void set_language(sx_str val) = 0;
			virtual IRdfContainer* GetLanguage(sx_bool reset = false) const = 0;

			virtual sx_str get_publisher() const = 0;
			virtual void set_publisher(sx_str val) = 0;
			virtual IRdfContainer* GetPublisher(sx_bool reset = false) const = 0;

			virtual sx_str get_relation() const = 0;
			virtual void set_relation(sx_str val) = 0;
			virtual IRdfContainer* GetRelation(sx_bool reset = false) const = 0;

			virtual sx_str get_rights() const = 0;
			virtual void set_rights(sx_str val) = 0;
			virtual IRdfContainer* GetRights(sx_bool reset = false) const = 0;

			virtual sx_str get_source() const = 0;
			virtual void set_source(sx_str val) = 0;

			virtual sx_str get_subject() const = 0;
			virtual void set_subject(sx_str val) = 0;
			virtual IRdfContainer* GetSubject(sx_bool reset = false) const = 0;

			virtual sx_str get_title() const = 0;
			virtual void set_title(sx_str val) = 0;
			virtual IRdfContainer* GetTitle(sx_bool reset = false) const = 0;

			virtual sx_str get_type() const = 0;
			virtual void set_type(sx_str val) = 0;
			virtual IRdfContainer* GetType(sx_bool reset = false) const = 0;
		};

		interface IXMPBasicScheme: public IScheme
		{
			virtual sx_str get_Advisory() const = 0;
			virtual void set_Advisory(sx_str val) = 0; // nullptr value only! 

			virtual sx_str get_BaseURL() const = 0;
			virtual void set_BaseURL(sx_str val) = 0;

			virtual sx_time get_CreateDate() const = 0;
			virtual void set_CreateDate(const sx_time &val) = 0;

			virtual sx_str get_CreatorTool() const = 0;
			virtual void set_CreatorTool(sx_str val) = 0;

			virtual sx_str get_Description() const = 0;
			virtual void set_Description(sx_str val) = 0; // nullptr value only! 

			virtual sx_str get_Identifier() const = 0;
			virtual void set_Identifier(sx_str val) = 0;
			virtual IRdfContainer* GetIdentifier(sx_bool reset = false) const = 0;

			virtual sx_str get_Label() const = 0;
			virtual void set_Label(sx_str val) = 0;

			virtual sx_time get_MetadataDate() const = 0;
			virtual void set_MetadataDate(const sx_time &val) = 0;

			virtual sx_time get_ModifyDate() const = 0;
			virtual void set_ModifyDate(const sx_time &val) = 0;

			virtual sx_str get_Nickname() const = 0;
			virtual void set_Nickname(sx_str val) = 0;

			virtual sx_str get_Rating() const = 0;
			virtual void set_Rating(sx_str val) = 0;

			virtual IRdfContainer* GetThumbnails(sx_bool reset = false) const = 0;

			virtual sx_str get_Title() const = 0;
			virtual void set_Title(sx_str val) = 0; // nullptr value only! 
		};

		interface IXMPMMScheme: public IScheme
		{
			virtual IScheme* GetDerivedFrom(sx_bool reset = false) const = 0;

			virtual sx_str get_DocumentID() const = 0;
			virtual void set_DocumentID(sx_str val) = 0;

			virtual IRdfContainer* GetHistory(sx_bool reset = false) const = 0;

			virtual IRdfContainer* GetIngredients(sx_bool reset = false) const = 0;

			virtual sx_str get_InstanceID() const = 0;
			virtual void set_InstanceID(sx_str val) = 0;

			virtual sx_str get_LastURL() const = 0;
			virtual void set_LastURL(sx_str val) = 0; // nullptr value only! 

			virtual IScheme* GetManagedFrom(sx_bool reset = false) const = 0;

			virtual sx_str get_Manager() const = 0;
			virtual void set_Manager(sx_str val) = 0;

			virtual sx_str get_ManageTo() const = 0;
			virtual void set_ManageTo(sx_str val) = 0;

			virtual sx_str get_ManageUI() const = 0;
			virtual void set_ManageUI(sx_str val) = 0;

			virtual sx_str get_ManagerVariant() const = 0;
			virtual void set_ManagerVariant(sx_str val) = 0;

			virtual sx_str get_Manifest() const = 0;
			virtual void set_Manifest(sx_str val) = 0; // nullptr value only! 

			virtual sx_str get_OriginalDocumentID() const = 0;
			virtual void set_OriginalDocumentID(sx_str val) = 0;

			virtual IRdfContainer* GetPantry(sx_bool reset = false) const = 0;

			virtual sx_str get_RenditionClass() const = 0;
			virtual void set_RenditionClass(sx_str val) = 0;

			virtual sx_str get_RenditionOf() const = 0;
			virtual void set_RenditionOf(sx_str val) = 0; // nullptr value only! 

			virtual sx_str get_RenditionParams() const = 0;
			virtual void set_RenditionParams(sx_str val) = 0;

			virtual sx_str get_SaveID() const = 0;
			virtual void set_SaveID(sx_str val) = 0; // nullptr value only! 

			virtual sx_str get_VersionID() const = 0;
			virtual void set_VersionID(sx_str val) = 0;

			virtual IRdfContainer* GetVersions(sx_bool reset = false) const = 0;
		};

		interface IXMPRMScheme: public IScheme
		{
			virtual sx_str get_Certificate() const = 0;
			virtual void set_Certificate(sx_str val) = 0;

			virtual sx_str get_Copyright() const = 0;
			virtual void set_Copyright(sx_str val) = 0; // nullptr value only!

			virtual sx_bool get_Marked() const = 0;
			virtual void set_Marked(sx_bool val) = 0;

			virtual sx_str get_Owner() const = 0;
			virtual void set_Owner(sx_str val) = 0;
			virtual IRdfContainer* GetOwner(sx_bool reset = false) const = 0;

			virtual sx_str get_UsageTerms() const = 0;
			virtual void set_UsageTerms(sx_str val) = 0;
			virtual IRdfContainer* GetUsageTerms(sx_bool reset = false) const = 0;

			virtual sx_str get_WebStatement() const = 0;
			virtual void set_WebStatement(sx_str val) = 0;
		};

		interface IPhotoshopScheme: public IScheme
		{
			virtual sx_str get_AuthorsPosition() const = 0;
			virtual void set_AuthorsPosition(sx_str val) = 0;

			virtual sx_str get_CaptionWriter() const = 0;
			virtual void set_CaptionWriter(sx_str val) = 0;

			virtual sx_str get_Category() const = 0;
			virtual void set_Category(sx_str val) = 0;

			virtual sx_str get_City() const = 0;
			virtual void set_City(sx_str val) = 0;

			virtual sx_uint get_ColorMode() const = 0;
			virtual void set_ColorMode(sx_uint val) = 0;

			virtual sx_str get_Country() const = 0;
			virtual void set_Country(sx_str val) = 0;

			virtual sx_str get_Credit() const = 0;
			virtual void set_Credit(sx_str val) = 0;

			virtual sx_time get_DateCreated() const = 0;
			virtual void set_DateCreated(const sx_time &val) = 0;

			virtual IRdfContainer* GetDocumentAncestors(sx_bool reset = false) const = 0;

			virtual sx_str get_Headline() const = 0;
			virtual void set_Headline(sx_str val) = 0;

			virtual sx_str get_History() const = 0;
			virtual void set_History(sx_str val) = 0;

			virtual sx_str get_ICCProfile() const = 0;
			virtual void set_ICCProfile(sx_str val) = 0;

			virtual sx_str get_Instructions() const = 0;
			virtual void set_Instructions(sx_str val) = 0;

			virtual sx_str get_Source() const = 0;
			virtual void set_Source(sx_str val) = 0;

			virtual sx_str get_State() const = 0;
			virtual void set_State(sx_str val) = 0;

			virtual IRdfContainer* GetSupplementalCategories(sx_bool reset = false) const = 0;

			virtual sx_str get_Title() const = 0;
			virtual void set_Title(sx_str val) = 0;

			virtual sx_str get_TransmissionReference() const = 0;
			virtual void set_TransmissionReference(sx_str val) = 0;

			virtual sx_uint get_Urgency() const = 0;
			virtual void set_Urgency(sx_uint val) = 0;
		};

		interface IAdobePDFScheme: public IScheme
		{
			virtual sx_str get_Keywords() const = 0;
			virtual void set_Keywords(sx_str val) = 0;

			virtual sx_str get_PDFVersion() const = 0;
			virtual void set_PDFVersion(sx_str val) = 0;

			virtual sx_str get_Producer() const = 0;
			virtual void set_Producer(sx_str val) = 0;
			
			virtual sx_str get_Trapped() const = 0;
			virtual void set_Trapped(sx_str val) = 0;
		};

		interface IPDFExtScheme : public IScheme
		{
			virtual sx_str GetProperty(sx_str name) const = 0;
			virtual void SetProperty(sx_str name, sx_str value) = 0;
		};

		interface IPDFAIdentScheme: public IScheme
		{
			virtual sx_str get_amd() const = 0;
			virtual void set_amd(sx_str val) = 0;

			virtual sx_str get_conformance() const = 0;
			virtual void set_conformance(sx_str val) = 0;

			virtual sx_uint get_part() const = 0;
			virtual void set_part(sx_uint val) = 0;
		};

		interface IPDFAExtScheme : public IScheme
		{
			virtual IRdfContainer* GetSchemas(sx_bool reset = false) const = 0;
			virtual IScheme* FindScheme(sx_str URI) = 0;
		};

		interface IZUGFeRDScheme: public IScheme
		{
			virtual sx_str get_DocumentType() const = 0;
			virtual void set_DocumentType(sx_str val) = 0;

			virtual sx_str get_DocumentFileName() const = 0;
			virtual void set_DocumentFileName(sx_str val) = 0;

			virtual sx_str get_Version() const = 0;
			virtual void set_Version(sx_str val) = 0;

			virtual sx_str get_ConformanceLevel() const = 0;
			virtual void set_ConformanceLevel(sx_str val) = 0;
		};

		interface IZUGFeRD2Scheme: public IScheme
		{
			virtual sx_str get_DocumentType() const = 0;
			virtual void set_DocumentType(sx_str val) = 0;

			virtual sx_str get_DocumentFileName() const = 0;
			virtual void set_DocumentFileName(sx_str val) = 0;

			virtual sx_str get_Version() const = 0;
			virtual void set_Version(sx_str val) = 0;

			virtual sx_str get_ConformanceLevel() const = 0;
			virtual void set_ConformanceLevel(sx_str val) = 0;
		};

		interface IFacturXScheme: public IScheme
		{
			virtual sx_str get_DocumentType() const = 0;
			virtual void set_DocumentType(sx_str val) = 0;

			virtual sx_str get_DocumentFileName() const = 0;
			virtual void set_DocumentFileName(sx_str val) = 0;

			virtual sx_str get_Version() const = 0;
			virtual void set_Version(sx_str val) = 0;

			virtual sx_str get_ConformanceLevel() const = 0;
			virtual void set_ConformanceLevel(sx_str val) = 0;
		};

		interface IPDFASchemaScheme : public IScheme
		{
			virtual sx_str get_namespaceURI() const = 0;
			virtual void set_namespaceURI(sx_str val) = 0;

			virtual sx_str get_prefix() const = 0;
			virtual void set_prefix(sx_str val) = 0;

			virtual sx_str get_schema() const = 0;
			virtual void set_schema(sx_str val) = 0;

			virtual IRdfContainer* GetProperty(sx_bool reset = false) const = 0;
			virtual IRdfContainer* GetValueType(sx_bool reset = false) const = 0;
		};

		interface IPDFAPropScheme : public IScheme
		{
			virtual sx_str get_category() const = 0;
			virtual void set_category(sx_str val) = 0;

			virtual sx_str get_description() const = 0;
			virtual void set_description(sx_str val) = 0;

			virtual sx_str get_name() const = 0;
			virtual void set_name(sx_str val) = 0;

			virtual sx_str get_valueType() const = 0;
			virtual void set_valueType(sx_str val) = 0;
		};

		interface IPDFAValTypeScheme : public IScheme
		{
			virtual sx_str get_description() const = 0;
			virtual void set_description(sx_str val) = 0;

			virtual sx_str get_namespaceURI() const = 0;
			virtual void set_namespaceURI(sx_str val) = 0;

			virtual sx_str get_prefix() const = 0;
			virtual void set_prefix(sx_str val) = 0;

			virtual sx_str get_type() const = 0;
			virtual void set_type(sx_str val) = 0;

			virtual IRdfContainer* GetField(sx_bool reset = false) const = 0;
		};

		interface IPDFAFieldScheme : public IScheme
		{
			virtual sx_str get_description() const = 0;
			virtual void set_description(sx_str val) = 0;

			virtual sx_str get_name() const = 0;
			virtual void set_name(sx_str val) = 0;

			virtual sx_str get_valueType() const = 0;
			virtual void set_valueType(sx_str val) = 0;
		};

		interface IResEventScheme : public IScheme
		{
			virtual sx_str get_action() const = 0;
			virtual void set_action(sx_str val) = 0;

			virtual sx_str get_instanceID() const = 0;
			virtual void set_instanceID(sx_str val) = 0;

			virtual sx_str get_parameters() const = 0;
			virtual void set_parameters(sx_str val) = 0;

			virtual sx_str get_softwareAgent() const = 0;
			virtual void set_softwareAgent(sx_str val) = 0;

			virtual sx_str get_when() const = 0;
			virtual void set_when(sx_str val) = 0;
		};

		interface ICustomScheme : public IScheme
		{
			virtual sx_str get_URI() const = 0;
			virtual sx_str get_Prefix() const = 0;
			
			virtual void Declare(sx_str prefix, sx_str TogetherWithURI = nullptr) = 0;
			virtual sx_str GetProperty(sx_str name) const = 0;
			virtual void SetProperty(sx_str name, sx_str value) = 0;

			virtual IRdfContainer* GetContainer(sx_str name) const = 0;
			virtual IRdfContainer* ResetContainer(sx_str name, rdf_cont_type t) const = 0;
		};

		interface IRdfProperty
		{
			virtual sx_str get_Value() const = 0;
			virtual void set_Value(sx_str s) = 0;
			virtual sx_str get_Lang() const = 0;
			virtual void set_Lang(sx_str s) = 0;
			virtual IScheme* GetSubScheme(sx_bool reset = false) = 0;
			virtual ICustomScheme* GetSubScheme(sx_str URI, sx_bool reset = false) = 0;
		};
	
		interface IRdfContainer
		{
			virtual rdf_cont_type get_Type() const = 0;

			virtual sx_uint get_Count() const = 0;
			virtual IRdfProperty* Item(sx_uint ind) const = 0;
			virtual IRdfProperty* Add() = 0;
			virtual void Move(sx_uint from, sx_uint to) = 0;
			virtual void Remove(sx_uint ind) = 0;
		};
	} // namespace XMP
} // namespace SX
