// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "../basic.h"
#include "../document.h"
#include "../misc.h"

namespace SX
{
	namespace Graphics { interface IBitmapData; }

	namespace ZUGFeRD
	{
		enum zf_profile
		{
			zf_profile_unknown  = 0,
			zf_profile_basic    = 1,
			zf_profile_comfort  = 2,
			zf_profile_extended = 3
		};
	
		enum zf_options
		{
			zf_options_def = 0,
			zf_options_pdfa_3b  = 1
		};

		//////////////////////////////////////////////////////////
		// QDT
		//////////////////////////////////////////////////////////

		// AllowanceChargeReasonCodeType
		enum reason_code
		{
			reason_code_unk = 0,
			reason_code_AA  = 'AA',
			reason_code_ADR = 'ADR',
			reason_code_AEO = 'AEO',
			reason_code_DI  = 'DI',
			reason_code_EAB = 'EAB',
			reason_code_FC  = 'FC',
			reason_code_IN  = 'IN',
			reason_code_MAC = 'MAC',
			reason_code_NAA = 'NAA',
			reason_code_PC  = 'PC',
			reason_code_RAA = 'RAA',
			reason_code_SH  = 'SH'
		};

		// DeliveryTermsCodeType
		enum delivery_code
		{
			delivery_code_unk = 0,
			delivery_code_EXW = 'EXW',
			delivery_code_FCA = 'FCA',
			delivery_code_FAS = 'FAS',
			delivery_code_FOB = 'FOB',
			delivery_code_CFR = 'CFR',
			delivery_code_CIF = 'CIF',
			delivery_code_DAT = 'DAT',
			delivery_code_DAP = 'DAP',
			delivery_code_CPT = 'CPT',
			delivery_code_CIP = 'CIP',
			delivery_code_DDP = 'DDP'
		};

		// DocumentCodeType
		enum doc_code
		{
			doc_code_unk = 0,
			doc_code_380 = '380',
			doc_code_84  = '84',
			doc_code_389 = '389'
		};

		// PaymentMeansCodeType
		enum payment_means_code
		{
			payment_means_code_unk = 0,
			payment_means_code_1   = '1',
			payment_means_code_3   = '3',
			payment_means_code_10  = '10',
			payment_means_code_20  = '20',
			payment_means_code_31  = '31',
			payment_means_code_42  = '42',
			payment_means_code_48  = '48',
			payment_means_code_49  = '49',
			payment_means_code_97  = '97'
		};

		// ReferenceCodeType
		enum ref_code
		{
			ref_code_unk = 0,
			ref_code_AAA = 'AAA',
			ref_code_AAB = 'AAB',
			ref_code_AAG = 'AAG',
			ref_code_AAJ = 'AAJ',
			ref_code_AAL = 'AAL',
			ref_code_AAM = 'AAM',
			ref_code_AAS = 'AAS',
			ref_code_ABT = 'ABT',
			ref_code_AER = 'AER',
			ref_code_AGG = 'AGG',
			ref_code_AJS = 'AJS',
			ref_code_ALQ = 'ALQ',
			ref_code_ALO = 'ALO',
			ref_code_API = 'API',
			ref_code_ASI = 'ASI',
			ref_code_AUD = 'AUD',
			ref_code_AWR = 'AWR',
			ref_code_BO  = 'BO',
			ref_code_BC  = 'BC',
			ref_code_CD  = 'CD',
			ref_code_DL  = 'DL',
			ref_code_MG  = 'MG',
			ref_code_OI  = 'OI',
			ref_code_PL  = 'PL',
			ref_code_PK  = 'PK',
			ref_code_POR = 'POR',
			ref_code_PP  = 'PP',
			ref_code_TIN = 'TIN',
			ref_code_VM  = 'VN'
		};

		// TaxCategoryCodeType
		enum tax_category_code
		{
			tax_category_code_unk = 0,
			tax_category_code_AE  = 'AE',
			tax_category_code_E   = 'E',
			tax_category_code_Z   = 'Z',
			tax_category_code_O   = 'O',
			tax_category_code_IC  = 'IC',
			tax_category_code_S   = 'S'
		};

		// TaxTypeCodeType
		enum tax_type_code
		{
			tax_type_code_unk              = 0,
			tax_type_code_ZF_INSURANCE_TAX = 1,
			tax_type_code_AAJ              = 'AAJ',
			tax_type_code_VAT              = 'VAT'
		};

		//////////////////////////////////////////////////////////
		// UDT
		//////////////////////////////////////////////////////////

		interface IElement
		{
			virtual sx_bool IsUsable() const = 0;
			virtual void Remove() = 0;
		};

		interface IProfileElement : public IElement
		{
			virtual zf_profile get_Profile() const = 0;
			virtual void set_Profile(zf_profile v) = 0;
		};

		// NumericType
		interface INumericElement : public IElement
		{
			virtual sx_double get_Value() const = 0;
			virtual void set_Value(sx_double v) = 0;
		};

		// AmountType
		interface IAmount : public INumericElement
		{
			virtual CurrCode3 get_currency() const = 0;
			virtual void set_currency(CurrCode3 v) = 0;
		};

		// CodeType
		interface ICodeElement : public IElement
		{
			virtual sx_str get_CodeID() const = 0;
			virtual void set_CodeID(sx_str v) = 0;
			virtual sx_str get_listID() const = 0;
			virtual void set_listID(sx_str v) = 0;
			virtual sx_str get_listVersionID() const = 0;
			virtual void set_listVersionID(sx_str v) = 0;
		};

		enum datetime_code
		{
			datetime_code_unk      = 0,
			datetime_code_CCYYMMDD = '102',
			datetime_code_CCYYMM   = '610',
			datetime_code_CCYYWW   = '616'
		};

		// DateTimeType
		interface IDateTime : public IElement
		{
			virtual sx_time get_Date() const = 0;
			virtual void set_Date(const sx_time& v) = 0;
			virtual datetime_code get_format() const = 0;
			virtual void set_format(datetime_code v) = 0;
		};

		// IDType
		interface IIDElement : public IElement
		{
			virtual sx_str get_ID() const = 0;
			virtual void set_ID(sx_str v) = 0;
			virtual sx_str get_schemeID() const = 0;
			virtual void set_schemeID(sx_str v) = 0;
			virtual sx_str get_schemeAgencyID() const = 0;
			virtual void set_schemeAgencyID(sx_str v) = 0;
		};

		enum measure_unit
		{
			measure_unit_unk = 0,
			measure_unit_C62 = 'C62',
			measure_unit_DAY = 'DAY',
			measure_unit_HAR = 'HAR',
			measure_unit_HUR = 'HUR',
			measure_unit_KGM = 'KGM',
			measure_unit_KTM = 'KTM',
			measure_unit_KWH = 'KWH',
			measure_unit_LS  = 'LS',
			measure_unit_LTR = 'LTR',
			measure_unit_MIN = 'MIN',
			measure_unit_MMK = 'MMK',
			measure_unit_MMT = 'MMT',
			measure_unit_MTK = 'MTK',
			measure_unit_MTQ = 'MTQ',
			measure_unit_MTR = 'MTR',
			measure_unit_NAR = 'NAR',
			measure_unit_NPR = 'NPR',
			measure_unit_P1  = 'P1',
			measure_unit_SET = 'SET',
			measure_unit_TNE = 'TNE',
			measure_unit_WEE = 'WEE'
		};

		// MeasureType
		interface IMeasuredQuantity : public INumericElement
		{
			virtual measure_unit get_unit() const = 0;
			virtual void set_unit(measure_unit v) = 0;
		};

		enum quantity_unit
		{
			quantity_unit_unk = 0,
			quantity_unit_BA  = 'BA',
			quantity_unit_BC  = 'BC',
			quantity_unit_BG  = 'BG',
			quantity_unit_BO  = 'BO',
			quantity_unit_BX  = 'BX',
			quantity_unit_CS  = 'CS',
			quantity_unit_CT  = 'CT',
			quantity_unit_CX  = 'CX',
			quantity_unit_NE  = 'NE',
			quantity_unit_PX  = 'PX',
			quantity_unit_RO  = 'RO',
			quantity_unit_SA  = 'SA'
		};

		// QuantityType
		interface IQuantity : public INumericElement
		{
			virtual quantity_unit get_unit() const = 0;
			virtual void set_unit(quantity_unit v) = 0;
		};

		// TextType
		interface ITextElement : public IElement
		{
			virtual sx_str get_Text() const = 0;
			virtual void set_Text(sx_str v) = 0;
		};

		//////////////////////////////////////////////////////////
		// RAM
		//////////////////////////////////////////////////////////

		// CreditorFinancialAccountType
		interface ICreditorAccount : public IElement
		{
			virtual ITextElement* GetName(sx_bool create = false) = 0;
			virtual IIDElement*   GetIBANID(sx_bool create = false) = 0;
			virtual IIDElement*   GetProprietaryID(sx_bool create = false) = 0;
		};

		// CreditorFinancialInstitutionType
		interface ICreditorInstitution : public IElement
		{
			virtual ITextElement* GetName(sx_bool create = false) = 0;
			virtual IIDElement*   GetBICID(sx_bool create = false) = 0;
			virtual IIDElement*   GetBLZID(sx_bool create = false) = 0;
		};

		// DebtorFinancialAccountType
		interface IDebtorAccount : public IElement
		{
			virtual IIDElement* GetIBANID(sx_bool create = false) = 0;
			virtual IIDElement* GetProprietaryID(sx_bool create = false) = 0;
		};

		// DebtorFinancialInstitutionType
		interface IDebtorInstitution : public IElement
		{
			virtual ITextElement* GetName(sx_bool create = false) = 0;
			virtual IIDElement*   GetBICID(sx_bool create = false) = 0;
			virtual IIDElement*   GetBLZID(sx_bool create = false) = 0;
		};

		// DocumentContextParameterType
		// see enum zf_profile
		
		interface INote;
		// DocumentLineDocumentType
		interface ILineDocument : public IElement
		{
			virtual IIDElement* GetLineID(sx_bool create = false) = 0;
			virtual INote*      GetIncludedNote(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// ExchangedDocumentContextType
		// see IInvoice

		interface ITimePeriod;
		interface ILanguageID : public IIDElement
		{
			virtual LangCode2 get_Lang() const = 0;
			virtual void set_Lang(LangCode2 v) = 0;
		};

		// ExchangedDocumentType
		interface IHeader : public IElement
		{
			virtual sx_bool get_Copy() const = 0;
			virtual void set_Copy(sx_bool v) = 0;
			virtual doc_code get_Type() const = 0;
			virtual void set_Type(doc_code v) = 0;

			virtual IIDElement*		GetID(sx_bool create = false) = 0;
			virtual ILanguageID*	GetLanguageID(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITextElement*	GetName(sx_bool create = false) = 0;
			virtual IDateTime*		GetIssueDate(sx_bool create = false) = 0;
			virtual ITimePeriod*	GetEffectivePeriod(sx_bool create = false) = 0;
			virtual INote*			GetIncludedNote(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		interface ITradeTax;
		// LogisticsServiceChargeType
		interface ILogisticsServiceCharge : public IElement
		{
			virtual ITextElement* GetDescription(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IAmount*      GetAppliedAmount(sx_bool create = false) = 0;
			virtual ITradeTax*    GetAppliedTradeTax(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		enum transport_means_code
		{
			transport_means_code_unk = 0,
			transport_means_code_1 = '1',
			transport_means_code_2 = '2',
			transport_means_code_3 = '3',
			transport_means_code_4 = '4',
			transport_means_code_5 = '5',
			transport_means_code_6 = '6',
			transport_means_code_7 = '7',
			transport_means_code_8 = '8',
			transport_means_code_9 = '9'
		};

		interface ITransportCode : public ICodeElement
		{
			virtual transport_means_code get_Code() const = 0;
			virtual void set_Code(transport_means_code v) = 0;
		};

		enum shipment_code
		{
			shipment_code_unk					= 0,
			shipment_code_SHIPMENT_REFERENCE	= 1,
			shipment_code_FLIGHT_NO				= 2,
			shipment_code_NUMBER_PLATE			= 3,
			shipment_code_SSCC					= 4,
			shipment_code_GINC					= 5,
			shipment_code_GSIN					= 6
		};

		interface IShipmentID : public IIDElement
		{
			virtual shipment_code get_Shipment() const = 0;
			virtual void set_Shipment(shipment_code v) = 0;
		};

		// LogisticsTransportMovementType
		interface ILogisticsTransportMovement : public IElement
		{
			virtual ITransportCode* GetMode(sx_bool create = false) = 0;
			virtual IShipmentID*    GetID(sx_bool create = false) = 0;
		};

		enum subj_code
		{
			subj_code_unk	= 0,
			subj_code_REG	= 'REG',
			subj_code_AAK	= 'AAK',
			subj_code_AAJ	= 'AAJ',
			subj_code_PMT	= 'PMT',
			subj_code_PRF	= 'PRF',
			subj_code_PRD	= 'PRD',
			subj_code_AAY	= 'AAY'
		};

		interface ISubjectCode : public ICodeElement
		{
			virtual subj_code get_Code() const = 0;
			virtual void set_Code(subj_code v) = 0;
		};

		enum cont_code
		{
			cont_code_unk	= 0,
			cont_code_ST1	= 'ST1',
			cont_code_ST2	= 'ST2',
			cont_code_ST3	= 'ST3',
			cont_code_EEV	= 'EEV',
			cont_code_WEV	= 'WEV',
			cont_code_VEV	= 'VEV'
		};

		interface IContentCode : public ICodeElement
		{
			virtual cont_code get_Code() const = 0;
			virtual void set_Code(cont_code v) = 0;
		};

		// NoteType
		interface INote : public IElement
		{
			virtual ITextElement*	GetContent(sx_bool create = false) = 0;
			virtual ISubjectCode*	GetSubjectCode(sx_bool create = false) = 0;
			virtual IContentCode*	GetContentCode(sx_bool create = false) = 0;
		};

		enum prod_prop_code
		{
			prod_prop_code_unk                  = 0,
			prod_prop_code_LENGTH               = 1,
			prod_prop_code_WIDTH                = 2,
			prod_prop_code_HEIGHT               = 3,
			prod_prop_code_DIAMETER             = 4,
			prod_prop_code_WEIGHT_GROSS         = 5,
			prod_prop_code_WEIGHT_NET           = 6,
			prod_prop_code_VOLUME_GROSS         = 7,
			prod_prop_code_VOLUME_NET           = 8,
			prod_prop_code_SIZE_TEXT            = 9,
			prod_prop_code_SIZE_CODE            = 10,
			prod_prop_code_SERIAL_NUMBER        = 11,
			prod_prop_code_LOT_NUMBER           = 12,
			prod_prop_code_EXPIRATION_DATE      = 13,
			prod_prop_code_MATERIAL             = 14,
			prod_prop_code_PACKAGING_MATERIAL   = 15,
			prod_prop_code_PACKAGING_TYPE       = 16,
			prod_prop_code_COLOR_TEXT           = 17,
			prod_prop_code_COLOR_CODE           = 18,
			prod_prop_code_ENERGY_CLASS         = 19,
			prod_prop_code_WEEE_NUMBER          = 20,
			prod_prop_code_PROMOTIONAL_VARIANT  = 21,
			prod_prop_code_METER_LOCATION       = 22,
			prod_prop_code_METER_NUMBER         = 23,
			prod_prop_code_SEAL_NUMBER          = 24,
			prod_prop_code_WASTE_CODE           = 25,
			prod_prop_code_WASTE_FRACTION       = 26,
			prod_prop_code_ORGANIC_CONTROL_BODY = 27,
			prod_prop_code_KIND_OF_ARTICLE      = 28,
			prod_prop_code_FEE                  = 29,
			prod_prop_code_COMMISSION           = 30,
			prod_prop_code_TRADING_UNIT         = 31,
			prod_prop_code_DEPOSIT_SYSTEM       = 32,
			prod_prop_code_DEPOSIT_TYPE         = 33
		};

		interface IProductCharacteristicCode : public ICodeElement
		{
			virtual prod_prop_code get_Code() const = 0;
			virtual void set_Code(prod_prop_code v) = 0;
		};

		// ProductCharacteristicType
		interface IProductCharacteristic : public IElement
		{
			virtual IProductCharacteristicCode* GetType(sx_bool create = false) = 0;
			virtual ITextElement*               GetDescription(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IMeasuredQuantity*          GetMeasure(sx_bool create = false) = 0;
			virtual ITextElement*               GetValue(sx_bool create = false) = 0;
		};

		enum prod_class_code
		{
			prod_class_code_unk             = 0,
			prod_class_code_GPC             = 1,
			prod_class_code_ECL             = 2,
			prod_class_code_UNSPSC          = 3,
			prod_class_code_HS              = 4,
			prod_class_code_CBV             = 5,
			prod_class_code_SELLER_ASSIGNED = 6,
			prod_class_code_BUYER_ASSIGNED  = 7
		};

		interface IProductClassificationCode: public ICodeElement
		{
			virtual prod_class_code get_Code() const = 0;
			virtual void set_Code(prod_class_code v) = 0;
		};

		// ProductClassificationType
		interface IProductClassification : public IElement
		{
			virtual IProductClassificationCode* GetCode(sx_bool create = false) = 0;
			virtual ITextElement*               GetName(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// ReferencedDocumentType
		interface IReferencedDocument : public IElement
		{
			virtual doc_code get_Type() const = 0;
			virtual void set_Type(doc_code v) = 0;
			virtual ref_code get_ReferenceType() const = 0;
			virtual void set_ReferenceType(ref_code v) = 0;
			virtual sx_time get_IssueDate() const = 0;
			virtual void set_IssueDate(const sx_time& v) = 0;

			virtual IIDElement* GetID(sx_bool create = false) = 0;
			virtual IIDElement* GetLineID(sx_bool create = false) = 0;
		};

		const sx_uint ICD_SWIFT		= 21;
		const sx_uint ICD_GS1_GLN	= 88;
		const sx_uint ICD_GS1_GTIN	= 160;
		
		interface IGlobalID : public IIDElement
		{
			virtual sx_uint get_ICD() const = 0;
			virtual void set_ICD(sx_uint v) = 0;
		};

		// ReferencedProductType
		interface IReferencedProduct: public IElement
		{
			virtual IGlobalID*		GetGlobalID(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IIDElement*		GetSellerID(sx_bool create = false) = 0;
			virtual IIDElement*		GetBuyerID(sx_bool create = false) = 0;
			virtual ITextElement*	GetName(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITextElement*	GetDescription(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IMeasuredQuantity*	GetUnitQuantity(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// SpecifiedPeriodType
		interface ITimePeriod : public IElement
		{
			virtual IDateTime* GetStartDate(sx_bool create = false) = 0;
			virtual IDateTime* GetEndDate(sx_bool create = false) = 0;
			virtual IDateTime* GetCompleteDate(sx_bool create = false) = 0;
		};

		// SupplyChainConsignmentType
		interface IChainConsignment : public IElement
		{
			virtual ILogisticsTransportMovement* GetTransportMovement(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// SupplyChainEventType
		interface IChainEvent : public IElement
		{
			virtual IDateTime* GetOccurrenceDate(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		interface ITradeParty;
		interface ITradeDeliveryTerms;
		interface ITradePrice;
		// SupplyChainTradeAgreementType
		interface ITradeAgreement : public IElement
		{
			virtual ITextElement*        GetBuyerReference(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradeParty*         GetSellerParty(sx_bool create = false) = 0;
			virtual ITradeParty*         GetBuyerParty(sx_bool create = false) = 0;
			virtual ITradeParty*         GetEndUserParty(sx_bool create = false) = 0;
			virtual ITradeDeliveryTerms* GetDeliveryTerms(sx_bool create = false) = 0;
			virtual IReferencedDocument* GetContract(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IReferencedDocument* GetBuyerOrder(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IReferencedDocument* GetAdditionalDocument(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradePrice*         GetGrossProductPrice(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradePrice*         GetNetProductPrice(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IReferencedDocument* GetCustomerOrder(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// SupplyChainTradeDeliveryType
		interface ITradeDelivery : public IElement
		{
			virtual IMeasuredQuantity*		GetBilledQuantity(sx_bool create = false) = 0;
			virtual IMeasuredQuantity*		GetChargeFreeQuantity(sx_bool create = false) = 0;
			virtual IQuantity*			GetPackageQuantity(sx_bool create = false) = 0;
			virtual IChainConsignment*		GetChainConsignment(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradeParty*			GetShipFromParty(sx_bool create = false) = 0;
			virtual ITradeParty*			GetShipToParty(sx_bool create = false) = 0;
			virtual ITradeParty*			GetUltimateShipToParty(sx_bool create = false) = 0;
			virtual IChainEvent*			GetDeliveryChainEvent(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IReferencedDocument*	GetDespatchAdviceDocument(sx_bool create = false) = 0;
			virtual IReferencedDocument*	GetReceivingAdviceDocument(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IReferencedDocument*	GetDeliveryNoteDocument(sx_bool create = false) = 0;
		};		

		interface ITradeSettlement;
		interface ITradeProduct;
		// SupplyChainTradeLineItemType
		interface ITradeLineItem : public IElement
		{
			virtual ILineDocument*		GetAssociatedDocument(sx_bool create = false) = 0;
			virtual ITradeAgreement*	GetAgreement(sx_bool create = false) = 0;
			virtual ITradeDelivery*		GetDelivery(sx_bool create = false) = 0;
			virtual ITradeSettlement*	GetSettlement(sx_bool create = false) = 0;
			virtual ITradeProduct*		GetProduct(sx_bool create = false) = 0;
		};

		interface IAccountingAccount;
		interface IPaymentMeans;
		interface ITradeAllowanceCharge;
		interface IPaymentTerms;
		interface IMonetarySummation;
		// SupplyChainTradeSettlementType
		interface ITradeSettlement : public IElement
		{
			virtual CurrCode3 get_InvoiceCurrency() const = 0;
			virtual void set_InvoiceCurrency(CurrCode3 v) = 0;

			virtual ITextElement*				GetPaymentReference(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradeParty*				GetInvoiceeParty(sx_bool create = false) = 0;
			virtual ITradeParty*				GetPayeeParty(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IPaymentMeans*				GetPaymentMeans(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradeTax*					GetApplicableTradeTax(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITimePeriod*				GetBillingSpecifiedPeriod(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradeAllowanceCharge*		GetAllowanceCharge(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ILogisticsServiceCharge*	GetLogisticsServiceCharge(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IPaymentTerms*			GetSpecifiedPaymentTerms(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IAccountingAccount*			GetSpecifiedAccountingAccount(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IMonetarySummation*			GetMonetarySummation(sx_bool create = false) = 0;
			virtual IAccountingAccount*			GetReceivableAccountingAccount(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// SupplyChainTradeTransactionType
		interface ITradeTransaction : public IElement
		{
			virtual ITradeAgreement*	GetAgreement(sx_bool create = false) = 0;
			virtual ITradeDelivery*		GetDelivery(sx_bool create = false) = 0;
			virtual ITradeSettlement*	GetSettlement(sx_bool create = false) = 0;
			virtual ITradeLineItem*		GetLineItem(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		enum tax_code
		{
			tax_code_unk = 0,
			tax_code_VA  = 'VA',
			tax_code_FC  = 'FC'
		};

		interface ITaxID : public IIDElement
		{
			virtual tax_code get_TaxScheme() const = 0;
			virtual void set_TaxScheme(tax_code v) = 0;
		};

		// TaxRegistrationType
		interface ITaxRegistration : public IElement
		{
			virtual ITaxID* GetID(sx_bool create = false) = 0;
		};

		// TradeAccountingAccountType
		interface IAccountingAccount : public IElement
		{
			virtual IIDElement* GetID(sx_bool create = false) = 0;
		};

		// TradeAddressType
		interface ITradeAddress : public IElement
		{
			virtual CountryCode2 get_Country() const = 0;
			virtual void set_Country(CountryCode2 v) = 0;
			virtual sx_str get_Postcode() const = 0;
			virtual void set_Postcode(sx_str v) = 0;

			virtual ITextElement* GetCity(sx_bool create = false) = 0;
			virtual ITextElement* GetLineOne(sx_bool create = false) = 0;
			virtual ITextElement* GetLineTwo(sx_bool create = false) = 0;
		};

		// TradeAllowanceChargeType
		interface ITradeAllowanceCharge : public IElement
		{
			virtual sx_bool get_Charge() const = 0;
			virtual void set_Charge(sx_bool v) = 0;
			virtual reason_code get_ReasonCode() const = 0;
			virtual void set_ReasonCode(reason_code v) = 0;

			virtual INumericElement* GetSequence(sx_bool create = false) = 0;
			virtual INumericElement* GetCalculationPercent(sx_bool create = false) = 0;
			virtual IAmount*         GetBasisAmount(sx_bool create = false) = 0;
			virtual IMeasuredQuantity*       GetBasisQuantity(sx_bool create = false) = 0;
			virtual IAmount*         GetActualAmount(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITextElement*    GetReason(sx_bool create = false) = 0;
			virtual ITradeTax*       GetCategoryTax(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		interface ICommunication;
		// TradeContactType
		interface ITradeContact : public IElement
		{
			virtual ITextElement*   GetPerson(sx_bool create = false) = 0;
			virtual ITextElement*   GetDepartment(sx_bool create = false) = 0;
			virtual ICommunication* GetPhone(sx_bool create = false) = 0;
			virtual ICommunication* GetFax(sx_bool create = false) = 0;
			virtual ICommunication* GetEmail(sx_bool create = false) = 0;
		};

		// TradeCountryType
		interface ITradeCountry : public IElement
		{
			virtual CountryCode2 GetCode(sx_uint ind = 0) const = 0;
			virtual void SetCode(CountryCode2 v, sx_uint ind = 0) = 0;
		};

		// TradeDeliveryTermsType
		interface ITradeDeliveryTerms : public IElement
		{
			virtual delivery_code get_Code() const = 0;
			virtual void set_Code(delivery_code v) = 0;
		};

		// TradePartyType
		interface ITradeParty : public IElement
		{
			virtual IIDElement*       GetID(sx_bool create = false) = 0;
			virtual IGlobalID*        GetGlobalID(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITextElement*     GetName(sx_bool create = false) = 0;
			virtual ITradeContact*    GetContact(sx_bool create = false) = 0;
			virtual ITradeAddress*    GetAddress(sx_bool create = false) = 0;
			virtual ITaxRegistration* GetTaxRegistration(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// TradePaymentDiscountTermsType
		interface IPaymentDiscountTerms : public IElement
		{
			virtual IDateTime*         GetBasisDate(sx_bool create = false) = 0;
			virtual IMeasuredQuantity* GetBasisPeriod(sx_bool create = false) = 0;
			virtual IAmount*           GetBasisAmount(sx_bool create = false) = 0;
			virtual INumericElement*   GetCalculationPercent(sx_bool create = false) = 0;
			virtual IAmount*           GetActualDiscountAmount(sx_bool create = false) = 0;
		};

		// TradePaymentPenaltyTermsType
		interface IPaymentPenaltyTerms : public IElement
		{
			virtual IDateTime*         GetBasisDate(sx_bool create = false) = 0;
			virtual IMeasuredQuantity* GetBasisPeriod(sx_bool create = false) = 0;
			virtual IAmount*           GetBasisAmount(sx_bool create = false) = 0;
			virtual INumericElement*   GetCalculationPercent(sx_bool create = false) = 0;
			virtual IAmount*           GetActualPenaltyAmount(sx_bool create = false) = 0;
		};

		// TradePaymentTermsType
		interface IPaymentTerms : public IElement
		{
			virtual ITextElement*			GetDescription(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IDateTime*				GetDueDate(sx_bool create = false) = 0;
			virtual IAmount*				GetPartialPayment(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IPaymentPenaltyTerms*	GetPenaltyTerms(sx_bool create = false) = 0;
			virtual IPaymentDiscountTerms*	GetDiscountTerms(sx_bool create = false) = 0;
		};

		// TradePriceType
		interface ITradePrice : public IElement
		{
			virtual IAmount*               GetCharge(sx_bool create = false) = 0;
			virtual IMeasuredQuantity*             GetQuantity(sx_bool create = false) = 0;
			virtual ITradeAllowanceCharge* GetAllowanceCharge(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// TradeProductType
		interface ITradeProduct : public IElement
		{
			virtual IIDElement*             GetGlobalID(sx_bool create = false) = 0;
			virtual IIDElement*             GetSellerID(sx_bool create = false) = 0;
			virtual IIDElement*             GetBuyerID(sx_bool create = false) = 0;
			virtual ITextElement*           GetName(sx_bool create = false) = 0;
			virtual ITextElement*           GetDescription(sx_bool create = false) = 0;
			virtual IProductCharacteristic* GetCharacteristic(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IProductClassification* GetClassification(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITradeCountry*          GetCountry(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IReferencedProduct*     GetReferencedProduct(sx_uint ind = 0, sx_bool create = false) = 0;
		};

		// TradeSettlementMonetarySummationType
		interface IMonetarySummation : public IElement
		{
			virtual IAmount* GetLineTotal(sx_bool create = false) = 0;
			virtual IAmount* GetChargeTotal(sx_bool create = false) = 0;
			virtual IAmount* GetAllowanceTotal(sx_bool create = false) = 0;
			virtual IAmount* GetTaxBasisTotal(sx_bool create = false) = 0;
			virtual IAmount* GetTaxTotal(sx_bool create = false) = 0;
			virtual IAmount* GetGrandTotal(sx_bool create = false) = 0;
			virtual IAmount* GetTotalPrepaid(sx_bool create = false) = 0;
			virtual IAmount* GetTotalAllowanceCharge(sx_bool create = false) = 0;
			virtual IAmount* GetDuePayable(sx_bool create = false) = 0;
		};

		// TradeSettlementPaymentMeansType
		interface IPaymentMeans : public IElement
		{
			virtual payment_means_code get_Type() const = 0;
			virtual void set_Type(payment_means_code v) = 0;

			virtual ITextElement*         GetInformation(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IIDElement*           GetID(sx_bool create = false) = 0;
			virtual IDebtorAccount*       GetDebtorAccount(sx_bool create = false) = 0;
			virtual ICreditorAccount*     GetCreditorAccount(sx_bool create = false) = 0;
			virtual IDebtorInstitution*   GetDebtorInstitution(sx_bool create = false) = 0;
			virtual ICreditorInstitution* GetCreditorInstitution(sx_bool create = false) = 0;
		};

		// TradeTaxType
		interface ITradeTax : public IElement
		{
			virtual tax_type_code get_Type() const = 0;
			virtual void set_Type(tax_type_code v) = 0;
			virtual tax_category_code get_Category() const = 0;
			virtual void set_Category(tax_category_code v) = 0;

			virtual IAmount*         GetCalculatedAmount(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual ITextElement*    GetExemptionReason(sx_bool create = false) = 0;
			virtual IAmount*         GetBasisAmount(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IAmount*         GetLineTotalBasisAmount(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual IAmount*         GetAllowanceChargeBasisAmount(sx_uint ind = 0, sx_bool create = false) = 0;
			virtual INumericElement* GetApplicablePercent(sx_bool create = false) = 0;
		};

		// UniversalCommunicationType
		interface ICommunication : public IElement
		{
			virtual ITextElement* GetCompleteNumber(sx_bool create = false) = 0;
			virtual IIDElement*   GetURI(sx_bool create = false) = 0;
		};

		//////////////////////////////////////////////////////////
		// ZUGFeRD Invoice
		//////////////////////////////////////////////////////////
		interface IValidationStatus;
		
		interface IInvoice
		{
			virtual sx_bool get_Test() const = 0;
			virtual void set_Test(sx_bool v) = 0;
			
			virtual IProfileElement*	GetProfile(sx_bool create = false) = 0;
			virtual IHeader*			GetHeader(sx_bool create = false) = 0;
			virtual ITradeTransaction*	GetTransaction(sx_bool create = false) = 0;
			
			virtual sx_bool Validate(sx_flags options = 0, IValidationStatus* pStat = nullptr) = 0;
		};
	
		interface IInvoiceStream : public IRefObject
		{
			virtual IInvoice* get_Invoice() const = 0;

			virtual void Open(ISequentialStream* pStream) = 0;
			virtual void Save(ISequentialStream* pStream) = 0;
			virtual void Reset() = 0;
		};

		enum zf_status
		{
			zf_status_not_pdf	= 1,
			zf_status_enc_pdf	= 2,
			zf_status_no_meta	= 3,
			zf_status_err_meta	= 4,
			zf_status_not_pdfa	= 5,
			zf_status_err_emb	= 6,
			zf_status_not_zf	= 7,
			zf_status_dif_prof	= 8,
			zf_status_valid		= 10
		};

		const object_type obj_type_invoice_document = 0x21;

		interface IInvoiceDocument : public IBaseDocument
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual IInvoice* get_Invoice() const = 0;
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str s) = 0;
			virtual IStr* get_Subject() const = 0;
			virtual void set_Subject(sx_str s) = 0;
			virtual IStr* get_Author() const = 0;
			virtual void set_Author(sx_str s) = 0;
			virtual IStr* get_Creator() const = 0;
			virtual void set_Creator(sx_str s) = 0;
			virtual sx_flags get_Options() const = 0;
			virtual void set_Options(sx_flags val) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual zf_status OpenStream(ISequentialStream* pStream, sx_bool ValidatePDFA = false) = 0;
			virtual zf_status OpenFile(sx_str sFilePath, sx_bool ValidatePDFA = false) = 0;
			virtual void CreateFromPDF(ISequentialStream* pStream) = 0;
			virtual void CreateFromPDFInvoice(ISequentialStream* pPDF, ISequentialStream* pInvoice) = 0;
			virtual void CreateFromDocument(IBaseDocument* pDoc, ISequentialStream* pInvoice = nullptr) = 0;
			virtual void AddPageFromImage(Graphics::IBitmapData* pImage, sx_double width = 0., sx_double height = 0.) = 0;
			virtual void SaveAsStream(ISequentialStream* pStream) = 0;
			virtual void SaveAsFile(sx_str sFilePath) = 0;
		};

		enum zf_val_option
		{
			zf_val_option_namespace	= 0x00,
			zf_val_option_schema	= 0x01,
			zf_val_option_rules		= 0x02,
			zf_val_option_codes		= 0x04,
			zf_val_option_profile	= 0x08
		};

		enum zf_val_code
		{
			zf_val_code_schema			= 1,
			zf_val_code_attr_notused	= 2,
			zf_val_code_elem_notused	= 3,
			zf_val_code_attr_req		= 4,
			zf_val_code_elem_req		= 5,
			zf_val_code_elem_one_req	= 6,
			zf_val_code_elem_one_opt	= 7,
			zf_val_code_value_req		= 8,
			zf_val_code_unknown_code	= 9,
			zf_val_code_out_of_profile	= 10
		};

		interface IValidationMessage
		{
			virtual sx_bool		get_CriticalError() const = 0;
			virtual zf_val_code	get_Code() const = 0;
			virtual sx_str		get_ExtendedInfo() const = 0;
		};

		interface IValidationStatus
		{
			virtual sx_bool OnMessage(IValidationMessage* pMess) = 0;
		};
	} // namespace ZUGFeRD
} // namespace SX
