// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once
#include "../basic.h"

namespace SX
{

	namespace Graphics { interface IInkImage; }

	namespace XPS
	{
		object_type obj_type_xps_annot = 0x1C;

		enum annot_type
		{
			annot_type_undefined	= 0,
			annot_type_text			= 0x00000001,
			annot_type_highlight	= 0x00000100,
			annot_type_ink			= 0x00004000
		};

		const sx_flags annot_all = annot_type_text | annot_type_highlight | annot_type_ink;;

		interface IInkLineSegment;

		//////////////////////////////////////////////////////////////////////////
		// annotation
		interface IAnnot : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual annot_type get_Type() const = 0;
			virtual IDocument* get_Doc() const = 0;
			virtual IPage* get_Page() const = 0;
			virtual IStr* get_Ident() const = 0;
			virtual void set_Ident(sx_str sIdent) = 0;
			virtual Graphics::IRegion* get_Region() const = 0;
			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_bool GetCreateDate(sx_time& time) const = 0;
			virtual sx_bool GetModDate(sx_time& time) const = 0;
			virtual void SetModDate(const sx_time& time) = 0;
			virtual void SetRegion(Graphics::IRegion* pReg) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// text markup annotations (highlight)
		interface ITextMarkup : public IAnnot
		{
			virtual const sx_color* GetColor() const = 0;
			virtual void SetColor(const sx_color& c) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IStickyNote : public IAnnot
		{
			virtual IStr* get_XAMLData() const = 0;
			virtual void set_XAMLData(sx_str sXAML) = 0;

			virtual sx_bool GetPopupArea(sx_rect& box) const = 0;
			virtual void SetPopupArea(const sx_rect& box) = 0;
			virtual sx_bool IsPopup() const = 0;
			virtual void OpenPopup(sx_bool bOpen = true) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IInkNote : public IAnnot
		{
			virtual ISequentialStream* get_InkData() const = 0;
			virtual void set_InkData(ISequentialStream* data) = 0;
			virtual Graphics::IInkImage* get_InkImage() const = 0;
			virtual void Reset(sx_bool bClear = false) = 0;

			virtual sx_bool GetPopupArea(sx_rect& box) const = 0;
			virtual void SetPopupArea(const sx_rect& box) = 0;
			virtual sx_bool IsPopup() const = 0;
			virtual void OpenPopup(sx_bool bOpen = true) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// annotations
		interface IAnnots
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IAnnot* Item(sx_uint uIndex) = 0;
			virtual IAnnot* ItemFromPage(sx_uint uPage, IAnnot* pPrev = nullptr) = 0;
			virtual IAnnot* ItemByLocation(sx_uint uPage, const sx_point &p, IAnnot* pPrev = nullptr) = 0;
			virtual IAnnot* Add(sx_uint uPage, annot_type type) = 0;
			virtual void Remove(IAnnot* pAnnot) = 0;
			virtual void RemoveFromPage(sx_uint uPage) = 0;
			virtual void RemoveAll() = 0;
		};
	} // namespace PDF
} // namespace SX