// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once
#include "../document.h"

namespace SX
{
	namespace Graphics { interface IBitmapData; }
	namespace PDF { interface IPage; }

	namespace XPS
	{
		const object_type obj_type_xps_document = 0x23;
		interface IDocStructure;
		interface INamedDests;
		interface IPage;
		interface ISignatureSpots;
		
		interface IDocument : public IBaseDocument
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual INamedDests* get_NamedDests()  = 0;
			virtual ISignatureSpots* get_SignatureSpots() = 0;
			virtual sx_uint get_Count() = 0; // pages

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IDocStructure* GetStructure(sx_bool bCreate = false) = 0;

			virtual IPage* Item(sx_uint uIndex) = 0;
			virtual IPage* AddPage() = 0;
			virtual IPage* InsertPage(sx_uint uTo) = 0;
			virtual void   MovePage(sx_uint uFrom, sx_uint uTo) = 0;
			virtual void   RemovePage(sx_uint uIndex) = 0;
			virtual sx_uint IndexOfPage(IPage* pPage) = 0;
			virtual IPage* CopyPage(IPage* pPage, sx_uint uTo, sx_uint uReserved = 0) = 0;
			virtual IPage* CopyPage(PDF::IPage* pPage, sx_uint uTo, sx_uint uReserved = 0) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& back, sx_uint uDPI = 96, sx_flags opt = 0, sx_uint start = 0, sx_uint end = INVALID_INDEX) = 0;
			virtual void Compact() = 0;
			virtual void SetAutoCompact(sx_uint uNumDeployedPages) = 0;

			virtual void StartCombine(IBaseDocument* pSourceDoc, sx_flags CombineOptions = 0) = 0;
			virtual void CombinePage(sx_uint uFrom, sx_uint uTo, sx_flags CombineOptions = 0) = 0; 
			virtual void CombineAllPages(sx_uint uTo, sx_flags CombineOptions = 0, ICombineStatus* pStatus = nullptr) = 0;
			virtual void EndCombine() = 0;

		};

	} // namespace XPS
} // namespace SX
