// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once

#include "../viewer.h"
#include "../search.h"
#include "../print.h"
#include "../document.h"

namespace SX
{
	namespace Graphics { interface IBitmapData; }

	namespace XPS
	{
		interface ICoreProperties;
		interface IDocuments;
		interface IDocument;
		interface IResources;
		interface ISignatures;
		interface IImage;
		interface IAnnots;

		const object_type obj_type_xps_package = 0x22;

		enum package_format
		{
			package_format_xps = 1,
			package_format_oxps
		};

		//////////////////////////////////////////////////////////////////////////
		// XPS package
		interface IPackage : public IBaseDocument
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_str get_FilePath() const = 0;
			virtual sx_bool get_Modified() const = 0;
			virtual package_format get_Format() const = 0;
			virtual void set_Format(package_format format) = 0;
			virtual IImage* get_Thumbnail() = 0;
			virtual ICoreProperties* get_Properties() = 0;
			virtual IDocuments* get_Documents()  = 0;
			virtual IResources* get_Resources() = 0;
			virtual ISignatures* get_Signatures() = 0;
			virtual IAnnots* get_Annots() = 0; 

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void OpenStream(ISequentialStream* pStream) = 0;
			virtual void OpenFile(sx_str sFilePath) = 0;
			virtual sx_qword SaveAsStream(ISequentialStream* pStream) = 0;
			virtual void SaveAsFile(sx_str sFilePath) = 0;
			virtual sx_uint GetPageIndex(SX::XPS::IDocument *pDoc, sx_uint pageInDoc) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& back, sx_uint uDPI = 96, sx_flags opt = 0, sx_uint start = 0, sx_uint end = INVALID_INDEX) = 0;
			virtual void SetThumbnail(sx_uint maxw, sx_uint maxh) = 0;
			virtual void ResetProperties(sx_bool bClear = false) = 0;
			virtual sx_bool IsClear() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// document collection in XPS package
		interface IDocuments
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_uint get_Count() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual IDocument* Item(sx_uint uIndex) = 0;
			virtual IDocument* Add() = 0;
			virtual void Move(sx_uint uFrom, sx_uint uTo) = 0;
			virtual void Remove(sx_uint uIndex) = 0;
		};

		interface ICoreProperties
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_str get_Category() const = 0;
			virtual void set_Category(const sx_str pStr) = 0;
			virtual sx_str get_ContentStatus() const = 0;
			virtual void set_ContentStatus(const sx_str pStr) = 0;
			virtual sx_str get_ContentType() const = 0;
			virtual void set_ContentType(const sx_str pStr) = 0;
			virtual sx_str get_Creator() const = 0;
			virtual void set_Creator(const sx_str pStr) = 0;
			virtual sx_str get_Description() const = 0;
			virtual void set_Description(const sx_str pStr) = 0;
			virtual sx_str get_Identifier() const = 0;
			virtual void set_Identifier(const sx_str pStr) = 0;
			virtual sx_str get_Keywords() const = 0;
			virtual void set_Keywords(const sx_str pStr) = 0;
			virtual sx_str get_Language() const = 0;
			virtual void set_Language(const sx_str pStr) = 0;
			virtual sx_str get_LastModifiedBy() const = 0;
			virtual void set_LastModifiedBy(const sx_str pStr) = 0;
			virtual sx_str get_Revision() const = 0;
			virtual void set_Revision(const sx_str pStr) = 0;
			virtual sx_str get_Subject() const = 0;
			virtual void set_Subject(const sx_str pStr) = 0;
			virtual sx_str get_Title() const = 0;
			virtual void set_Title(const sx_str pStr) = 0;
			virtual sx_str get_Version() const = 0;
			virtual void set_Version(const sx_str pStr) = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual sx_bool GetCreated(sx_time& time) const = 0;
			virtual void SetCreated(const sx_time& time) = 0;
			virtual sx_bool GetModified(sx_time& time) const = 0;
			virtual void SetModified(const sx_time& time) = 0;
			virtual sx_bool GetLastPrinted(sx_time& time) const = 0;
			virtual void SetLastPrinted(const sx_time& time) = 0;
		};
	} // namespace XPS
} // namespace SX
