// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once

#include "../graphics.h"

namespace SX
{
	namespace XPS
	{
		const object_type 
			obj_type_font = 0x1E,
			obj_type_image = 0x1F;

		//////////////////////////////////////////////////////////////////////////
		interface IImage : public IRefObject
		{
			virtual sx_uint get_Width() const = 0;
			virtual sx_uint get_Height() const = 0;
			virtual Graphics::ImageFormat get_Format() const = 0;
			virtual Graphics::PixelFormat get_PixelFormat() const = 0;
			virtual Graphics::IBitmapData* GetBitmapData() const = 0;

		};

		//////////////////////////////////////////////////////////////////////////
		interface IFont : public IRefObject
		{
			virtual IStr* get_PostScriptName() const = 0;
			virtual IStr* get_FamilyName() const = 0;
			virtual sx_uint get_Weight() const = 0;
			virtual sx_uint get_Stretch() const = 0;
			virtual sx_bool get_Italic() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IResources
		{
			virtual void Rescan(sx_bool bClear = false) = 0;

			virtual sx_uint get_Images() const = 0;
			virtual IStr* GetImageKey(sx_uint uIndex) const = 0;
			virtual IImage* GetImage(sx_str sKey) = 0;
			virtual IImage* AddImage(ISequentialStream* pStream, Graphics::ImageFormat format, sx_str pKey = nullptr) = 0;
			virtual IImage* AddBitmap(Graphics::IBitmapData* pBitmap, Graphics::ImageFormat format, sx_str pKey = nullptr) = 0;


			virtual sx_uint get_Fonts() const = 0;
			virtual IStr* GetFontKey(sx_uint uIndex) const = 0;
			virtual IFont* GetFont(sx_str sKey) = 0;
			virtual IFont* AddFont(sx_str FontFamily, sx_uint weight = 400, sx_uint style = 0, sx_uint stretch = 5, sx_str pKey = nullptr) = 0;
			virtual IFont* AddCustomFont(ISequentialStream* pStream, sx_str pKey = nullptr) = 0;

		};

	} // namespace XPS
} // namespace SX
