// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once
#include "../basic.h"

namespace SX
{
	namespace XPS
	{
		const object_type obj_type_xps_signature = 0x26;

		enum signature_status
		{
			signature_status_unknown,
			signature_status_unsigned,
			signature_status_invalid,
			signature_status_valid
		};

		enum signature_format
		{
			signature_format_unknown,
			signature_format_rsa_sha1	= 1,
			signature_format_rsa_sha256	= 256,
			signature_format_rsa_sha384	= 384,
			signature_format_rsa_sha512	= 512
		};

		enum signature_digest
		{
			signature_digest_unknown,
			signature_digest_sha1	= 1,
			signature_digest_sha256	= 256,
			signature_digest_sha384	= 384,
			signature_digest_sha512	= 512
		};

		enum signature_optional_parts
		{
			signature_optional_parts_none,
			signature_optional_parts_sign_orig	= 0x001,
			signature_optional_parts_sign_certs	= 0x002,
			signature_optional_parts_core_props	= 0x010,
			signature_optional_parts_prn_ticket	= 0x020,
			signature_optional_parts_disc_ctrl	= 0x040,
			signature_optional_parts_all		= 0xFFF
		};

		interface ISignature : IRefObject
		{
			virtual IStr* get_ID() const = 0;
			virtual signature_status get_Status() const = 0;
			virtual IStr* get_Signer() const = 0;
			virtual ICryptoCert* get_SignerCert() const = 0;
			virtual signature_format get_Format() const = 0;
			virtual signature_digest get_Digest() const = 0;
			virtual IStr* get_TimeFormat() const = 0;
			virtual sx_bool GetSigningTime(sx_time& t) = 0;
			virtual void SetOptions( signature_format format = signature_format_rsa_sha1, signature_digest digest = signature_digest_sha1, signature_optional_parts parts = signature_optional_parts_core_props, sx_str timeformat = nullptr) = 0;
			virtual void Sign( ICryptoCert* pCert, ICryptoKey* pKey = nullptr ) = 0;
		};

		interface ISignatures
		{
			virtual ISignature* FirstSignature() = 0;
			virtual ISignature* NextSignature(ISignature* pPrev) = 0;
			virtual ISignature* FindSignature(sx_str ID) = 0;
			virtual ISignature* AddSignature(sx_str ID) = 0;
			virtual void RemoveSignature(ISignature* pSign) = 0;
		};

		interface ISignatureSpot : IRefObject
		{
			virtual IStr* get_ID() const = 0;
			virtual IStr* get_Reason() const = 0;
			virtual void set_Reason(sx_str s) = 0;
			virtual IStr* get_Location() const = 0;
			virtual void set_Location(sx_str s) = 0;
			virtual sx_bool GetDeadline(sx_time& t) = 0;
			virtual void SetDeadline(const sx_time& time) = 0;
			virtual sx_uint GetSpotArea(sx_point& p) const = 0;
			virtual void SetSpotArea(sx_uint page, const sx_point& p) = 0;
			virtual void Hide() = 0;
		};

		interface ISignatureSpots
		{
			virtual ISignatureSpot* FirstSpot() = 0;
			virtual ISignatureSpot* NextSpot(ISignatureSpot* pPrev) = 0;
			virtual ISignatureSpot* FindSpot(sx_str ID) = 0;
			virtual ISignatureSpot* AddSpot(sx_str ID) = 0;
			virtual void RemoveSpot(sx_str ID) = 0;
		};
	} // namespace XPS
} // namespace SX
