// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "graphics.h"

namespace SX
{
	namespace Barcode
	{
		enum barcode_format
		{
			barcode_format_unknown,
			barcode_format_pdf417,
			barcode_format_qrcode,
			barcode_format_datamatrix
		};
		interface IEncodeOptions;

		interface IBarcode : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual barcode_format get_Format() const = 0;
			virtual IEncodeOptions* get_EncodeOptions() = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual Graphics::IBitmapData* Encode(sx_str Data) = 0;
			virtual IStr* Decode(Graphics::IBitmapData* Image) = 0;
		};

		enum barcode_scale
		{
			barcode_scale_fit_free = 1,
			barcode_scale_fit_prop,
			barcode_scale_snap_mode
		};

		interface IEncodeOptions
		{
			virtual sx_uint get_Width() = 0;
			virtual void set_Width(sx_uint Width) = 0;
			virtual sx_uint get_Height() = 0;
			virtual void set_Height(sx_uint Height) = 0;
			virtual sx_uint get_HRatio() = 0;
			virtual void set_HRatio(sx_uint Ratio) = 0;
			virtual sx_uint get_VRatio() = 0;
			virtual void set_VRatio(sx_uint Ratio) = 0;
			virtual barcode_scale get_Scale() = 0;
			virtual void set_Scale(barcode_scale Scale) = 0;
		};

		enum PDF417_format
		{
			PDF417_format_standard = 1,
			PDF417_format_truncated,
			PDF417_format_hibc,
			PDF417_format_micro,
			PDF417_format_microhibc
		};

		enum PDF417_ECC
		{
			PDF417_ECC_auto = -1,
			PDF417_ECC_2,
			PDF417_ECC_8,
			PDF417_ECC_16,
			PDF417_ECC_32,
			PDF417_ECC_64,
			PDF417_ECC_128,
			PDF417_ECC_256,
			PDF417_ECC_512
		};

		interface IEncodePDF417 : public IEncodeOptions
		{
			virtual PDF417_format get_Format() = 0;
			virtual void set_Format(PDF417_format f) = 0;
			virtual PDF417_ECC get_ECC() = 0;
			virtual void set_ECC(PDF417_ECC c) = 0;
			virtual sx_uint get_DataColumns() = 0;
			virtual void set_DataColumns(sx_uint c) = 0;
		};

		enum QRcode_format
		{
			QRcode_format_standard = 1,
			QRcode_format_hibc,
		};

		enum QRcode_ECC
		{
			QRcode_ECC_auto = 0,
			QRcode_ECC_20,
			QRcode_ECC_37,
			QRcode_ECC_55,
			QRcode_ECC_65
		};

		interface IEncodeQRcode : public IEncodeOptions
		{
			virtual QRcode_format get_Format() = 0;
			virtual void set_Format(QRcode_format f) = 0;
			virtual QRcode_ECC get_ECC() = 0;
			virtual void set_ECC(QRcode_ECC c) = 0;
			virtual sx_uint get_Size() = 0;
			virtual void set_Size(sx_uint s) = 0;
		};

		enum DataMatrix_size
		{
			DataMatrix_size_auto = 0,
			DataMatrix_size_10x10,
			DataMatrix_size_12x12,
			DataMatrix_size_8x18,
			DataMatrix_size_14x14,
			DataMatrix_size_8x32,
			DataMatrix_size_16x16,
			DataMatrix_size_12x26,
			DataMatrix_size_18x18,
			DataMatrix_size_20x20,
			DataMatrix_size_12x36, 
			DataMatrix_size_22x22,
			DataMatrix_size_16x36,
			DataMatrix_size_24x24,
			DataMatrix_size_26x26,
			DataMatrix_size_16x48,
			DataMatrix_size_32x32,
			DataMatrix_size_36x36,
			DataMatrix_size_40x40,
			DataMatrix_size_44x44,
			DataMatrix_size_48x48,
			DataMatrix_size_52x52,
			DataMatrix_size_64x64,
			DataMatrix_size_72x72,
			DataMatrix_size_80x80,
			DataMatrix_size_88x88,
			DataMatrix_size_96x96,
			DataMatrix_size_104x104,
			DataMatrix_size_120x120,
			DataMatrix_size_132x132,
			DataMatrix_size_144x144
		};

		interface IEncodeDataMatrix : public IEncodeOptions
		{
			virtual DataMatrix_size get_Size() = 0;
			virtual void set_Size(DataMatrix_size s) = 0;
			virtual sx_bool get_SuppressRectangular() = 0;
			virtual void set_SuppressRectangular(sx_bool val) = 0;
		};

	} // end Barcode
} // end SX