// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once
#include "basic.h"

namespace SX
{
	interface ISearchProvider;
	interface IPrintProvider;
	interface IOutlineItem;
	interface IAcquisitionDevices;
	interface IAcquisitionDevice;
	interface IDocEvents;
	namespace Viewer { interface IViewableDocument; }
	namespace Graphics { interface IBitmapData; enum PixelFormat; }

	object_type
		obj_type_image_document			=	0x29,
		obj_type_fragment_document		=	0x2A,
		obj_type_composite_document		=	0x2E,
		obj_type_text_document			=	0x2F,
		obj_type_acquisition_document	=	0x30;

	struct page_props : sx_marg
	{
		sx_double width;
		sx_double height;
	};

	enum combine
	{
		combine_named_dests		= 0x01,
		combine_outlines		= 0x02, 
		combine_form_fields		= 0x04,
		combine_layers			= 0x08,
		combine_links			= 0x10,
		combine_log_struct		= 0x20,
		combine_jscript			= 0x40,

		combine_emb_unk_fonts		= 0x01000,
		combine_emb_win_fonts		= 0x02000,
		combine_flatten_at			= 0x04000,
		combine_preserve_layers		= 0x08000,
		combine_xps_fail_vect_text	= 0x10000
	};
	
	enum recognize_bb
	{
		recognize_bb_def,
		recognize_bb_ignore_bounds, 
		recognize_bb_ignore_clipping,
		recognize_bb_pass_hidden_layers,
		recognize_bb_pass_all_layers
	};

	//////////////////////////////////////////////////////////////////////////
	interface IBaseDocument : public IRefObject
	{
		virtual Viewer::IViewableDocument* GetViewableDocument() = 0;
		virtual ISearchProvider* GetSearchProvider() = 0;
		virtual IPrintProvider* GetPrintProvider(sx_str sJobName, sx_str sPrinterName, sx_byte* pDevMode = nullptr, ISequentialStream* pOutput = nullptr) = 0;
		virtual IDocEvents* GetEventHandler() const = 0;
		virtual void SetEventHandler(IDocEvents* pEventHandler) = 0;
		virtual void Clear() = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ISimpleDocument : public IBaseDocument
	{
		virtual sx_uint get_Pages() const = 0;
		virtual IOutlineItem* get_Outline() const = 0;
		virtual IOutlineItem* FindOutlineItem(sx_uint page) const = 0;

		virtual void ResolveLinks(sx_flags opt) = 0;
		virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& back, sx_uint uDPI = 72, sx_flags opt = 0, sx_flags annots = 0, sx_uint start = 0, sx_uint end = INVALID_INDEX) = 0;
		virtual IStr* GetPlainText(sx_flags opt = 0, sx_uint start = 0, sx_uint end = INVALID_INDEX) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ITextDocument : public IBaseDocument
	{
		virtual void SetTextFont(sx_str FontFamily, sx_uint weight = 400, sx_uint style = 0, sx_uint stretch = 5, sx_double size = 12.) = 0;
		virtual void SetText(ISequentialStream* pText, const page_props &props, sx_bool bXMLFormatted = false, sx_flags Reserved = 0) = 0;
		virtual void SetTextColor(const sx_color& color) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IImageDocument : public IBaseDocument
	{
		virtual Graphics::IBitmapData* GetImage() = 0;
		virtual void SetImage(Graphics::IBitmapData* pImage, sx_uint DPI) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IFragmentDocument : public IBaseDocument
	{
		//////////////////////////////////////////////////////////////////////////
		// ref doc
		virtual void SetFragment(IBaseDocument* pDoc, sx_str sPageRange) = 0;
		virtual IBaseDocument* GetRefDoc() = 0;

		//////////////////////////////////////////////////////////////////////////
		// page mapping
		virtual sx_uint GetRefPage(sx_uint uIndex) const = 0;
		virtual sx_uint GetFragmentPage(sx_uint uIndex) const = 0;
		virtual void CompactPage(sx_uint ind) = 0;
		virtual void SetAutoCompact(sx_uint uNumDeployedPages) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ICompositeDocument : public IBaseDocument
	{
		//////////////////////////////////////////////////////////////////////////
		// ref docs
		virtual sx_uint get_Count() const = 0;
		virtual IBaseDocument* Item(sx_uint uIndex) = 0;
		virtual void Insert(IBaseDocument* pDoc, sx_uint uTo) = 0;
		virtual void Move(sx_uint uFrom, sx_uint uTo) = 0;
		virtual void Remove(sx_uint uIndex) = 0;

		//////////////////////////////////////////////////////////////////////////
		// page mapping
		virtual sx_uint GlobalPageToDoc(sx_uint uIndex) const = 0;
		virtual sx_uint GlobalPageToDocPage(sx_uint uIndex) const = 0;
		virtual sx_uint DocPageToGlobalPageIndex(sx_uint uDoc, sx_uint uIndex) const = 0;

		virtual void CompactPage(sx_uint ind) = 0;
		virtual void SetAutoCompact(sx_uint uNumDeployedPages) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IAcquisitionDocument : public IBaseDocument
	{
		virtual sx_uint get_Pages() const = 0;
		virtual sx_uint Acquire(IAcquisitionDevice* pDev, sx_uint uTo) = 0;
		virtual Graphics::IBitmapData* Page(sx_uint uIndex) = 0;
		virtual void Insert(Graphics::IBitmapData* image, sx_uint uTo) = 0;
		virtual void Move(sx_uint uFrom, sx_uint uTo) = 0;
		virtual void Remove(sx_uint uIndex) = 0;
		virtual IAcquisitionDevices* GetDevices(const sx_uint vers) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IAcquisitionDevices : public IRefObject
	{
		virtual IAcquisitionDevice* GetNextDevice(IAcquisitionDevice* pPrevDev = nullptr) = 0;
		virtual IAcquisitionDevice* FindDevice(sx_str sName) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IAcquisitionDevice : public IRefObject
	{
		virtual sx_str get_Name() const = 0;
		virtual sx_str get_ID() const = 0;
		virtual sx_uint get_DPI() const = 0;
		virtual void set_DPI(sx_uint DPI) = 0;
		virtual Graphics::PixelFormat get_ImageFormat() const = 0;
		virtual void set_ImageFormat(Graphics::PixelFormat format) = 0;
		virtual sx_bool GetRectArea(sx_rect& rect) const = 0;
		virtual void SetRectArea(const sx_rect& rect) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ILinkItem : public IRefObject
	{
		virtual sx_bool get_IsLocal() const = 0;
		virtual sx_uint get_Page() const = 0;
		virtual sx_double get_X() const = 0;
		virtual sx_double get_Y() const = 0;
		virtual sx_str get_URI() const = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IOutlineItem : public ILinkItem
	{
		virtual sx_str get_Title() const = 0;
		virtual IOutlineItem* FirstChild() = 0;
		virtual IOutlineItem* NextChild(IOutlineItem* pPrev) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface ICombineStatus
	{
		virtual sx_bool OnPage(sx_uint ind, sx_uint count) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	enum doc_event
	{
		doc_event_dest_doc = 1,
		doc_event_clear_doc,
		doc_event_mod_doc,
		doc_event_pre_load,
		doc_event_post_load,
		doc_event_pre_save,
		doc_event_post_save,
		doc_event_pre_print,
		doc_event_post_print,
		doc_event_new_page,
		doc_event_del_page,
		doc_event_move_page,
		doc_event_resize_page,
		doc_event_req_rc_page,
		doc_event_upd_rc_page,
		// PDF
		doc_event_req_rc_img = 50,
		doc_event_upd_rc_img,
		doc_event_form_field,
		doc_event_cjk_font,
		// XPS
		doc_event_new_doc = 70,
		doc_event_del_doc
	};

	//////////////////////////////////////////////////////////////////////////
	interface IDocEvent
	{
		virtual doc_event get_Type() const = 0;
		virtual sx_bool get_Handled() const = 0;
		virtual void set_Handled(sx_bool bHandled = true) = 0;
		virtual IBaseDocument* get_Doc() const = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IDocPageEvent : public IDocEvent
	{
		virtual sx_uint get_Page() const = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IDocEvents
	{
		virtual sx_bool OnEvent(IDocEvent* pEvent) = 0;
	};

	//////////////////////////////////////////////////////////////////////////
	interface IDocLoader : public ILoadProvider
	{
		virtual sx_str get_URI() const = 0;
		virtual sx_str get_Ident() const = 0;
		virtual sx_bool OnLink(sx_str pURI) = 0;
		virtual sx_bool OnPage(sx_uint ind) = 0;
	};

#ifdef SX_WINDESKTOP_DLL
	enum wia_device
	{
		wia_device_unknown,
		wia_device_scanner,
		wia_device_camera
	};

	//////////////////////////////////////////////////////////////////////////
	interface IWIADevice : public IAcquisitionDevice
	{
		virtual wia_device get_Type() const = 0;
		virtual sx_bool get_ShowSettings() const = 0;
		virtual void SetShowSettings(sx_bool show, HWND hParentWnd = nullptr) = 0;
	};
#endif
} // namespace SX
