// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "basic.h"
//////////////////////////////////////////////////////////////////////////
// WIC, Direct2D
interface IWICBitmapSource; 
typedef IWICBitmapSource* sx_wic_bitmap_source;
interface ID2D1RenderTarget;
typedef ID2D1RenderTarget* sx_d2d_target;
typedef void* sx_gdi_target;
typedef void* sx_gdi_bitmap;

namespace SX
{
	namespace Graphics
	{
		interface IRegion;

		const object_type 
			obj_type_bitmap				= 0x06,
			obj_type_region				= 0x07,
			obj_type_drawing_ctx		= 0x08,
			obj_type_drawing_d2d_ctx	= 0x09;

		enum FontFormat
		{
			FontFormatUndefined,
			FontFormatType1,
			FontFormatType1Compact,
			FontFormatType3,
			FontFormatTrueType,
			FontFormatOpenType
		};

		enum ImageFormat
		{
			ImageFormat_Unknown,
			ImageFormat_BMP,
			ImageFormat_JPEG,
			ImageFormat_PNG,
			ImageFormat_TIFF,
			ImageFormat_GIF,
			ImageFormat_HDPhoto
		};

		enum PixelFormat
		{
			PixelFormat_Unknown,
			PixelFormat_BlackWhite = 0x0111,
			PixelFormat_Grayscale2 = 0x0112,
			PixelFormat_Grayscale4 = 0x0114,
			PixelFormat_Grayscale  = 0x0118,
			PixelFormat_Monochrome = 0x0211,
			PixelFormat_Indexed2   = 0x0212,
			PixelFormat_Indexed4   = 0x0214,
			PixelFormat_Indexed    = 0x0218,
			PixelFormat_RGB        = 0x0438,
			PixelFormat_RGBA       = 0x0448,
			PixelFormat_BGR        = 0x0838,
			PixelFormat_BGRA       = 0x0848,
			PixelFormat_CMYK       = 0x8048
		};

		enum ImageInterpolation
		{
			ImageInterpolation_neighbor = 1,
			ImageInterpolation_linear = 2,
			ImageInterpolation_bilinear = 3,
			ImageInterpolation_bicubic = 4,
			ImageInterpolation_blackman = 5,
			ImageInterpolation_catmul_rom = 6, 
			ImageInterpolation_lanzcos = 7,
			ImageInterpolation_gauss= 8,
			ImageInterpolation_spline = 9
		};

		enum TextAntialias
		{
			TextAntialias_none = 0,
			TextAntialias_system = 1,
			TextAntialias_grayscale = 2,
			TextAntialias_cleartype = 3
		};

		enum TextRender
		{
			TextRender_glyphs_only = 0,
			TextRender_glyphs_for_embed = 1,
			TextRender_force_text = 2
		};

		enum Drawer
		{
			Drawer_Unknown,
			Drawer_Direct2D,
			Drawer_GDI
		};

		enum Blockiness
		{
			Blockiness_one_block,
			Blockiness_big_blocks = 18,
			Blockiness_middle_blocks = 16,
			Blockiness_small_blocks = 12
		};

		//////////////////////////////////////////////////////////////////////////
		// image bitmap
		interface IPaletteData
		{
			virtual sx_uint get_Count() = 0;
			virtual PixelFormat get_Format() = 0;
			virtual const sx_byte* GetColors() const = 0;
			virtual void ResetColors(PixelFormat format, sx_uint count, const sx_byte* pColors = nullptr) = 0;
		};

		interface IBitmapData : public IRefObject
		{
			virtual sx_uint get_Width() = 0;
			virtual sx_uint get_Height() = 0;
			virtual sx_double get_DPI() = 0;
			virtual void set_DPI(sx_double dpi) = 0;
			virtual PixelFormat get_PixelFormat() = 0;
			virtual IPaletteData* get_Palette() = 0;
			virtual const sx_byte* GetPixels() = 0;
			virtual const sx_byte* GetScanLine(sx_uint Line) = 0;
			virtual IBitmapData* GetNextFrame() = 0;
			virtual IBitmapData* CopyAs(PixelFormat format) = 0;
			virtual IBitmapData* Crop(sx_rect rc) = 0;
			virtual IBitmapData* Rotate(sx_rotation_angle r) = 0;
			virtual IBitmapData* Resize(sx_double scale, ImageInterpolation interpol) = 0;
			virtual sx_qword SaveToStream(ISequentialStream* pStream, ImageFormat format, sx_uint Quality = 50) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// color helper
		interface IColor
		{
			virtual void AsColor(sx_color& src, sx_color& dst) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// matrix helper
		interface IMatrix
		{
			virtual sx_bool IsIdentity(const sx_matrix& m) = 0;
			virtual void Identity(sx_matrix& m) = 0;
			virtual void Translate(sx_matrix&  m, sx_double x, sx_double y, sx_bool bAppend = false) = 0;
			virtual void Scale(sx_matrix& m, sx_double cx, sx_double cy, sx_bool bAppend = false) = 0;
			virtual void Rotate(sx_matrix& m, sx_double a, sx_bool bAppend = false) = 0;
			virtual void Multiply(sx_matrix& m, const sx_matrix& mul, sx_bool bAppend = false) = 0;
			virtual sx_bool Invert(sx_matrix& m) = 0;

			virtual void TransformPoint(const sx_matrix& m, sx_point & pt) = 0;
			virtual void TransformRect(const sx_matrix& m, sx_rect& rect) = 0;
			virtual void TransformQuad(const sx_matrix& m, sx_quad& quad) = 0;
			virtual void TransformRegion(const sx_matrix& m, IRegion* reg) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IRegion : public IRefObject
		{
			virtual sx_uint get_QuadsCount() const = 0;
			virtual void GetQuad(sx_uint uIndex, sx_quad& quad) const = 0;
			virtual void AddQuad(sx_quad& quad) = 0;
			virtual void GetBBox(sx_rect& rect) const = 0;
			virtual sx_uint PointInQuad(const sx_double &x, const sx_double &y) const = 0;
			virtual sx_bool PointIn(const sx_double &x, const sx_double &y) const = 0;
			virtual IRegion* Copy(sx_matrix* pTansform = nullptr, sx_bool bMergeQuads = false) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// array of points
		interface IPointsArray
		{
			virtual sx_uint get_PointsCount() const = 0;
			virtual void GetPoint(sx_uint uInd, sx_point& pt) const = 0;
			virtual void SetPoint(sx_uint uInd, const sx_point &pt) = 0;
			virtual void InsertPoint(sx_uint uInd, const sx_point &pt) = 0;
			virtual void RemovePoint(sx_uint uInd) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// segment of vector image 
		interface IInkLineSegment : public IRefObject, public IPointsArray
		{
			virtual void GetMatrix(sx_matrix& m) const = 0;
			virtual void SetMatrix(const sx_matrix& m) = 0;
			virtual const sx_color* GetColor() const = 0;
			virtual void SetColor(const sx_color& c) = 0;
			virtual sx_double get_LineWidth() const = 0;
			virtual void set_LineWidth(const sx_double &dWidth) = 0;
			virtual sx_flags get_LineOptions() const = 0;
			virtual void set_LineOptions(sx_flags opt) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// IInkImage
		interface IInkImage : public IRefObject
		{
			virtual sx_bool get_Modified() const = 0;
			virtual sx_uint GetUPI() const = 0;
			virtual sx_bool GetBBox(sx_rect& box) const = 0;
			virtual void SetBBox(const sx_rect& box) = 0;
			virtual void ResetBBox() = 0;
			virtual void SaveToStream(ISequentialStream *pStream) = 0;
			// image segments
			virtual sx_uint get_SegmentsCount() const = 0;
			virtual IInkLineSegment* GetSegment(sx_uint uInd) = 0;
			virtual IInkLineSegment* InsertSegment(sx_uint uBefore = INVALID_INDEX) = 0;
			virtual void RemoveSegment(sx_uint uInd) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// ink pen options
		enum ink_pen
		{
			ink_pen_default				= 0x00000000,
			ink_pen_fit_to_curve		= 0x00000001,
			ink_pen_subtractive_transp	= 0x00000002,
			ink_pen_ignore_pressure		= 0x00000004,
			ink_pen_anti_aliased		= 0x00000010,
			ink_pen_ignore_rotation		= 0x00000020,
			ink_pen_ignore_angle		= 0x00000040,
			ink_pen_tip_rect			= 0x10000000
		};



		//////////////////////////////////////////////////////////////////////////
		// drawing context
		interface IDrawingContext : public IRefObject
		{
			virtual Drawer get_Drawer() const = 0;
			virtual sx_bool get_Opened() const = 0;
			virtual TextRender get_TextRender () const = 0;
			virtual void set_TextRender(TextRender mode) = 0;
			virtual TextAntialias get_TextAntialias () const = 0;
			virtual void set_TextAntialias(TextAntialias mode) = 0;
			virtual ImageInterpolation get_ImageInterpolation () const = 0;
			virtual void set_ImageInterpolation(ImageInterpolation mode) = 0;			

			// context state
			virtual sx_bool Open() = 0;
			virtual void Close() = 0;

			// block processing of content
			virtual Blockiness get_Blockiness() const = 0;
			virtual void set_Blockiness(Blockiness bl) = 0;
			virtual sx_bool ProcessNextBlock(sx_bool skip = false) = 0;

			virtual sx_flags GetDrawStatus() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// Direct2D drawing context
		interface ID2DContext : public IDrawingContext
		{
			virtual sx_d2d_target get_Target() const = 0;
			virtual void set_Target(sx_d2d_target pID2D1Target) = 0;
		};

		enum draw_opt
		{
			draw_opt_rasterize_blended			= 0x01,
			draw_opt_skip_blended				= 0x02,
			draw_opt_recover_annot_appearance	= 0x04,
			draw_opt_highlight_fields			= 0x10,
			draw_opt_highlight_req_fields		= 0x20,
			draw_opt_empty_sign_icon			= 0x40,
			draw_opt_sign_status_icon			= 0x80,
			draw_opt_field_name					= 0x100,
			draw_opt_highlight_outline_fields	= 0x200,
			draw_opt_force_blend				= 0x400,
			draw_opt_flatten_image_mask			= 0x800,
			draw_opt_ignore_text_color			= 0x1000,
			draw_opt_ignore_text_background		= 0x2000,
			draw_opt_stroke_text_background		= 0x4000,
			draw_opt_allow_smooth_lines			= 0x8000,
			draw_opt_force_unicode_text			= 0x10000,
			draw_opt_fail_if_err				= 0x800000
		};

		enum draw_status
		{
			draw_status_ok						= 0,
			draw_status_unk_font				= 0x01,
			draw_status_inv_image				= 0x02,
			draw_status_inv_mask				= 0x04,
			draw_status_blend_failed			= 0x10,
			draw_status_system_failure			= 0x1000000
		};
			
		//////////////////////////////////////////////////////////////////////////
		// GDI drawing context
		interface IGDIContext : public IDrawingContext
		{
			virtual sx_gdi_target GetTarget() const = 0;
			virtual sx_bool SetTarget(sx_gdi_target hdc, const sx_rect& rect) = 0;
			virtual sx_gdi_bitmap GetBitmap() = 0;
			virtual sx_bool SetBitmap(const sx_size& size, const sx_color_i_rgb& backColor) = 0;
		};

		enum GDI_opt
		{
			GDI_opt_fail_unk_font		=0x000001,
			GDI_opt_fail_rop_op			=0x000002,
			GDI_opt_fail_wmf_img		=0x000004,
			GDI_opt_fail_unk_img		=0x000008,
			GDI_opt_fail_img_attr		=0x000010,

			GDI_opt_sim_rop_op			=0x001000,
			GDI_opt_bidirectional		=0x002000,
			GDI_opt_pixel_lines			=0x004000,
			GDI_opt_adapt_dashes		=0x008000,
			GDI_opt_optimize_regions	=0x010000,
			GDI_opt_import_plus			=0x100000,
			GDI_opt_import_grad			=0x200000,
			GDI_opt_import_grad2		=0x400000
		};
	}
}