// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once
#include "document.h"
#include "rich-content.h"

namespace SX
{
	namespace PDF { interface ICompositeImage; interface IAutoText; }

	object_type obj_type_layout_document =	0x31;

	namespace Layout
	{
		interface IPage;
		interface IPlaceholder;
		interface IDecoElement;
		interface IProvider;		
		interface IPackage;		
		interface ILayer;		

		enum prov_type
		{
			prov_type_unknown	= 0,
			prov_type_basic		= 1,
			prov_type_office	= 2,
			prov_type_transform	= 3,
			prov_type_poster	= 10,
			prov_type_booklet	= 11,
			prov_type_table		= 12,
			prov_type_multi		= 50,
			prov_type_ref		= 51
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDocument : public IBaseDocument
		{
			// ref doc
			virtual IBaseDocument* GetRefDoc() = 0;
			virtual void SetRefDoc(IBaseDocument* pDoc) = 0;
			// pages
			virtual sx_uint GetPageCount() const = 0;
			virtual IPage*  GetPage(sx_uint ind) = 0;
			virtual IPage*  AddPage(sx_double w, sx_double h) = 0;
			virtual IPage*  InsertPage(sx_uint to, sx_double w, sx_double h) = 0;
			virtual void    MovePage(sx_uint from, sx_uint to) = 0;
			virtual void    RemovePage(sx_uint ind) = 0;
			virtual sx_uint IndexOfPage(IPage* pPage) = 0;
			// misc
			virtual IDecoElement* CreateDeco() = 0;	
			virtual IProvider* CreateProvider(prov_type t) = 0;
			virtual IPackage* CreatePackage() = 0;
			virtual ILayer* GetLayers(sx_flags opt = 0) = 0;
			virtual void ImportNavigation(sx_flags opt = 0) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPage
		{
			virtual sx_double get_Width() const = 0;
			virtual void set_Width(sx_double w) = 0;
			virtual sx_double get_Height() const = 0;
			virtual void set_Height(sx_double h) = 0;
			// placeholders
			virtual sx_uint GetCount() const = 0;
			virtual IPlaceholder* GetItem(sx_uint ind) = 0;
			virtual IPlaceholder* AddItem() = 0;
			virtual IPlaceholder* AddAttachment(IBaseDocument* pDoc, sx_uint PageInd) = 0;
			virtual void MoveItem(sx_uint from, sx_uint to) = 0;
			virtual void RemoveItem(sx_uint ind) = 0;
			// deco
			virtual IDecoElement* GetBackground() = 0;
			virtual void SetBackground(IDecoElement* pRC) = 0;
			virtual IDecoElement* GetOverlay() = 0;
			virtual void SetOverlay(IDecoElement* pRC) = 0;
			virtual ILayer* GetDecoLayer(sx_bool overlay = false) = 0;
			virtual void SetDecoLayer(ILayer* pLayer, sx_bool overlay = false) = 0;
			// misc
			virtual void CalcMatrix(sx_matrix& m, sx_double dZoom = 1.0, sx_rotation_angle angle = sx_rotation_angle_0, sx_double dx = 0., sx_double dy = 0., sx_uint uDPI = 72 ) const = 0;
			virtual sx_bool StartDraw(Graphics::IDrawingContext* ctx, sx_matrix *pTransform = nullptr, sx_flags flags = 0, sx_flags annots = 0) = 0;
			virtual sx_bool StartPrint(Graphics::IDrawingContext* ctx, sx_matrix *pTransform = nullptr, sx_flags flags = 0, sx_flags annots = 0) = 0;
			virtual Graphics::IBitmapData* CopyToBitmap(const sx_color_i_rgba& backColor, sx_matrix *pTransform = nullptr,  sx_flags flags = 0, sx_flags annots = 0) = 0;
			virtual Viewer::IViewableDocument* GetDecoPreview(IDecoElement* pDeco, sx_uint ph = INVALID_INDEX) = 0;
		};
	
		//////////////////////////////////////////////////////////////////////////
		interface IPlaceholder
		{
			virtual sx_bool GetAttachedPage() const = 0;
			virtual sx_uint GetRefPage() const = 0;
			virtual void SetRefPage(sx_uint ind) = 0;
			virtual void GetRectArea(sx_rect& box) const = 0;
			virtual void SetRectArea(const sx_rect& box) = 0;
			virtual void GetMatrix(sx_matrix& m) const = 0;
			virtual void SetMatrix(const sx_matrix& m) = 0;
			virtual ILayer* GetLayer() = 0;
			virtual void SetLayer(ILayer* pLayer) = 0;
			// deco
			virtual IDecoElement* GetBackground() = 0;
			virtual void SetBackground(IDecoElement* pRC) = 0;
			virtual IDecoElement* GetOverlay() = 0;
			virtual void SetOverlay(IDecoElement* pRC) = 0;
			virtual ILayer* GetDecoLayer(sx_bool overlay = false) = 0;
			virtual void SetDecoLayer(ILayer* pLayer, sx_bool overlay = false) = 0;
		};
	
		//////////////////////////////////////////////////////////////////////////
		interface IDecoElement : public IRefObject
		{
			virtual Content::IRichCollection* get_Objects() = 0;
			virtual sx_bool get_Dynamic() const = 0;
			virtual sx_bool get_Modified() const = 0;
			virtual void GetBBox(sx_rect& rc) const = 0;
			virtual sx_bool GetAbsBBox(sx_rect& rc) const = 0;
			// factory
			virtual Content::IPath* CreatePath() = 0;
			virtual Content::IPen* CreatePen(const sx_double& width) = 0;
			virtual Content::IBrush* CreateBrush(Content::brush_type type) = 0;
			virtual Content::ISolidBrush* CreateSolidBrush(const sx_color& color) = 0;
			virtual Content::IDashStyle* CreateDashStyle() = 0;
			virtual Content::IColorStops* CreateColorStops() = 0;
			virtual Content::IFont* CreateFont(sx_str family, sx_uint weight = 400, sx_bool italic = 0, sx_uint stretch = 5) = 0; 
			virtual Content::ITextStyle* CreateTextStyle() = 0;
			virtual Content::IRichContent* CreateRichContent() = 0;
			virtual Content::IRichContent* ImportPDFContent(PDF::ICompositeImage* pImage) = 0;
			virtual sx_bool TransformBBox(const sx_double &w, const sx_double & h, const sx_double & hoffset, const sx_double & voffset) = 0;
		};

		interface ILayer
		{
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str sTitle) = 0;
			virtual sx_bool get_Visible() const = 0;
			virtual void set_Visible(sx_bool bVisible) = 0;
			virtual sx_bool GetVisibility() const = 0;

			virtual ILayer* Parent() = 0;
			virtual ILayer* FirstChild() = 0;
			virtual ILayer* NextChild(ILayer* pItem) = 0;
			virtual ILayer* FindChild(sx_str sTitle) = 0;
			virtual ILayer* AddChild(sx_str sTitle) = 0;
			virtual void RemoveChild(ILayer* pItem) = 0;
			virtual void RemoveAllChilds() = 0;
		};

		enum src_pages
		{
			src_pages_all = 1,
			src_pages_all_excover,
			src_pages_odd,
			src_pages_odd_excover,
			src_pages_even,
			src_pages_even_excover,
			src_pages_range,
			src_pages_range_excl,
			src_pages_single
		};

		enum fit_style
		{
			fit_style_none = 1,
			fit_style_page,
			fit_style_shrink_page,
			fit_style_width,
			fit_style_shrink_width,
			fit_style_height,
			fit_style_shrink_height,
			fit_style_fill
		};

		enum hor_align
		{
			hor_align_left = 1,
			hor_align_center,
			hor_align_right
		};

		enum vert_align
		{
			vert_align_top = 1,
			vert_align_middle,
			vert_align_bottom
		};

		enum ln_style
		{
			ln_style_solid = 1,
			ln_style_dot,
			ln_style_dash,
			ln_style_dash_dot,
			ln_style_dash_dot_dot
		};

		//////////////////////////////////////////////////////////////////////////
		interface IProvider : public IRefObject
		{
			virtual prov_type get_Type() const = 0;
			virtual src_pages get_SrcSet() const = 0;
			virtual sx_str    get_SrcRange() const = 0;
			virtual sx_uint   get_SrcIndex() const = 0;
			virtual sx_bool   get_SrcReverse() const = 0;

			virtual void SetSrcPages(src_pages set, sx_str range, sx_uint ind = 0, sx_bool reverse = false) = 0;
			virtual Viewer::IViewableDocument* GetPreview(IDecoElement* pDeco = nullptr, sx_flags flags = 0) = 0;

			virtual sx_uint Start(sx_uint pos = INVALID_INDEX) = 0;
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IBasicProvider : public IProvider
		{
			virtual IDecoElement* ResetBackground(sx_bool bCreate = false, sx_bool bClear = false) = 0;
			virtual IDecoElement* ResetOverlay(sx_bool bCreate = false, sx_bool bClear = false) = 0;
		};

		enum deco_range
		{
			deco_range_all,
			deco_range_landscape	= 0x1,
			deco_range_first		= 0x2,
			deco_range_even			= 0x4
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IOfficeProvider : public IProvider
		{
			virtual sx_bool get_DifferentLandscape() const = 0;
			virtual void set_DifferentLandscape(sx_bool v) = 0;
			virtual sx_bool get_DifferentFirstPage() const = 0;
			virtual void set_DifferentFirstPage(sx_bool v) = 0;
			virtual sx_bool get_DifferentOddEven() const = 0;
			virtual void set_DifferentOddEven(sx_bool v) = 0;

			virtual sx_size ResetCoverPage(sx_size sz, sx_bool bSet = false) = 0;
			virtual IDecoElement* ResetCoverImage(sx_bool bCreate = false, sx_bool bClear = false) = 0;

			virtual IDecoElement* ResetBackground(sx_bool bCreate = false, sx_bool bClear = false, sx_flags range = 0) = 0;
			virtual IDecoElement* ResetHeader(sx_bool bCreate = false, sx_bool bClear = false, sx_flags range = 0) = 0;
			virtual IDecoElement* ResetFooter(sx_bool bCreate = false, sx_bool bClear = false, sx_flags range = 0) = 0;

			virtual sx_size ResetAddendumPage(sx_size sz, sx_bool bSet = false) = 0;
			virtual IDecoElement* ResetAddendumImage(sx_bool bCreate = false, sx_bool bClear = false) = 0;
		};

		enum orient_style
		{
			orient_style_0,
			orient_style_90,
			orient_style_180,
			orient_style_270,
			orient_style_auto
		};

		interface ITransformProvider : public IProvider
		{
			virtual sx_size GetSheetSize() = 0;
			virtual sx_marg GetSheetMargins() = 0;
			virtual fit_style GetFit() = 0;
			virtual hor_align GetHAlignment() = 0;
			virtual vert_align GetVAlignment() = 0;
			virtual orient_style GetOrientation() = 0;
			virtual sx_double GetScale() = 0;

			virtual void SetSheet(sx_size sz, sx_marg margins) = 0;
			virtual void SetAlignment(fit_style fit, hor_align ha, vert_align va, orient_style orient, sx_double scale = 1.) = 0;
			virtual IDecoElement* ResetBackground(sx_bool bCreate = false, sx_bool bClear = false) = 0;
			virtual IDecoElement* ResetOverlay(sx_bool bCreate = false, sx_bool bClear = false) = 0;
		};

		enum booklet_collate
		{
			booklet_collate_ebook = 1,
			booklet_collate_folding
		};

		//////////////////////////////////////////////////////////////////////////
		interface IBookletProvider : public IProvider
		{
			virtual sx_size GetSheetSize() = 0;
			virtual sx_marg GetSheetMargins() = 0;
			virtual sx_double GetSheetMiddleMargin(sx_bool right = false) = 0;
			virtual booklet_collate GetCollate() = 0;
			virtual fit_style GetFit() = 0;
			virtual hor_align GetHAlignment(sx_bool right = false) = 0;
			virtual vert_align GetVAlignment(sx_bool right = false) = 0;

			virtual void SetSheet(sx_size sz, sx_marg margins, sx_double mlm, sx_double mrm) = 0;
			virtual void SetCollate(booklet_collate col) = 0;
			virtual void SetAlignment(fit_style fit, hor_align hal, vert_align val, hor_align har, vert_align var) = 0;
			virtual IDecoElement* ResetHalfBackground(sx_bool right, sx_bool bCreate = false, sx_bool bClear = false) = 0;
			virtual IDecoElement* ResetHalfOverlay(sx_bool right, sx_bool bCreate = false, sx_bool bClear = false) = 0;
		};

		enum table_collate
		{
			table_collate_L2R_T2B = 1,
			table_collate_R2L_T2B,
			table_collate_L2R_B2T,
			table_collate_R2L_B2T,
			table_collate_T2B_L2R,
			table_collate_T2B_R2L,
			table_collate_B2T_L2R,
			table_collate_B2T_R2L
		};

		//////////////////////////////////////////////////////////////////////////
		interface ITableProvider : public IProvider
		{
			virtual sx_uint get_Cols() const = 0;
			virtual sx_uint get_Rows() const = 0;
			virtual table_collate get_Collate() const = 0;

			virtual sx_size GetSheetSize() = 0;
			virtual sx_marg GetSheetMargins() = 0;
			virtual fit_style GetCellFit(sx_uint col, sx_uint row) const = 0;
			virtual hor_align GetCellHAlignment(sx_uint col, sx_uint row) const = 0;
			virtual vert_align GetCellVAlignment(sx_uint col, sx_uint row) const = 0;
			virtual sx_bool GetCellSkip(sx_uint col, sx_uint row) const = 0;
			virtual sx_size GetTableSpacing() const = 0;

			virtual void SetSheet(sx_size sz, sx_marg margins, sx_double hm, sx_double vm) = 0;
			virtual void SetTable(sx_uint cols, sx_uint rows, table_collate collate) = 0;
			virtual void SetCellAlignment(sx_uint col, sx_uint row, fit_style fit, hor_align ha, vert_align va) = 0;
			virtual void SetCellSkip(sx_uint col, sx_uint row, sx_bool skip) = 0;
			virtual IDecoElement* ResetCellBackground(sx_uint col, sx_uint row, sx_bool bCreate = false, sx_bool bClear = false) = 0;
			virtual IDecoElement* ResetCellOverlay(sx_uint col, sx_uint row, sx_bool bCreate = false, sx_bool bClear = false) = 0;
			virtual void SetCellBackground(sx_uint col, sx_uint row, IDecoElement* pDeco) = 0;
			virtual void SetCellOverlay(sx_uint col, sx_uint row, IDecoElement* pDeco) = 0;
		};

		struct poster_grid 
		{
			sx_uint cols;
			sx_uint rows;
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IPosterProvider : public IProvider
		{
			virtual sx_size GetSheetSize() = 0;
			virtual sx_marg GetSheetMargins() = 0;
			virtual void SetSheet(sx_size sz, sx_marg margins) = 0;

			virtual sx_size GetSize() = 0;
			virtual sx_marg GetPadding() = 0;
			virtual sx_bool GetAutoRotate() = 0;
			virtual void SetPoster(sx_size sz, sx_marg padding, sx_bool autorotate = true) = 0;
			
			virtual sx_double GetGlueingZoneWidth() const = 0;
			virtual const sx_color* GetGlueingZoneColor() const = 0;
			virtual sx_bool GetGlueingZoneCropImage() const = 0;
			virtual void SetGlueingZone(sx_double width, const sx_color& c, sx_bool crop_image = true) = 0;

			virtual sx_double GetGlueingLineWeight() const = 0;
			virtual const sx_color* GetGlueingLineColor() const = 0;
			virtual ln_style GetGlueingLineStyle() const = 0;
			virtual void SetGlueingLine(sx_double weight, const sx_color& c, ln_style st = ln_style_solid) = 0;

			virtual sx_double GetCuttingLineWeight() const = 0;
			virtual const sx_color* GetCuttingLineColor() const = 0;
			virtual ln_style GetCuttingLineStyle() const = 0;
			virtual void SetCuttingLine(sx_double weight, const sx_color& c, ln_style st = ln_style_solid) = 0;

			virtual sx_str GetLabelFont() const = 0;
			virtual const sx_color* GetLabelColor() const = 0;
			virtual sx_double GetLabelFontHeight() const = 0;
			virtual sx_uint GetLabelFontWeight() const = 0;
			virtual sx_bool GetLabelFontStyle() const = 0;
			virtual sx_uint GetLabelFontStretch() const = 0;
			virtual void SetNumberingLabel(sx_str font, sx_double height, const sx_color& c, sx_uint weight = 400, sx_bool italic = false, sx_uint stretch = 5) = 0;

			virtual poster_grid GetGridInfo() = 0;
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IRefProvider : public IBasicProvider
		{
			virtual IBaseDocument* GetRefDoc() = 0;
			virtual void SetRefDoc(IBaseDocument* pDoc) = 0;
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IMultiProvider : public IProvider
		{
			virtual sx_uint GetCount() const = 0;
			virtual IProvider* GetItem(sx_uint ind) = 0;
			virtual IProvider* AddItem(prov_type t) = 0;
			virtual void MoveItem(sx_uint from, sx_uint to) = 0;
			virtual void RemoveItem(sx_uint ind) = 0;
			virtual sx_str GetItemTitle(sx_uint ind) const = 0;
			virtual void SetItemTitle(sx_uint ind, sx_str t) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPackage : public IRefObject
		{
			virtual IStr* get_Title() const = 0;
			virtual void set_Title(sx_str title) = 0;

			virtual void OpenStream(ISequentialStream* pStream) = 0;
			virtual void SaveAsStream(ISequentialStream* pStream) = 0;

			virtual prov_type get_Provider() const = 0;
			virtual IProvider* LoadProvider() = 0;
			virtual void UploadProvider(IProvider* pProv) = 0;

			virtual sx_bool get_Layout() const = 0;
			virtual void LoadLayout() = 0;
			virtual void UploadLayout() = 0;

			virtual sx_bool get_Source() const = 0;
			virtual IBaseDocument* LoadSource() = 0;
			virtual void UploadSource() = 0;
		};
	} // namespace Layout
} // namespace SX
