// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "basic.h"

namespace SX
{
	namespace Graphics { interface IBitmapData; }

	namespace Media
	{
		const object_type obj_type_media_player = 0x2B;
		interface IPlayerStatus;

		//////////////////////////////////////////////////////////////////////////
		interface IClipStream : public IRefObject
		{
			virtual ISequentialStream* get_Stream() const = 0;
			virtual void set_Stream(ISequentialStream* pStream) = 0;
			virtual sx_double get_Duration() const = 0; // ms
			// audio
			virtual sx_str get_AudioFormat() const = 0;
			virtual sx_str get_AudioLanguage() const = 0;
			virtual sx_uint get_AudioBitsPerSample() const = 0;
			virtual sx_uint get_AudioChannels() const = 0;
			virtual sx_uint get_AudioBitRate() const = 0; // Kb per sec
			virtual sx_uint get_AudioSampleRate() const = 0; // Samples per sec
			virtual sx_uint get_AudioThreads() const = 0;
			virtual sx_uint get_AudioActThread() const = 0;
			virtual void set_AudioActThread(sx_uint ind) = 0;
			// video
			virtual sx_str get_VideoFormat() const = 0;
			virtual sx_str get_VideoLanguage() const = 0;
			virtual sx_uint get_VideoBitRate() const = 0; // Kbps
			virtual sx_size get_FrameSize() const = 0; // pixels
			virtual sx_double get_FrameRate() const = 0; // FPS
			virtual SX::Graphics::IBitmapData* GetFrame(sx_double pos) = 0; // pos in ms
			virtual sx_uint get_VideoThreads() const = 0;
			virtual sx_uint get_VideoActThread() const = 0;
			virtual void set_VideoActThread(sx_uint ind) = 0;
		};

		enum player_state
		{
			player_state_none,
			player_state_opening,
			player_state_stopped,
			player_state_paused,
			player_state_playing
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPlayer : public IRefObject
		{
			virtual IClipStream* get_Clip() const = 0;
			virtual player_state get_State() const = 0;
			virtual sx_double get_Position() const = 0; // ms

			virtual sx_double get_Volume() const = 0;
			virtual void set_Volume(const sx_double &dVolume) = 0;
			virtual sx_bool get_Mute() const = 0;
			virtual void set_Mute(sx_bool bMute) = 0;
			virtual sx_bool get_FullScreenVideo() const = 0;
			virtual void set_FullScreenVideo(sx_bool bFS) = 0;


			virtual sx_bool Open(IClipStream* pClip, sx_bool bSyncLoad = true) = 0;
			virtual sx_bool Play(sx_uint uRepeatEventRate = 0) = 0; // ms
			virtual sx_bool Pause() = 0;
			virtual sx_bool Stop() = 0;
			virtual sx_bool Seek(sx_double pos) = 0; // ms

			
			virtual void SetStatusHandler(IPlayerStatus* pHandler) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPlayerStatus
		{
			virtual sx_bool OnEvent(player_state st, sx_flags reserved) = 0;
		};

		enum device_state
		{
			device_state_none,
			device_state_stopped,
			device_state_paused,
			device_state_recording
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IMediaDevice : public IRefObject
		{
			virtual sx_str get_Name() const = 0;
			virtual sx_str get_ID() const = 0;
			virtual sx_bool get_Video() const = 0;
			virtual device_state get_State() const = 0;

			virtual sx_str get_Format() const = 0;
			virtual void set_Format(sx_str f) = 0;

			virtual sx_double get_Duration() const = 0; // ms
			virtual ISequentialStream* get_Stream() const = 0; // available in stopped state only

			virtual sx_bool Record(IMediaDevice* pSecondaryDevice = nullptr) = 0; // only video -> audio relation is allowed here
			virtual sx_bool Pause() = 0;
			virtual sx_bool Stop() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IMedia
		{
			virtual IPlayer* CreatePlayer() const = 0;
			virtual IClipStream* CreateClipStream() const = 0;
			virtual IMediaDevice* GetNextDevice(IMediaDevice* pPrevDev = nullptr) = 0;
			virtual IMediaDevice* FindDevice(sx_str sName) = 0;
		};
	}
}