// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "types.h"
#include "basic.h"

namespace SX
{
	namespace NAV
	{
		object_type obj_type_pdf_navigator	= 0x1D;

		enum load_type
		{
			load_type_unknown,
			load_type_default,
			load_type_module
		};

		enum field_type
		{
			field_type_unknown,
			field_type_string,
			field_type_date,
			field_type_numeric,
			field_type_file_name,
			field_type_file_desc,
			field_type_file_cdate,
			field_type_file_mdate,
			field_type_file_size,
			field_type_comp_size
		};

		//////////////////////////////////////////////////////////////////////////
		interface ILocale : public IRefObject
		{
			virtual IStr* get_Name() const = 0;
			virtual sx_uint get_Count() const = 0;

			virtual IStr* GetID(sx_uint ind) const = 0;
			virtual IStr* Get(sx_str id) const = 0;
			virtual void Set(sx_str id, sx_str s) = 0;
			virtual void Remove(sx_str id) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface ILocales
		{
			virtual sx_uint get_Count() const = 0;

			virtual ILocale* Item(sx_uint ind) const = 0;
			virtual ILocale* Get(sx_str name) const = 0;
			virtual ILocale* Add(sx_str name) = 0;
			virtual void Remove(sx_str name) = 0;
			virtual ILocale* GetDefault() const = 0;
			virtual void SetDefault(sx_str name) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IResources
		{
			virtual sx_uint get_Count() const = 0;

			virtual IStr* GetID(sx_uint ind) const = 0;
			virtual ISequentialStream* GetResStream(sx_str id) const = 0;
			virtual void SetResStream(sx_str id, ISequentialStream* pStream) = 0;
			virtual void Remove(sx_str id) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IField : public IRefObject
		{
			virtual field_type get_Type() const = 0;
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str name) = 0;
			virtual sx_bool get_Visible() const = 0;
			virtual void set_Visible(sx_bool set) = 0;
			virtual sx_bool get_Editable() const = 0;
			virtual void set_Editable(sx_bool set) = 0;
			virtual sx_uint get_Order() const = 0;
			virtual void set_Order(sx_uint ord) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IFields
		{
			virtual sx_uint get_Count() const = 0;

			virtual IStr* GetID(sx_uint ind) const = 0;
			virtual IField* Get(sx_str id) const = 0;
			virtual IField* Add(sx_str id, field_type type) = 0;
			virtual void Remove(sx_str id) = 0;
		};

		interface ISortFields
		{
			virtual sx_uint get_Count() const = 0;

			virtual IStr* GetName(sx_uint ind) const = 0;
			virtual sx_bool GetAsc(sx_uint ind) const = 0;
			virtual void Set(sx_str name, sx_uint prio, sx_bool bAsc = true) = 0;
			virtual void Clear(sx_uint name) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface INavigator : public IRefObject
		{
			virtual IStr* get_Name() const = 0;
			virtual void set_Name(sx_str Name) = 0;
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str sDescription) = 0;
			virtual IStr* get_ID() const = 0;
			virtual void set_ID(sx_str ID) = 0;
			virtual IStr* get_Version() const = 0;
			virtual void set_Version(sx_str Version) = 0;
			virtual IStr* get_APIVersion() const = 0;
			virtual void set_APIVersion(sx_str Version) = 0;
			virtual load_type get_LoadType() const = 0;
			virtual void set_LoadType(load_type set) = 0;
			virtual IStr* get_StartRes() const = 0;
			virtual void set_StartRes(sx_str Res) = 0;
			virtual IStr* get_IconRes() const = 0;
			virtual void set_IconRes(sx_str Res) = 0;

			virtual IFields* GetInitialFields() const = 0;
			virtual ILocales* GetLocales() const = 0;
			virtual IResources* GetResources() const = 0;
			virtual void CopyTo(INavigator* pNav) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface INavPackage : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// properties
			virtual sx_str get_FilePath() const = 0;
			virtual sx_bool get_Modified() const = 0;

			//////////////////////////////////////////////////////////////////////////
			// methods
			virtual void OpenStream(ISequentialStream* pStream) = 0;
			virtual void OpenFile(sx_str sFilePath) = 0;
			virtual sx_qword SaveAsStream(ISequentialStream* pStream) = 0;
			virtual void SaveAsFile(sx_str sFilePath) = 0;

			virtual INavigator* GetNavigator() = 0;
		};
	} // namespace NAV
} // namespace SX
