// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "types.h"
#include "basic.h"

namespace SX
{
	namespace Graphics { interface IBitmapData; }

	namespace OCR
	{
		enum engine_status { engine_status_ready = 1, engine_status_no_dict = 2 };
		enum tess_mode { tess_mode_norm = 1, tess_mode_cube = 2, tess_mode_combined = 3 };

		//////////////////////////////////////////////////////////////////////////
		// word enumerator
		interface IOCRWords
		{
			virtual IStr* NextWord(sx_rect& bbox) = 0;
			virtual void Reset() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// ocr'ed image
		interface IOCRImage : public IRefObject
		{
			virtual IStr* GetText() = 0;
			virtual void GetCharBox(sx_uint index, sx_rect& rc) const = 0;
			virtual IOCRWords* get_Words() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// ocr engine
		interface IOCREngine : public IRefObject
		{
			virtual sx_uint get_MinImageSize() const = 0;
			virtual void set_MinImageSize(sx_uint m) = 0;
			virtual engine_status get_Status() = 0;

			virtual IOCRImage* CreateImage(Graphics::IBitmapData* pBitmapData) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// Tesseract engine
		interface ITesseract : public IOCREngine
		{
			virtual sx_str get_ResourcesFolder() const = 0;
			virtual void set_ResourcesFolder(sx_str sFolder) = 0;
			virtual sx_str get_Languages() const = 0;
			virtual void set_Languages(sx_str sLang) = 0;
			virtual tess_mode get_Mode() const = 0;
			virtual void set_Mode(tess_mode mode) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// main library interface
		interface IOCRTools
		{
			virtual ITesseract* GetTesseract() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// library entry point
		typedef IOCRTools* (__cdecl* OCRLibraryEntryPoint)();
	} // namespace OCR
} // namespace SX
