// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once

#include "document.h"

namespace SX
{
	object_type obj_type_ooxml_document = 0x35;

	namespace OOXML
	{
		interface IPackageInfo;

		//////////////////////////////////////////////////////////////////////////
		enum package_format
		{
			package_format_not_opc,
			package_format_not_ooxml,
			package_format_enc_ooxml,
			package_format_wordprocessing,
			package_format_spreadsheet,
			package_format_presentation
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPackage : public IRefObject
		{
			virtual package_format get_Format() const = 0;
			virtual IPackageInfo* get_Info() = 0;
			virtual void OpenStream(ISequentialStream* pStream) = 0;
			virtual sx_bool SetPassword(sx_str sPassword) = 0;
			virtual ISimpleDocument* LoadDocument(ILoadStatus* pStatus = nullptr) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum prop_part
		{
			prop_part_core = 1,
			prop_part_extended,
			prop_part_custom
		};

		enum prop_type
		{
			prop_type_text = 1,
			prop_type_num,
			prop_type_date,
			prop_type_bool,
			prop_type_group = 10
		};

		//////////////////////////////////////////////////////////////////////////
		interface IProperties
		{
			// entries
			virtual sx_uint   get_Count() const = 0;
			virtual sx_str    GetPropName(sx_uint ind) const = 0;
			virtual prop_type GetPropType(sx_uint ind) const = 0;
			// entry values
			virtual sx_str    GetText(sx_str name) const = 0;
			virtual sx_double GetNum(sx_str name) const = 0;
			virtual sx_time   GetDate(sx_str name) const = 0;
			virtual sx_bool   GetBool(sx_str name) const = 0;
			virtual IProperties* GetGroup(sx_str name) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IPackageInfo : public IRefObject
		{
			virtual IProperties* GetProperties(prop_part part = prop_part_core) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// Word documents
		//////////////////////////////////////////////////////////////////////////
		interface IWordImport
		{
			virtual sx_bool get_HiddenText() const = 0;
			virtual void set_HiddenText(sx_bool b) = 0;
		};

		interface IWordProcessing : public IPackageInfo
		{
			virtual IWordImport* get_ImportOptions() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// Excel spreadsheets
		//////////////////////////////////////////////////////////////////////////
		interface IExcelImport
		{
			virtual sx_bool get_Import() const = 0;
			virtual void set_Import(sx_bool b) = 0; // is not relevant for workbook 
			virtual sx_bool get_SplitSheet() const = 0;
			virtual void set_SplitSheet(sx_bool b) = 0;
			virtual sx_bool get_Gridlines() const = 0;
			virtual void set_Gridlines(sx_bool b) = 0;
			virtual sx_bool get_Headings() const = 0;
			virtual void set_Headings(sx_bool b) = 0;
			virtual sx_bool get_DownThenOver() const = 0;
			virtual void set_DownThenOver(sx_bool b) = 0;
		};

		interface IExcelWorksheet
		{
			virtual IExcelImport* get_ImportOptions() = 0;
			virtual sx_str get_Name() const = 0;
		};

		interface IExcelWorkbook : public IPackageInfo
		{
			virtual sx_uint get_ActiveSheet() const = 0;
			virtual sx_uint get_Sheets() = 0;
			virtual IExcelWorksheet* GetSheet(sx_uint ind) = 0;
			virtual IExcelWorksheet* GetSheet(sx_str name) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// PowerPoint presentations
		//////////////////////////////////////////////////////////////////////////
		interface IPPointSection
		{
			virtual sx_str get_Name() const = 0;
		};

		interface IPPointPresentation : public IPackageInfo
		{
			virtual sx_uint get_Sections() = 0;
			virtual IPPointSection* GetSection(sx_uint ind) = 0;
			virtual IPPointSection* GetSection(sx_str name) = 0;
		};
	} // namespace OOXML
} // namespace SX