// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "graphics.h"

namespace SX
{
	enum print_page_set
	{
		print_page_set_all,
		print_page_set_odd,
		print_page_set_even,
		print_page_set_range,
		print_simulate_duplex_part1,
		print_simulate_duplex_part2
	};

	enum print_page_scale
	{
		print_page_scale_orig,
		print_page_scale_fit4print,
		print_page_scale_shrink4print,
		print_page_scale_fit2sheet,
		print_page_scale_shrink2sheet
	};

	enum print_content_as
	{
		print_content_as_is,
		print_content_as_text2curves,
		print_content_as_bitmap,
		print_content_with_ttf
	};

	interface IPrintStatus;
	interface IStr;
	namespace Graphics{interface IBitmapData;}	

	interface IPrintProvider : public IRefObject
	{
		//////////////////////////////////////////////////////////////////////////
		// properties
		virtual print_page_set get_PageSet() const = 0;
		virtual void set_PageSet(print_page_set set) = 0;
		virtual IStr* get_PageRange() const = 0;
		virtual void set_PageRange(sx_str sRange) = 0;
		virtual sx_bool get_ReverseOrder() const = 0;
		virtual void set_ReverseOrder(sx_bool bRev) = 0;
		virtual print_page_scale get_PageScale() const = 0;
		virtual void set_PageScale(print_page_scale scale) = 0;
		virtual sx_bool get_AutoRotate() const = 0;
		virtual void set_AutoRotate(sx_bool bRotate) = 0;
		virtual sx_bool get_Centered() const = 0;
		virtual void set_Centered(sx_bool bCentered) = 0;
		virtual print_content_as get_PrintAs() const = 0;
		virtual void set_PrintAs(print_content_as cont) = 0;
		virtual sx_flags get_DrawOptions() const = 0;
		virtual void set_DrawOptions(sx_flags opt) = 0; 
		virtual sx_flags get_PDFAnnots() const = 0;
		virtual void set_PDFAnnots(sx_flags annots) = 0;
		virtual sx_flags get_AdjustPaperSize() const = 0;
		virtual void set_AdjustPaperSize(sx_flags settings) = 0;

		//////////////////////////////////////////////////////////////////////////
		// methods
		virtual void StartPrint(IPrintStatus* pStatusHandler = nullptr) = 0;
		virtual sx_uint GetSheetsCount() const = 0;
		virtual Graphics::IBitmapData* GetSheetThumbnail(sx_uint sheet, sx_uint width, sx_uint height) const = 0;
	};

	interface IPrintStatus
	{
		virtual sx_bool OnSheet(sx_uint doc_ind, sx_uint ind, sx_uint count) = 0;
	};
	
} // end SX