// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once

#include "basic.h"
#include "types.h"

namespace SX
{
	interface ITextRegion;
	namespace Graphics { interface IBitmapData; }
	namespace PDF { interface ICompositeImage; interface IMarkInfo; interface IAutoText; }

	namespace Content
	{
		interface IRichCollection;
		interface IRichClipboard;
		interface IObjectTrunk;
		interface IObjectFinder;
		interface IRichObject;
		interface IRichGroup;
		interface ITextStyle;
		interface IPath;
		interface IBrush;
		interface ISolidBrush;
		interface IPen;
		interface IColorStops;
		interface IDashStyle;
		interface IFont;
		interface IFontStream;
		interface IAutoText;
		interface IAutoTextPattern;		
		interface IShapeObject;
		interface IBitmap;
		enum brush_type;

		object_type obj_type_rich_content = 0x2C;
		object_type obj_type_trunk = 0x2D;

		interface IRichContent : public IRefObject
		{
			// misc
			virtual IRichCollection* get_Objects() = 0;
			virtual sx_bool get_Modified() const = 0;
			virtual sx_bool get_SynchronizeChanges() const = 0;
			virtual void set_SynchronizeChanges(sx_bool bSyncChanges) = 0;

			virtual sx_uint GetUPI() const = 0;
			virtual void GetMatrix(sx_matrix& m) const = 0;
			virtual void GetBBox(const sx_matrix& m, sx_rect &rc) const = 0;
			virtual sx_bool GetAbsBBox(sx_rect& rc) const = 0;
			virtual sx_bool GetBBox(sx_rect& rc) const = 0;
			virtual sx_bool UpdateAutoText(IAutoText* pAutoText = nullptr, sx_uint uPage = INVALID_INDEX ) = 0;
			virtual sx_bool TransformBBox(const sx_double &w, const sx_double & h, const sx_double & hoffset, const sx_double & voffset) = 0;
			virtual void Clear() = 0;

			// factory
			virtual IRichClipboard* GetDocClipboard() = 0;
			virtual IPath* CreatePath() = 0;
			virtual IPen* CreatePen(const sx_double& width) = 0;
			virtual IBrush* CreateBrush(brush_type type) = 0;
			virtual ISolidBrush* CreateSolidBrush(const sx_color& color) = 0;
			virtual IDashStyle* CreateDashStyle() = 0;
			virtual IColorStops* CreateColorStops() = 0;
			virtual IFont* CreateFont(sx_str family, sx_uint weight = 400, sx_bool italic = 0, sx_uint stretch = 5) = 0; 
			virtual ITextStyle* CreateTextStyle() = 0;
			virtual IRichContent* CreateRichContent(sx_double w, sx_double h) = 0;
			virtual IObjectFinder* CreateObjectFinder() = 0;
			virtual IBitmap* CreateSharedBitmap() const = 0;
			virtual IFontStream* CreateSharedFont() const = 0;
			virtual IFontStream* CreateSharedFont(sx_str family, sx_uint weight = 400, sx_bool italic = 0, sx_uint stretch = 5) const = 0; 
			virtual IFontStream* CreateSharedFont(ISequentialStream* pData, sx_uint weight = 400, sx_bool italic = 0, sx_uint stretch = 5) const = 0;
		};

		enum rich_type
		{
			rich_type_blank = 0,
			rich_type_group = 1,
			rich_type_vector = 2,
			rich_type_raster = 4,
			rich_type_text = 8,
			rich_type_composite = 16
		};

		enum dyn_props
		{
			dyn_props_def           = 0,
			dyn_props_locked        = 0x01,
			dyn_props_hidden		= 0x02
		};

		enum find_opt
		{
			find_opt_def			= 0,
			find_opt_atomic_comp	= 0x01,
			find_opt_underlay_comp	= 0x02,
			find_opt_atomic_group	= 0x04,
			find_opt_underlay_group	= 0x08,
			find_opt_reverse_find	= 0x10,
			find_opt_entire_rect	= 0x20,
			find_opt_ignore_locked	= 0x40,
			find_opt_smart_text		= 0x80,
			find_opt_smart_line		= 0x100
		};

		enum conv_options
		{
			conv_options_auto_groups	= 0x1,
			conv_options_sep_text		= 0x2,
			conv_options_recover_props	= 0x4,
			conv_options_backup_props	= 0x8,
			conv_options_mark_groups	= 0x10,
			conv_options_comp_fonts		= 0x20,
			conv_options_bidi_text		= 0x40
		};

		enum text_layout
		{
			text_layout_linearize	= 0x01,
			text_layout_wrap		= 0x02,
			text_layout_align		= 0x04,
			text_layout_clear_nl	= 0x10,
			text_layout_clear_marg	= 0x20,
			text_layout_check_line_spacing = 0x40
		};

		enum text_alignment
		{
			text_alignment_left = 1,
			text_alignment_center,
			text_alignment_right
		};

		enum shape_type;

		interface IRichCollection
		{
			virtual IRichContent* get_Root() = 0;
			virtual IRichGroup* get_Group() = 0;
			virtual sx_uint get_Count() const = 0;
			virtual IRichObject* Item(sx_uint uIndex) = 0;
			virtual sx_uint IndexOf(IRichObject* pItem) const = 0;
			virtual IRichObject* Insert(sx_uint index, rich_type type) = 0;
			virtual IRichObject* InsertCopy(sx_uint index, IRichObject* pObject) = 0;
			virtual IRichGroup*  InsertShape(sx_uint index, shape_type shape) = 0;
			virtual void CopyFrom(sx_uint index, IRichCollection* pCollection) = 0;
			virtual void Ungroup(sx_uint index) = 0;
			virtual void Move(sx_uint from, sx_uint to) = 0;
			virtual void Remove(sx_uint index) = 0;
		};

		interface IRichObject
		{
			virtual rich_type get_Type() const = 0;
			virtual IRichCollection* get_Collection() const = 0;
			virtual IPath* get_Clip() const = 0;
			virtual void set_Clip(IPath* pPath) = 0;
			virtual sx_double get_Opacity() const = 0;
			virtual void set_Opacity(const sx_double &opacity) = 0;
			virtual IRichObject* get_OpacityMask() const = 0;
			virtual void set_OpacityMask(IRichObject* pMask) = 0;
			virtual IStr* get_BlendMode() const = 0;
			virtual void set_BlendMode(sx_str pMode) = 0;
			virtual sx_flags get_DynProps() const = 0;
			virtual void set_DynProps(sx_flags props) = 0;

			virtual void GetMatrix(sx_matrix& m) const = 0;
			virtual void SetMatrix(const sx_matrix& m) = 0;
			virtual void GetTransformation(sx_matrix& m) const = 0;
			virtual void ApplyTransformation(const sx_matrix &transform) = 0;
			virtual void GetBBox(const sx_matrix& m, sx_rect &rc) const = 0;

			virtual IObjectTrunk* CreateTrunk() const = 0;
			virtual PDF::IMarkInfo* GetMarkInfo() const = 0;
			virtual PDF::IMarkInfo* SetMarkInfo(sx_str Name) = 0;
		};

		interface IRichGroup : public IRichObject
		{
			virtual IRichCollection* get_Objects() = 0;
			virtual IShapeObject* get_Shape() = 0;
		};

		interface IRichRaster : public IRichObject
		{
			virtual Graphics::IBitmapData* get_Bitmap() = 0;
			virtual void set_Bitmap(Graphics::IBitmapData* pBitmap) = 0;
			virtual IBitmap* get_SharedBitmap() const = 0;
			virtual void set_SharedBitmap(IBitmap* pBitmap) = 0;
		};
		
		interface IRichComposite : public IRichObject
		{
			virtual IRichContent* get_Content() const = 0;
			virtual void set_Content(IRichContent* pContent) = 0;
			virtual PDF::ICompositeImage* get_PDFImage() const = 0;
			virtual void GetContentTransformation(sx_matrix& m) const = 0;
			virtual void SpliceTrunk(IObjectTrunk* pTrunk) const = 0;
		}; 

		interface IRichText : public IRichObject
		{
			virtual sx_uint get_Length() const = 0;
			virtual IStr* get_Text() const = 0;
			virtual ITextStyle* GetDefaultStyle() const = 0;
			virtual ITextStyle* GetCharStyle(sx_uint uIndex) const = 0;
			virtual void SetCharStyle(sx_uint Start, sx_uint Count, ITextStyle* pStyle) = 0;
			virtual void InsertChars(sx_uint uIndex, sx_str sText, ITextStyle* pStyle = nullptr) = 0;
			virtual void InsertLineBreak(sx_uint uIndex) = 0;
			virtual void RemoveChars(sx_uint Start, sx_uint Count) = 0;
			virtual ITextRegion* GetTextRegion() = 0;
			virtual ITextRegion* GetTextRegion(const sx_matrix &m) = 0;
			virtual IAutoTextPattern* GetAutoTextPattern() = 0;
			virtual sx_bool UpdateAutoText(IAutoText *pAutoText = nullptr, sx_uint uPage = INVALID_INDEX) = 0;
			virtual sx_bool Relayout(sx_double width, sx_flags layout, text_alignment ta = text_alignment_left) = 0;
		};

		interface IRichVector : public IRichObject
		{
			virtual IPath* get_Path() const = 0;
			virtual void set_Path(IPath* pPath) = 0;
			virtual IPen* get_Pen() const = 0;
			virtual void set_Pen(IPen* pPen) = 0;
			virtual IBrush* get_StrokeBrush() const = 0;
			virtual void set_StrokeBrush(IBrush* pBrush) = 0;
			virtual IBrush* get_FillBrush() const = 0;
			virtual void set_FillBrush(IBrush* pBrush) = 0;
		};
		
		interface IObjectTrunk : public IRefObject
		{			
			virtual IRichContent* GetRoot() const = 0;
			virtual IRichObject* GetTargetObject() const = 0;
			virtual void GetTransformation(sx_matrix& m) const = 0;
			virtual void ApplyTransformation(const sx_matrix &transform) = 0;
			virtual IRichComposite* GetIntermediateNode(sx_int level) const = 0;
			virtual void Splice(IObjectTrunk* pTrunk) const = 0;
			virtual sx_bool Exists() const = 0;
			virtual sx_bool Equals(IObjectTrunk* pObj) const = 0;
			virtual sx_bool Contains(IObjectTrunk* pObj) const = 0;
		};

		interface IObjectFinder : public IRefObject
		{
			virtual void FindAtPoint(const sx_point& pt, sx_flags opt = find_opt_def) = 0;
			virtual void FindWithinArea(const sx_rect& rc, sx_flags opt = find_opt_def) = 0;
			virtual void FindObject(IRichObject* pRO, sx_flags opt = find_opt_def) = 0;
			virtual void FindText(sx_str text, sx_flags opt = find_opt_def) = 0;
			virtual void FindByMCID(sx_uint mcid, sx_flags opt = find_opt_def) = 0;

			virtual IObjectTrunk* FindNext() = 0;
		};
		
		interface IRichClipboard
		{
			virtual sx_uint get_Count() const = 0;
			virtual IRichObject* Item(sx_uint index) = 0;
			virtual IRichObject* Insert(sx_uint index, IRichObject* pObject) = 0;
			virtual void Remove(sx_uint index) = 0;
			virtual void Clear() = 0;
		};

		interface IBitmap : public IRefObject
		{
			virtual void SetBitmap(Graphics::IBitmapData* pBitmap) = 0;
			virtual void SetJpegBitmap(ISequentialStream* pJpeg) = 0;
		};
		
		interface IFontStream : public IRefObject
		{
			virtual void SetFontData(ISequentialStream* pData) = 0;
		};

		typedef const sx_short macro_field;
		macro_field
			macro_field_page_num		= 0xE001,
			macro_field_page_label		= 0xE002,
			macro_field_page_count		= 0xE003,
			macro_field_doc_title		= 0xE004,
			macro_field_doc_subject		= 0xE005,
			macro_field_doc_author		= 0xE006,

			macro_field_ph_num			= 0xE00A,
			macro_field_ph_count		= 0xE00B,
			macro_field_ref_page_num	= 0xE00C,
			macro_field_ref_page_count	= 0xE00D,
			macro_field_rng_page_num	= 0xE00E,
			macro_field_rng_page_count	= 0xE00F,

			macro_field_user			= 0xE011,
			macro_field_timeshort		= 0xE021,
			macro_field_timelong		= 0xE022,
			macro_field_dateshort		= 0xE023,
			macro_field_datelong		= 0xE024,

			macro_field_custom			= 0xE080;

		interface IAutoTextMacro
		{
			virtual sx_uint get_Page() const = 0;
			virtual sx_str get_Pattern() const = 0;
			virtual void SetText(sx_str sText) = 0;
		};

		interface IAutoText
		{
			virtual sx_bool OnMacro(IAutoTextMacro* pMacro) = 0;
		};

		interface IAutoTextPattern
		{
			virtual IStr* get_Pattern() const = 0;
			virtual void set_Pattern(sx_str pPattern) = 0;
			virtual text_alignment get_Alignment() const = 0;
			virtual void set_Alignment(text_alignment al) = 0;
		};

		interface IFont
		{
			virtual IStr* get_Family() const = 0;
			virtual sx_bool get_Italic() const = 0;
			virtual sx_uint get_Weight() const = 0;
			virtual sx_uint get_Stretch() const = 0;
			virtual IStr* get_PSName() const = 0;
		};

		interface ITextStyle
		{
			virtual IFont* get_Font() const = 0;
			virtual void set_Font(IFont* pFont) = 0;
			virtual sx_double get_Size() const = 0;
			virtual void set_Size(const sx_double &dFontSize) = 0;
			virtual sx_double get_LetterStretch() const = 0;
			virtual void set_LetterStretch(const sx_double &uStretch) = 0;
			virtual sx_double get_Rise() const = 0;
			virtual void set_Rise(const sx_double &dRise) = 0;
			virtual sx_double get_LetterSpacing() const = 0;
			virtual void set_LetterSpacing(const sx_double &letterSpacing) = 0;
			virtual sx_double get_WordSpacing() const = 0;
			virtual void set_WordSpacing(const sx_double &wordSpacing) = 0;
			virtual sx_double get_Leading() const = 0;
			virtual void set_Leading(const sx_double &leading) = 0;
			virtual const sx_color* get_Color() const = 0;
			virtual void set_Color(const sx_color& color) = 0;
			virtual IBrush* get_Fill() const = 0;
			virtual void set_Fill(IBrush* pBrush) = 0;
			virtual IBrush* get_Stroke() const = 0; //PDF only
			virtual void set_Stroke(IBrush* pBrush) = 0; //PDF only
			virtual IPen *get_StrokePen() const = 0; //PDF only
			virtual void set_StrokePen(IPen *pPen) = 0; //PDF only
			virtual sx_bool get_DirectionRTL() const = 0;
			virtual void set_DirectionRTL(sx_bool bRTL) = 0;
			virtual sx_bool get_DirectionVertical() const = 0;
			virtual void set_DirectionVertical(sx_bool bVert) = 0;

			virtual void SetFont(sx_str family, const sx_double & size, sx_uint weight = 400, sx_bool italic = false, sx_uint stretch  = 5) = 0;
			virtual void SetFont(const sx_color& color, sx_str family, const sx_double &  size, sx_uint weight = 400, sx_bool italic = false, sx_uint stretch  = 5) = 0;
			virtual void SetFont(ISequentialStream* pFontStream) = 0;
			virtual void SetFont(IFontStream* pFontStream) = 0;
			virtual ITextStyle *Clone() const = 0;
			virtual sx_bool IsEqual(ITextStyle *pOtherStyle) = 0;
		};

		enum line_cap
		{
			line_cap_butt = 0, 
			line_cap_round = 1,
			line_cap_square = 2
		};
		
		enum line_join
		{
			line_join_miter = 0, 
			line_join_round = 1,
			line_join_bevel = 2
		};

		interface IResource
		{
			virtual IRichContent* get_Root() = 0;
		};

		interface IPen : public IResource
		{
			virtual sx_double get_Width() const = 0;
			virtual void set_Width(const sx_double &width) = 0;
			virtual line_cap get_LineCap() const = 0;
			virtual void set_LineCap(line_cap cap) = 0;
			virtual line_join get_LineJoin() const = 0;
			virtual void set_LineJoin(line_join join) = 0;
			virtual sx_double get_MiterLimit() const = 0;
			virtual void set_MiterLimit(const sx_double &miterLimit) = 0;
			virtual IDashStyle *get_DashStyle() const = 0;
			virtual void set_DashStyle(IDashStyle *pStyle) = 0;
		};

		interface IDashStyle : public IResource
		{
			virtual sx_uint get_Count() const = 0;
			virtual sx_double get_Phase() const = 0;
			virtual void set_Phase(const sx_double& dPhase) = 0;
			virtual sx_double GetItem(sx_uint uIndex) const = 0;
			virtual void Add(const sx_double& dash) = 0;
			virtual void Clear() = 0;
		};

		enum brush_type
		{
			brush_type_solid = 1,
			brush_type_lgradient,
			brush_type_rgradient,
			brush_type_image,
			brush_type_pattern
		};

		interface IBrush : public IResource
		{
			virtual brush_type get_Type() const = 0;
			virtual sx_double get_Opacity() const = 0;
			virtual void set_Opacity(const sx_double &opacity) = 0;
			virtual void GetMatrix(sx_matrix &m) const = 0;
			virtual void SetMatrix(const sx_matrix &m) = 0;
		};

		interface ISolidBrush : public IBrush
		{
			virtual const sx_color* get_Color() const = 0;
			virtual void set_Color(const sx_color& color) = 0;
		};

		enum gradient_spread_method
		{
			gradient_spread_method_none,	//PDF only
			gradient_spread_method_pad,
			gradient_spread_method_repeat,	//XPS only
			gradient_spread_method_reflect	//XPS only
		};

		interface ILinearGradientBrush : public IBrush
		{
			virtual IColorStops* get_ColorStops() const = 0;
			virtual void set_ColorStops(IColorStops* pColorStops) = 0;
			virtual gradient_spread_method get_SpreadMethod() const = 0;
			virtual void set_SpreadMethod(gradient_spread_method spreadMethod) = 0;

			virtual void GetStart(sx_point &ptStart) const = 0;
			virtual void SetStart(const sx_point &ptStart) = 0;
			virtual void GetEnd(sx_point &ptEnd) const = 0;
			virtual void SetEnd(const sx_point &ptEnd) = 0;
		};

		interface IRadialGradientBrush : public IBrush
		{
			virtual IColorStops* get_ColorStops() const = 0;
			virtual void set_ColorStops(IColorStops* pColorStops) = 0;
			virtual gradient_spread_method get_SpreadMethod() const = 0;
			virtual void set_SpreadMethod(gradient_spread_method spreadMethod) = 0;

			virtual void GetStart(sx_point &ptStart, sx_double &dRadius) const = 0;
			virtual void SetStart(const sx_point &ptStart, const sx_double &dRadius) = 0;
			virtual void GetEnd(sx_point &ptEnd, sx_double &dRadius) const = 0;
			virtual void SetEnd(const sx_point &ptEnd, const sx_double &dRadius) = 0;
		};

		enum brush_tile_mode
		{
			brush_tile_mode_none,
			brush_tile_mode_tile,
			brush_tile_mode_flipx,
			brush_tile_mode_flipy,
			brush_tile_mode_flipxy
		};

		interface IPatternBrush: public IBrush
		{
			virtual brush_tile_mode get_TileMode() const = 0;
			virtual void set_TileMode(brush_tile_mode tileMode) = 0;
			virtual IRichContent *get_Content() const = 0;
			virtual void set_Content(IRichContent *pContent) = 0;
			virtual void GetStep(sx_point &step) const = 0;
			virtual void SetStep(const sx_point &step) = 0;			
		};

		interface IImageBrush : public IBrush
		{
			virtual Graphics::IBitmapData *get_Image() = 0;
			virtual void set_Image(Graphics::IBitmapData * pImage) = 0;
			virtual IBitmap* get_SharedBitmap() const = 0;
			virtual void set_SharedBitmap(IBitmap* pBitmap) = 0;

			virtual brush_tile_mode get_TileMode() const = 0;
			virtual void set_TileMode(brush_tile_mode tileMode) = 0;

			virtual void GetViewBox(sx_rect &viewBox) const = 0;
			virtual void SetViewBox( const sx_rect &viewBox) = 0;
			virtual void GetViewPort(sx_rect &viewPort) const = 0;
			virtual void SetViewPort( const sx_rect &viewPort) = 0;
		};
		
		interface IColorStops : public IResource
		{
			virtual sx_uint get_Count() const = 0;
			virtual const sx_color* get_Color(sx_uint uColorStop) const = 0;
			virtual sx_double get_Stop(sx_uint uColorStop) const = 0;
			virtual void Add(const sx_double& dStop, const sx_color& color) = 0;
			virtual void Clear() = 0;
		};


		enum fill_rule
		{
			fill_rule_even_odd,
			fill_rule_non_zero = 0x01000000
		};

		interface IFigure;

		interface IPath : public IResource
		{
			virtual sx_uint get_Count() const = 0;
			virtual fill_rule get_FillRule() const = 0;
			virtual void set_FillRule(fill_rule fr) = 0;

			virtual IFigure* AddLine(const sx_point &pt1, const sx_point &pt2) = 0;
			virtual IFigure* AddBox(const sx_rect &box, sx_bool bStroke = true, sx_bool bFill = true) = 0;
			virtual IFigure* AddEllipse(const sx_rect &box, sx_bool bStroke = true, sx_bool bFill = true) = 0;
			virtual sx_bool AddUnderline(IRichText* pText, sx_uint start, sx_uint end) = 0;
			virtual sx_bool AddStrikeout(IRichText* pText, sx_uint start, sx_uint end) = 0;
			virtual IFigure* GetFigure(sx_uint index) = 0;
			virtual IFigure* InsertFigure(sx_uint index) = 0;
			virtual IFigure* CopyFigure(sx_uint index, IFigure* pFrom) = 0;
			virtual void RemoveFigure(sx_uint index) = 0;
			virtual void GetBBox(sx_rect& rc) const = 0;
			virtual void ApplyTransformation(const sx_matrix& transform) = 0;
			virtual void Clear() = 0;
		};

		enum segment_type
		{
			segment_type_line = 1,
			segment_type_sline = 2,
			segment_type_bezier = 3,
			segment_type_qbezier = 4,
			segment_type_arc = 5
		};

		interface ISegment;

		enum figure_type
		{
			figure_type_empty,
			figure_type_line,
			figure_type_arc,
			figure_type_rect,
			figure_type_ellipse,
			figure_type_simple_polyline,
			figure_type_simple_polygone,
			figure_type_complex
		};

		interface IFigure
		{
			virtual sx_bool get_Closed() const = 0;
			virtual void set_Closed(sx_bool bClosed) = 0;
			virtual sx_bool get_Fill() const = 0;
			virtual void set_Fill(sx_bool bFill) = 0;
			virtual sx_uint get_Count() const = 0;

			virtual void GetStartPoint(sx_point& pt) = 0;
			virtual void SetStartPoint(const sx_point& pt) = 0;
			virtual ISegment* GetSegment(sx_uint index) const = 0;
			virtual ISegment* InsertSegments(segment_type type, const sx_uint index, sx_uint count = 1) = 0;
			virtual void RemoveSegments(sx_uint index, sx_uint count = 1) = 0;
			virtual IFigure* NextFigure() = 0;
			virtual figure_type GetType() = 0;
		};

		interface ISegment
		{
			virtual segment_type get_Type()  = 0;
			virtual sx_bool get_Stroke() const = 0;
			virtual void set_Stroke(sx_bool bStroke) = 0;
			virtual ISegment* NextSegment() = 0;
		};

		interface ILine : public ISegment
		{
			virtual void GetInfo(sx_point &pt) const = 0;
			virtual void SetInfo(const sx_point &pt) = 0;
		};

		interface ISLine : public ISegment
		{
			virtual void GetInfo(sx_point &pt1, sx_point &pt2) const = 0;
			virtual void SetInfo(const sx_point &pt1, const sx_point &pt2) = 0;
		};

		interface IBezier : public ISegment
		{
			virtual void GetInfo(sx_point &pt1, sx_point &pt2, sx_point &pt3) const = 0;
			virtual void SetInfo(const sx_point &pt1, const sx_point &pt2, const sx_point &pt3) = 0;
		};

		interface IQBezier : public ISegment
		{
			virtual void GetInfo(sx_point &pt1, sx_point &pt2) const = 0;
			virtual void SetInfo(const sx_point &pt1, const sx_point &pt2) = 0;
		};

		interface IArc : public ISegment
		{
			virtual void GetInfo(sx_point& pt, sx_size& rad, sx_double& rot, sx_bool& bLarge, sx_bool& bSweep) const = 0;
			virtual void SetInfo(const sx_point& pt, const sx_size& rad, const sx_double& rot, sx_bool bLarge, sx_bool bSweep) = 0;
		};
	} // namespace Content

	namespace PDF
	{
		interface ILayer;
		interface IRawProps;

		interface IMarkInfo
		{
			virtual IStr* get_Name() const = 0;
			virtual IRawProps* get_Props() const = 0;
			virtual sx_int get_MCID() = 0;
			virtual void set_MCID(sx_int id) = 0;
			virtual ILayer* get_Layer() = 0;
			virtual void set_Layer(ILayer* pLayer) = 0;
			virtual void ResetProps(sx_bool bClear = false, sx_bool bIndirect = false) = 0;
		};
	}
} // namespace SX