// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "graphics.h"

namespace SX
{
	enum plain_text
	{
		plain_text_default					=	0,
		plain_text_skip_hidden				=	0x0010,
		plain_text_consider_ligatures		=	0x0020,
		plain_text_unix_line_feed			=	0x0040,
		plain_text_skip_present_forms		=	0x0080,
		plain_text_simulate_spaces			=	0x0100,
		plain_text_simulate_tabs			=	0x0200,
		plain_text_simulate_lines			=	0x0400,
		plain_text_skip_overlapped_chars	=	0x0800,
		plain_text_sort_line_left2right		=	0x1000,
		plain_text_sort_lines_top2bottom	=	0x2000,
		plain_text_sort_bidi_chars			=	0x4000
	};

	enum search_opt
	{
		search_opt_case_sensitive		=	0x1,
		search_opt_whole_word			=	0x2,
		search_opt_reg_ex				=	0x4,
		search_opt_consider_text_opt	=	0x8,
		search_opt_consider_ligatures	=	plain_text_consider_ligatures
	};

	enum replace_opt
	{
		replace_opt_reset_font			=	0x1
	};

	interface ISearchStatus;
	namespace Graphics { interface IRegion; }

	const object_type 
		obj_type_search_provider =			0x20;

	interface ISearchProvider : public IRefObject
	{
		//////////////////////////////////////////////////////////////////////////
		// properties
		virtual sx_str get_Text() const = 0;
		virtual sx_uint get_Page() const = 0;
		virtual Graphics::IRegion* get_Selection() const = 0;

		//////////////////////////////////////////////////////////////////////////
		// methods
		virtual void Reset(sx_str sText, sx_uint Page = 0, sx_flags searchFlags = 0) = 0;
		virtual sx_bool SearchForward(ISearchStatus* pStatusHandler = nullptr) = 0;
		virtual sx_bool SearchBack(ISearchStatus* pStatusHandler = nullptr) = 0;
		virtual IStr* GetContext(sx_uint MaxLen = 64) = 0;
		virtual sx_bool Replace(sx_str sText, sx_flags opt = 0) = 0;
	};

	interface ISearchStatus
	{
		virtual sx_bool OnPage(sx_uint ind, sx_uint count) = 0;
	};

	interface ITextRegion : public IRefObject
	{
		virtual sx_uint get_Count() const = 0;
		virtual sx_short GetChar(sx_uint uIndex) const = 0;
		virtual void GetQuad(sx_uint uIndex, sx_quad& quad) const = 0;
		virtual IStr* GetText(sx_uint uStart, sx_uint uEnd) = 0;
		virtual Graphics::IRegion* GetSelection(sx_uint uStart, sx_uint uEnd) const = 0;
	};
} // end SX