// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once

#include "rich-content.h"

namespace SX
{
	namespace Content
	{
		enum shape_type
		{
			shape_type_unknown				= 0,

			shape_type_line					= 1,
			shape_type_arc					= 2,
			shape_type_curve				= 3,
			shape_type_elbow				= 4,
			
			shape_type_text_block			= 50,
			shape_type_circle				= 51,
			shape_type_rect					= 52,
			shape_type_triangle				= 53,
			shape_type_rhombus				= 54,
			shape_type_trapezoid			= 55,
			shape_type_parallelogram		= 56,
			shape_type_pentagon				= 57,
			shape_type_hexagon				= 58,
			shape_type_octagon				= 59,
			shape_type_lfigure				= 60,
			shape_type_half_frame			= 61,
			shape_type_partial_circle		= 62,
			shape_type_tear_drop			= 63,
			shape_type_hollow_rect			= 64,
			shape_type_hollow_circle		= 65,
			shape_type_block_arc			= 66,
			shape_type_cross				= 67,
			shape_type_plaque				= 68,
			shape_type_cube					= 69,
			shape_type_cylinder				= 70,
			shape_type_moon					= 71,
			shape_type_bevel				= 72,
			shape_type_cloud				= 73,

			shape_type_block_arrow			= 91,
			shape_type_bent_arrow			= 92,
			shape_type_lru_arrow			= 93,
			shape_type_quad_arrow			= 94,
			shape_type_uturn_arrow			= 95,
			shape_type_striped_arrow		= 96,
			shape_type_notched_arrow		= 97,
			shape_type_pentagon_arrow		= 98,
			shape_type_chevron_arrow		= 99,
			shape_type_callout_arrow		= 100,
			shape_type_callout_lr_arrow		= 101,
			shape_type_callout_quad_arrow	= 102,
			shape_type_curved_arrow			= 103,
			shape_type_rect_callout			= 104,
			shape_type_circle_callout		= 105,
			shape_type_balloon_legend		= 106,
			shape_type_cloud_balloon		= 107,
			shape_type_folded_corner		= 108,
			shape_type_wave					= 109,
			shape_type_scroll				= 110,
			shape_type_ribbon				= 111,

			shape_type_table				= 200
		};

		interface ITextBlock;

		interface IShapeObject 
		{
			virtual shape_type get_Type() = 0;
			virtual IRichGroup* get_Group() = 0;

			virtual sx_point get_Offset() const = 0;
			virtual void set_Offset(const sx_point& pt) = 0;
			virtual sx_size get_Scale() const = 0;
			virtual void set_Scale(const sx_size& sc) = 0;
			virtual sx_double get_Angle() const = 0;
			virtual void set_Angle(const sx_double& ang) = 0;
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str d) = 0;

			virtual sx_bool Update() = 0;
		};

		enum  arrow_head
		{
			arrow_head_simple	= 1,
			arrow_head_stealth	= 2,
			arrow_head_open		= 3,
			arrow_head_square	= 4,
			arrow_head_diamond	= 5,
			arrow_head_circle	= 6,
			arrow_head_butt		= 7
		};

		interface IArrowHead
		{
			virtual arrow_head get_Type() = 0;
			virtual void set_Type(arrow_head type) = 0;
			virtual sx_double get_Weight() = 0;
			virtual void set_Weight(const sx_double& weight) = 0; 
			virtual sx_double get_Length() = 0;
			virtual void set_Length(const sx_double& length) = 0;
		};

		interface IShapeLine: public IShapeObject
		{
			virtual sx_double get_Length() const = 0;
			virtual void set_Length(const sx_double& len) = 0;
			virtual IArrowHead* get_StartHead() = 0;
			virtual IArrowHead* get_EndHead() = 0;
			virtual void SetHeads(sx_bool start, sx_bool end) = 0;
		};

		interface IShapeArc: public IShapeObject
		{
			virtual sx_double get_Start() const = 0;
			virtual void set_Start(const sx_double& val) = 0;
			virtual sx_double get_Length() const = 0;
			virtual void set_Length(const sx_double& val) = 0;
		};

		interface IShapeCurve: public IShapeObject
		{
			virtual sx_point get_StartControlPoint() const = 0;
			virtual void set_StartControlPoint(const sx_point& pt) = 0;
			virtual sx_point get_EndControlPoint() const = 0;
			virtual void set_EndControlPoint(const sx_point& pt) = 0;
		};

		interface IShapeElbow : public IShapeObject
 		{
			virtual sx_bool get_Curved() const = 0;
			virtual void set_Curved(sx_bool v) = 0;
			virtual sx_bool get_Coaxial() const = 0;
			virtual void set_Coaxial(sx_bool v) = 0;
			virtual sx_double get_Treshold() const = 0;
			virtual void set_Treshold(const sx_double& treshold) = 0;
			virtual IArrowHead* get_StartHead() = 0;
			virtual IArrowHead* get_EndHead() = 0;
			virtual void SetHeads(sx_bool Start, sx_bool End) = 0;
		};

		interface IClosedShape: public IShapeObject
		{
			virtual ITextBlock* get_TextBlock() = 0;
		};
		
		enum corners_type
		{
			corners_type_all_normal,
			corners_type_all_rnd,
			corners_type_all_trnk,
			corners_type_one_rnd,
			corners_type_one_trnk,
			corners_type_two_rnd,
			corners_type_two_trnk,
			corners_type_diag_rnd,
			corners_type_diag_trnk,
			corners_type_one_rnd_trnk,
			corners_type_diag_rnd_trnk
		};

		interface IShapeRect : public IClosedShape
		{
			virtual corners_type get_Corners() const = 0; 
			virtual void set_Corners(const corners_type c) = 0;
			virtual sx_double get_CornerSize() const = 0;
			virtual void set_CornerSize(const sx_double sz) = 0;
			virtual sx_double get_SecondaryCornerSize() const = 0;
			virtual void set_SecondaryCornerSize(const sx_double sz) = 0;
		};

		interface IShapeTriangle : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeTrapezoid : public IClosedShape
		{
			virtual sx_bool get_Symmetric() const = 0;
			virtual void set_Symmetric(sx_bool v) = 0;
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
			virtual sx_double get_SecondarySkew() const = 0;
			virtual void set_SecondarySkew(const sx_double& val) = 0;
		};

		interface IShapeParallelogram : public IClosedShape
		{
			virtual sx_bool get_Right() const = 0;
			virtual void set_Right(sx_bool v) = 0;
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeHexagon : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeOctagon : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapePartialCircle : public IClosedShape
		{
			virtual sx_double get_Start() const = 0;
			virtual void set_Start(const sx_double& val) = 0;
			virtual sx_double get_Length() const = 0;
			virtual void set_Length(const sx_double& val) = 0;
			virtual sx_bool get_Chord() const = 0;
			virtual void set_Chord(sx_bool b) = 0;
		};

		interface IShapeTearDrop : public IClosedShape
		{
			virtual sx_bool get_Straight() const = 0;
			virtual void set_Straight(sx_bool b) = 0;
			virtual sx_double get_Tail() const = 0;
			virtual void set_Tail(const sx_double& val) = 0;
		};

		interface IShapeLFigure : public IClosedShape
		{
			virtual sx_double get_Cutout() const = 0;
			virtual void set_Cutout(const sx_double& val) = 0;
			virtual sx_double get_SecondaryCutout() const = 0;
			virtual void set_SecondaryCutout(const sx_double& val) = 0;
		};

		interface IShapeHalfFrame : public IClosedShape
		{
			virtual sx_double get_Cutout() const = 0;
			virtual void set_Cutout(const sx_double& val) = 0;
			virtual sx_double get_SecondaryCutout() const = 0;
			virtual void set_SecondaryCutout(const sx_double& val) = 0;
		};

		interface IShapeHollowRect : public IClosedShape
		{
			virtual sx_double get_Depth() const = 0;
			virtual void set_Depth(const sx_double& val) = 0;
			virtual sx_bool get_Rounded() const = 0;
			virtual void set_Rounded(sx_bool b) = 0;
			virtual sx_double get_CornerSize() const = 0;
			virtual void set_CornerSize(const sx_double sz) = 0;
		};

		interface IShapeHollowCircle : public IClosedShape
		{
			virtual sx_double get_Depth() const = 0;
			virtual void set_Depth(const sx_double& val) = 0;
		};
		
		interface IShapeBlockArc : public IClosedShape
		{
			virtual sx_double get_Start() const = 0;
			virtual void set_Start(const sx_double& val) = 0;
			virtual sx_double get_Length() const = 0;
			virtual void set_Length(const sx_double& val) = 0;
			virtual sx_double get_Depth() const = 0;
			virtual void set_Depth(const sx_double& val) = 0;
		};

		interface IShapeCross : public IClosedShape
		{
			virtual sx_double get_Cutout() const = 0;
			virtual void set_Cutout(const sx_double& val) = 0;
		};

		interface IShapePlaque : public IClosedShape
		{
			virtual sx_double get_Cutout() const = 0;
			virtual void set_Cutout(const sx_double& val) = 0;
		};

		interface IShapeCube : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeCylinder : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeMoon : public IClosedShape
		{
			virtual sx_double get_Fullness() const = 0;
			virtual void set_Fullness(const sx_double& val) = 0;
		};

		interface IShapeBevel : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeBlockArrow : public IClosedShape
		{
			virtual sx_bool get_DoubleHeaded() const = 0;
			virtual void set_DoubleHeaded(sx_bool v) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
		};

		interface IShapeBentArrow : public IClosedShape
		{
			virtual sx_bool get_Right() const = 0;
			virtual void set_Right(sx_bool v) = 0;
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
			virtual sx_bool get_DoubleHeaded() const = 0;
			virtual void set_DoubleHeaded(sx_bool v) = 0;
			virtual sx_bool get_Rounded() const = 0;
			virtual void set_Rounded(sx_bool b) = 0;
			virtual sx_double get_CornerSize() const = 0;
			virtual void set_CornerSize(const sx_double& sz) = 0;
		};

		interface IShapeLRUArrow : public IClosedShape
		{
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
		};

		interface IShapeQuadArrow : public IClosedShape
		{
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
		};

		interface IShapeUTurnArrow : public IClosedShape
		{
			virtual sx_bool get_Right() const = 0;
			virtual void set_Right(sx_bool v) = 0;
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_HeadOffset() const = 0;
			virtual void set_HeadOffset(const sx_double& offset) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
			virtual sx_bool get_Rounded() const = 0;
			virtual void set_Rounded(sx_bool b) = 0;
			virtual sx_double get_CornerSize() const = 0;
			virtual void set_CornerSize(const sx_double& sz) = 0;
		};

		interface IShapeCalloutArrow : public IClosedShape
		{
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
			virtual sx_double get_BoxLength() const = 0;
			virtual void set_BoxLength(const sx_double& val) = 0;
		};

		interface IShapeCalloutLRArrow : public IClosedShape
		{
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
			virtual sx_double get_BoxLength() const = 0;
			virtual void set_BoxLength(const sx_double& val) = 0;
		};

		interface IShapeCalloutQuadArrow : public IClosedShape
		{
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
			virtual sx_double get_BoxLength() const = 0;
			virtual void set_BoxLength(const sx_double& val) = 0;
		};

		interface IShapeCurvedArrow : public IClosedShape
		{
			virtual sx_bool get_Right() const = 0;
			virtual void set_Right(sx_bool v) = 0;
			virtual sx_double get_HeadWeight() const = 0;
			virtual void set_HeadWeight(const sx_double& weight) = 0;
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
		};

		interface IShapeStripedArrow : public IClosedShape
		{
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
		};

		interface IShapeNotchedArrow : public IClosedShape
		{
			virtual sx_double get_HeadLength() const = 0;
			virtual void set_HeadLength(const sx_double& length) = 0;
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& val) = 0;
		};

		interface IShapePentagonArrow : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IShapeChevronArrow : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IBalloonCallout : public IClosedShape
		{
			virtual sx_bool get_StraightTail() const = 0;
			virtual void set_StraightTail(sx_bool b) = 0;
			virtual sx_double get_TailWeight() const = 0;
			virtual void set_TailWeight(const sx_double& val) = 0;
			virtual sx_point get_Anchor() const = 0;
			virtual void set_Anchor(const sx_point& pt) = 0;
		};

		interface IRectBalloon : public IBalloonCallout
		{
			virtual sx_bool get_Rounded() const = 0;
			virtual void set_Rounded(sx_bool b) = 0;
			virtual sx_double get_CornerSize() const = 0;
			virtual void set_CornerSize(const sx_double& sz) = 0;
		};
		
		interface IBalloonLegend : public IClosedShape
		{
			virtual sx_uint get_TailSegments() const = 0;
			virtual void set_TailSegments(sx_uint val) = 0;
			virtual sx_double get_TailOriginX() const = 0;
			virtual void set_TailOriginX(const sx_double& val) = 0;
			virtual sx_double get_TailOriginY() const = 0;
			virtual void set_TailOriginY(const sx_double& val) = 0;
			virtual sx_point GetSegmentEndPoint(sx_uint id) const = 0;
			virtual void SetSegmentEndPoint(sx_uint id, const sx_point& pt) = 0;

			virtual sx_bool get_Rounded() const = 0;
			virtual void set_Rounded(sx_bool b) = 0;
			virtual sx_double get_CornerSize() const = 0;
			virtual void set_CornerSize(const sx_double& sz) = 0;
		};
		
		interface ICloudBalloon : IClosedShape
		{
			virtual sx_point get_Anchor() const = 0;
			virtual void set_Anchor(const sx_point& pt) = 0;
		};


		interface IFoldedCorner : public IClosedShape
		{
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IWave : public IClosedShape
		{
			virtual sx_uint get_Waves() const = 0;
			virtual void set_Waves(sx_uint val) = 0;
			virtual sx_double get_Amplitude() const = 0;
			virtual void set_Amplitude(const sx_double& val) = 0;
		};

		interface IScroll : public IClosedShape
		{
			virtual sx_bool get_Right() const = 0;
			virtual void set_Right(sx_bool v) = 0;
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
		};

		interface IRibbon : public IClosedShape
		{
			virtual sx_bool get_Upper() const = 0;
			virtual void set_Upper(sx_bool v) = 0;
			virtual sx_bool get_Curved() const = 0;
			virtual void set_Curved(sx_bool v) = 0;
			virtual sx_double get_Skew() const = 0;
			virtual void set_Skew(const sx_double& val) = 0;
			virtual sx_double get_Slope() const = 0;
			virtual void set_Slope(const sx_double& val) = 0;
		};

		/////////////////////////////////////////////////////////////////////
		// text shapes
		/////////////////////////////////////////////////////////////////////
		enum box_side
		{
			box_side_left	= 0x01,
			box_side_top	= 0x02,
			box_side_right	= 0x04,
			box_side_bottom	= 0x08,
			box_side_horz	= 0x03,
			box_side_vert	= 0x0C,
			box_side_all	= 0x0F,
			// table borders only
			box_side_cols	= 0x10,
			box_side_rows	= 0x20,
			box_side_inside	= 0x30,
			box_side_grid	= 0x3F
		};
		
		enum border_style
		{
			border_style_solid = 1,
			border_style_dot,
			border_style_dash,
			border_style_dash_dot,
			border_style_dash_dot_dot
		};

		enum text_align
		{
			text_align_left = 1,
			text_align_center,
			text_align_right,
			text_align_justify
		};

		enum vert_align
		{
			vert_align_top = 1,
			vert_align_middle,
			vert_align_bottom
		};

		enum text_underline
		{
			text_underline_none,
			text_underline_single
		};

		interface IMargins : public IRefObject
		{
			virtual sx_double GetMargin(box_side sd) const = 0;
			virtual void SetMargin(const sx_double& m, sx_flags sd = box_side_all) = 0;
		};
		
		interface ICharStyle : public IRefObject
		{
			virtual ITextStyle* get_Style() const = 0;
			virtual void set_Style(ITextStyle* pStyle) = 0;
			virtual text_underline get_Underline() const = 0;
			virtual void set_Underline(text_underline tu) = 0;
			virtual ICharStyle* Clone() = 0;
		};
		
		interface IBorderLine : public IRefObject
		{
			virtual IPen* get_Pen() const = 0;
			virtual void set_Pen(IPen* pPen) = 0;
			virtual const sx_color* get_Color() const = 0;
			virtual void set_Color(const sx_color& color) = 0;
			virtual border_style get_Style() const = 0;
			virtual void set_Style(border_style st) = 0;
		};

		interface ITextBlock
		{
			// box
			virtual text_align get_TextAlign() const = 0;
			virtual void set_TextAlign(text_align al) = 0;
			virtual vert_align get_VertAlign() const = 0;
			virtual void set_VertAlign(vert_align al) = 0;
			virtual sx_rotation_angle get_TextOrientation() const = 0;
			virtual void set_TextOrientation(sx_rotation_angle angle) = 0;
			virtual sx_bool get_WrapText() const = 0;
			virtual void set_WrapText(sx_bool v) = 0;
			virtual IMargins* GetMargins() const = 0;
			virtual void SetMargins(IMargins* pMg) = 0;
			virtual void SetMargins(const sx_double& l, const sx_double& t, const sx_double& r, const sx_double& b) = 0;

			// text
			virtual sx_str get_Text() const = 0;
			virtual void set_Text(sx_str sText) = 0;
			virtual sx_uint get_TextLength() const = 0;
			virtual ICharStyle* GetCharStyle(sx_uint ind = INVALID_INDEX) const = 0;
			virtual void SetCharStyle(ICharStyle* pStyle, sx_uint start = INVALID_INDEX, sx_uint end = INVALID_INDEX) = 0;
		};

		interface ITableSpan
		{
			virtual sx_uint get_Blocks() const = 0;
			virtual sx_uint GetFirstCol(sx_uint ind) const = 0;
			virtual sx_uint GetFirstRow(sx_uint ind) const = 0;
			virtual sx_uint GetLastCol(sx_uint ind) const = 0;
			virtual sx_uint GetLastRow(sx_uint ind) const = 0;
			virtual void AddBlock(sx_uint fc, sx_uint fr, sx_uint lc, sx_uint lr) = 0;
			virtual void RemoveBlock(sx_uint ind) = 0;
		};

		interface ITableCell
		{
			virtual ITextBlock* get_TextBlock() = 0;
			virtual void SetBackColor(const sx_color& color) = 0;
		};

		interface ITableCol
		{
			virtual sx_double get_Width() const = 0;
			virtual sx_int GetPreferredWidth() = 0;
			virtual void SetPreferredWidth(sx_int w) = 0;

			virtual sx_uint get_Cells() const = 0;
			virtual ITableCell* GetCell(sx_uint ind) = 0;

			virtual IBorderLine* GetLeftBorder() const = 0;
			virtual void SetLeftBorder(IBorderLine* pBrd) = 0;
			virtual void SetCellsMargins(IMargins* pMg) = 0;
			virtual void SetCellsBackColor(const sx_color& color) = 0;
			virtual void SetCellsCharStyle(ICharStyle* pStyle) = 0;
		};

		interface ITableRow
		{
			virtual sx_double get_Height() const = 0;
			virtual sx_int GetPreferredHeight() = 0;
			virtual void SetPreferredHeight(sx_int h) = 0;

			virtual sx_uint get_Cells() const = 0;
			virtual ITableCell* GetCell(sx_uint ind) = 0;

			virtual IBorderLine* GetTopBorder() const = 0;
			virtual void SetTopBorder(IBorderLine* pBrd) = 0;
			virtual void SetCellsMargins(IMargins* pMg) = 0;
			virtual void SetCellsBackColor(const sx_color& color) = 0;
			virtual void SetCellsCharStyle(ICharStyle* pStyle) = 0;
		};
		
		interface IShapeTable: public IShapeObject
		{
			virtual sx_uint get_Columns() const = 0;
			virtual sx_uint get_Rows() const = 0;
			virtual ITableSpan* get_SpanBlocks() const = 0;
			virtual void SetGrid(sx_uint cols, sx_uint rows) = 0;
			virtual ITableCell* GetCell(sx_uint col, sx_uint row) = 0;

			virtual ITableCol* GetColumn(sx_uint ind) = 0;
			virtual ITableCol* InsertColumn(sx_uint before, sx_int w = 0, ITableCol* pStyle = nullptr) = 0;
			virtual void RemoveColumn(sx_uint ind) = 0;

			virtual ITableRow* GetRow(sx_uint ind) = 0;
			virtual ITableRow* InsertRow(sx_uint before, sx_int h = 0, ITableRow* pStyle = nullptr) = 0;
			virtual void RemoveRow(sx_uint ind) = 0;

			virtual IBorderLine* CreateBorderLine(IPen* pPen, const sx_color& color) = 0;
			virtual IBorderLine* GetBorder(box_side sd) const = 0;
			virtual void SetBorder(IBorderLine* pBrd, box_side sd = box_side_all) = 0;

			virtual void SetCellsMargins(IMargins* pMg) = 0;
			virtual void SetCellsBackColor(const sx_color& color) = 0;
			virtual void SetCellsCharStyle(ICharStyle* pStyle) = 0;
		};
	} // namespace Content
} // namespace SX