// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once

#ifndef interface
#define interface struct
#endif
#ifdef SX_UNIX_DLL
#define nullptr 0
#define __int64 long long
#endif


namespace SX
{
	//////////////////////////////////////////////////////////////////////////
	// basic types
	typedef bool				sx_bool;
	typedef unsigned char		sx_byte;
	typedef int					sx_int;
	typedef unsigned int		sx_uint;
	typedef	unsigned short		sx_short;
	typedef unsigned __int64	sx_qword;
	typedef sx_uint				sx_flags;
	typedef float				sx_float;
	typedef double				sx_double;
	typedef const wchar_t*		sx_str;

	//////////////////////////////////////////////////////////////////////////
	// Const
	const sx_uint INVALID_INDEX = 0xFFFFFFFF; 

	//////////////////////////////////////////////////////////////////////////
	// Enums
	enum sx_rotation_angle 
	{
		sx_rotation_angle_0 =   0,
		sx_rotation_angle_90 =  90,
		sx_rotation_angle_180 = 180,
		sx_rotation_angle_270 = 270
	};

#pragma pack(push,1)
	//////////////////////////////////////////////////////////////////////////
	// Structures
	struct sx_time 
	{
		sx_short wYear;
		sx_short wMonth;
		sx_short wDayOfWeek;
		sx_short wDay;
		sx_short wHour;
		sx_short wMinute;
		sx_short wSecond;
		sx_short wMilliseconds;
	};

	struct sx_point 
	{
		sx_double x;
		sx_double y;
	};

	struct sx_size 
	{
		sx_double width;
		sx_double height;
	};

	struct sx_rect 
	{
		sx_double x;
		sx_double y;
		sx_double width;
		sx_double height;
	};

	struct sx_marg 
	{
		sx_double left;
		sx_double top;
		sx_double right;
		sx_double bottom;
	};

	struct sx_quad
	{
		union
		{
			sx_double c[8];
			sx_point  p[4];
		};
	};

	struct sx_matrix 
	{
		sx_double m11;
		sx_double m12;
		sx_double m21;
		sx_double m22;
		sx_double dx;
		sx_double dy;
	};

	struct sx_3d_matrix
	{
		sx_double m11;
		sx_double m12;
		sx_double m13;
		sx_double m21;
		sx_double m22;
		sx_double m23;
		sx_double m31;
		sx_double m32;
		sx_double m33;
		sx_double dx;
		sx_double dy;
		sx_double dz;
	};

	// color formats
	typedef const sx_byte sx_color_t;
	sx_color_t
		sx_color_none	= 0x00,
		sx_color_ti_g	= 0x01,
		sx_color_ti_rgb	= 0x02,
		sx_color_ti_rgba= 0x03,
		sx_color_ti_cmyk= 0x04,
		sx_color_tf_g	= 0x21,
		sx_color_tf_rgb	= 0x22,
		sx_color_tf_rgba= 0x23,
		sx_color_tf_cmyk= 0x24;

	struct sx_color
	{
	protected:
		sx_byte t;
		sx_color(sx_color_t ct) : t(ct) {}
	public:
		sx_color() : t(sx_color_none) {}
		sx_color_t type() const { return t; }
	};

	struct sx_color_i_g : public sx_color
	{
		sx_byte g;
		sx_color_i_g() : sx_color(sx_color_ti_g), g(0) {}
		sx_color_i_g(sx_byte gc) : sx_color(sx_color_ti_g), g(gc) {}
	};

	struct sx_color_i_rgb : public sx_color
	{
		sx_byte r;
		sx_byte g;
		sx_byte b;
		sx_color_i_rgb() : sx_color(sx_color_ti_rgb), r(0), g(0), b(0) {}
		sx_color_i_rgb(sx_byte rc, sx_byte gc, sx_byte bc) : sx_color(sx_color_ti_rgb), r(rc), g(gc), b(bc) {}
	};

	struct sx_color_i_rgba : public sx_color
	{
		sx_byte r;
		sx_byte g;
		sx_byte b;
		sx_byte a;
		sx_color_i_rgba() : sx_color(sx_color_ti_rgba), r(0), g(0), b(0), a(255) {}
		sx_color_i_rgba(sx_byte rc, sx_byte gc, sx_byte bc, sx_byte ac) : sx_color(sx_color_ti_rgba), r(rc), g(gc), b(bc), a(ac) {}
	};

	struct sx_color_i_cmyk : public sx_color
	{
		sx_byte c;
		sx_byte m;
		sx_byte y;
		sx_byte k;
		sx_color_i_cmyk() : sx_color(sx_color_ti_cmyk), c(255), m(255), y(255), k(255) {}
		sx_color_i_cmyk(sx_byte cc, sx_byte mc, sx_byte yc, sx_byte kc) : sx_color(sx_color_ti_cmyk), c(cc), m(mc), y(yc), k(kc) {}
	};

	struct sx_color_f_g : public sx_color
	{
		sx_float g;
		sx_color_f_g() : sx_color(sx_color_tf_g), g(0.) {}
		sx_color_f_g(sx_float gc) : sx_color(sx_color_tf_g), g(gc) {}
	};

	struct sx_color_f_rgb : public sx_color
	{
		sx_float r;
		sx_float g;
		sx_float b;
		sx_color_f_rgb():sx_color(sx_color_tf_rgb), r(0.), g(0.), b(0.) {}
		sx_color_f_rgb(sx_float rc, sx_float gc, sx_float bc):sx_color(sx_color_tf_rgb), r(rc), g(gc), b(bc) {}
	};

	struct sx_color_f_rgba : public sx_color
	{
		sx_float r;
		sx_float g;
		sx_float b;
		sx_float a;
		sx_color_f_rgba() : sx_color(sx_color_tf_rgba), r(0.), g(0.), b(0.), a(1.) {}
		sx_color_f_rgba(sx_float rc, sx_float gc, sx_float bc, sx_float ac) : sx_color(sx_color_tf_rgba), r(rc), g(gc), b(bc), a(ac) {}
	};

	struct sx_color_f_cmyk : public sx_color
	{
		sx_float c;
		sx_float m;
		sx_float y;
		sx_float k;
		sx_color_f_cmyk() : sx_color(sx_color_tf_cmyk), c(1.), m(1.), y(1.), k(1.) {}
		sx_color_f_cmyk(sx_float cc, sx_float mc, sx_float yc, sx_float kc) : sx_color(sx_color_tf_cmyk), c(cc), m(mc), y(yc), k(kc) {}
	};
#pragma pack(pop)
};
