// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************

#pragma once
#include "types.h"
#include "graphics.h"
#include "PDF\events.h"

namespace SX
{
	interface IBaseDocument; interface ILinkItem; interface IDocEvent;
	namespace PDF { interface IDocument; interface IAnnot; interface IInk; interface IPopup; interface IFreeText; interface IFormField; }
	namespace XPS { interface ILink; }
	namespace Media { interface IPlayer; }
	namespace XHTML { interface IRichText; interface ITextBox; }
	namespace Content { interface IRichContent; interface IObjectTrunk; }
	namespace Viewer
	{
		interface IViewer;
		interface IViewerCanvas;
		interface IViewerConfig;
		interface IViewerConfigPDF;
		interface IViewerConfigXPS;
		interface IViewerEvents;
		interface IViewerEvent;
		interface IToolState;
		interface IDynNotes;
		interface IDynNote;
		interface IUndoStack;
		interface IZoomScale;

		const object_type obj_type_viewable_document = 0x27;

		//////////////////////////////////////////////////////////////////////////
		interface IViewableDocument : public IRefObject
		{
			virtual sx_uint GetUPI() const = 0;
			virtual sx_uint GetPageCount() const = 0;
			virtual void GetPageSize(sx_uint uIndex, sx_size& size) const = 0; // in doc units, usually in points
			virtual sx_bool DrawPage(sx_uint uIndex, Graphics::IDrawingContext* ctx, sx_matrix *pTransform = nullptr, sx_flags flags = 0) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerSnapshot : public IRefObject 
		{
			virtual IViewableDocument* get_Document() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum tool_mode
		{
			tool_mode_none = 0,
			tool_mode_custom,
			tool_mode_tracker,
			tool_mode_cropper,
			tool_mode_rect_mark,
			tool_mode_select_text,
			tool_mode_snapshot,
			tool_mode_lens,
			tool_mode_text_box,
			tool_mode_stylus,
			tool_mode_measure,
			tool_mode_page_content,
			tool_mode_annot = 64,
			tool_mode_free_text,
			tool_mode_ink,
			tool_mode_eraser,
			tool_mode_shape
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewer : public IRefObject
		{
			//////////////////////////////////////////////////////////////////////////
			// viewable document
			virtual IViewableDocument* get_Document() = 0;
			virtual void set_Document(IViewableDocument* pDoc) = 0;
			virtual void Navigate(IViewableDocument* pDoc, sx_uint page, sx_bool bTakeSnaphot = false) = 0;
			virtual void Navigate(IViewerSnapshot* pSnapshot) = 0;
			virtual IViewerSnapshot* TakeSnapshot(sx_flags reserved = 0) = 0;

			//////////////////////////////////////////////////////////////////////////
			// viewport
			virtual sx_uint get_Width() const = 0; // in pixels
			virtual sx_uint get_Height() const = 0; // in pixels
			virtual void Resize(sx_uint w, sx_uint h) = 0;
			virtual sx_bool Redraw(Graphics::IDrawingContext* ctx, sx_bool bFirstBlock, sx_rect *pDrawRect = nullptr) = 0;
			virtual void Relayout( sx_uint uNewActPage, sx_bool bAdoptPageOffset = false ) = 0;
			virtual void Invalidate( sx_bool bPages = false, sx_bool bPDFAnnots = false ) = 0;

			//////////////////////////////////////////////////////////////////////////
			// canvas and config
			virtual IViewerCanvas* get_Canvas() = 0;
			virtual IViewerConfig* get_Config() = 0;

			//////////////////////////////////////////////////////////////////////////
			// interactivity
			virtual tool_mode get_ToolMode() const = 0;
			virtual void set_ToolMode(tool_mode mode) = 0;
			virtual void Focus(sx_bool bSet) = 0;
			virtual sx_bool MouseActivity(sx_uint x, sx_uint y, sx_flags flags) = 0;
			virtual sx_bool KeyboardActivity(sx_byte k, sx_flags flags) = 0;
			virtual sx_bool ShowPopup(sx_uint page, PDF::IPopup* pPopup, sx_bool bShow = true) = 0;
			virtual sx_bool ShowMediaPlayer(Media::IPlayer* pPlayer, sx_uint page = INVALID_INDEX, sx_rect* pRect = nullptr) = 0;

			//////////////////////////////////////////////////////////////////////////
			// misc
			virtual Graphics::IBitmapData* GetPageThumb(sx_uint uPage, sx_uint maxw, sx_uint maxh) = 0;
			virtual void SetEventHandler(IViewerEvents* pEventHandler) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum viewer_fit
		{
			viewer_fit_none,
			viewer_fit_page,
			viewer_fit_page_width,
			viewer_fit_page_height
		};

		//////////////////////////////////////////////////////////////////////////
		enum marking_type 
		{
			marking_type_none,
			marking_type_custom, 
			marking_type_text,
			marking_type_bitmap,
			marking_type_rectangle,
			marking_type_line,
			marking_type_pdf_field,
			marking_type_pdf_annot,
			marking_type_dyn_note
		};

		enum label_place
		{
			label_place_none,		// no label displaed
			label_place_under_page,	// under page in page spacing, hor. centered
			label_place_rb_corner	// at the page, right-bottom corner
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerThumbs
		{
			// additional settings
			virtual sx_bool get_MultiSelect() const = 0;
			virtual void set_MultiSelect(sx_bool bMS) = 0;
			virtual const sx_color_i_rgb& get_SelBorderColor() const = 0;
			virtual void set_SelBorderColor(const sx_color_i_rgb& c) = 0;
			virtual sx_uint get_SelBorderThickness() const = 0;
			virtual void set_SelBorderThickness(sx_uint uThickness) = 0;
			virtual sx_bool get_InteractiveSelect() const = 0;
			virtual void set_InteractiveSelect(sx_bool bInteractiveSelect) = 0;
			// manage 
			virtual sx_uint get_SelectedThumbs() const = 0;
			virtual sx_uint GetSelectedThumb(sx_uint uIndex) const = 0;
			virtual void SetThumbState(sx_uint uPageIndex, sx_bool bSt) = 0;
			// page label
			virtual sx_uint get_PageLabelSize() const = 0;
			virtual void set_PageLabelSize(sx_uint uSize) = 0;
			virtual label_place get_PageLabelPlace() const = 0;
			virtual void set_PageLabelPlace(label_place pl) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerCanvas
		{
			// size and offset in vp
			virtual sx_uint get_Width() const = 0;
			virtual sx_uint get_Height() const = 0;
			virtual sx_uint get_VerticalOffset() const = 0;
			virtual void set_VerticalOffset(sx_uint offset) = 0;
			virtual sx_uint get_HorizontalOffset() const = 0;
			virtual void set_HorizontalOffset(sx_uint offset) = 0;
			// layout
			virtual sx_bool get_SeparateCover() const = 0;
			virtual void set_SeparateCover(sx_bool bSep) = 0;
			virtual sx_rotation_angle get_Rotation() const = 0;
			virtual void set_Rotation(sx_rotation_angle angle)= 0;
			virtual sx_double get_Zoom() const = 0;
			virtual void set_Zoom(sx_double Zoom) = 0;
			virtual sx_bool SnapZoom(sx_bool bDecrease) = 0;
			virtual viewer_fit get_Fit() const = 0;
			virtual void set_Fit(viewer_fit Fit) = 0;
			virtual sx_bool get_VerticalLayout() const = 0;
			virtual void set_VerticalLayout(sx_bool bVert) = 0;
			virtual sx_uint get_LayoutMultiplicity() const = 0;
			virtual void set_LayoutMultiplicity(sx_uint Mult) = 0;
			virtual sx_uint get_Columns() const = 0;
			virtual sx_uint get_Rows() const = 0;
			// layouted pages
			virtual sx_uint get_ActivePage() const = 0;
			virtual sx_uint get_VisiblePages() const = 0;
			virtual sx_uint GetVisiblePage(sx_uint uIndex) const = 0;
			virtual sx_bool GetPageRect(sx_uint uIndex, sx_rect& rect) const = 0;
			virtual sx_bool GetPageMatrix(sx_uint uPageIndex, sx_matrix &m) const = 0;
			virtual sx_uint GetPageColumn(sx_uint uIndex) const = 0;
			virtual sx_uint GetPageRow(sx_uint uIndex) const = 0;
			virtual sx_uint GetPage(sx_uint uRow, sx_uint uCol) const = 0;
			virtual sx_uint GetPageIndex(sx_uint uCanvasX, sx_uint uCanvasY) const = 0;
			virtual void CanvasToPage(sx_uint uIndex, const sx_point& ptCanvas, sx_point& ptPage) const = 0;
			virtual void PageToCanvas(sx_uint uIndex, const sx_point& ptPage, sx_point& ptCanvas) const = 0;
			// navigation
			virtual void Navigate(sx_uint uToPage, const sx_double& dX = 0, const sx_double& dY = 0) = 0;
			virtual void Navigate(sx_uint uToPage, const sx_rect &rect, sx_bool bEnsureVisible = true, sx_bool bZoomByRect = false) = 0;
			virtual void Navigate(IRefObject *pObject) = 0; //SX::Viewer::ISearchProvider, SX::PDF::IPage, SX::PDF::IOutlineItem, SX::PDF::ILink, SX::PDF::IDest, SX::IStr (destination name)
			// history
			virtual sx_bool History(sx_bool back, sx_bool bGo = true) = 0;
			virtual void ClearHistory() = 0;
			// marking
			virtual marking_type get_MarkingType() const = 0;
			virtual Graphics::IRegion* get_MarkingRegion() const = 0;
			virtual IStr* get_MarkedText() const = 0;
			virtual sx_uint get_MarkedPage() const = 0;
			virtual void ResetMarking(IRefObject *pMarking = nullptr, sx_uint uPage = INVALID_INDEX) = 0; //  SX::Viewer::ISearchProvider, SX::Graphics::IRegion			
			virtual PDF::IAnnot* get_MarkedAnnot() const = 0;
			// misc
			virtual IZoomScale* get_ZoomScale() = 0;
			virtual IUndoStack* get_UndoStack() = 0;
			virtual IViewerThumbs* get_Thumbs() = 0;
			virtual IDynNotes* get_Notes() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum viewer_mode
		{
			viewer_mode_normal = 1,
			viewer_mode_thumbs,
			viewer_mode_flip,
			viewer_mode_smart_flip
		};

		//////////////////////////////////////////////////////////////////////////
		enum vk_layout
		{
			vk_layout_windesktop,
			vk_layout_winrt,
			vk_layout_winforms,
			vk_layout_wininput,
			vk_layout_winphone8
		};

		//////////////////////////////////////////////////////////////////////////
		enum viewer_popups
		{
			viewer_popups_disable,
			viewer_popups_single,
			viewer_popups_multi,
			viewer_popups_conform
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerConfig
		{
			// viewer
			virtual viewer_mode get_Mode() const = 0;
			virtual void set_Mode(viewer_mode mode) = 0;
			virtual sx_bool get_TouchDevice() const = 0;
			virtual void set_TouchDevice(sx_bool bTouch) = 0;
			virtual sx_bool get_MarkInputSource() const = 0;
			virtual void set_MarkInputSource(sx_bool bTouch) = 0;
			virtual vk_layout get_KeyboardLayout() const = 0;
			virtual void set_KeyboardLayout(vk_layout layout) = 0;
			virtual sx_uint get_DPI() = 0;
			virtual void set_DPI(sx_uint uDPI) = 0;
			virtual Graphics::Blockiness get_Blockiness() const = 0;
			virtual void set_Blockiness(Graphics::Blockiness bl) = 0;
			virtual Graphics::TextRender get_TextRender() const = 0;
			virtual void set_TextRender(Graphics::TextRender textRender) = 0;
			virtual Graphics::TextAntialias get_TextAntialias () const = 0;
			virtual void set_TextAntialias(Graphics::TextAntialias mode) = 0;
			virtual Graphics::ImageInterpolation get_ImageInterpolation () const = 0;
			virtual void set_ImageInterpolation(Graphics::ImageInterpolation mode) = 0; 
			virtual sx_flags get_DrawOptions() const = 0;
			virtual void set_DrawOptions(sx_flags opt) = 0; 
			virtual sx_bool get_UseCache() const = 0;
			virtual void set_UseCache(sx_bool bUseCache) = 0;
			virtual sx_uint get_PageCacheSize() const = 0;
			virtual void set_PageCacheSize(sx_uint uSize) = 0;
			virtual sx_uint get_PageCacheInvalidation() const = 0;
			virtual void set_PageCacheInvalidation(sx_uint uDelay) = 0;
			virtual sx_uint get_HistorySize() const = 0;
			virtual void set_HistorySize(sx_uint uSize) = 0;
			// canvas
			virtual sx_uint get_VerticalCanvasPadding() const = 0;
			virtual void set_VerticalCanvasPadding(sx_uint uPadding) = 0;
			virtual sx_uint get_HorizontalCanvasPadding() const = 0;
			virtual void set_HorizontalCanvasPadding(sx_uint uPadding) = 0;
			virtual const sx_color_i_rgb & get_CanvasColor() const = 0;
			virtual void set_CanvasColor(const sx_color_i_rgb & c) = 0;
			virtual const sx_color_i_rgb & get_MarkingColor() const = 0;
			virtual void set_MarkingColor(const sx_color_i_rgb & c) = 0;
			virtual sx_uint get_PageSpacing() const = 0;
			virtual void set_PageSpacing(sx_uint uSpacing) = 0;
			virtual const sx_color_i_rgb &get_PageColor() const = 0;
			virtual void set_PageColor(const sx_color_i_rgb& c) = 0;
			virtual const sx_color_i_rgb &get_PageBorderColor() const = 0;
			virtual void set_PageBorderColor(const sx_color_i_rgb& c) = 0;
			virtual const sx_color_i_rgb &get_UniformTextColor() const = 0;
			virtual void set_UniformTextColor(const sx_color_i_rgb& c) = 0;
			
			virtual sx_uint get_PageBorderThickness() const = 0;
			virtual void set_PageBorderThickness(sx_uint uThickness) = 0;
			virtual sx_uint get_LineScrollHeight() const = 0;
			virtual void set_LineScrollHeight(sx_uint uLineHeight) = 0;
			virtual sx_bool get_AllowLocalLinks() const = 0;
			virtual void set_AllowLocalLinks(sx_bool bAllow) = 0;
			virtual sx_bool get_RelayDocEvents() const = 0;
			virtual void set_RelayDocEvents(sx_bool bRelay) = 0;
			// PDF specific
			virtual IViewerConfigPDF *get_PDF() = 0;
			// XPS specific
			virtual IViewerConfigXPS *get_XPS() = 0;
			// notify viewer
			virtual void Update() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerConfigPDF
		{
			// common
			virtual sx_flags get_AnnotsVisible() const = 0;
			virtual void set_AnnotsVisible(sx_flags mask) = 0;
			virtual sx_flags get_AnnotsSensitive() const = 0;
			virtual void set_AnnotsSensitive(sx_flags mask) = 0;
			virtual sx_flags get_AnnotsFocusable() const = 0;
			virtual void set_AnnotsFocusable(sx_flags mask) = 0;
			// popups
			virtual viewer_popups get_Popups() const = 0;
			virtual void set_Popups(viewer_popups popups) = 0;
			virtual sx_double get_PopupOpacity() const = 0;
			virtual void set_PopupOpacity(const sx_double &dOpacity) = 0;
			virtual sx_str get_PopupFontFace() const = 0;
			virtual void set_PopupFontFace(sx_str szFontFace) = 0;
			virtual sx_double get_PopupFontSize() const = 0;
			virtual void set_PopupFontSize(const sx_double &dFontSize) = 0;
			virtual const sx_color_i_rgb &get_PopupTextColor() const = 0;
			virtual void set_PopupTextColor(const sx_color_i_rgb &color) = 0;
			// form fields
			virtual sx_bool get_EnableFormFilling() const = 0;
			virtual void set_EnableFormFilling(sx_bool bEnable) = 0 ;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerConfigXPS
		{
			// common
			virtual sx_flags get_AnnotsVisible() const = 0;
			virtual void set_AnnotsVisible(sx_flags mask) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IZoomScale
		{
			virtual sx_double get_Lowest() const = 0;
			virtual sx_double get_Highest() const = 0;
			virtual sx_uint get_Levels() const = 0;
			virtual sx_double GetLevel(sx_uint ind) const = 0;
			virtual void AddLevel(sx_double level) = 0;
			virtual void Reset(sx_double lowest, sx_double highest, sx_double first_level) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IRichTextEditor
		{
			virtual IStr* get_Chars() const = 0;
			virtual sx_uint get_NumberChars() const = 0;
			virtual XHTML::IRichText* get_RichText() const = 0;

			virtual sx_bool get_Moveable() const = 0;
			virtual void set_Moveable(sx_bool bMoveable) = 0;
			virtual sx_bool get_ReadOnly() const = 0;
			virtual sx_uint get_Caret() = 0;
			virtual void set_Caret(sx_uint pos) = 0;
			virtual sx_uint get_SelectionStart() = 0;
			virtual sx_uint get_SelectionEnd() = 0;

			virtual PDF::IAnnot* get_ContainerAnnot() const = 0;
			virtual IDynNote* get_ContainerNote() const = 0;


			virtual void SetSelection(sx_uint start = 0, sx_uint end = 0) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// Viewer events
		//////////////////////////////////////////////////////////////////////////
		enum viewer_event
		{
			// info events, can't be canceled
			viewer_event_attach_doc = 1,
			viewer_event_detach_doc,
			viewer_event_doc_activity,
			viewer_event_invalidate,
			viewer_event_change_act_page,
			viewer_event_change_page_visibility,
			viewer_event_change_layout,
			viewer_event_change_scroll,
			viewer_event_change_config,
			viewer_event_change_marking,
			viewer_event_change_thumb_marking,
			viewer_event_pdf,
			viewer_event_xps_link,
			viewer_event_popup,
			viewer_event_link,
			viewer_event_undo,
			// cancelable events
			viewer_event_cancelable = 0x1001,
			viewer_event_set_page,
			viewer_event_scroll_canvas,
			viewer_event_mouse_activity,
			viewer_event_keyboard_activity,
			viewer_event_tool_activity,
			viewer_event_text_editor,
			viewer_event_media_player
		};

		//////////////////////////////////////////////////////////////////////////
		enum mouse_flags : sx_uint
		{
			mouse_flags_delta		= 0x0000FFFF,
			mouse_flags_lbdown		= 0x00010000,
			mouse_flags_lbup		= 0x00020000,
			mouse_flags_lbdbl		= 0x00030000,
			mouse_flags_rbdown		= 0x00040000,
			mouse_flags_rbup		= 0x00050000, 
			mouse_flags_rbdbl		= 0x00060000,
			mouse_flags_vert_wheel	= 0x00070000,
			mouse_flags_horz_wheel	= 0x00080000,
			mouse_flags_move		= 0x00090000,
			mouse_flags_cursor		= 0x000A0000,
			mouse_flags_enter		= 0x000D0000,
			mouse_flags_exit		= 0x000E0000,
			mouse_flags_event		= 0x000F0000,
			mouse_flags_control		= 0x00100000,
			mouse_flags_shift		= 0x00200000, 
			mouse_flags_barrel		= 0x00400000, 
			mouse_flags_eraser		= 0x00800000, 
			mouse_flags_invert		= 0x01000000,
			mouse_flags_keys		= 0x01F00000,
			mouse_flags_pen			= 0x10000000, 
			mouse_flags_touch		= 0x20000000, 
			mouse_flags_input		= 0xF0000000 
		};

		//////////////////////////////////////////////////////////////////////////
		enum key_flags
		{
			key_flags_repeat_count = 0xFFFF,
			key_flags_pressed = 0x10000,
			key_flags_shift = 0x20000,
			key_flags_lcontrol = 0x40000,
			key_flags_lalt = 0x80000,
			key_flags_rcontrol = 0x100000,
			key_flags_ralt = 0x200000,
			key_flags_control = key_flags_lcontrol | key_flags_rcontrol,
			key_flags_alt = key_flags_lalt | key_flags_ralt 
		};

		//////////////////////////////////////////////////////////////////////////
		enum hotspot_flags
		{
			hotspot_flags_up	= 0x1,
			hotspot_flags_down	= 0x2,
			hotspot_flags_enter	= 0x4,
			hotspot_flags_exit	= 0x8
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerEvents
		{
			virtual sx_bool OnEvent(IViewerEvent* pEvent) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerEvent
		{
			virtual viewer_event get_Type() const = 0;
			virtual sx_bool get_Handled() const = 0;
			virtual void set_Handled(sx_bool bHandled = true) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerEvent_Doc : public IViewerEvent
		{
			virtual IBaseDocument* get_Doc() const = 0;
			virtual IViewableDocument* get_VDoc() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerEvent_DocActivity : public IViewerEvent_Doc
		{
			virtual IDocEvent* get_DocEvent() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerEvent_Page : public IViewerEvent
		{
			virtual sx_uint get_Lose() const = 0;
			virtual sx_uint get_Gain() const = 0;
			virtual sx_bool get_Terminal() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IViewerEvent_PDF : public IViewerEvent
		{
			virtual SX::PDF::IDocument* get_Doc() const = 0;
			virtual SX::PDF::pdf_event  get_PDFType() const = 0;
			virtual SX::PDF::IPDFEvent* get_PDFEvent() const = 0;
		};

		interface IViewerEvent_PDF_Page : public IViewerEvent_PDF
		{
			virtual sx_uint get_Page() const = 0;			
		};

		interface IViewerEvent_PDF_Annot : public IViewerEvent_PDF
		{
			virtual PDF::IAnnot* get_Annot() const = 0;
		};

		interface IViewerEvent_XPS_Link : public IViewerEvent
		{
			virtual hotspot_flags get_Activity() const = 0;
			virtual SX::XPS::ILink* get_Link() const = 0;
		};

		interface IViewerEvent_Link : public IViewerEvent
		{
			virtual hotspot_flags get_Activity() const = 0;
			virtual ILinkItem* get_Link() const = 0;
		};

		enum popup_activity
		{
			popup_activity_visibility,
			popup_activity_activate,
			popup_activity_rect,
			popup_activity_text
		};
		interface IDynPopup;

		interface IViewerEvent_PopupActivity : public IViewerEvent
		{
			virtual popup_activity get_Activity() = 0;
			virtual sx_bool get_Visible() const = 0;
			virtual sx_bool get_Active() const = 0;
			virtual PDF::IPopup* get_Popup() const = 0;
			virtual IDynPopup* get_DynPopup() const = 0;
			virtual IRichTextEditor* get_TextEditor() const = 0;

			virtual void GetViewportRect(sx_rect &rect) = 0;
		};

		enum undo_activity
		{
			undo_activity_stack_state,
			undo_activity_undo,
			undo_activity_redo
		};

		interface IViewerEvent_Undo : public IViewerEvent
		{
			virtual undo_activity get_Activity() = 0;
		};
		
		interface IViewerCancelableEvent : public IViewerEvent
		{
			virtual sx_bool get_Canceled() const = 0;
			virtual void Cancel() = 0;
		};

		interface IViewerEvent_SetActivePage : public IViewerCancelableEvent 
		{
			virtual sx_uint get_Page() const = 0;
		};

		interface IViewerEvent_ScrollCanvas : public IViewerCancelableEvent 
		{
			virtual sx_uint get_VerticalOffset() const = 0;
			virtual sx_uint get_HorizontalOffset() const = 0;
		};

		interface IViewerEvent_MouseActivity : public IViewerCancelableEvent
		{
			virtual mouse_flags get_Activity() const = 0;
			virtual sx_uint get_X() const = 0;
			virtual sx_uint get_Y() const = 0;
			virtual sx_int get_Delta() const = 0;
			virtual sx_bool get_Ctrl() const = 0;
			virtual sx_bool get_Shft() const = 0;
			virtual sx_flags get_Keys() const = 0;
			virtual sx_flags get_Input() const = 0;
		};
		
		interface IViewerEvent_KeyboardActivity : public IViewerCancelableEvent
		{
			virtual sx_byte get_Key() const = 0;
			virtual sx_bool get_Pressed() const = 0;
			virtual sx_uint get_RepeatCount() const = 0;
			virtual sx_bool get_Shft() const = 0;
			virtual sx_bool get_Ctrl() const = 0;
			virtual sx_bool get_Alt() const = 0;
			virtual sx_bool get_RightCtrl() const = 0;
			virtual sx_bool get_RightAlt() const = 0;
		};

			
		interface IViewerEvent_ToolActivity : public IViewerCancelableEvent
		{
			virtual IToolState* get_State() const = 0;
		};

		enum viewer_text_editor
		{
			viewer_text_editor_get_focus = 1,
			viewer_text_editor_lost_focus,
			viewer_text_editor_insert,
			viewer_text_editor_remove,
			viewer_text_editor_remove_back,
			viewer_text_editor_caret
		};

		interface IViewerEvent_TextEditor: public IViewerCancelableEvent
		{
			virtual viewer_text_editor get_Activity() = 0;
			virtual IRichTextEditor* get_TextEditor() = 0;
			virtual sx_str get_NewText() const = 0;
		};
		
		enum player_activity
		{
			player_activity_hide,
			player_activity_show			
		};


		interface IMediaPlayer
		{
			virtual Media::IPlayer* get_BasePlayer() const = 0;
			// to be
		};

		interface IViewerEvent_MediaPlayer : public IViewerCancelableEvent 
		{
			virtual player_activity get_Activity() const = 0; // show / hide
			virtual IMediaPlayer* get_Player() const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// Viewer tools
		//////////////////////////////////////////////////////////////////////////
		typedef const sx_byte sx_tool_act;
		sx_tool_act
			sx_tool_change		= 1,
			sx_tool_start		= 2,
			sx_tool_apply		= 3,
			sx_tool_cancel		= 4,
			sx_tool_change_page = 5;

		enum tracker_zone
		{
			tracker_zone_none,			
			tracker_zone_body,	
			tracker_zone_text,
			tracker_zone_lt,
			tracker_zone_rt,
			tracker_zone_lb,
			tracker_zone_rb,
			tracker_zone_l,
			tracker_zone_t,
			tracker_zone_r,
			tracker_zone_b,
			tracker_zone_move,
			tracker_zone_rotate,
			tracker_zone_control,
			tracker_zone_node
		};

		//////////////////////////////////////////////////////////////////////////
		interface IToolState
		{
			virtual tool_mode get_ToolMode() const = 0;
			virtual sx_tool_act get_Activity() const = 0;
			virtual sx_uint get_Page() const = 0;
			virtual void set_Page(sx_uint uPage) = 0;
			virtual sx_bool get_CtrlKey() const = 0;
			virtual void set_CtrlKey(sx_bool bPressed) = 0;
			virtual sx_bool get_ShftKey() const = 0;
			virtual void set_ShftKey(sx_bool bPressed) = 0;
			virtual sx_bool get_AutoScroll() const = 0;
			virtual void set_AutoScroll(sx_bool bAutoScroll) = 0;

			virtual void ResetTool() = 0;
			virtual tracker_zone GetTrackerZone(sx_uint x, sx_uint y) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum line_style
		{
			line_style_solid = 1,
			line_style_dot,
			line_style_dash,
			line_style_dash_dot,
			line_style_dash_dot_dot
		};

		//////////////////////////////////////////////////////////////////////////
		enum tracker_style
		{
			tracker_style_frame			= 0,
			tracker_style_moveable		= 0x1,
			tracker_style_resizeable	= 0x2,
			tracker_style_transformable	= 0x3,
			tracker_style_rotateable	= 0x7,
			tracker_style_crop_left		= 0x1000,
			tracker_style_crop_up		= 0x2000,
			tracker_style_crop_right	= 0x4000,
			tracker_style_crop_down		= 0x8000,
			tracker_style_crop_locked	= 0x10000
		};

		//////////////////////////////////////////////////////////////////////////
		enum tracker_handle_style
		{
			tracker_handle_style_rect = 1,
			tracker_handle_style_circle,
			tracker_handle_style_line,
			tracker_handle_style_srect,
			tracker_handle_style_scircle,
			tracker_handle_style_rrect,
			tracker_handle_style_diamond,
			tracker_handle_style_sdiamond
		};

		//////////////////////////////////////////////////////////////////////////
		interface IEditorTrackerStyle
		{
			virtual void SetFrameWidth(const sx_double &width) = 0;
			virtual void SetFrameColor(const sx_color_i_rgb& color) = 0;
			virtual void SetFrameStyle(line_style style) = 0;
			virtual void SetFrameOpacity(sx_byte op) = 0;

			virtual void SetResizeHandleSize(const sx_double &size) = 0;
			virtual void SetResizeHandleStyle(tracker_handle_style style) = 0;
			virtual void SetResizeHandleColor(const sx_color_i_rgb& color) = 0;

			virtual void SetRotateStalkSize(const sx_double &size) = 0;
			virtual void SetRotateHandleSize(const sx_double &size) = 0;
			virtual void SetRotateHandleStyle(tracker_handle_style style) = 0;
			virtual void SetRotateHandleColor(const sx_color_i_rgb& color) = 0;

			virtual void SetControlHandleSize(const sx_double &size) = 0;
			virtual void SetControlHandleStyle(tracker_handle_style style) = 0;
			virtual void SetControlHandleColor(const sx_color_i_rgb& color) = 0;

			virtual void SetOutlineWidth(const sx_double &size) = 0;
			virtual void SetOutlineColor(const sx_color_i_rgb& color) = 0;
			virtual void SetOutlineStyle(line_style style) = 0;

			virtual void SetNodeHandleSize(const sx_double &size) = 0;
			virtual void SetNodeHandleStyle(tracker_handle_style style) = 0;
			virtual void SetNodeHandleColor(const sx_color_i_rgb& color) = 0;
			
		};


		//////////////////////////////////////////////////////////////////////////
		interface ITrackerStyle
		{
			virtual sx_flags get_Style() = 0;
			virtual void set_Style(sx_flags st) = 0;
			virtual sx_size get_MinSize() const = 0;
			virtual void set_MinSize(const sx_size &minSize) = 0;

			virtual void SetHandleStyle(tracker_handle_style style) = 0;
			virtual void SetHandleColor(const sx_color_i_rgb& color) = 0;
			virtual void SetHandleSize(const sx_double &size) = 0;

			virtual void SetFrameWidth(const sx_double &width) = 0;
			virtual void SetFrameStyle(line_style st) = 0;
			virtual void SetFrameColor(const sx_color_i_rgb& color) = 0;
			virtual void SetFrameOpacity(sx_byte op) = 0;

			virtual void SetFillColor(const sx_color_i_rgb& color) = 0;
			virtual void SetFillOpacity(sx_byte op) = 0;

			virtual void SetPrimaryStyle(line_style st) = 0;
			virtual void SetPrimaryColor(const sx_color_i_rgb& color) = 0;
			virtual void SetPrimaryOpacity(sx_byte op) = 0;

			virtual void SetOrdinaryStyle(line_style st) = 0;
			virtual void SetOrdinaryColor(const sx_color_i_rgb& color) = 0;
			virtual void SetOrdinaryOpacity(sx_byte op) = 0;

			virtual IEditorTrackerStyle* get_EditorStyle() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum tracker_type
		{
			tracker_type_rect = 1,
			tracker_type_circle,
			tracker_type_line,
			tracker_type_polyline,
			tracker_type_polygone,
			tracker_type_rect_brush,
			tracker_type_circle_brush,
			tracker_type_cross_brush
		};

		sx_tool_act sx_tool_change_tracker = 10,
					sx_tool_tracker_new_point = 11;

		//////////////////////////////////////////////////////////////////////////
		interface IToolTracker : public IToolState
		{
			// settings
			virtual tracker_type get_Type() const = 0;
			virtual void set_Type(tracker_type type) = 0;
			virtual sx_double get_Width() const = 0;
			virtual void set_Width(sx_double dWidth) = 0;
			virtual sx_double get_BrushSize() const = 0;
			virtual void set_BrushSize(sx_double dSize) = 0;
			virtual void GetColor(sx_color_i_rgb& color) const = 0;
			virtual void SetColor(const sx_color_i_rgb& color) = 0;
			// state
			virtual void GetBeginPoint(sx_point &pt) const = 0;
			virtual void GetEndPoint(sx_point &pt) const = 0;
		};

		sx_tool_act sx_tool_change_crop_area = 15;

		//////////////////////////////////////////////////////////////////////////
		interface IToolCropper : public IToolState
		{
			// settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;

			//state
			virtual void GetPageArea(sx_rect& rect) = 0;
			virtual void SetPageArea(const sx_rect& rect) = 0;
		};

		sx_tool_act sx_tool_rectmark_change_page = sx_tool_change_page, sx_tool_rectmark_change_primary = 53, sx_tool_rectmark_add = 54, sx_tool_rectmark_transform = 55;

		//////////////////////////////////////////////////////////////////////////
		interface IToolRectMark : public IToolState
		{
			//settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual sx_bool get_AllowNew() const = 0;
			virtual void set_AllowNew(sx_bool bNew) = 0;
			virtual void SetThumbnail(Graphics::IBitmapData* pThumb, sx_uint reserved = 0) = 0;

			//state
			virtual sx_uint get_Count() const = 0;
			virtual void GetItem(sx_uint uIndex, sx_rect &rect) = 0;
			virtual sx_uint GetPrimaryItem() const = 0;
			virtual void SetPrimaryItem(sx_uint uItem) = 0;
			virtual void Add(sx_rect &rect) = 0;
			virtual void Remove(sx_uint uIndex) = 0;
			virtual void RemoveAll() = 0;
		};

		sx_tool_act sx_tool_change_text_selection = 20, sx_tool_continue_text_selection = 21, sx_tool_before_text_selection = 22;
		
		//////////////////////////////////////////////////////////////////////////
		interface IToolTextSelect : public IToolState
		{
			// settings
			virtual sx_bool get_TouchStyle() const = 0;
			virtual void set_TouchStyle(sx_bool bTouch) = 0;
			virtual sx_bool get_CaretOnly() const = 0;
			virtual void set_CaretOnly(sx_bool bCaretOnly) = 0;
			virtual sx_flags get_TextOptions() const = 0;
			virtual void set_TextOptions(sx_flags options) = 0;

			// state
			virtual void GetToolPoint(sx_point &pt) const = 0;
			virtual IStr* GetText() const = 0;
			virtual Graphics::IRegion* GetRegion() const = 0;
			virtual sx_bool LeftSideCaret() const = 0;
		};

		sx_tool_act sx_tool_change_snapshot = 15;

		//////////////////////////////////////////////////////////////////////////
		interface IToolSnapshot : public IToolCropper
		{
			//state
			virtual Graphics::IBitmapData* GetBitmap() = 0;
			virtual void Draw(Graphics::IDrawingContext* ctx) = 0;
		};

		sx_tool_act sx_tool_move_lens = 12, sx_tool_change_lens = 13;

		//////////////////////////////////////////////////////////////////////////
		interface IToolLens : public IToolState
		{
			// settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;

			//state
			virtual sx_double get_Zoom() const = 0;
			virtual void set_Zoom(const sx_double &bZoom) = 0;

			virtual void SetLensSize(sx_uint w, sx_uint h) = 0;
			virtual void SetPagePos(const sx_point& pt) = 0;
			virtual void GetPageArea(sx_rect& rect) = 0;
			virtual Graphics::IBitmapData* GetBitmap() = 0;
			virtual void Draw(Graphics::IDrawingContext* ctx) = 0;
		};
		
		sx_tool_act sx_tool_text_box_create = 48, sx_tool_text_box_change = 49;

		interface IToolTextBox : public IToolState
		{
			//settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual sx_bool get_AllowNew() const = 0;
			virtual void set_AllowNew(sx_bool bNew) = 0;
			virtual sx_bool get_AllowEditText() const = 0;
			virtual void set_AllowEditText(sx_bool bAllow) = 0;
			virtual sx_bool get_Wrap() const = 0;
			virtual void set_Wrap(sx_bool bWrap) = 0;
			virtual XHTML::ITextBox* get_TextBox() const = 0;
			virtual void set_TextBox(XHTML:: ITextBox*pTextBox) = 0;

			//state
			virtual IRichTextEditor* get_TextEditor() const = 0;
			virtual void GetPageArea(sx_rect& rect) = 0;
			virtual void SetPageArea(const sx_rect& rect) = 0;
			virtual void Invalidate() = 0;
		};

		sx_tool_act sx_tool_start_segment = 60, sx_tool_complete_segment = 61, sx_tool_stylus_erase =80;
		
		//////////////////////////////////////////////////////////////////////////
		interface IToolStylus : public IToolState
		{
			//settings
			virtual Graphics::IInkImage* get_Ink() const = 0;
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual void GetPageArea(sx_rect& rect) = 0;
			virtual void SetPageArea(const sx_rect& rect) = 0;
			virtual sx_uint get_ShowPressure()  const = 0;
			virtual void set_ShowPressure(sx_uint showPressure) = 0;
			virtual sx_bool get_EraseMode() const = 0;
			virtual void set_EraseMode(sx_bool bEraseMode) = 0;
			virtual sx_double get_EraserSize() const = 0;
			virtual void set_EraserSize(const sx_double &dEraserSize) = 0;
			//state
			virtual void GetToolPoint(sx_point& pt) = 0;
		};
		
		const sx_tool_act 
			sx_tool_pc_dbl_click		= 63, 
			sx_tool_pc_change_selection	= 64,
			sx_tool_pc_transform		= 65,
			sx_tool_pc_act_text			= 66,
			sx_tool_pc_deact_text		= 67,
			sx_tool_pc_out_of_text		= 68,
			sx_tool_pc_out_of_vector	= 69,
			sx_tool_pc_create_shape		= 70,
			sx_tool_pc_change_vector	= 71,
			sx_tool_pc_change_shape		= 72,
			sx_tool_pc_reset_mode		= 73,
			sx_tool_pc_start_transform	= 74;

		enum tool_pc_ht
		{
			tool_pc_ht_none			= 0, 
			tool_pc_ht_sel_page		= 0x01,
			tool_pc_ht_sel_objects	= 0x02,
			tool_pc_ht_sel_in_composite	= 0x04,
			tool_pc_ht_sel_in_group	= 0x08,
			tool_pc_ht_rect_select	= 0x10,
			tool_pc_ht_entire_rect	= 0x20
		};

		enum tool_pc_invalidate
		{
			tool_pc_invalidate_def = 0,
			tool_pc_invalidate_page = 0x01
		};

		enum tool_pc_mode
		{
			tool_pc_mode_select = 1,
			tool_pc_mode_add,
			tool_pc_mode_add_rect,
			tool_pc_mode_add_line,
			tool_pc_mode_add_ellipse,
			tool_pc_mode_add_shape,
			tool_pc_mode_edit_text,
			tool_pc_mode_edit_vector,
			tool_pc_mode_edit_shape,
			tool_pc_mode_comp_bbox,
			tool_pc_mode_add_polyline,
			tool_pc_mode_add_freeline,
			tool_pc_mode_edit_linear_grad,
			tool_pc_mode_edit_radial_grad
		};

		//////////////////////////////////////////////////////////////////////////
		interface IToolPageContent : public IToolState
		{
			// properties
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual Content::IRichContent* get_Content() = 0;
			virtual IRichTextEditor* get_TextEditor() = 0;
			virtual sx_bool get_MultiSelect() = 0;
			virtual void set_MultiSelect(sx_bool ms) = 0;
			virtual tool_pc_mode get_Mode() const = 0;
			virtual void set_Mode(tool_pc_mode mode) = 0;
			virtual sx_flags get_HitTest() = 0;
			virtual void set_HitTest(sx_flags ht) = 0;
			virtual sx_flags get_RCOptions() = 0;
			virtual void set_RCOptions(sx_flags opt) = 0;
			virtual sx_bool get_EnableUndo() const = 0;
			virtual void set_EnableUndo(sx_bool bEnable) const = 0;
			
			// state
			virtual void GetBeginPoint(sx_point &pt) const = 0;
			virtual void GetEndPoint(sx_point &pt) const = 0;
			virtual void GetClipArea(sx_rect& rect) const = 0;
			virtual void Invalidate(sx_flags opt = tool_pc_invalidate_def) = 0;
			// selection
			virtual sx_uint get_SelCount() const = 0;
			virtual sx_uint get_PrimSelObject() const = 0;
			virtual Content::IObjectTrunk* GetSelObject(sx_uint ind) = 0;
			virtual void SetObjectSel(Content::IObjectTrunk* pObj, sx_bool sel = true, sx_bool prim = true) = 0;
			virtual void ClearSelection(sx_bool remain_prim = false) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		enum tool_annot_status
		{
			tool_annot_status_unchanged, 
			tool_annot_status_added, 
			tool_annot_status_removed 
		};

		sx_tool_act sx_tool_change_annot_set = 30, sx_tool_annot_move = 31, sx_tool_annot_resize = 32;

		//////////////////////////////////////////////////////////////////////////
		interface IToolAnnot : public IToolState
		{
			// settings			
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual sx_bool get_MultiSelect() const = 0;
			virtual void set_MultiSelect(sx_bool bMS) = 0;
			virtual sx_flags get_MarkMask() const = 0;
			virtual void set_MarkMask(sx_flags flags) = 0;
			
			// state - marked annots
			virtual sx_uint get_Count() const = 0;
			virtual PDF::IAnnot* GetItem(sx_uint uAnnot) const = 0;
			virtual tool_annot_status GetStatus(sx_uint uAnnot) const = 0;
			virtual sx_uint GetPrimaryItem() const = 0;
			virtual void MarkAnnot(PDF::IAnnot* pAnnot) = 0;
			virtual void UnmarkAnnot(PDF::IAnnot* pAnnot) = 0;
			virtual void GetMarkedArea(sx_quad& quad) const = 0;
			virtual void GetTransformation(sx_matrix& mx) const = 0;
		};

		sx_tool_act sx_tool_free_text_create = 42, sx_tool_free_text_change_shape = 40, sx_tool_free_text_change_text = 41;

		interface IToolFreeText : public IToolState
		{
			//settings
			virtual sx_bool get_AllowNew() const = 0;
			virtual void set_AllowNew(sx_bool bNew) = 0;
			virtual sx_bool get_AltCallout() const = 0;
			virtual void set_AltCallout(sx_bool bAlt) = 0;
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual PDF::IFreeText* get_FreeText() const = 0;
			virtual void set_FreeText(PDF::IFreeText *pFreeText) = 0;

			//state
			virtual IRichTextEditor* get_TextEditor() const = 0;
			virtual void GetToolPoint(sx_point &pt) const = 0;
		};

		sx_tool_act sx_tool_ink_segment = 75;

		interface IToolInk : public IToolState
		{

			//settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual sx_bool get_ShowFrame() const = 0;
			virtual void set_ShowFrame(sx_bool bShow) = 0;
			virtual sx_double get_LastSegmentPressure() const = 0;
			virtual sx_uint get_ShowPressure()  const = 0;
			virtual void set_ShowPressure(sx_uint showPressure) = 0;

			virtual PDF::IInk* get_Ink() const = 0;
			virtual void set_Ink( PDF::IInk* pInk) = 0;

			//state
			virtual void GetToolPoint(sx_point& pt) = 0;
		};

		interface IToolEraser : public IToolState
		{
			//settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual sx_bool get_ShowFrame() const = 0;
			virtual void set_ShowFrame(sx_bool bShow) = 0;

			virtual sx_double get_Size() const = 0;
			virtual void set_Size(const sx_double& dSize)  = 0;
			virtual sx_uint get_Inks() const = 0;
			virtual PDF::IInk* GetInk(sx_uint uInk) = 0;
			virtual void AddInk(PDF::IInk* pInk) = 0;
			//state
			virtual void GetToolPoint(sx_point& pt) = 0;
		};

		sx_tool_act sx_tool_shape_change = 50, sx_tool_shape_create = 51, sx_tool_shape_discard = 52;

		interface IToolShape : public IToolState
		{
			//settings
			virtual ITrackerStyle* get_TrackerStyle() = 0;
			virtual PDF::IAnnot* get_Shape() const = 0;
			virtual void set_Shape(PDF::IAnnot* pShape) = 0;
			virtual sx_bool Stop() = 0;

			//state
			virtual void GetToolPoint(sx_point &pt) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// undo manager
		interface IUndoTransaction : public IRefObject { };

		interface IUndoStack
		{
			virtual sx_uint get_MaxLength() const = 0;
			virtual void set_MaxLength(sx_uint len) = 0;
			virtual sx_uint get_UndoCount() const = 0;
			virtual sx_uint get_RedoCount() const = 0;
			virtual sx_bool get_Locked() const = 0;

			virtual IUndoTransaction* StartTransaction() = 0;			
			virtual IUndoTransaction* StartPDFTransaction() = 0;
			virtual sx_bool Undo(sx_bool bRedo = false) = 0;
			virtual sx_bool Redo() = 0;
			virtual void Clear() = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		// dynamic user notes
		enum dyn_note
		{
			dyn_note_undefined,
			dyn_note_sticky		= 0x0001,
			dyn_note_highlight	= 0x0002,
			dyn_note_underline	= 0x0004,
			dyn_note_strikeout	= 0x0008,
			dyn_note_line		= 0x0010,
			dyn_note_square		= 0x0020,
			dyn_note_circle		= 0x0040,
			dyn_note_polyline	= 0x0080,
			dyn_note_ink		= 0x0100,
			dyn_note_stamp		= 0x0200
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynPopup
		{
			virtual sx_bool get_Visible() const = 0;
			virtual void set_Visible(sx_bool v) = 0;
			virtual IStr* get_Subject() const = 0;
			virtual void set_Subject(sx_str s) = 0;
			virtual IStr* get_Description() const = 0;
			virtual void set_Description(sx_str d) = 0;

			virtual void GetColor(sx_color_i_rgb& c) const = 0;
			virtual void SetColor(const sx_color_i_rgb& c) = 0;
			virtual void GetRectArea(sx_rect& box) const = 0;
			virtual void SetRectArea(const sx_rect& box) = 0;
		};

		object_type obj_type_dyn_note = 0x28;

		//////////////////////////////////////////////////////////////////////////
		interface IDynNote : public IRefObject
		{
			virtual dyn_note get_Type() const = 0;
			virtual sx_uint get_Page() const = 0;
			virtual IStr* get_Ident() const = 0;
			virtual void set_Ident(sx_str sIdent) = 0;
			virtual sx_bool get_Visible() const = 0;
			virtual void set_Visible(sx_bool v) = 0;
			virtual IDynPopup* get_Popup() const = 0;
			virtual void ResetPopup(sx_bool bClear = false) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynStickyNote : public IDynNote
		{
			virtual void GetAnchor(sx_point& pt) const = 0;
			virtual void SetAnchor(const sx_point& pt) = 0;
			virtual void SetIcon(Graphics::IBitmapData* pIcon, sx_uint DPI, sx_uint x, sx_uint y) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynTextMarkup : public IDynNote
		{
			virtual Graphics::IRegion* GetRegion() const = 0;
			virtual void SetRegion(Graphics::IRegion *pReg) = 0;
			virtual void GetColor(sx_color_i_rgb& c) const = 0;
			virtual void SetColor(const sx_color_i_rgb& c) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynPath : public IDynNote
		{
			virtual sx_double get_Thickness() const = 0;
			virtual void set_Thickness(const sx_double& t) = 0;
			virtual line_style get_Style() const = 0;
			virtual void set_Style(line_style s) = 0;
			virtual void GetColor(sx_color_i_rgba& c) const = 0;
			virtual void SetColor(const sx_color_i_rgba& c) = 0;
			virtual void GetFillColor(sx_color_i_rgba& c) const = 0;
			virtual void SetFillColor(const sx_color_i_rgba& c) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynLine : public IDynPath
		{
			virtual void GetStart(sx_point& pt) const = 0;
			virtual void SetStart(const sx_point& pt) = 0;
			virtual void GetEnd(sx_point& pt) const = 0;
			virtual void SetEnd(const sx_point& pt) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynSquare : public IDynPath
		{
			virtual void GetBox(sx_rect& bbox) const = 0;
			virtual void SetBox(const sx_rect& bbox) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynCircle : public IDynPath
		{
			virtual void GetBox(sx_rect& bbox) const = 0;
			virtual void SetBox(const sx_rect& bbox) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynPolyline : public IDynPath, public Graphics::IPointsArray
		{
			virtual sx_bool get_Closed() const = 0;
			virtual void set_Closed(sx_bool v) = 0;
			virtual void GetBox(sx_rect& bbox) const = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynInk : public IDynNote
		{
			virtual Graphics::IInkImage* get_Ink() const = 0;
			virtual void set_Ink(Graphics::IInkImage* pInk) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynStamp : public IDynNote
		{
			virtual void GetBox(sx_rect& bbox) const = 0;
			virtual void SetBox(const sx_rect& bbox) = 0;
			virtual void SetImage(Graphics::IBitmapData* pImage) = 0;
		};

		//////////////////////////////////////////////////////////////////////////
		interface IDynNotes
		{
			virtual sx_uint get_Count() const = 0;
			virtual sx_flags get_Visible() const = 0;
			virtual void set_Visible(sx_flags mask) = 0;
			virtual sx_flags get_Focusable() const = 0;
			virtual void set_Focusable(sx_flags mask) = 0;
			virtual viewer_popups get_Popups() const = 0;
			virtual void set_Popups(viewer_popups popups) = 0;

			virtual IDynNote* Item(sx_uint uIndex) = 0;
			virtual IDynNote* ItemAtPage(sx_uint uPage, IDynNote* pPrev = nullptr) = 0;
			virtual IDynNote* ItemByLocation(sx_uint uPage, const sx_point &p, IDynNote* pPrev = nullptr) = 0;
			virtual IDynNote* Add(sx_uint uPage, dyn_note type) = 0;
			virtual void Remove(IDynNote* pAnnot) = 0;
			virtual void RemoveAtPage(sx_uint uPage) = 0;
			virtual void RemoveAll() = 0;
		};

#ifdef SX_WINDESKTOP_DLL
		//////////////////////////////////////////////////////////////////////////
		// Window based viewer
		typedef HWND sx_window;

		enum viewer_flags
		{
			viewer_flags_none = 0,
			viewer_flags_scrollbar_always = 2,
			viewer_flags_scrollbar_autohide = 4,
			viewer_flags_border_flat = 8,
			viewer_flags_border_3D = 16
		};

		interface ISystemClipboard
		{
			virtual sx_bool get_EnableHotkeys() const = 0;
			virtual void set_EnableHotkeys(sx_bool bEnable) = 0;
			virtual sx_bool get_AllowPaste() const = 0;
			virtual sx_bool get_AllowCopy() const = 0;
			virtual sx_bool get_AllowCut() const = 0;
			virtual sx_bool Paste() = 0;
			virtual sx_bool Copy() = 0;
			virtual sx_bool Cut() = 0;
		};



		//////////////////////////////////////////////////////////////////////////
		interface IWinViewer
		{
			//////////////////////////////////////////////////////////////////////////
			// object
			virtual IViewer* get_Viewer() = 0;
			virtual sx_window get_Window() = 0;
			virtual ISystemClipboard* get_Clipboard() = 0;

			//////////////////////////////////////////////////////////////////////////
			// options
			virtual sx_bool get_EnableKeysScroll() const = 0; 
			virtual void set_EnableKeysScroll(sx_bool bEnable)  = 0;
			virtual sx_bool get_EnableBarScroll() const = 0;
			virtual void set_EnableBarScroll(sx_bool bEnable) = 0;
			virtual sx_bool get_EnableWheelScroll() const = 0;
			virtual void set_EnableWheelScroll(sx_bool bEnable) = 0;
			virtual sx_bool get_EnableMouseDrag() const = 0;
			virtual void set_EnableMouseDrag(sx_bool bEnable) = 0;

			virtual sx_bool get_EnableZoomGesture() const = 0; 
			virtual void set_EnableZoomGesture(sx_bool bEnable) = 0;
			virtual sx_bool get_EnableScrollGesture() const = 0; 
			virtual void set_EnableScrollGesture(sx_bool bEnable) = 0;
			virtual sx_bool get_EnableRotateGesture() const = 0; 
			virtual void set_EnableRotateGesture(sx_bool bEnable) = 0;

			virtual void SetEventHandler(IViewerEvents* pEventHandler) = 0;
		};
#endif
	}
} // namespace SX