// ********************************************************************
// This is a part of the PDF Xpansion SDK 13.
// Copyright (C) 2002-2018 soft Xpansion
// All rights reserved.
//
// This source code is only intended as a supplement to the PDF Xpansion SDK 
// and related electronic documentation provided with the libraries.
// ********************************************************************
#pragma once

#include "document.h"

namespace SX
{
	namespace Content
	{
		interface IAutoText;
		interface IAutoTextPattern;
	} //Content
	
	object_type obj_type_rich_text_document = 0x34;

	namespace XHTML
	{
		enum text_alignment
		{
			text_alignment_left = 1,
			text_alignment_center,
			text_alignment_right,
			text_alignment_justify
		};

		enum vert_alignment
		{
			vert_alignment_top = 1,
			vert_alignment_middle,
			vert_alignment_bottom
		};

		enum line_style
		{
			line_style_none,
			line_style_solid,
			line_style_dot,
			line_style_dash,
			line_style_dash_dot,
			line_style_dash_dot_dot
		};

		enum border_mask
		{
			border_mask_left	= 0x1,
			border_mask_top		= 0x2,
			border_mask_right	= 0x4,
			border_mask_bottom	= 0x8
		};

		interface IRichTextStyle : public IRefObject
		{
			virtual IStr* get_FontFamily() const = 0;
			virtual void set_FontFamily(sx_str sName) = 0;
			virtual sx_double get_FontSize() const = 0;
			virtual void set_FontSize(const sx_double dSize) = 0;
			virtual sx_bool get_Italic() const = 0;
			virtual void set_Italic(sx_bool bStyle) = 0;
			virtual sx_uint get_Weight() const = 0;
			virtual void set_Weight(sx_uint uWeight) = 0;
			virtual sx_uint get_FontStretch() const = 0;
			virtual void set_FontStretch(sx_uint uStretch) = 0;
			virtual sx_bool get_Underline() const = 0;
			virtual void set_Underline(sx_bool bStyle) = 0;
			virtual sx_bool get_Strikeout() const = 0;
			virtual void set_Strikeout(sx_bool bStyle) = 0;
			virtual const sx_color* get_Color() const = 0;
			virtual void set_Color(const sx_color& c) = 0;
			virtual text_alignment get_Alignment() const = 0; 
			virtual void set_Alignment(text_alignment eAlign) = 0;
			virtual sx_double get_LineHeight() const = 0;
			virtual void set_LineHeight(const sx_double dSp) = 0;
			virtual sx_double get_LetterSpacing() const = 0;
			virtual void set_LetterSpacing(const sx_double dSp) = 0;
			virtual sx_double get_Rise() const = 0;
			virtual void set_Rise(const sx_double dRise) = 0;

			virtual IRichTextStyle* Clone() const = 0;
		};

		interface IRichText : public IRefObject
		{
			// text edit
			virtual sx_uint get_Length() const = 0;
			virtual IStr* get_Text() const = 0;
			virtual IRichTextStyle* GetDefaultStyle() const = 0;
			virtual IRichTextStyle* GetCharStyle(sx_uint uIndex) const = 0;
			virtual void SetCharStyle(sx_uint Start, sx_uint Count, IRichTextStyle* pStyle) = 0;
			virtual void InsertChars(sx_uint uIndex, sx_str sText, IRichTextStyle* pStyle = nullptr) = 0;
			virtual void InsertLineBreak(sx_uint uIndex) = 0;
			virtual void RemoveChars(sx_uint Start, sx_uint Count) = 0;
		};

		interface ITextBox : public IRefObject
		{
			virtual IRichText* get_Content() = 0;
			virtual vert_alignment get_VertAlignment() const = 0;
			virtual void set_VertAlignment(vert_alignment eAlign) = 0;

			virtual const sx_color* get_BackColor() const = 0;
			virtual void set_BackColor(const sx_color& c) = 0;
			virtual const sx_color* get_BorderColor() const = 0;
			virtual void set_BorderColor(const sx_color& c) = 0;
			virtual sx_double get_BorderThickness() const = 0;
			virtual void set_BorderThickness(const sx_double &th) = 0;
			virtual line_style get_BorderStyle() const = 0;
			virtual void set_BorderStyle(line_style st) = 0;
			virtual sx_flags get_BorderMask() const = 0;
			virtual void set_BorderMask(sx_flags mask) = 0;
			virtual sx_double get_TextOpacity() const = 0;
			virtual void set_TextOpacity(sx_double dOpacity) = 0;
			virtual sx_double get_PaddingL() const = 0;
			virtual sx_double get_PaddingT() const = 0;
			virtual sx_double get_PaddingR() const = 0;
			virtual sx_double get_PaddingB() const = 0;

			virtual void SetPadding(const sx_double &l, const sx_double &t, const sx_double &r, const sx_double &b) = 0;
			virtual Content::IAutoTextPattern* GetAutoTextPattern(Content::IAutoText* pAutoText) = 0;
		};

		enum rich_text_import
		{
			rich_text_import_wrap_lines			= 0x00001,
			rich_text_import_justify_words		= 0x00002,
			rich_text_import_bidi_text			= 0x00004,
		};

		enum rich_text_image
		{
			rich_text_image_adjust_page_size	= 0x00010,
			rich_text_image_shrink_size			= 0x00020,
			rich_text_image_fit_size			= 0x00040,
		};

		enum rich_text_html
		{
			rich_text_html_skip_anchors			= 0x010000,
			rich_text_html_id_to_anchors		= 0x020000,
			rich_text_html_generate_toc			= 0x040000,
			rich_text_html_skip_colors			= 0x100000,
			rich_text_html_skip_font_family		= 0x200000,
			rich_text_html_inc_fixed_blocks		= 0x400000
		};

		enum content_type
		{
			content_type_text			= 0x0001,
			content_type_raster			= 0x0002,
			content_type_svg			= 0x0004,
			content_type_mml			= 0x0008
		};

		//////////////////////////////////////////////////////////////////////////
		interface IImportProvider
		{
			virtual sx_bool get_ImportLinked() const = 0;
			virtual void set_ImportLinked(const sx_bool b) = 0;
			virtual sx_bool get_Siblings() const = 0;
			virtual void set_Siblings(const sx_bool b) = 0;
			virtual sx_uint get_Descendants() const = 0;
			virtual void set_Descendants(const sx_uint depth) = 0;
			virtual sx_flags get_ContentType() const = 0;
			virtual void set_ContentType(const sx_flags opt) = 0;
			virtual sx_flags get_Flags() const = 0;
			virtual void set_Flags(const sx_flags opt) = 0;
			
			virtual void ImportLocal(sx_str sPath) = 0;
			virtual void ImportRemote(sx_str sURI, sx_str sUsername = nullptr, sx_str sPassword = nullptr) = 0;
		};
		
		//////////////////////////////////////////////////////////////////////////
		interface IRichTextDocument : public ISimpleDocument
		{
			virtual IRichTextStyle* GetDefaultStyle() const = 0;
			virtual IImportProvider* GetImportProvider(sx_bool lock = true) = 0;
			virtual void SetPage(const page_props &props, sx_double MiddleMargin = -1.) = 0;
			virtual void SetBackColor(const sx_color& c) = 0;
			virtual void SetLoader(IDocLoader* pLoader) = 0;

			virtual sx_bool SearchPlainText(ISequentialStream* pStream, sx_str sText, sx_uint TxtEnc = 0, sx_flags Opt = 0) = 0;
			virtual void ImportPlainText(ISequentialStream* pStream, sx_uint TxtEnc = 0, sx_flags flags = rich_text_import_wrap_lines) = 0;
			virtual void ImportImage(ISequentialStream* pStream, sx_uint DPI = 0, sx_flags flags = 0) = 0;
			virtual void ImportHTML(ISequentialStream* pStream, sx_flags flags = 0) = 0;                                 
			virtual void ImportSVG(ISequentialStream* pStream, sx_uint DPI = 72, sx_flags flags = 0) = 0;
			virtual void ImportMML(ISequentialStream* pStream, sx_uint DPI = 72, sx_str FontName = nullptr, sx_uint FontSize = 12, sx_flags flags = 0) = 0;
		};
	} // namespace XHTML
} // namespace SX