﻿#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::ProcessInvoice(SX::ZUGFeRD::IInvoice *pInvoice)
	{
		CString mess = "Invoice information\n\nProfile: ";
		switch(pInvoice->GetProfile()->get_Profile())
		{
			case SX::ZUGFeRD::zf_profile_basic: mess.Append(L"basic"); break;
			case SX::ZUGFeRD::zf_profile_comfort: mess.Append(L"comfort"); break;
			case SX::ZUGFeRD::zf_profile_extended: mess.Append(L"extended"); break;
			case SX::ZUGFeRD::zf_profile_unknown: mess.Append(L"unknown"); break;
		}
		if(SX::ZUGFeRD::ITradeTransaction* pTransaction = pInvoice->GetTransaction())
		{
			if(SX::ZUGFeRD::ITradeAgreement* pAgreement = pTransaction->GetAgreement())
			{
				SX::ZUGFeRD::ITradeParty* pBuyerParty = pAgreement->GetBuyerParty();
				if (pBuyerParty != nullptr)
				{
					SX::ZUGFeRD::ITextElement* pBuyerText = pBuyerParty->GetName();
					if (pBuyerText != nullptr)
						mess.AppendFormat(_T("\nBuyer: %s"),pBuyerText->get_Text());
				}

				SX::ZUGFeRD::ITradeParty* pSallerParty = pAgreement->GetBuyerParty();
				if (pSallerParty != nullptr)
				{
					SX::ZUGFeRD::ITextElement* pSallerText = pBuyerParty->GetName();
					if (pSallerText != nullptr)
						mess.AppendFormat(L"\nSeller: %s",pSallerText->get_Text());
				}
			}
			if(SX::ZUGFeRD::ITradeSettlement* pSettlement = pTransaction->GetSettlement())
				if(SX::ZUGFeRD::IAmount* pAmount = pSettlement->GetMonetarySummation()->GetGrandTotal())
				{
					CString sCurr;
					switch(pAmount->get_currency())
					{
						case CurrCode3_EUR: sCurr=L"EUR";	break;
						case CurrCode3_USD: sCurr=L"USD";	break;
					}
					mess.AppendFormat(_T("\nTotal: %.2f %s"), pAmount->get_Value(), sCurr);
				}
		}
		m_pUI->MessageBox(mess,MB_OK);
	}

	void CPDFXpansionSDK::OpenInvoice()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
		{
			try {
				SX::ObjPtr<SX::ZUGFeRD::IInvoiceDocument> ptrInvoiceDoc = m_pApp->get_Factory()->CreateInvoiceDocument();
				SX::ZUGFeRD::zf_status docStatus = ptrInvoiceDoc->OpenFile(sPath);
				if (docStatus == SX::ZUGFeRD::zf_status_valid)
				{
					SX::ZUGFeRD::IInvoice* pInvoice = ptrInvoiceDoc->get_Invoice();
					if(pInvoice->Validate((SX::ZUGFeRD::zf_val_option_schema | SX::ZUGFeRD::zf_val_option_rules | SX::ZUGFeRD::zf_val_option_codes), this))
					{
						// show document in viewer
						IVDocPtr ptrVDoc = ptrInvoiceDoc->GetViewableDocument();
						IViewerPtr ptrV = m_pDefViewer->get_Viewer();
						ptrV->set_Document(ptrVDoc);
						// process invoice
						ProcessInvoice(pInvoice); 
					}
					else m_pUI->MessageBox(L"Invoice validation failed!", MB_OK | MB_ICONSTOP);
				}
				else
				{
					CString mess;
					switch(docStatus)
					{
						case ZUGFeRD::zf_status_not_pdf	: mess=L"ZUGFeRD file must be a PDF file."; break;
						case ZUGFeRD::zf_status_enc_pdf	: mess=L"ZUGFeRD file must be a PDF file without encryption."; break;
						case ZUGFeRD::zf_status_no_meta	: mess=L"ZUGFeRD file must be a PDF file with metadata."; break;
						case ZUGFeRD::zf_status_err_meta: mess=L"Metadata of this file isn't conform to ZUGFeRD format."; break;
						case ZUGFeRD::zf_status_not_pdfa: mess=L"ZUGFeRD file isn't conform to PDF/A-3 format."; break;
						case ZUGFeRD::zf_status_err_emb	: mess=L"ZUGFeRD file hasn't embedded invoice or embedding is incorrect."; break;
						case ZUGFeRD::zf_status_not_zf	: mess=L"Embedded invoice isn't well-formed XML or ZUGFeRD."; break;
						case ZUGFeRD::zf_status_dif_prof: mess=L"The profile specified in metadata and invoice profile are different."; break;
					}
					m_pUI->MessageBox(mess, MB_OK | MB_ICONSTOP);
				}
			} catch (SX::IException* e) { auto err = e->get_ErrorCode(); }
		}
	}

	void CPDFXpansionSDK::OpenInvoiceXML()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XML Files *.xml\0*.xml\0\0", sPath))
		{
			try {
				SX::ObjPtr<SX::ZUGFeRD::IInvoiceStream> ptrInvoiceStream = m_pApp->get_Factory()->CreateInvoiceStream();
				SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateReadStream(sPath);
				ptrInvoiceStream->Open(ptrStream);

				if(SX::ZUGFeRD::IInvoice* pInvoice = ptrInvoiceStream->get_Invoice())
				{
					if(pInvoice->Validate((SX::ZUGFeRD::zf_val_option_schema | SX::ZUGFeRD::zf_val_option_rules | SX::ZUGFeRD::zf_val_option_codes),this))
					{
						// process invoice
						ProcessInvoice(pInvoice);
					}
					else m_pUI->MessageBox(L"Invoice validation failed!", MB_OK | MB_ICONSTOP);
				}
				else m_pUI->MessageBox(L"Invoice isn't well-formed XML or ZUGFeRD.", MB_OK | MB_ICONSTOP);

			} catch (SX::IException* e) { auto err = e->get_ErrorCode(); }
		}
	}

	//////////////////////////////////////////////////////////////////////////
	const CString Note1 = L"Rechnung gemäß Vertrag/Auftrag  03-07/14v, ZV1";
	const CString Note2 = L"soft Xpansion GmbH & Co. KG\nKönigsallee 45\n44789 Bochum\nDeutschland\nGeschäftsführer: Dr. Juri Stern\nHandelsregister HRA 5869 (AG Bochum)";
	const CString Note3 = L"Zahlungsziel: 14 Tage Netto ohne Abzug";
	
	void CPDFXpansionSDK::CreateInvoice()
	{
		try
		{
			SX::ObjPtr<SX::ZUGFeRD::IInvoiceDocument> ptrInvoiceDoc = m_pApp->get_Factory()->CreateInvoiceDocument();
			// load printable invoice
			SX::ObjPtr<SX::ISequentialStream> ptrReadStream = m_pApp->get_Factory()->CreateReadStream(GetFilePath("inv.pdf"));
			ptrInvoiceDoc->CreateFromPDF(ptrReadStream);
			SX::ZUGFeRD::IInvoice* pInvoice = ptrInvoiceDoc->get_Invoice();
			// set invoice header
			SX::ZUGFeRD::IHeader* pHeader = pInvoice->GetHeader(true);
			pHeader->GetID(true)->set_ID(L"123456");
			pHeader->GetName(true)->set_Text(L"RECHUNG");
			pHeader->set_Type(SX::ZUGFeRD::doc_code_380);
			SX::sx_time dt;	dt.wYear = 2015; dt.wMonth= 8; dt.wDay = 19;
			pHeader->GetIssueDate(true)->set_Date(dt);
			pHeader->GetIssueDate()->set_format(SX::ZUGFeRD::datetime_code_CCYYMMDD);
			pHeader->GetIncludedNote(0, true)->GetContent(true)->set_Text(Note1);
			pHeader->GetIncludedNote(1, true)->GetContent(true)->set_Text(Note2);
			pHeader->GetIncludedNote(2, true)->GetContent(true)->set_Text(Note3);
			SX::ZUGFeRD::ITradeTransaction* pTransact = pInvoice->GetTransaction(true);

			// set agreement info
			SX::ZUGFeRD::ITradeAgreement* pAgreement = pTransact->GetAgreement(true);
			SX::ZUGFeRD::ITradeParty* pSeller = pAgreement->GetSellerParty(true);
			pSeller->GetName(true)->set_Text(L"soft Xpansion GmbH & Co. KG");

			pAgreement->GetBuyerParty(true)->GetName(true)->set_Text(L"Musterfirma GmbH");
			pTransact->GetLineItem(0, true)->GetAssociatedDocument(true);
			pTransact->GetLineItem(0, true)->GetDelivery(true)->GetBilledQuantity(true)->set_unit(SX::ZUGFeRD::measure_unit_C62);
			pTransact->GetLineItem(0, true)->GetDelivery(true)->GetBilledQuantity(true)->set_Value(0);
			{
				SX::ZUGFeRD::IAmount *am = pTransact->GetLineItem(0, true)->GetSettlement(true)->GetMonetarySummation(true)->GetLineTotal(true);
				am->set_Value(4125.0); am->set_currency(SX::CurrCode3_EUR);
			}
			pTransact->GetLineItem(0, true)->GetProduct(true)->GetName(true)->set_Text(L"PDF Xpansion SDK");

			SX::ZUGFeRD::ITradeAddress* pAddress = pSeller->GetAddress(true);
			pAddress->set_Postcode(L"44789");
			pAddress->GetLineOne(true)->set_Text(L"Königsallee 45");
			pAddress->GetCity(true)->set_Text(L"Bochum");
			pAddress->set_Country(SX::CountryCode2_DE);

			SX::ZUGFeRD::ITaxID* pTax = pSeller->GetTaxRegistration(0, true)->GetID(true);
			pTax->set_TaxScheme(SX::ZUGFeRD::tax_code_VA);
			pTax->set_ID(L"DE814621954");
			pTax = pSeller->GetTaxRegistration(1, true)->GetID(true);
			pTax->set_TaxScheme(SX::ZUGFeRD::tax_code_FC);
			pTax->set_ID(L"306/5826/0758");

			SX::ZUGFeRD::ITradeParty* pBuyer = pAgreement->GetBuyerParty(true);
			pBuyer->GetName(true)->set_Text(L"Deutsche Bank PGK AG");
			pAddress = pBuyer->GetAddress(true);
			pAddress->set_Postcode(L"65432");
			pAddress->GetLineOne(true)->set_Text(L"Hauptstr. 12");
			pAddress->GetCity(true)->set_Text(L"Musterstadt");
			pAddress->set_Country(SX::CountryCode2_DE);

			// set delivery info
			SX::sx_time dTime; dTime.wMonth = 8; dTime.wYear = 2015; dTime.wDay=1; 
			SX::ZUGFeRD::IDateTime* pDate = pTransact->GetDelivery(true)->GetDeliveryChainEvent(0, true)->GetOccurrenceDate(0, true);
			pDate->set_Date(dTime);

			// set settlement info
			SX::ZUGFeRD::ITradeSettlement* pSettlement = pTransact->GetSettlement(true);
			pSettlement->GetPaymentReference(0, true)->set_Text(L"2015-123456");
			pSettlement->set_InvoiceCurrency(SX::CurrCode3_EUR);

			SX::ZUGFeRD::IPaymentMeans* pPaymentMeans = pSettlement->GetPaymentMeans(0,true);
			pPaymentMeans->GetCreditorAccount(true)->GetIBANID(true)->set_ID(L"DE51430700240151329000");
			pPaymentMeans->GetCreditorInstitution(true)->GetBICID(true)->set_ID(L"DEUTDEDB430");

			SX::ZUGFeRD::ITradeTax* pTradeTax = pSettlement->GetApplicableTradeTax(0, true);
			SX::ZUGFeRD::IAmount* pAmount = pTradeTax->GetCalculatedAmount(0, true);
			pAmount->set_Value(783.75); pAmount->set_currency(SX::CurrCode3_EUR);
			pTradeTax->set_Type(SX::ZUGFeRD::tax_type_code_VAT);
			pAmount = pTradeTax->GetBasisAmount(0, true);
			pAmount->set_Value(4125); pAmount->set_currency(SX::CurrCode3_EUR);
			pTradeTax->GetApplicablePercent(true)->set_Value(19);

			SX::ZUGFeRD::IMonetarySummation* pSumm = pSettlement->GetMonetarySummation(true);
			pAmount = pSumm->GetLineTotal(true);
			pAmount->set_Value(4125.); pAmount->set_currency(SX::CurrCode3_EUR);
			pAmount = pSumm->GetChargeTotal(true);
			pAmount->set_Value(0.); pAmount->set_currency(SX::CurrCode3_EUR);
			pAmount = pSumm->GetAllowanceTotal(true);
			pAmount->set_Value(0.); pAmount->set_currency(SX::CurrCode3_EUR);
			pAmount = pSumm->GetTaxBasisTotal(true);
			pAmount->set_Value(4125.); pAmount->set_currency(SX::CurrCode3_EUR);
			pAmount = pSumm->GetTaxTotal(true);
			pAmount->set_Value(783.75); pAmount->set_currency(SX::CurrCode3_EUR);
			pAmount = pSumm->GetGrandTotal(true);
			pAmount->set_Value(4908.75); pAmount->set_currency(SX::CurrCode3_EUR);

			// set line item info
			SX::ZUGFeRD::ITradeLineItem* pLineItem = pTransact->GetLineItem(0, true);
			pLineItem->GetAssociatedDocument(true);
			pSettlement = pLineItem->GetSettlement(true);

			// check required elements of invoice
			if(!pInvoice->Validate(SX::ZUGFeRD::zf_val_option_rules,this))
			{
				m_pUI->MessageBox(L"Invoice isn't complete yet.", MB_OK | MB_ICONSTOP);
				return;
			}
			// save invoice
			CString sPath;
			if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			{
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));
				ptrInvoiceDoc->SaveAsFile(sPath);
			}
			// show document in viewer
			IVDocPtr ptrVDoc = ptrInvoiceDoc->GetViewableDocument();
			IViewerPtr ptrV = m_pDefViewer->get_Viewer();
			ptrV->set_Document(ptrVDoc);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	CString CPDFXpansionSDK::GetFilePath(const CString& sFileName)
	{
		CPath path;
		LPWSTR str = path.m_strPath.GetBuffer(MAX_PATH);
		GetModuleFileName(NULL, str, MAX_PATH);
		path.RemoveFileSpec();
		path.Append(L"..");
		path.Append(sFileName);
		return path.m_strPath;
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	//////////////////////////////////////////////////////////////////////////
	SX::sx_bool CPDFXpansionSDK::OnMessage(SX::ZUGFeRD::IValidationMessage* pMess)
	{
		CString sMess = pMess->get_CriticalError() ? L"Error: " : L"Warning: ";
		switch (pMess->get_Code())
		{
			case SX::ZUGFeRD::zf_val_code_schema: sMess.Append(L"invalid XML schema"); break;
			case SX::ZUGFeRD::zf_val_code_attr_notused: sMess.Append(L"attribute not used in the given context"); break;
			case SX::ZUGFeRD::zf_val_code_elem_notused: sMess.Append(L"element not used in the given context"); break;
			case SX::ZUGFeRD::zf_val_code_attr_req: sMess.Append(L"attribute required"); break;
			case SX::ZUGFeRD::zf_val_code_elem_req: sMess.Append(L"element required"); break;
			case SX::ZUGFeRD::zf_val_code_elem_one_req: sMess.Append(L"element must occur exactly 1 times"); break;
			case SX::ZUGFeRD::zf_val_code_elem_one_opt: sMess.Append(L"element may occur at maximum 1 times"); break;
			case SX::ZUGFeRD::zf_val_code_value_req: sMess.Append(L"element value required"); break;
			case SX::ZUGFeRD::zf_val_code_out_of_profile: sMess.Append(L"element or attribute not used in the given profile"); break;
			default: sMess.Append(L"unknown code");
		}
		sMess.Append(L"\nAdditional info: ");
		sMess.Append(pMess->get_ExtendedInfo());
		sMess.Append(L"\n");
		ATL::AtlTrace(sMess);
		return true;
	}

} // namespace Sample