#include "stdafx.h"
#include "MainFrame.h"

namespace Sample
{
	LPCWSTR sWndTitle = L"PDF Xpansion Reader";

	//////////////////////////////////////////////////////////////////////////
	CMainFrame::CMainFrame(IPDFXpansionSDK* pSDK) : m_pSDK(pSDK), m_hViewer(NULL) { }

	//////////////////////////////////////////////////////////////////////////
	// CMainFrame
	bool CMainFrame::CreateFrame(int nShowWindow)
	{
		HMENU hMenu = LoadMenu(NULL, MAKEINTRESOURCE(ID_MENU_MAIN_FRAME));
		if(NULL == Create(NULL, NULL, sWndTitle, WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, WS_EX_APPWINDOW, hMenu, NULL))
			return false;
		RECT rc; GetClientRect(&rc);
		m_hViewer = m_pSDK->CreateViewer(m_hWnd, rc.right, rc.bottom);
		if(m_hViewer == NULL)
			MessageBox(L"Can't create viewer window.", MB_OK | MB_ICONSTOP);
		ShowWindow(nShowWindow);
		return true;
	}

	//////////////////////////////////////////////////////////////////////////
	int CMainFrame::MessageBox(LPCTSTR lpText, UINT uType)
	{
		return ::MessageBox(m_hWnd, lpText, sWndTitle, uType);
	}

	//////////////////////////////////////////////////////////////////////////
	bool CMainFrame::OpenFileDialog( LPCWSTR ext, CString &sPath )
	{
		OPENFILENAME ofn;
		ZeroMemory(&ofn, sizeof (ofn));
		ofn.hInstance = _AtlBaseModule.GetResourceInstance();
		ofn.lStructSize = sizeof (ofn);
		ofn.hwndOwner = m_hWnd;
		ofn.lpstrFilter = ext;
		ofn.lpstrFile = sPath.GetBuffer(MAX_PATH);
		ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST;
		ofn.nMaxFile = MAX_PATH;
		bool bOK = FALSE != GetOpenFileName(&ofn);
		sPath.ReleaseBuffer();
		return bOK;
	}

	//////////////////////////////////////////////////////////////////////////
	void CMainFrame::OnFinalMessage( _In_ HWND )
	{
		PostQuitMessage(0);
		delete this;
	}

	//////////////////////////////////////////////////////////////////////////
	LRESULT CMainFrame::OnSize( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled )
	{
		if(m_hViewer && wParam != SIZE_MINIMIZED)
		{
			RECT rc; GetClientRect(&rc);
			::MoveWindow(m_hViewer, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, true);
		}
		bHandled = TRUE;
		return 0;
	}

	//////////////////////////////////////////////////////////////////////////
	LRESULT CMainFrame::OnInitMenuPopup( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled )
	{		
		if(HIWORD(lParam) == 0)
		{
			HMENU hMenu = HMENU(wParam);
			IViewerPtr ptrV = m_pSDK->GetViewer();
			if(ptrV.get())
				for( UINT ind = 0, cnt = ::GetMenuItemCount(hMenu); ind < cnt; ++ind )
					switch(::GetMenuItemID(hMenu, ind))
					{
						case ID_MODE_NORMAL:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Config()->get_Mode() == Viewer::viewer_mode_normal ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_MODE_FLIP:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Config()->get_Mode() == Viewer::viewer_mode_flip ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_LAYOUT_VERTICAL:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_VerticalLayout() ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_LAYOUT_1COL:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_LayoutMultiplicity() == 1 ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_LAYOUT_2COL:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_LayoutMultiplicity() == 2 ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_LAYOUT_4COL:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_LayoutMultiplicity() == 4 ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_LAYOUT_COVERPAGE:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_SeparateCover() ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_ZOOM_FITWIDTH:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_Fit() == Viewer::viewer_fit_page_width ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_ZOOM_FITHEIGHT:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_Fit() == Viewer::viewer_fit_page_height ? MF_CHECKED : MF_UNCHECKED)); break;
						case ID_ZOOM_FITPAGE:
							::CheckMenuItem(hMenu, ind, MF_BYPOSITION | (ptrV->get_Canvas()->get_Fit() == Viewer::viewer_fit_page ? MF_CHECKED : MF_UNCHECKED)); break;
					}
		}

		bHandled = TRUE;
		return 0;
	}

	//////////////////////////////////////////////////////////////////////////
	LRESULT CMainFrame::OnCloseCommand( UINT uMsg, WPARAM wParam, HWND hWnd, BOOL &bHandled )
	{
		PostMessage(WM_CLOSE);
		bHandled = TRUE;
		return 0;
	}

	//////////////////////////////////////////////////////////////////////////
	LRESULT CMainFrame::OnFileCommand( UINT uMsg, WPARAM wParam, HWND hWnd, BOOL &bHandled )
	{
		switch (LOWORD(wParam))
		{
			case ID_FILE_OPENPDF:
				m_pSDK->OpenPDF();
				break;
			case ID_FILE_OPENXPS:
				m_pSDK->OpenXPS();
				break;
		}
		bHandled = TRUE;
		return 0;
	}

	//////////////////////////////////////////////////////////////////////////
	LRESULT CMainFrame::OnViewerCommand( UINT uMsg, WPARAM wParam, HWND hWnd, BOOL &bHandled )
	{
		IViewerPtr ptrV = m_pSDK->GetViewer();
		if(ptrV.get())
			switch (LOWORD(wParam))
			{
				case ID_MODE_NORMAL:
					ptrV->get_Config()->set_Mode(Viewer::viewer_mode_normal); ptrV->get_Config()->Update(); break;
				case ID_MODE_FLIP:
					ptrV->get_Config()->set_Mode(Viewer::viewer_mode_flip); ptrV->get_Config()->Update(); break;
				case ID_MODE_TNUMBNAILS:
					ptrV->get_Config()->set_Mode(Viewer::viewer_mode_thumbs); ptrV->get_Config()->Update(); break;
				case ID_LAYOUT_VERTICAL:
					ptrV->get_Canvas()->set_VerticalLayout(!ptrV->get_Canvas()->get_VerticalLayout()); break;
				case ID_LAYOUT_1COL:
					ptrV->get_Canvas()->set_LayoutMultiplicity(1); break;
				case ID_LAYOUT_2COL:
					ptrV->get_Canvas()->set_LayoutMultiplicity(2); break;
				case ID_LAYOUT_4COL:
					ptrV->get_Canvas()->set_LayoutMultiplicity(4); break;
				case ID_LAYOUT_COVERPAGE:
					ptrV->get_Canvas()->set_SeparateCover(!ptrV->get_Canvas()->get_SeparateCover()); break;
				case ID_ROTATE_CW:
					ptrV->get_Canvas()->set_Rotation(sx_rotation_angle((ptrV->get_Canvas()->get_Rotation() + 90) % 360)); break;
				case ID_ROTATE_CCW:
					ptrV->get_Canvas()->set_Rotation(sx_rotation_angle((ptrV->get_Canvas()->get_Rotation() + 270) % 360)); break;
				case ID_ZOOM_50:
					ptrV->get_Canvas()->set_Zoom(0.5); break;
				case ID_ZOOM_100:
					ptrV->get_Canvas()->set_Zoom(1.); break;
				case ID_ZOOM_200:
					ptrV->get_Canvas()->set_Zoom(2.); break;
				case ID_ZOOM_500:
					ptrV->get_Canvas()->set_Zoom(5.); break;
				case ID_ZOOM_FITWIDTH:
					ptrV->get_Canvas()->set_Fit(Viewer::viewer_fit_page_width); break;
				case ID_ZOOM_FITHEIGHT:
					ptrV->get_Canvas()->set_Fit(Viewer::viewer_fit_page_height); break;
				case ID_ZOOM_FITPAGE:
					ptrV->get_Canvas()->set_Fit(Viewer::viewer_fit_page); break;
			}
		bHandled = TRUE;
		return 0;
	}
} // namespace Sample