#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"
#include <fstream>
#include <sstream>

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() {m_ptrDoc = nullptr;  if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	const SX::sx_double PAGE_WIDTH = 595;
	const SX::sx_double PAGE_HEIGHT = 842;

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				//Enable save and combine menu
				m_ptrDoc = ptrDoc.release();
			
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);

			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				
				//Disable save and combine menu
				m_ptrDoc = nullptr;
		}catch (SX::IException*) {  }	
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	void CPDFXpansionSDK::ShowDocument(SX::IBaseDocument* pBaseDoc)
	{
		IViewerPtr ptrV = m_pDefViewer->get_Viewer();
		ptrV->Relayout(ptrV->get_Canvas()->get_ActivePage());
	}

	void CPDFXpansionSDK::CombineDocuments(SX::IBaseDocument* pBaseDoc, SX::sx_flags flags)
	{
		try {
			SX::PDF::IPages* pPages = m_ptrDoc->get_Pages();
			m_ptrDoc->StartCombine(pBaseDoc,flags);
			//Adds the selected document to the end of the main
			m_ptrDoc->CombineAllPages(pPages->get_Count(),flags);
			m_ptrDoc->EndCombine();
			ShowDocument(m_ptrDoc);

		} catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	inline SX::sx_bool CPDFXpansionSDK::IsPDF()
	{
		return m_ptrDoc != nullptr;
	}

	inline SX::sx_bool CPDFXpansionSDK::IsModified()
	{
		if (m_ptrDoc == nullptr)
			return false;

		return m_ptrDoc->get_Properties()->get_Modified();
	}

	void CPDFXpansionSDK::SplitPDF()
	{
		SX::PDF::IPages* pPages = m_ptrDoc->get_Pages();
		SX::sx_uint pageCount = pPages->get_Count();

		if (pageCount == 0)
			return;

		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
		{
			SX::sx_flags combineFlags =  SX::combine_named_dests | SX::combine_outlines | SX::combine_form_fields | SX::combine_layers | SX::combine_links;
			try 
			{
				//split document by pages
				for (SX::sx_uint index = 0; index<pageCount; index++)
				{
					IPDFDocPtr ptrNewDoc = m_pApp->get_Factory()->CreatePDFDocument();
					ptrNewDoc->StartCombine(m_ptrDoc,combineFlags);
					ptrNewDoc->CombinePage(index,0,combineFlags);
					ptrNewDoc->EndCombine();
					CString sFileName; 
					sFileName.Format(_T("%s%d%s"),sPath,index+1,_T(".pdf"));
					ptrNewDoc->SaveAsFile(sFileName);
				}
			}
			catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
		}
	}

	void CPDFXpansionSDK::ExportXPS()
	{
		try
		{
			// create new XPS document
			IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();
			SX::XPS::IDocuments* pDocuments = ptrPack->get_Documents();
			SX::ObjPtr<SX::XPS::IDocument> ptrXpsDoc = pDocuments->Add();

			// combine XPS with current PDF
			SX::sx_flags combineFlags = SX::combine_named_dests | SX::combine_outlines | SX::combine_links | SX::combine_emb_unk_fonts;
			ptrXpsDoc->StartCombine(m_ptrDoc, combineFlags);
			ptrXpsDoc->CombineAllPages(0,combineFlags);
			ptrXpsDoc->EndCombine();

			//save XPS package
			CString sPath;
			if (m_pUI->SaveFileDialog(L"XPS Files *.xps\0*.xps\0\0", sPath))
			{
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".xps"))
					sPath.Append(_T(".xps"));

				ptrPack->SaveAsFile(sPath);
			}
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	void CPDFXpansionSDK::CombineWithPDF()
	{
		//Open PDF document
		CString sPath = L"";
		IPDFDocPtr ptrDoc = nullptr;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;
		}
		catch (SX::IException*) 
		{
			m_pUI->MessageBox(L"File not open", MB_OK | MB_ICONSTOP);
			return;
		}

		SX::sx_flags combineFlags =  SX::combine_named_dests | SX::combine_outlines | SX::combine_form_fields | SX::combine_layers | SX::combine_links;
		CombineDocuments(ptrDoc, combineFlags);
	}

	void CPDFXpansionSDK::CombineWithXPS()
	{
		//Open XPS document
		CString sPath;
		IXPSPackPtr ptrPack;
		if(!m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			return;

		SX::ObjPtr<SX::XPS::IDocument> ptrXPSDoc = nullptr;
		try {
			ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
			ptrPack->OpenFile(sPath);

			SX::XPS::IDocuments* pXPSDocs = ptrPack->get_Documents();
			ptrXPSDoc = pXPSDocs->Item(0);

		}catch (SX::IException* e) 
		{
			ShowErrorMessage(e->get_ErrorCode());
			return;
		}

		SX::sx_flags combineFlags = SX::combine_named_dests | SX::combine_outlines | SX::combine_links | SX::combine_emb_unk_fonts;
		CombineDocuments(ptrXPSDoc, combineFlags);
	}

	void CPDFXpansionSDK::ImportSVG()
	{
		//Open SVG
		CString sPath;
		if(!m_pUI->OpenFileDialog(L"SVG Files *.svg\0*.svg\0\0", sPath))
			return;
		try {
	
			SX::ObjPtr<SX::ISequentialStream> ptrSVGStream = m_pApp->get_Factory()->CreateReadStream(sPath);
			//Add SVG to resources
			SX::PDF::IDocResources* pDocResources = m_ptrDoc->get_Resources();			
			SX::ObjPtr<SX::PDF::ICompositeImage> pSVGImage = pDocResources->AddSvgFragment(ptrSVGStream, 72, 595.3, 841.9,  0, &CLoadProvider(sPath, m_pApp));

			if (pSVGImage != nullptr)
			{
				//Create new page in main document
				IPDFPagePtr ptrSVGPage = m_ptrDoc->get_Pages()->Add();
				//Add SVG image to page
				SX::sx_matrix matr;
				m_pApp->get_Factory()->Matrix()->Identity(matr);
				ptrSVGPage->InsertImage(pSVGImage.get(),matr);

				ShowDocument(m_ptrDoc);
			}
		}catch (SX::IException* e) {

			ShowErrorMessage(e->get_ErrorCode());
			return;
		}
	}

	void CPDFXpansionSDK::ImportEMF()
	{
		//Open emf document
		CString sPath;
		SX::ObjPtr<SX::ISequentialStream> ptrFileStream;
		if(!m_pUI->OpenFileDialog(L"EMF Files *.emf\0*.emf\0", sPath))
			return;

		try 
		{
			ptrFileStream = m_pApp->get_Factory()->CreateReadStream(sPath);
			SX::PDF::IDocResources* pDocResources = m_ptrDoc->get_Resources();
			pDocResources->set_ShareNewImages(true);
			SX::ObjPtr<SX::PDF::ICompositeImage> pEMFImage = pDocResources->AddGdiMetafile(ptrFileStream);

			if (pEMFImage != nullptr)
			{
				IPDFPagePtr ptrImagePage = m_ptrDoc->get_Pages()->Add();

				//Add emf document to page
				SX::sx_matrix matr;
				m_pApp->get_Factory()->Matrix()->Identity(matr);
				ptrImagePage->InsertImage(pEMFImage.get(),matr);

				ShowDocument(m_ptrDoc);
			}
		}
		catch(SX::IException* e)
		{
			ShowErrorMessage(e->get_ErrorCode());
		}
	}

	void CPDFXpansionSDK::ImportTXT()
	{
		//Open text document
		CString sPath;
		ITextDocumentPtr ptrTextDoc;
		if(m_pUI->OpenFileDialog(L"Text Files *.txt\0*.txt\0\0", sPath))
			try {
				ptrTextDoc = m_pApp->get_Factory()->CreateTextDocument();	

				//set text document params 
				SX::page_props propsTextPageProperties;
				propsTextPageProperties.height = PAGE_HEIGHT;					//in points
				propsTextPageProperties.width = PAGE_WIDTH;						//in points
				propsTextPageProperties.right = propsTextPageProperties.left = 5;	//in points
				propsTextPageProperties.top = propsTextPageProperties.bottom = 15;	//in points

				//Read document from stream
				SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateReadStream(sPath);
				ptrTextDoc->SetTextColor(SX::sx_color_i_rgb(0,0,0));
				ptrTextDoc->SetTextFont(L"Times New Roman");
				ptrTextDoc->SetText(ptrStream ,propsTextPageProperties);


		}catch (SX::IException*) 
		{
			m_pUI->MessageBox(L"File not open", MB_OK | MB_ICONSTOP);
			return;
		}

		try
		{
			m_ptrDoc->StartCombine(ptrTextDoc);
			m_ptrDoc->CombineAllPages(m_ptrDoc->get_Pages()->get_Count());
			m_ptrDoc->EndCombine();

			ShowDocument(m_ptrDoc);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	void CPDFXpansionSDK::ImportIMG()
	{
		//Open image
		CString sPath;
		SX::ObjPtr<SX::IImageDocument> ptrImagePage;		
		if(!m_pUI->OpenFileDialog(L"PNG Files *.png\0*.png\0JPG Files *.JPG\0*.jpg\0\0", sPath))
			return;

		try {
			ptrImagePage = m_pApp->get_Factory()->CreateImageDocument();	
			SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateReadStream(sPath);
			SX::ObjPtr<SX::Graphics::IBitmapData> ptrBmp =  m_pApp->get_Factory()->CreateBitmapFromStream(ptrStream);
			SX::PDF::IDocResources* pDocResources = m_ptrDoc->get_Resources();
			SX::ObjPtr<SX::PDF::IRasterImage> ptrRasterImage = pDocResources->AddRasterImage(ptrBmp, SX::PDF::Filter_JPEG, 80);
			
			IPDFPagePtr ptrImagePage = m_ptrDoc->get_Pages()->Add();
			SX::sx_matrix matr;
			m_pApp->get_Factory()->Matrix()->Identity(matr);
			ptrImagePage->InsertImage(ptrRasterImage.get(),matr);

			ShowDocument(m_ptrDoc);
		}catch (SX::IException* e) 
		{
			ShowErrorMessage(e->get_ErrorCode());
		}
	}

	void CPDFXpansionSDK::SavePDF()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));

				m_ptrDoc->SaveAsFile(sPath);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	//////////////////////////////////////////////////////////////////////////
	CLoadProvider::CLoadProvider(CString rootObjPath, IApplication *pApp):m_sDir(rootObjPath), m_pApp(pApp)
	{
		PathRemoveFileSpec(m_sDir.GetBuffer());
		m_sDir.ReleaseBuffer();
	}

	//////////////////////////////////////////////////////////////////////////
	SX::ISequentialStream* CLoadProvider::GetExtResource(sx_str sURL)
	{
		CString path;
		if (PathIsRelative(sURL))
		{
			path = m_sDir;
			PathAppend(path.GetBuffer(MAX_PATH),sURL);
		}
		else
			path = sURL;

		try{
			return m_pApp->get_Factory()->CreateReadStream(path);
		}catch(SX::IException *){
			return nullptr;
		}
	}


} // namespace Sample