#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"
#include <atlpath.h>
#include "GDI2PDF.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { m_ptrDoc = nullptr; if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	CString CPDFXpansionSDK::GetFilePath(const CString& sFileName)
	{
		CPath path;
		LPWSTR str = path.m_strPath.GetBuffer(MAX_PATH);
		GetModuleFileName(NULL, str, MAX_PATH);
		path.RemoveFileSpec();
		path.Append(L"..");
		path.Append(sFileName);
		return path.m_strPath;
	}

	SX::PDF::IPage* CPDFXpansionSDK::CreatePage(SX::PDF::IDocument* pDoc, sx_double w, sx_double h)
	{
		SX::ObjPtr<SX::PDF::IPage> ptrPage = pDoc->get_Pages()->Add();
		SX::sx_rect pageRect; pageRect.x = pageRect.y = 0; pageRect.width = w; pageRect.height = h;	
		ptrPage->SetMediaBox(pageRect);
		return ptrPage.release();
	}

	//////////////////////////////////////////////////////////////////////////
	void CPDFXpansionSDK::CreatePDF()
	{
		// crete new PDF document
		m_ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();
		// add new empty page
		SX::ObjPtr<SX::PDF::IPage> ptrPage = CreatePage(m_ptrDoc.get(), 595.3, 842.); // A4 size in points
		// layout content
		SX::ObjPtr<SX::Content::IRichContent> ptrRichContent = ptrPage->GetRichContent();
		InitTextStyles(ptrRichContent);
		FillPage(ptrRichContent.get());
		// load doc in viewer
		IVDocPtr ptrVDoc = m_ptrDoc->GetViewableDocument();
		IViewerPtr ptrV = m_pDefViewer->get_Viewer();
		ptrV->set_Document(ptrVDoc);
	}

	void CPDFXpansionSDK::SavePDF()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));

				m_ptrDoc->SaveAsFile(sPath);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	//////////////////////////////////////////////////////////////////////////
	void CPDFXpansionSDK::CreateEMF()
	{
		m_ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();
		SX::sx_matrix matr;
		SX::Graphics::IMatrix* pMatrix = m_pApp->get_Factory()->Matrix();
		pMatrix->Identity(matr);
		CGDI2PDF gdi2pdf(m_pApp->get_Factory());
		for (sx_uint ind = 0; ind < 3; ind ++)
		{
			SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Add();
			SX::ObjPtr<SX::ISequentialStream> ptrStream = gdi2pdf.DrawPage(ind);
			SX::ObjPtr<SX::PDF::ICompositeImage> emfImage = m_ptrDoc->get_Resources()->AddGdiMetafile(ptrStream, 0, SX::Graphics::GDI_opt_import_plus);
			SX::ObjPtr<SX::PDF::IPage> pCurrPage = m_ptrDoc->get_Pages()->Item(ind);
			pCurrPage->InsertImage(emfImage, matr, false);
		}
		// load doc in viewer
		IVDocPtr ptrVDoc = m_ptrDoc->GetViewableDocument();
		IViewerPtr ptrV = m_pDefViewer->get_Viewer();
		ptrV->set_Document(ptrVDoc);
	}

	//////////////////////////////////////////////////////////////////////////
	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	SX::sx_bool CPDFXpansionSDK::IsPDF()
	{
		return m_ptrDoc != nullptr; 
	}

	//////////////////////////////////////////////////////////////////////////
	// Layout content
	//////////////////////////////////////////////////////////////////////////
	const CString HEADER_1 = " for application development that has the unprecedented potential to enable developers to build rich interactive experiences, powered by vast data stores, that are available on any device. Although the boundaries of the platform continue to evolve, industry leaders speak nearly in unison about how HTML5 will be the cornerstone for this platform. But the full strength of the platform relies on many more technologies that W3C and its partners are creating, including CSS, SVG, WOFF, the Semantic Web stack, XML, and a variety of APIs.";
	const CString HEADER_2 = "W3C develops these technical specifications and guidelines through a process designed to maximize consensus about the content of a technical report, to ensure high technical and editorial quality, and to earn endorsement by W3C and the broader community.";
	const CString BODIES[] = 
	{	
		"Web Design and Applications involve the standards for building and Rendering Web pages, including HTML, CSS, SVG, Ajax, and other technologies for Web Applications (WebApps). This section also includes information on how to make pages accessible to people with disabilities (WCAG), to internationalize them, and make them work on mobile devices.",
		"W3C is focusing on technologies to enable Web access anywhere, anytime, using any device. This includes Web access from mobile phones and other mobile devices as well as use of Web technology in consumer electronics, printers, interactive television, and even automobiles.",
		"Web Architecture focuses on the foundation technologies and principles which sustain the Web, including URIs and HTTP.",
		"In addition to the classic Web of documents W3C is helping to build a technology stack to support a Web of data, the sort of data you find in databases. The ultimate goal of the Web of data is to enable computers to do more useful work and to develop systems that can support trusted interactions over the network. The term Semantic Web refers to W3Cs vision of the Web of linked data. Semantic Web technologies enable people to create data stores on the Web, build vocabularies, and write rules for handling data. Linked data are empowered by technologies such as RDF, SPARQL, OWL, and SKOS.",
		"XML Technologies including XML, XML Namespaces, XML Schema, XSLT, Efficient XML Interchange (EXI), and other related standards."
	};
	const CString HEADERS[] =
	{	
		"Web Design and Applications",
		"Web of Devices",
		"Web Architecture",
		"Semantic Web",
		"XML Technology"
	};
	const CString IMAGE_FILES[] =
	{
		"palette.png",
		"phone.png",
		"hammer.png",
		"box.png",
		"xml.png"
	};

	void CPDFXpansionSDK::InitTextStyles(SX::Content::IRichContent* pRichContent)
	{
		// create text styles
		m_pBlackNormal10 = pRichContent->CreateTextStyle();
		m_pBlackNormal10->SetFont(SX::sx_color_i_rgb(0,0,0), L"Arial", 10, 400, false);	 //Font weight: 400 - normal, 700 - bold
		m_pBlackNormal10->set_Leading(15);

		m_pBlackBold10 = pRichContent->CreateTextStyle();
		m_pBlackBold10->SetFont(SX::sx_color_i_rgb(0,0,0), L"Arial", 10, 700, false);	 //Font weight: 400 - normal, 700 - bold
		m_pBlackBold10->set_Leading(15);

		m_pBlueNormal10 = pRichContent->CreateTextStyle();
		m_pBlueNormal10->SetFont(SX::sx_color_i_rgb(0,51,102), L"Arial", 10, 400, false);	 //Font weight: 400 - normal, 700 - bold
		m_pBlueNormal10->set_Leading(15);

		m_pBlackNormal22 = pRichContent->CreateTextStyle();
		m_pBlackNormal22->SetFont(SX::sx_color_i_rgb(0,0,0), L"Arial", 22, 400, false);	 //Font weight: 400 - normal, 700 - bold
		m_pBlackNormal22->set_Leading(15);

		m_pWhiteNormal22 = pRichContent->CreateTextStyle();
		m_pWhiteNormal22->SetFont(SX::sx_color_i_rgb(255,255,255), L"Arial", 22, 400, false);	 //Font weight: 400 - normal, 700 - bold
		m_pWhiteNormal22->set_Leading(15);

		m_pBlueNormal17 = pRichContent->CreateTextStyle();
		m_pBlueNormal17->SetFont(SX::sx_color_i_rgb(0,51,102), L"Arial", 17.5, 400, false);	 //Font weight: 400 - normal, 700 - bold
		m_pBlueNormal17->set_Leading(15);
	}

	void CPDFXpansionSDK::FillPage(SX::Content::IRichContent* pRichContent)
	{
		SX::Graphics::IMatrix* pMatrix = m_pApp->get_Factory()->Matrix();
		SX::Content::IRichCollection* objects = pRichContent->get_Objects();

		// add page title
		SX::sx_rect rect; rect.x = 48; rect.y = 27; rect.width = 495; rect.height = 50; // in points
		SX::Content::IPath* pPath = pRichContent->CreatePath();
		pPath->AddBox(rect, true, true);
		SX::Content::IRichVector* pTitleBack = static_cast<SX::Content::IRichVector*>(objects->Insert(objects->get_Count(), SX::Content::rich_type_vector));
		pTitleBack->set_Path(pPath);	
		pTitleBack->set_Pen(pRichContent->CreatePen(1));
		pTitleBack->set_StrokeBrush(pRichContent->CreateSolidBrush(SX::sx_color_i_rgb(174,194,211)));
		pTitleBack->set_FillBrush(pRichContent->CreateSolidBrush(SX::sx_color_i_rgb(219,231,240)));
		rect.x +=20; rect.y = rect.y + 15; // text offset
		AddText(pRichContent, rect, "STANDARDS", m_pWhiteNormal22);  // text shadow
		rect.x-=1; rect.y-=1;
		AddText(pRichContent, rect, "STANDARDS", m_pBlackNormal22); // text

		// main text
		rect.x = rect.x + 20; rect.y = rect.y + rect.height + 20; rect.width = 475;
		SX::Content::IRichText* pTxtBody = AddText(pRichContent,rect,L"W3C standards define an ",m_pBlackNormal10);
		pTxtBody = AddText(pRichContent,rect,L"Open Web Platform",m_pBlackBold10,pTxtBody);
		pTxtBody = AddText(pRichContent,rect,HEADER_1,m_pBlackNormal10,pTxtBody);
		SX::sx_rect trect; SX::sx_matrix mx;
		pTxtBody->GetTransformation(mx);
		pTxtBody->GetBBox(mx,trect);
		rect.y=trect.y + trect.height + 20;
		pTxtBody = AddText(pRichContent,rect,HEADER_2, m_pBlackNormal10);
		// blocks
		pTxtBody->GetTransformation(mx);
		pTxtBody->GetBBox(mx,trect);
		rect.y = trect.height + trect.y + 32;
		for (int blockIndex=0; blockIndex<5; blockIndex++)
		{
			rect = AddBlock(pRichContent,rect,GetFilePath(IMAGE_FILES[blockIndex]), HEADERS[blockIndex],BODIES[blockIndex]);
			rect.y+= rect.height + 20;
		}
		// footer
		SX::sx_rect pageRect;
		pRichContent->GetMatrix(mx);
		pRichContent->GetBBox(mx,pageRect);
		rect.height = rect.x = rect.y = 0;
		rect.width = pageRect.width;
		SX::Content::IRichText* pTxtFooter = AddText(pRichContent,rect,L"Copyright  2015 W3C  ", m_pBlackNormal10);
		AddText(pRichContent,rect,L"( MIT , ERCIM , Keio, Beihang) Usage policies apply.",m_pBlueNormal10, pTxtFooter);
		pMatrix->Identity(mx);
		pTxtFooter->GetBBox(mx,trect);
		pMatrix->Translate(mx,(pageRect.width - trect.width)/2, pageRect.height-20);
		pTxtFooter->ApplyTransformation(mx);
	}

	SX::Content::IRichText* CPDFXpansionSDK::AddText(SX::Content::IRichContent* rc, const SX::sx_rect& rect, const CString& text, SX::Content::ITextStyle* pTextStyle, SX::Content::IRichText* richText)
	{
		SX::Content::IRichCollection* pRichCollection = rc->get_Objects();

		//add text
		SX::Content::IRichText* pText = richText;
		if (richText == nullptr)
			pText = static_cast<SX::Content::IRichText*>(pRichCollection->Insert(pRichCollection->get_Count(), SX::Content::rich_type_text));

		pText->InsertChars(pText->get_Length(),text, pTextStyle);
		pText->Relayout(rect.width, SX::Content::text_layout_wrap);

		if (richText != nullptr)
			return pText;

		// transform text area to PDF space
		SX::Graphics::IMatrix* pMatr = m_pApp->get_Factory()->Matrix();
		SX::sx_matrix textMatr;
		SX::sx_rect textRect;

		pMatr->Identity(textMatr);
		pText->GetBBox(textMatr,textRect);

		pMatr->Translate(textMatr, rect.x  , rect.y+ textRect.height+textRect.y);
		pText->ApplyTransformation(textMatr);

		return pText;
	}

	SX::Content::IRichObject* CPDFXpansionSDK::AddImage(SX::Content::IRichContent* rc, const CString& imagePath, const SX::sx_rect& rect)
	{
		SX::Content::IRichCollection* pRichCollection = rc->get_Objects();
		SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateReadStream(imagePath);
		SX::ObjPtr<SX::Graphics::IBitmapData> ptrBmp =  m_pApp->get_Factory()->CreateBitmapFromStream(ptrStream);
		ptrBmp->set_DPI(72);
		SX::Content::IRichRaster* pRastr = static_cast<SX::Content::IRichRaster*>(pRichCollection->Insert(pRichCollection->get_Count(), SX::Content::rich_type_raster));
		pRastr->set_Bitmap(ptrBmp);
		SX::Graphics::IMatrix* pMatr = m_pApp->get_Factory()->Matrix();
		SX::sx_matrix matr;
		SX::sx_rect imgRect;
		pMatr->Identity(matr);
		pRastr->GetBBox(matr,imgRect);
		pMatr->Translate(matr,rect.x,rect.y);
		pRastr->ApplyTransformation(matr);
		return pRastr;
	}

	SX::sx_rect& CPDFXpansionSDK::AddBlock(SX::Content::IRichContent* rc, SX::sx_rect& rect, const CString& sImagePath, const CString& sHeader, const CString& sBody)
	{
		SX::sx_rect resRect;
		resRect.x = rect.x;
		resRect.y = rect.y;
		resRect.width = rect.width;

		// main image
		AddImage(rc,sImagePath,rect);
		// header
		rect.x+=60;
		SX::Content::IRichText* pText = AddText(rc,rect,sHeader,m_pBlueNormal17);
		// body text
		SX::sx_matrix mx;
		pText->GetMatrix(mx);
		SX::sx_rect mrect;
		pText->GetBBox(mx,mrect);
		mrect.width = rect.width - 64;
		mrect.y = rect.y + mrect.height + 4;
		pText = AddText(rc,mrect,sBody,m_pBlackNormal10);

		pText->GetTransformation(mx);
		pText->GetBBox(mx,mrect);
		resRect.height = mrect.y + mrect.height - resRect.y;


		return resRect;
	}

} // namespace Sample