﻿#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDFInvoice()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
		{
			SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateReadStream(sPath);
			SX::ObjPtr<SX::EInvoice::IInvoiceDocument> ptrPDFInvoice = m_pApp->get_Factory()->CreateEInvoiceDocument();
			SX::EInvoice::einv_status invStatus = ptrPDFInvoice->Open(ptrStream, SX::EInvoice::einv_options_check_xml);
			if (invStatus == SX::EInvoice::einv_status_valid)
			{
				// optionally display PDF in viewer
				IVDocPtr ptrVDoc = ptrPDFInvoice->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer(); ptrV->set_Document(ptrVDoc);
				// extract XML invoice to file
				switch(ptrPDFInvoice->get_Standard())
				{
					case SX::EInvoice::einv_standard_zugferd	: m_pUI->MessageBox(L"Invoice is conform to ZUGFeRD 1.0", MB_OK); break;
					case SX::EInvoice::einv_standard_zugferd20	: m_pUI->MessageBox(L"Invoice is conform to ZUGFeRD 2.0", MB_OK); break;
					case SX::EInvoice::einv_standard_facturx	: m_pUI->MessageBox(L"Invoice is conform to FACTUR-X", MB_OK); break;
				}
				if(m_pUI->SaveFileDialog(L"XML Files *.xml\0*.xml\0\0", sPath))
				{
					int len = sPath.GetLength(); if(sPath.Right(4) != _T(".xml")) sPath.Append(_T(".xml"));
					SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateWriteStream(sPath);
					ptrPDFInvoice->GetInvoice(ptrStream);
				}
			}
			else switch(invStatus)
			{
				case SX::EInvoice::einv_status_not_pdf		: m_pUI->MessageBox(L"It should be valid PDF file", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_enc_pdf		: m_pUI->MessageBox(L"It should be non-encrypted PDF", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_not_pdfa		: m_pUI->MessageBox(L"It is non-conform to PDF/A-3", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_no_meta		: m_pUI->MessageBox(L"No metadata was found in this file", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_err_meta		: m_pUI->MessageBox(L"Metadata is non-conform to PDF/A-3", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_no_emb		: m_pUI->MessageBox(L"No embedded invoice was found in this file", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_err_emb		: m_pUI->MessageBox(L"Invoice is not embedded correctly", MB_OK | MB_ICONSTOP); break;
				case SX::EInvoice::einv_status_nonconform	: m_pUI->MessageBox(L"Invoice XML is non-conform or declares other profile", MB_OK | MB_ICONSTOP); break;
			}
		}
	}

	//////////////////////////////////////////////////////////////////////////
	void CPDFXpansionSDK::CreatePDFInvoice()
	{
		CString sXMLPath;
		if(m_pUI->OpenFileDialog(L"XML Files *.xml\0*.xml\0\0", sXMLPath))
		{
			CString sPDFPath;
			if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPDFPath))
			{
				try
				{
					SX::ObjPtr<SX::EInvoice::IInvoiceDocument> ptrInvoiceDoc = m_pApp->get_Factory()->CreateEInvoiceDocument();
					SX::ObjPtr<SX::ISequentialStream> ptrPDFStream = m_pApp->get_Factory()->CreateReadStream(sPDFPath);
					SX::ObjPtr<SX::ISequentialStream> ptrXMLStream = m_pApp->get_Factory()->CreateReadStream(sXMLPath);
					ptrInvoiceDoc->CreateFromPDF(ptrXMLStream, ptrPDFStream, SX::EInvoice::einv_options_zugferd20); // use 0 for ZUGFeRD 1.0 and einv_options_facturx for FacturX
					CString sPath;
					if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
					{
						int len = sPath.GetLength(); if(sPath.Right(4) != _T(".pdf")) sPath.Append(_T(".pdf"));
						SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateWriteStream(sPath);
						ptrInvoiceDoc->Save(ptrStream);
					}
				}
				catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
			}
		}
	}

	//////////////////////////////////////////////////////////////////////////
	void CPDFXpansionSDK::BuildPDFInvoice()
	{
		CString sXMLPath;
		if(m_pUI->OpenFileDialog(L"XML Files *.xml\0*.xml\0\0", sXMLPath))
		{
			try
			{
				SX::ObjPtr<SX::EInvoice::IInvoiceDocument> ptrInvoiceDoc = m_pApp->get_Factory()->CreateEInvoiceDocument();
				SX::ObjPtr<SX::ISequentialStream> ptrXMLStream = m_pApp->get_Factory()->CreateReadStream(sXMLPath);
				ptrInvoiceDoc->Create(ptrXMLStream, SX::EInvoice::einv_options_zugferd20); // use 0 for ZUGFeRD 1.0 and einv_options_facturx for FacturX
				BuildPage(ptrInvoiceDoc.get());
				CString sPath = L"inv.pdf";
				if(m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
				{
					int len = sPath.GetLength(); if (sPath.Right(4) != _T(".pdf")) sPath.Append(_T(".pdf"));
					SX::ObjPtr<SX::ISequentialStream> ptrStream = m_pApp->get_Factory()->CreateWriteStream(sPath);
					ptrInvoiceDoc->Save(ptrStream);
				}
			}
			catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
		}
	}

	void CPDFXpansionSDK::BuildPage(SX::EInvoice::IInvoiceDocument* pPDFInv)
	{
		SX::ObjPtr<SX::EInvoice::IPage> ptrPage = pPDFInv->CreatePage(595.3, 841.9);
		SX::ObjPtr<SX::ISequentialStream> ptrFile = m_pApp->get_Factory()->CreateReadStream(L"logo.png");
		if(ptrFile.get())
		{
			SX::ObjPtr<SX::EInvoice::IRasterImage> ptrLogo = ptrPage->LoadImage(ptrFile);
			if(ptrLogo.get()) ptrPage->PlaceImage(515, 30, 50, 50, ptrLogo);
		}
		SX::ObjPtr<SX::EInvoice::ILineStyle> ptrNL = ptrPage->CreateLineStyle(0.5);
		SX::ObjPtr<SX::EInvoice::ILineStyle> ptrBL = ptrPage->CreateLineStyle(1.5);
		SX::ObjPtr<SX::EInvoice::ITextStyle> ptrST = ptrPage->CreateTextStyle(L"Calibri", 8);
		SX::ObjPtr<SX::EInvoice::ITextStyle> ptrTT = ptrPage->CreateTextStyle(L"Calibri", 8, 700);
		SX::ObjPtr<SX::EInvoice::ITextStyle> ptrNT = ptrPage->CreateTextStyle(L"Calibri", 12);
		SX::ObjPtr<SX::EInvoice::ITextStyle> ptrBT = ptrPage->CreateTextStyle(L"Calibri", 12, 700);

		ptrPage->PlaceText(50, 70, L"PDF Xpansion GmbH · Königsallee 45 · D-44789 Bochum", ptrST);
		ptrPage->PlaceLine(50, 82, 565, 82, ptrNL);

		ptrPage->PlaceText(50, 120, L"Fachhochschule Dortmund\nSonnenstraße 96\n44139 Dortmund", ptrBT);

		ptrPage->PlaceText(50, 190, L"Rechnung Nr. 1234567", ptrBT);

		ptrPage->PlaceLine(405, 185, 565, 185, ptrBL); ptrPage->PlaceLine(565, 185, 565, 240, ptrBL);
		ptrPage->PlaceLine(565, 240, 405, 240, ptrBL); ptrPage->PlaceLine(405, 240, 405, 185, ptrBL);
		ptrPage->PlaceText(410, 190, L"Datum", ptrBT); ptrPage->PlaceText(560, 190, L"15.04.2019", ptrNT, 2);
		ptrPage->PlaceText(410, 205, L"Vertrag", ptrBT); ptrPage->PlaceText(560, 205, L"V12-2345-678", ptrNT, 2);
		ptrPage->PlaceText(410, 220, L"Lieferung", ptrBT); ptrPage->PlaceText(560, 220, L"April 2019", ptrNT, 2);

		ptrPage->PlaceText(50, 270, L"Wir bedanken uns für Ihre Treue und erlauben uns, Ihnen folgenden Betrag in Rechnung zu stellen:", ptrNT);

		ptrPage->PlaceLine(50, 305, 565, 305, ptrBL);
		ptrPage->PlaceText(50, 305, L"Nr.    Produkt, Leistung", ptrBT);
		ptrPage->PlaceText(330, 305, L"Einzelpreis, €                  Menge             Summe, €", ptrBT);
		ptrPage->PlaceLine(50, 322, 565, 322, ptrBL);
		ptrPage->PlaceText(50, 335, L"         PDF Xpansion SDK", ptrBT);
		ptrPage->PlaceText(50, 350, L"1       PDF-Datei lesen", ptrNT);
		ptrPage->PlaceText(330, 350, L"1.000                                 1                               1.000", ptrNT);
		ptrPage->PlaceText(50, 365, L"2       PDF-Datei speichern", ptrNT);
		ptrPage->PlaceText(330, 365, L"1.000                                 1                               1.000", ptrNT);
		ptrPage->PlaceText(50, 380, L"3       PDF-Dokumente trennen / zusammenfügen", ptrNT);
		ptrPage->PlaceText(330, 380, L"500                                    1                               500", ptrNT);
		ptrPage->PlaceLine(50, 405, 565, 405, ptrBL);
		ptrPage->PlaceText(50, 405, L"G e s a m t", ptrBT); ptrPage->PlaceText(565, 405, L"2.500", ptrBT, 2);
		ptrPage->PlaceLine(50, 422, 565, 422, ptrBL);

		ptrPage->PlaceText(565, 430, L"Nettobetrag                                  2.500 €", ptrNT, 2);
		ptrPage->PlaceText(565, 445, L"MwSt, 19%                                       475 €", ptrNT, 2);
		ptrPage->PlaceText(565, 460, L"Rechnungsbetrag                        2.975 €", ptrBT, 2);
		ptrPage->PlaceText(50, 490, L"Zahlungsziel: 14 Tage Netto ohne Abzug", ptrNT);

		ptrPage->PlaceLine(50, 750, 565, 750, ptrNL);
		ptrPage->PlaceText(50, 755, L"PDF Xpansion GmbH", ptrTT);
		ptrPage->PlaceText(250, 755, L"Bankverbindung", ptrTT);
		ptrPage->PlaceText(450, 755, L"Handelsregister", ptrTT);
		ptrPage->PlaceText(50, 770, L"Königsallee 45, D-44789 Bochum\nTel. +49 (0)234-123 45 67\nFax +49 (0)234-123 45 89\nE-Mail: info@pdf-xpansion.com\nhttp://www.pdf-xpansion.com", ptrST);
		ptrPage->PlaceText(250, 770, L"Deutsche Bank PGK AG, Bochum\nBLZ 430 700 24, Konto 1234567\nIBAN DE51 4307 0024 01234567 00\nUSt.-IdNr.: DE12345678\nSteuer Nr.: 123/4567/1234", ptrST);
		ptrPage->PlaceText(450, 770, L"HRA 1234 (AG Bochum)\nGeschäftsführer: Frank Müller", ptrST);
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
			case ErrorOutOfMemory: mess = L"not enough memory"; break;
			case ErrorOutOfSpace: mess = L"not enough disk space"; break;
			case ErrorOutOfRange: mess = L"index out of range"; break;
			case ErrorInvalidFormat: mess = L"invalid data format"; break;
			case ErrorInvalidUse: mess = L"invalid using"; break;
			case ErrorInvalidParam: mess = "invalid argument"; break;
			default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}
} // namespace Sample