
#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { m_ptrDoc = nullptr; if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	SX::sx_bool CPDFXpansionSDK::OnMessage(SX::PDF::IMessagePDFA* pMess)
	{
		if ((pMess->get_Type() & SX::PDF::pdfa_mess_cat) == SX::PDF::pdfa_mess_error)
		{
			sx_flags messType = pMess->get_Type() & SX::PDF::pdfa_mess_type;
			CString sErrMess; sErrMess.Format(L"Document cant converted, error %d", messType);
			m_pUI->MessageBox(sErrMess, 0);
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(0);
				
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				m_ptrDoc = ptrDoc.release();
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::ExportText()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"Text Files *.txt\0*.txt\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".txt"))
					sPath.Append(_T(".txt"));

				// get document pages
				SX::PDF::IPages* pPages = m_ptrDoc->get_Pages();
				SX::sx_uint pagesCount = pPages->get_Count();
				CString sDocText;
				// get text from pages
				for(SX::sx_uint pageIndex = 0; pageIndex < pagesCount; pageIndex++)
				{
					SX::ObjPtr<SX::PDF::IPage> page = pPages->Item(pageIndex);
					SX::ObjPtr<SX::IStr> ptrPageText = page->GetPlainText(SX::plain_text_simulate_spaces | SX::plain_text_simulate_lines | SX::plain_text_skip_overlapped_chars);
					sDocText.Append(ptrPageText->get_str());
				}
				
				// save text to file
				USES_CONVERSION;
				LPSTR lpANSI = W2A( sDocText );
				HANDLE hFile = CreateFile(sPath, GENERIC_READ | GENERIC_WRITE, 0, NULL,	CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
				DWORD dwBytesWritten = 0;
				WriteFile(hFile,lpANSI,sDocText.GetLength(),&dwBytesWritten,NULL);
				CloseHandle(hFile);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	void CPDFXpansionSDK::ExportPagesAsImages()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PNG Files *.png\0*.png\0\0", sPath))
		try
		{
				// get document pages
				SX::PDF::IPages* pPages = m_ptrDoc->get_Pages();
				SX::sx_uint pagesCount = pPages->get_Count();
				// save pages as images
				for (SX::sx_uint pageIndex = 0; pageIndex< pagesCount; pageIndex++)
				{
					// get current page
					SX::ObjPtr<SX::PDF::IPage> page = pPages->Item(pageIndex);
					// transform Bitmap to PDF space
					SX::sx_matrix pageMatr;
					page->CalcMatrix(pageMatr);
					SX::ObjPtr<SX::Graphics::IBitmapData> ptrBitmap = page->CopyToBitmap(&pageMatr);
					CString sFilePath; sFilePath.Format(_T("%s%d.png"),sPath,pageIndex+1);
					SX::ObjPtr<SX::ISequentialStream> pSaveStream = m_pApp->get_Factory()->CreateWriteStream(sFilePath);
					ptrBitmap->SaveToStream(pSaveStream, SX::Graphics::ImageFormat_PNG,100);
				}
				
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	bool CPDFXpansionSDK::CanExport()
	{
		return m_ptrDoc != nullptr;
	}

	void CPDFXpansionSDK::ExportPDFA()
	{
		SX::PDF::IDocument* pDocPDF = static_cast<SX::PDF::IDocument*>(m_ptrDoc.get());
		try 
		{
			m_ptrDoc->ConformToPDFA(SX::PDF::pdfa_3a, 0, this);
			CString sPath;
			if (m_pUI->SaveFileDialog(L"PDFA Files *.pdf\0*.pdf\0\0", sPath))
			{
				m_ptrDoc->SaveAsFile(sPath + L".pdf");
			}
		}
		catch (SX::IException * e) 
		{
			if (e->get_ErrorCode() != SX::ErrorNonConform)
				ShowErrorMessage(e->get_ErrorCode());	//for all other errors
		}
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				m_ptrDoc = nullptr;
		}catch (SX::IException*) {  }	
	}
} // namespace Sample