#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{
	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() 
	{ 
		m_ptrDoc = nullptr; 
		if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); 
	}

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);

				pConf->get_PDF()->set_EnableFormFilling(true);
				pConf->set_DrawOptions(pConf->get_DrawOptions() | SX::Graphics::draw_opt_highlight_fields | SX::Graphics::draw_opt_highlight_req_fields | SX::Graphics::draw_opt_empty_sign_icon | SX::Graphics::draw_opt_sign_status_icon);

				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	inline SX::sx_bool CPDFXpansionSDK::IsPDF()
	{
		return m_ptrDoc != nullptr;
	}

	inline SX::sx_bool CPDFXpansionSDK::IsModified()
	{
		if (m_ptrDoc == nullptr)
			return false;

		return m_ptrDoc->get_Properties()->get_Modified();
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;
				
				SX::PDF::ISignatures* pSignature = ptrDoc->get_Properties()->GetSignatures();
				CheckSignatures(pSignature);

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				m_ptrDoc = ptrDoc.release();
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::CheckSignatures(SX::PDF::ISignatures* pSignatures)
	{
		SX::ObjPtr<SX::PDF::IFormField> pFormField = pSignatures->FirstSignature();
			while(pFormField.get() != nullptr)
			{
				ShowSignatureInfo(pFormField);
				pFormField = pSignatures->NextSignature(pFormField);
			}
	}

	void CPDFXpansionSDK::ShowSignatureInfo(SX::PDF::IFormField* pFormField)
	{
		CString msg;
		SX::ObjPtr<SX::PDF::ISignature> ptrSignature = (static_cast<SX::PDF::IFieldSignature*>(pFormField))->GetSignature();
		if (ptrSignature)
		{
			SX::ObjPtr<SX::IStr> pStr = ptrSignature->get_Signer();
			if (pStr != NULL)
			{
				msg.AppendFormat(L"%s \nStatus: ", pStr->get_str());
				switch(ptrSignature->get_Status())
				{
					case SX::PDF::signature_status_invalid: msg.Append(L"invalid"); break;
					case SX::PDF::signature_status_unknown: msg.Append(L"unknown"); break;
					case SX::PDF::signature_status_valid: msg.Append(L"valid"); break;
					case SX::PDF::signature_status_valid_for_revision: msg.Append(L"valid, but document was updated"); break;
				}
				m_pUI->MessageBox(msg,0);
			}
		}
	}

	void CPDFXpansionSDK::TimeStructCopy(SYSTEMTIME &source, SX::sx_time &destination)
	{
		destination.wDay = source.wDay;
		destination.wDayOfWeek = source.wDayOfWeek;
		destination.wHour = source.wHour;
		destination.wMilliseconds = source.wMilliseconds;
		destination.wMinute = source.wMinute;
		destination.wMonth = source.wMonth;
		destination.wSecond = source.wSecond;
		destination.wYear = source.wYear;
	}

	void CPDFXpansionSDK::Sign()
	{
		SX::PDF::IPages* pPages = m_ptrDoc->get_Pages();
		if (pPages->get_Count() == 0)
			return;

		IPDFPagePtr ptrLastPage = pPages->Item(pPages->get_Count()-1);

		try
		{
			//Create new Certificate
			SX::ObjPtr<SX::ICryptoCert> ptrCryptoCert = m_pApp->get_Factory()->CreateCryptoCert();
			SX::sx_time tFrom, tTo;
			SYSTEMTIME sysTime = {0};
			GetSystemTime(&sysTime);
			TimeStructCopy(sysTime, tFrom);
			TimeStructCopy(sysTime, tTo);
			tTo.wYear++;

			ptrCryptoCert->Create(L"CN=PDF Xpansion Sample,E=pdf@soft-xpansion.com", tFrom, tTo, SX::sx_cert_ku_dig_sign);

			//add new signature field
			SX::PDF::ISignatures* pSignatures = m_ptrDoc->get_Properties()->GetSignatures();
			SX::ObjPtr<SX::PDF::IFormField> ptrNewFormFieldSignature = pSignatures->AddSignature(L"New signature");

			//Create widget for signature
			SX::PDF::IAnnots* pAnnots = ptrLastPage->get_Annots();
			SX::ObjPtr<SX::PDF::IWidget> ptrWidget = pAnnots->InsertNewWidget(ptrNewFormFieldSignature);
			
			SX::sx_rect rc;
			ptrLastPage->GetMediaBox(rc);
			rc.x += rc.width * 1/2;		//right half
			rc.y += rc.height / 4;		//bottom quarter
			rc.width = rc.height = 250; //will be replaced by image size
			ptrWidget->SetRectArea(rc);

			//Get image path
			CPath path;
			LPWSTR str = path.m_strPath.GetBuffer(MAX_PATH);
			GetModuleFileName(NULL, str, MAX_PATH);
			path.RemoveFileSpec();
			path.Append(L"..\\sign.png");

			//Set widget image
			SX::PDF::IDocResources* pDocResources = m_ptrDoc->get_Resources();
			SX::ObjPtr<SX::Graphics::IBitmapData> ptrBmp =  m_pApp->get_Factory()->CreateBitmapFromStream(m_pApp->get_Factory()->CreateReadStream(path.m_strPath));
			SX::ObjPtr<SX::PDF::IRasterImage> ptrRasterImage = pDocResources->AddRasterImage(ptrBmp, SX::PDF::Filter_Flate);
			SX::ObjPtr<SX::PDF::ICompositeImage> ptrCompositImage = pDocResources->AddCompositeImage(ptrRasterImage);
			ptrWidget->SetAppearance(ptrCompositImage);

			//sign it
			SX::ObjPtr<SX::PDF::IFieldSignature> ptrFieldSignature = static_cast<SX::PDF::IFieldSignature*>(ptrNewFormFieldSignature.get());
			SX::ObjPtr<SX::PDF::ISignature> pSignature = ptrFieldSignature->GetSignature(true);
			pSignature->SignatureInquiry(ptrCryptoCert);


			SavePDF();
			IViewerPtr ptrV = m_pDefViewer->get_Viewer();
			ptrV->Invalidate(false,true);
		}
		catch (SX::IException* e)
		{
			ShowErrorMessage(e->get_ErrorCode());
		}
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				m_ptrDoc = nullptr;
		}catch (SX::IException*) {  }	
	}

	void CPDFXpansionSDK::SavePDF()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));

				m_ptrDoc->SaveAsFile(sPath);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}
} // namespace Sample