#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { m_ptrDoc = nullptr; if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				
				// set viewer tool mode
				ptrV->set_ToolMode(SX::Viewer::tool_mode_none);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				// set pdf settings
				SX::Viewer::IViewerConfigPDF* pPdfConf = pConf->get_PDF(); 
				pPdfConf->set_AnnotsVisible(pPdfConf->get_AnnotsVisible() | SX::PDF::annot_markup);
				pPdfConf->set_AnnotsSensitive(pPdfConf->get_AnnotsSensitive() | SX::PDF::annot_markup);
				pPdfConf->set_AnnotsFocusable(pPdfConf->get_AnnotsFocusable() | SX::PDF::annot_markup);
				m_pDefViewer->SetEventHandler(this);
				pPdfConf->set_Popups(SX::Viewer::viewer_popups_conform);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				m_ptrDoc = ptrDoc.release();
				ptrV->set_ToolMode(SX::Viewer::tool_mode_annot);
				
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				ptrV->set_ToolMode(SX::Viewer::tool_mode_none);
				m_ptrDoc = nullptr;
		}catch (SX::IException*) {  }	
	}

	void CPDFXpansionSDK::SetInkTool()
	{
		if (m_ptrDoc)
		{
			IViewerPtr ptrV = m_pDefViewer->get_Viewer();
			ptrV->set_ToolMode(SX::Viewer::tool_mode_ink);
		}
	}

	void CPDFXpansionSDK::SetEraserTool()
	{
		if (m_ptrDoc)
		{
			IViewerPtr ptrV = m_pDefViewer->get_Viewer();
			ptrV->set_ToolMode(SX::Viewer::tool_mode_eraser);
		}
	}

	SX::sx_bool CPDFXpansionSDK::IsPDF()
	{
		return m_ptrDoc != nullptr;
	}

	SX::sx_bool CPDFXpansionSDK::IsInk()
	{
		if (m_ptrDoc)
		{
			IViewerPtr ptrV = m_pDefViewer->get_Viewer();
			return SX::Viewer::tool_mode_ink == ptrV->get_ToolMode();
		}
		return false;
	}

	SX::sx_bool CPDFXpansionSDK::IsEraser()
	{
		if (m_ptrDoc)
		{
			IViewerPtr ptrV = m_pDefViewer->get_Viewer();
			return SX::Viewer::tool_mode_eraser == ptrV->get_ToolMode();
		}
		return false;
	}

	SX::sx_bool CPDFXpansionSDK::OnEvent(SX::Viewer::IViewerEvent* pEvent)
	{
		if (m_ptrDoc == nullptr)
			return false;

		if (pEvent->get_Type() != SX::Viewer::viewer_event_tool_activity)
			return false;

		SX::Viewer::IViewerEvent_ToolActivity* pToolsActivity = static_cast<SX::Viewer::IViewerEvent_ToolActivity*>(pEvent);
		SX::Viewer::IToolState* pToolsState = pToolsActivity->get_State();

		switch (pToolsState->get_ToolMode())
		{
		case SX::Viewer::tool_mode_ink:
			OnToolInk(static_cast<SX::Viewer::IToolInk*>(pToolsState));
			break;
		case SX::Viewer::tool_mode_eraser:
			OnToolEraser(static_cast<SX::Viewer::IToolEraser*>(pToolsState));
			break;
		}

		return false;
	}

	void CPDFXpansionSDK::OnToolInk(SX::Viewer::IToolInk *pInkTool)
	{
		switch (pInkTool->get_Activity())
		{
		case SX::Viewer::sx_tool_change:
			{
				SX::Viewer::ITrackerStyle *pTS = pInkTool->get_TrackerStyle();
				pInkTool->set_ShowFrame(false);
			}
			break;
		case SX::Viewer::sx_tool_start:
			{
				SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(pInkTool->get_Page());
				if (ptrPage)
				{
					SX::PDF::IAnnots *pAnnots = ptrPage->get_Annots();
					SX::sx_point pt;
					pInkTool->GetToolPoint(pt);
					SX::sx_matrix m;
					ptrPage->CalcMatrix(m);
					SX::Graphics::IMatrix *pMatrix = m_pApp->get_Factory()->Matrix();
					pMatrix->Invert(m);
					pMatrix->TransformPoint(m, pt);
					SX::ObjPtr<SX::PDF::IInk> ptrInk = static_cast<SX::PDF::IInk*>(pAnnots->FindItem(pt, SX::PDF::annot_type_ink));
					if (!ptrInk) // change existing annotation
					{
						ptrInk = static_cast<SX::PDF::IInk*>(pAnnots->InsertNew(SX::PDF::annot_type_ink));
						if (ptrInk)
						{
							ptrInk->SetColor(SX::sx_color_f_rgb(1, 0, 0));
							ptrInk->set_Opacity(0.5);
							ptrInk->set_BorderWidth(10);
							pInkTool->set_Ink(ptrInk);
						}
						else return;
					}
					else pInkTool->set_Ink(ptrInk);
				}
			}
			break;
		case SX::Viewer::sx_tool_cancel:
			{
				SX::sx_uint uPage = pInkTool->get_Page();
				if (uPage != SX::INVALID_INDEX)
				{
					SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(uPage);
					if (ptrPage)
					{
						SX::PDF::IAnnots *pAnnots = ptrPage->get_Annots();
						SX::ObjPtr<SX::PDF::IInk> ptrInk = pInkTool->get_Ink();
						if (ptrInk && ptrInk->get_SegmentsCount() == 0) // remove empty annotation
						{
							pAnnots->Remove(ptrInk);
							pInkTool->ResetTool();
						}
					}
				}
			}
			break;
		}
	}

	void CPDFXpansionSDK::OnToolEraser(SX::Viewer::IToolEraser *pEraser)
	{
		switch (pEraser->get_Activity())
		{
		case SX::Viewer::sx_tool_change: // setup tool
			{
				SX::Viewer::ITrackerStyle *pTS = pEraser->get_TrackerStyle();
				pEraser->set_ShowFrame(false);
				pEraser->set_Size(30);
			}
			break;
		case SX::Viewer::sx_tool_start: // active page is changed
			{
				SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(pEraser->get_Page());
				if (ptrPage)
				{
					SX::PDF::IAnnots *pAnnots = ptrPage->get_Annots();
					sx_point pt;
					pEraser->GetToolPoint(pt);
					sx_matrix m;
					ptrPage->CalcMatrix(m);
					SX::Graphics::IMatrix *pMatrix = m_pApp->get_Factory()->Matrix();
					pMatrix->Invert(m);
					pMatrix->TransformPoint(m, pt);
					SX::sx_uint uAnnotCount = ptrPage->get_Annots()->get_Count();
					for (SX::sx_uint uAnnot = 0; uAnnot < uAnnotCount; uAnnot++)
					{
						SX::ObjPtr<SX::PDF::IAnnot> ptrAnnot = ptrPage->get_Annots()->Item(uAnnot);
						if (ptrAnnot && ptrAnnot->get_Type() == SX::PDF::annot_type_ink)
							pEraser->AddInk(static_cast<SX::PDF::IInk*>(ptrAnnot.get()));
					}
				}
				break;
			}
		case SX::Viewer::sx_tool_apply: // erase operation is completed
			{
				SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(pEraser->get_Page());
				if (ptrPage)
				{
					SX::PDF::IAnnots *pAnnots = ptrPage->get_Annots();
					SX::sx_uint uCount = pEraser->get_Inks();
					for (sx_uint u = 0; u < uCount; u++)
					{
						SX::ObjPtr<SX::PDF::IInk> ptrInk = pEraser->GetInk(u);
						if (ptrInk->get_SegmentsCount() == 0) // remove empty annotations
							pAnnots->Remove(ptrInk);
					}
				}
			}
			break;
		}
	}

	inline SX::sx_bool CPDFXpansionSDK::IsModified()
	{
		if (m_ptrDoc == nullptr)
			return false;

		return m_ptrDoc->get_Properties()->get_Modified();
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	void CPDFXpansionSDK::SavePDF()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));

				m_ptrDoc->SaveAsFile(sPath);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}
} // namespace Sample