﻿#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{
	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK()
		: m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr),
		m_hJSLib(NULL), m_pJSTools(nullptr), m_pJSApp(nullptr), m_pJSDoc(nullptr), m_pPDFDoc(nullptr)
	{
	}

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK()
	{
		if (m_pJSApp)	m_pJSApp->ReleaseApp();
		if (m_hJSLib) ::FreeLibrary(m_hJSLib);
		if (m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion);
	}

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
				InitJSTools(pUI);
			} catch(IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	sx_bool CPDFXpansionSDK::InitJSTools(ISampleAppUI* pUI)
	{
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-js-x64.dll");
#else
		CString sPath(L"pdf-xpansion-js-x86.dll");
#endif
		m_hJSLib = ::LoadLibrary(sPath);
		if (m_hJSLib == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find JScript library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if (JS::JSLibraryEntryPoint entry = reinterpret_cast<JS::JSLibraryEntryPoint>(GetProcAddress(m_hJSLib, "EntryPoint")))
		{
			m_pJSTools = entry();
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		m_pJSApp = m_pJSTools->CreateApp(m_pApp, LangCode2_en, L"PDF Xpansion Reader", L"");
		m_pJSApp->SetEventHandler(this);
		//
		IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();
		ptrDoc->OpenFile(L"forms.pdf");
		m_pJSApp->AttachScript(ptrDoc, nullptr);
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				// configurate the viewer here
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(Viewer::viewer_popups_single);
				// config JS
				pConf->get_PDF()->set_EnableFormFilling(true);
				pConf->set_RelayDocEvents(true);
				m_pDefViewer->SetEventHandler(this);

				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if (m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
		{
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();
				ptrDoc->OpenFile(sPath);

				m_pPDFDoc = ptrDoc.get();

				//	Check document security
				PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;
				//	JScript document
				PDF::IFormFields *pFormFields = ptrDoc->get_FormFields();
				pFormFields->set_RaiseEvents(true);
				pFormFields->set_AutoCalculate(true);
				if (m_pJSDoc) m_pJSApp->ReleaseDoc(m_pJSDoc);
				m_pJSDoc = m_pJSApp->CreateDoc(ptrDoc);
				m_pJSApp->AttachScript(ptrDoc, m_pJSDoc);
				// attach in viewer
				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
			}
			catch (IException*) {}
		}
	}

	bool CPDFXpansionSDK::CheckSecurity(PDF::IDocSecurity* pDocSecurity)
	{
		PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == PDF::doc_security_password)
		{
			PDF::IPasswordSecurity* pPassSec = static_cast<PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);

				m_pPDFDoc = nullptr;

				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
		}catch (IException*) {  }	
	}

	sx_bool CPDFXpansionSDK::OnEvent(Viewer::IViewerEvent* pEvent)
	{
		if (pEvent)
		{
			switch (pEvent->get_Type())
			{
			case Viewer::viewer_event_pdf:
				{
					Viewer::IViewerEvent_PDF *pEvPDF = (Viewer::IViewerEvent_PDF *)pEvent;
					switch (pEvPDF->get_PDFType())
					{
					case PDF::pdf_event_annot_lost_focus:
					case PDF::pdf_event_annot_mouse_up:
					case PDF::pdf_event_annot_mouse_down:
					case PDF::pdf_event_annot_mouse_enter:
					case PDF::pdf_event_annot_mouse_exit:
					case PDF::pdf_event_annot_get_focus:
						{
							Viewer::IViewerEvent_PDF_Annot *pEvAnnot = (Viewer::IViewerEvent_PDF_Annot *)pEvPDF;
							OnAnnotEvent(pEvAnnot);
						}
						break;
					}
				}
				break;
			case Viewer::viewer_event_doc_activity:
				{
					Viewer::IViewerEvent_DocActivity *pEvDoc = static_cast<Viewer::IViewerEvent_DocActivity*>(pEvent);
					IDocEvent* pEvent = pEvDoc->get_DocEvent();
					if (pEvent && pEvent->get_Type() == doc_event_form_field)
						OnEventFormField(static_cast<IDocFieldEvent*>(pEvent));
				}
				break;
			};
		}
		return true;
	}

	void CPDFXpansionSDK::OnAnnotEvent(SX::Viewer::IViewerEvent_PDF_Annot *pEvAnnot)
	{
		ObjPtr<PDF::IAnnot> ptrAnnot = pEvAnnot->get_Annot();
		ObjPtr<PDF::IPDFActions> ptrActions = ptrAnnot->GetEventActions(pEvAnnot->get_PDFType());
		if (ptrActions.get())
		{
			sx_uint cnt = ptrActions->get_Count();
			for (sx_uint i = 0; i < cnt; ++i)
			{
				ObjPtr<PDF::IPDFAction> ptrAction = ptrActions->Item(i);
				if (ptrAction)
				{
					switch (ptrAction->get_Type())
					{
					case PDF::pdf_action_exec_jscript:
						{
							ObjPtr<IStr> sJS = static_cast<PDF::IActionJScript*>(ptrAction.get())->get_JScript();
							if (sJS.get())
							{
								m_pJSApp->AttachScript(sJS->get_str(), m_pJSDoc, pEvAnnot->get_PDFEvent());
								if (!pEvAnnot->get_Handled()) pEvAnnot->set_Handled(true);
							}
						}
						break;
					case PDF::pdf_action_reset_form:
						{
							m_pPDFDoc->get_FormFields()->ResetValues();
							m_pPDFDoc->get_FormFields()->ResetAppearances(false);

							IViewerPtr ptrV = m_pDefViewer->get_Viewer();
							ptrV->Invalidate(true);
						}
						break;
					}
				}
			}
		}
	}

	void CPDFXpansionSDK::OnEventFormField(IDocFieldEvent *pEvField)
	{
		PDF::IPDFEvent *pPdfEvent = pEvField->get_PDFEvent();
		ObjPtr<PDF::IFormField> ptrFormField = pEvField->get_Field();
		ObjPtr<PDF::IPDFActions> ptrActions = ptrFormField->GetEventActions(pPdfEvent->get_Type());
		if (ptrActions.get())
		{
			sx_uint cnt = ptrActions->get_Count();
			for (sx_uint i = 0; i < cnt; ++i)
			{
				ObjPtr<PDF::IPDFAction> ptrAction = ptrActions->Item(i);
				if (ptrAction && ptrAction->get_Type() == PDF::pdf_action_exec_jscript)
				{
					ObjPtr<IStr> sJS = static_cast<PDF::IActionJScript*>(ptrAction.get())->get_JScript();
					if (sJS.get())
					{
						m_pJSApp->AttachScript(sJS->get_str(), m_pJSDoc, pPdfEvent);
						if (!pEvField->get_Handled()) pEvField->set_Handled(true);
					}
				}
			}
		}
	}

	sx_bool CPDFXpansionSDK::OnEvent(JS::IJSEvent* pEvent)
	{
		if (pEvent)
		{
			switch (pEvent->get_Type())
			{
			case JS::js_event_invalidate:
				{
					IViewerPtr ptrV = m_pDefViewer->get_Viewer();
					ptrV->Invalidate(true);
				}
				break;
			case JS::js_event_app_escript:
				{
					JS::IJSEvEScript *pEvEScript = (JS::IJSEvEScript *)pEvent;
					switch (pEvEScript->get_IDS())
					{
					case JS::js_escript_str_ge:
						pEvEScript->SetString(L"Ungültiger Wert: nur Werte größer oder gleich %s sind gültig.");
						break;
					case JS::js_escript_str_gt_lt:
						pEvEScript->SetString(L"Ungültiger Wert: nur Werte größer oder gleich %s und kleiner oder gleich %s sind gültig.");
						break;
					case JS::js_escript_str_le:
						pEvEScript->SetString(L"Ungültiger Wert: nur Werte kleiner oder gleich %s sind gültig.");
						break;
					case JS::js_escript_str_inv_mnt:
						pEvEScript->SetString(L"** Ungültig **");
						break;
					case JS::js_escript_str_inv_date:
						pEvEScript->SetString(L"Datum oder Zeit ungültig. Bitte prüfen Sie Ihre Angaben. Feld");
						break;
					case JS::js_escript_str_inv_date2:
						pEvEScript->SetString(L"sollte folgendes Format haben");
						break;
					case JS::js_escript_str_inv_value:
						pEvEScript->SetString(L"Das Format des eingegebenen Werts passt nicht zum Feldformat.");
						break;
					case JS::js_escript_str_month_info:
						pEvEScript->SetString(L"Januar[1]Februar[2]März[3]April[4]Mai[5]Juni[6]Juli[7]August[8]September[9]Oktober[10]November[11]Dezember[12]Sept[9]Jan[1]Feb[2]Mrz[3]Apr[4]Jun[6]Jul[7]Aug[8]Sep[9]Okt[10]Nov[11]Dez[12]");
						break;
					}
				}
				break;			
			case JS::js_event_wsh_error:
				{
					JS::IJSEvError *pEvError = (JS::IJSEvError *)pEvent;

					sx_str desc = pEvError->get_Desc();
					sx_str src = pEvError->get_Source();				

					CString title;
					title.Format(L"Line: %u, Char: %u", pEvError->get_Line(), pEvError->get_Char());

					m_pUI->MessageBox(desc, MB_OK | MB_ICONERROR, title.GetBuffer());
				}
				break;
			case JS::js_event_app_alert:
				{
					JS::IJSEvAlert *pEvAlert = (JS::IJSEvAlert *)pEvent;

					sx_str msg = pEvAlert->get_Msg();
					sx_str title = pEvAlert->get_Title();

					m_pUI->MessageBox(msg, MB_OK, title);
				}
				break;
			}
		}

		return true;
	}
} // namespace Sample