// COutlinesDlg.cpp : Implementation of CCOutlines

#include "stdafx.h"
#include "OutlinesDlg.h"
#include "..\..\..\include\PDF\document.h"
#include "..\..\..\include\PDF\outline.h"


//////////////////////////////////////////////////////////////////////////
void CCOutlinesDlg::LoadDocumentOutlines()
{
	SX::PDF::IOutlineItem* pRootOutline = m_pDocument->get_Outline();
	m_hRootItem = AddTreeItem(L"Root", pRootOutline);
	// recursive filling outlines
	if (pRootOutline != nullptr)
		AddChilds(pRootOutline, m_hRootItem);
	TreeView_Expand(m_cwTreeView.m_hWnd, m_hRootItem, TVE_EXPAND);
}

//////////////////////////////////////////////////////////////////////////
HTREEITEM CCOutlinesDlg::AddTreeItem(CString text, SX::PDF::IOutlineItem* pOutline, HTREEITEM hParrent)
{
	// insert new node into the tree
	m_vOutlines.push_back(pOutline);

	TVITEM item;
	item.pszText = text.GetBuffer();
	item.mask = TVIF_TEXT | TVIF_PARAM;
	// save outline pointer as LParam
	item.lParam = (LPARAM)pOutline;
	TVINSERTSTRUCT itemStruct;

	itemStruct.item = item;
	itemStruct.hParent=hParrent;
	itemStruct.hInsertAfter = TVI_LAST;
	return TreeView_InsertItem(m_cwTreeView.m_hWnd, &itemStruct); 
}

//////////////////////////////////////////////////////////////////////////
void CCOutlinesDlg::AddChilds(SX::PDF::IOutlineItem* pParentOutline, HTREEITEM hParentNode)
{
	SX::PDF::IOutlineItem* ptrCurOutline = pParentOutline->FirstChild();
	while(ptrCurOutline != nullptr)
	{
		SX::ObjPtr<SX::IStr> ptrTitle = ptrCurOutline->get_Title();
		HTREEITEM hNewItem = AddTreeItem(ptrTitle->get_str(), ptrCurOutline, hParentNode);
		AddChilds(ptrCurOutline, hNewItem);
		ptrCurOutline = pParentOutline->NextChild(ptrCurOutline);
		
	}
}

//////////////////////////////////////////////////////////////////////////
SX::PDF::IOutlineItem* CCOutlinesDlg::GetSelectedOutline(OUT HTREEITEM& hSelectedNode)
{
	hSelectedNode = TreeView_GetSelection(m_cwTreeView.m_hWnd);
	TVITEM item; 
	ZeroMemory(&item, sizeof(item));
	// get outline pointer
	item.mask =  TVIF_PARAM;
	item.hItem = hSelectedNode;
	if (TreeView_GetItem(m_cwTreeView.m_hWnd, &item))
		return (SX::PDF::IOutlineItem*)item.lParam;
	else
		return nullptr;
}


//////////////////////////////////////////////////////////////////////////
SX::sx_bool CCOutlinesDlg::HasDestination(SX::PDF::IOutlineItem* pOutline)
{
	SX::ObjPtr<SX::PDF::IDest> ptrDest = pOutline->get_Dest();
	if (ptrDest) return true;

	SX::PDF::INamedDests* pNamedDests = m_pDocument->get_NamedDests();
	if (SX::ObjPtr<SX::IStr> pNamedDest = pOutline->get_NamedDest())
	{
		
		ptrDest = pNamedDests->Get(pNamedDest->get_str());
		if (ptrDest) return true;
	}

	SX::ObjPtr<SX::PDF::IPDFActions> ptrActions = pOutline->GetActions();
	sx_uint uActionCount = ptrActions->get_Count();
	for (sx_uint i=0; i < uActionCount; i++)
	{
		SX::ObjPtr<SX::PDF::IPDFAction> ptrAction = ptrActions->Item(i);
		if (ptrAction->get_Type() == SX::PDF::pdf_action_goto)
		{
			SX::ObjPtr<SX::PDF::IActionGoTo> pActionGoTo = static_cast<SX::PDF::IActionGoTo*>(ptrAction.get());
			ptrDest = pActionGoTo->get_Dest();
			if (ptrDest) return true;

			ObjPtr<IStr> pNamedDest = pActionGoTo->get_NamedDest();
			ptrDest = pNamedDests->Get(pNamedDest->get_str());
			if (ptrDest) return true;
		}
	}

	return false;
}

LRESULT CCOutlinesDlg::ID_TREE_SEL_CHANGED(int /*idCtrl*/, LPNMHDR pNMHDR, BOOL& /*bHandled*/)
{
	HTREEITEM hSelectedNode;
	if (SX::PDF::IOutlineItem* ptrCurrOutline = GetSelectedOutline(hSelectedNode))
	{
		m_cwAddButton.EnableWindow(TRUE);
		m_cwGoToButton.EnableWindow((HasDestination(ptrCurrOutline)  && hSelectedNode != m_hRootItem)? TRUE : FALSE);
	}
	else
	{
		m_cwAddButton.EnableWindow(FALSE);
		m_cwGoToButton.EnableWindow(FALSE);
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////
LRESULT CCOutlinesDlg::OnClickedAddNew(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	HTREEITEM hSelectedNode;
	if (SX::ObjPtr<SX::PDF::IOutlineItem> ptrCurrOutline = GetSelectedOutline(hSelectedNode))
	{
		SX::PDF::IPages* pPages = m_pDocument->get_Pages();
		SX::sx_uint uPageCount = pPages->get_Count();
		CCAddOutlineDlg dlg(uPageCount);
		if (dlg.DoModal() == IDOK)
		{
			// add child outline
			SX::ObjPtr<SX::PDF::IOutlineItem> ptrChildOutline = ptrCurrOutline->InsertChild(dlg.get_Title());
			// create outline destination
			SX::ObjPtr<SX::PDF::IDest> ptrDest = ptrChildOutline->ResetDest();
			SX::ObjPtr<SX::PDF::IPage> ptrPage = pPages->Item(dlg.get_PageNum()); 
			ptrDest->set_Page(ptrPage);

			// create and show new tree node
			HTREEITEM hNewNode = AddTreeItem(dlg.get_Title(), ptrChildOutline, hSelectedNode);
			TreeView_Expand(m_cwTreeView.m_hWnd, hNewNode, TVE_EXPANDPARTIAL);
		}
	}
	return 0;
}

LRESULT CCOutlinesDlg::OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	CAxDialogImpl<CCOutlinesDlg>::OnInitDialog(uMsg, wParam, lParam, bHandled);
	m_cwTreeView = GetDlgItem(IDC_TREE_VIEW);
	m_cwAddButton = GetDlgItem(IDC_ADD_BUTTON);
	m_cwGoToButton = GetDlgItem(IDC_GOTO_BUTTON);
	m_cwAddButton.EnableWindow(FALSE);
	m_cwGoToButton.EnableWindow(FALSE);
	LoadDocumentOutlines();
	bHandled = TRUE;
	return 1;  // Let the system set the focus
}

//////////////////////////////////////////////////////////////////////////
LRESULT CCOutlinesDlg::OnClickedGoTo(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	HTREEITEM hSelectedNode;
	if (SX::ObjPtr<SX::PDF::IOutlineItem> ptrCurrOutline = GetSelectedOutline(hSelectedNode))
		m_pViewer->get_Canvas()->Navigate(ptrCurrOutline);

	EndDialog(wID);
	return 0;
}

//////////////////////////////////////////////////////////////////////////
LRESULT CCOutlinesDlg::OnClickedCancel(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	EndDialog(wID);
	return 0;
}