#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"
#include "../../../../include/app.h"
#include "../../../../include/layout.h"

namespace Sample
{
	#define  HEADER_FOOTER_HEIGHT  30
	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() 
	{ 
		//Free open document before library disposed
		m_ptrBaseDoc = nullptr; 
		if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); 
	}

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				m_ptrBaseDoc = ptrDoc.release();
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				m_ptrBaseDoc = ptrPack.release();
		}catch (SX::IException*) {  }	
	}

	void CPDFXpansionSDK::PrintDocument()
	{
		if (!m_ptrBaseDoc)
		{
			m_pUI->MessageBox(L"File not open!", MB_OK);
			return;
		}
		//Get current page
		IViewerPtr ptrV = m_pDefViewer->get_Viewer();
		sx_uint curPage =ptrV->get_Canvas()->get_ActivePage();
		SX::ObjPtr<SX::Viewer::IViewableDocument> ptrViewDoc = ptrV->get_Document();
		SX::sx_uint count = ptrViewDoc->GetPageCount();

		//Show printer dialog
		m_pUI->PrintFileDialog(m_ptrBaseDoc, curPage,count);
	}

	void CPDFXpansionSDK::PrintWithHeaderFooter()
	{
		if (!m_ptrBaseDoc)
		{
			m_pUI->MessageBox(L"Document not open!", MB_OK);
			return;
		}

		// create layout document
		SX::ObjPtr<SX::Layout::IDocument> ptrLayout = m_pApp->get_Factory()->CreateLayoutDocument();
		ptrLayout->SetRefDoc(m_ptrBaseDoc);
		SX::ObjPtr<SX::Layout::IBasicProvider> ptrProvider = static_cast<SX::Layout::IBasicProvider*>(ptrLayout->CreateProvider(SX::Layout::prov_type_basic));
		
		SX::ObjPtr<SX::Layout::IDecoElement> ptrDeco = CreatePageDecorator(ptrLayout);
		ptrProvider->SetOverlay(ptrDeco);
		ptrProvider->Start(0);

		m_pUI->PrintFileDialog(ptrLayout, 0,ptrLayout->GetPageCount());

	}

	SX::Layout::IDecoElement* CPDFXpansionSDK::CreatePageDecorator(SX::Layout::IDocument *pLayout)
	{
		SX::Graphics::IMatrix *pMatrix = m_pApp->get_Factory()->Matrix();

		// retrieve size of first page
		SX::sx_size sizePage;
		SX::ObjPtr<SX::Viewer::IViewableDocument> ptrViewDoc = m_ptrBaseDoc->GetViewableDocument();
		ptrViewDoc->GetPageSize(0, sizePage);

		// create header decorator
		SX::ObjPtr<SX::Layout::IDecoElement> ptrDeco = pLayout->CreateDeco();
		SX::Content::IRichCollection *pColl = ptrDeco->get_Objects();

		// add autotext to show document title
		SX::Content::IRichText *pHeaderText = static_cast<SX::Content::IRichText*>(pColl->Insert(pColl->get_Count(), SX::Content::rich_type_text));
		SX::Content::ITextStyle *pHeaderTextStyle = pHeaderText->GetDefaultStyle();
		pHeaderTextStyle->SetFont(SX::sx_color_i_rgb(0,0,0), L"Times", 14, 700); // times new roman, 14, bold

		// E004 - document title
		CString sPattern; sPattern.Format(L"Document title: %c", SX::Content::macro_field_doc_title);
		pHeaderText->GetAutoTextPattern()->set_Pattern(sPattern);
		pHeaderText->GetAutoTextPattern()->set_Alignment(SX::Content::text_alignment_center);
		SX::sx_matrix mx; pMatrix->Identity(mx);
		pMatrix->Translate(mx, sizePage.width / 2., 20);
		pHeaderText->ApplyTransformation(mx);

		// add autotext to show page number
		SX::Content::IRichText *pFooterText = static_cast<SX::Content::IRichText*>(pColl->Insert(pColl->get_Count(), SX::Content::rich_type_text));
		SX::Content::ITextStyle *pFooterTextStyle = pFooterText->GetDefaultStyle();
		pFooterTextStyle->SetFont(SX::sx_color_i_rgb(0,0,0), L"Times", 12, 400);

		// E001 - page number, E003 - page count
		sPattern.Format(L"Page %c of %c", SX::Content::macro_field_page_num, SX::Content::macro_field_page_count);
		pFooterText->GetAutoTextPattern()->set_Pattern(sPattern);
		pFooterText->GetAutoTextPattern()->set_Alignment(SX::Content::text_alignment_right);
		pMatrix->Identity(mx);
		pMatrix->Translate(mx, sizePage.width - 10, sizePage.height - 12);
		pFooterText->ApplyTransformation(mx);

		// add header line 
		SX::Content::IRichVector *pVector = static_cast<SX::Content::IRichVector*>(pColl->Insert(pColl->get_Count(), SX::Content::rich_type_vector));
		SX::Content::IPath *pPath = ptrDeco->CreatePath();
		SX::sx_point pt1 = {0,HEADER_FOOTER_HEIGHT}; SX::sx_point pt2 = {sizePage.width, HEADER_FOOTER_HEIGHT};
		pPath->AddLine(pt1, pt2);	
		pVector->set_Path(pPath);
		SX::Content::ISolidBrush* pBrush = ptrDeco->CreateSolidBrush(sx_color_i_rgb(197,197,197)); // gray color
		SX::Content::IPen* pPen = ptrDeco->CreatePen(1);	// pen width = 1 pt.
		pVector->set_Pen(pPen);
		pVector->set_StrokeBrush(pBrush);

		pt1.y = pt2.y = sizePage.height - HEADER_FOOTER_HEIGHT;
		pVector = static_cast<SX::Content::IRichVector*>(pColl->Insert(pColl->get_Count(), SX::Content::rich_type_vector));
		pPath = ptrDeco->CreatePath();
		pPath->AddLine(pt1, pt2);	
		pVector->set_Path(pPath);
		pVector->set_Pen(pPen);
		pVector->set_StrokeBrush(pBrush);

		
		return ptrDeco.release();
	}

} // namespace Sample