// SecurityDialog.h : Declaration of the CSecurityDialog

#pragma once

#include "resource.h"       // main symbols

#include <atlhost.h>
#include <Windowsx.h>

using namespace ATL;

// CSecurityDialog

class CSecurityDialog : 
	public CAxDialogImpl<CSecurityDialog>
{
public:
	CSecurityDialog(SX::PDF::encryption_algorithm alg, SX::PDF::encryption_key_length keyLen)
	{
		m_alg = alg;
		m_keyLen = keyLen;
	}

	~CSecurityDialog()
	{
	}

	const CString &GetUserPassword() const;
	const CString &GetOwnerPassword() const;
	const SX::PDF::encryption_algorithm GetAlgorithm() const;
	const SX::PDF::encryption_key_length GetKeyLength() const;

	enum { IDD = IDD_DIALOG_SECURITY };

BEGIN_MSG_MAP(CSecurityDialog)
	MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
	COMMAND_HANDLER(IDOK, BN_CLICKED, OnClickedOK)
	COMMAND_HANDLER(IDCANCEL, BN_CLICKED, OnClickedCancel)
	COMMAND_HANDLER(IDC_ALGORITHM, CBN_SELCHANGE, OnComboChanged)
	COMMAND_HANDLER(IDC_KEYLENGTH, CBN_SELCHANGE, OnComboChanged)
	CHAIN_MSG_MAP(CAxDialogImpl<CSecurityDialog>)
END_MSG_MAP()

// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	LRESULT OnInitDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		CAxDialogImpl<CSecurityDialog>::OnInitDialog(uMsg, wParam, lParam, bHandled);
		bHandled = TRUE;
		
		m_cwComboKeyLen = GetDlgItem(IDC_KEYLENGTH);
		m_cwComboAlg = GetDlgItem(IDC_ALGORITHM);

		int pos;
		ComboBox_SetItemData(m_cwComboAlg,ComboBox_AddString(m_cwComboAlg, L"RC4"),SX::PDF::encryption_algorithm_rc4);
		ComboBox_SetItemData(m_cwComboAlg,ComboBox_AddString(m_cwComboAlg, L"AES"),SX::PDF::encryption_algorithm_aes);
		
		switch(m_keyLen)
		{
			case SX::PDF::encryption_key_length_40:  pos = 0; break;
			case SX::PDF::encryption_key_length_128: pos = 1; break;
			case SX::PDF::encryption_key_length_256: pos = 2; break;
			case SX::PDF::encryption_key_length_384: pos = 3; break;
			case SX::PDF::encryption_key_length_512: pos = 4; break;
			default: pos = 0; break;
		}
		ComboBox_SetItemData(m_cwComboKeyLen, ComboBox_AddString(m_cwComboKeyLen, L"40"), SX::PDF::encryption_key_length_40);
		ComboBox_SetItemData(m_cwComboKeyLen, ComboBox_AddString(m_cwComboKeyLen, L"128"), SX::PDF::encryption_key_length_128);
		ComboBox_SetItemData(m_cwComboKeyLen, ComboBox_AddString(m_cwComboKeyLen, L"256"), SX::PDF::encryption_key_length_256);
		ComboBox_SetItemData(m_cwComboKeyLen, ComboBox_AddString(m_cwComboKeyLen, L"384"), SX::PDF::encryption_key_length_384);
		ComboBox_SetItemData(m_cwComboKeyLen, ComboBox_AddString(m_cwComboKeyLen, L"512"), SX::PDF::encryption_key_length_512);

		ComboBox_SetCurSel(m_cwComboAlg, (m_alg == SX::PDF::encryption_algorithm_none)?0:UINT(m_alg)-1);
		ComboBox_SetCurSel(m_cwComboKeyLen, pos);

		return 1;  // Let the system set the focus
	}

	LRESULT OnComboChanged(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		m_alg = (SX::PDF::encryption_algorithm)(ComboBox_GetItemData(m_cwComboAlg,ComboBox_GetCurSel(m_cwComboAlg)));
		UINT keylen = ComboBox_GetItemData(m_cwComboKeyLen,ComboBox_GetCurSel(m_cwComboKeyLen));

		if ((m_alg == SX::PDF::encryption_algorithm_rc4 && keylen > 128) || (m_alg == SX::PDF::encryption_algorithm_aes && keylen < 128))
		{
			MessageBox(L"The current key length is not compatible with the selected algorithm",L"Error", MB_OK | MB_ICONSTOP);
			GetDlgItem(IDOK).EnableWindow(false);
		}
		else
		{
			GetDlgItem(IDOK).EnableWindow(true);
		}
		return 1;
	}

	LRESULT OnClickedOK(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		GetDlgItemText(IDC_OWNER_PASSWORD, m_sOwnerPassword);
		GetDlgItemText(IDC_USER_PASSWORD, m_sUserPassword);
		m_alg = (SX::PDF::encryption_algorithm)(ComboBox_GetItemData(m_cwComboAlg,ComboBox_GetCurSel(m_cwComboAlg)));
		m_keyLen = (SX::PDF::encryption_key_length)ComboBox_GetItemData(m_cwComboKeyLen,ComboBox_GetCurSel(m_cwComboKeyLen));
		EndDialog(wID);
		bHandled = TRUE;
		return 0;
	}

	LRESULT OnClickedCancel(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
	{
		EndDialog(wID);
		return 0;
	}

private:
	SX::PDF::encryption_algorithm m_alg;
	SX::PDF::encryption_key_length m_keyLen;
	CString m_sOwnerPassword;
	CString m_sUserPassword;
	CWindow m_cwComboKeyLen;
	CWindow m_cwComboAlg;
};


