#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { m_ptrDoc = nullptr; if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0, 0, (sx_double)w, (sx_double)h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				ptrV->set_ToolMode(SX::Viewer::tool_mode_none);

				// set pdf settings
				SX::Viewer::IViewerConfigPDF* pPdfConf = pConf->get_PDF(); 
				pPdfConf->set_AnnotsVisible(pPdfConf->get_AnnotsVisible() | SX::PDF::annot_markup);
				pPdfConf->set_AnnotsSensitive(pPdfConf->get_AnnotsSensitive() | SX::PDF::annot_markup);
				pPdfConf->set_AnnotsFocusable(pPdfConf->get_AnnotsFocusable() | SX::PDF::annot_markup);
				pPdfConf->set_Popups(SX::Viewer::viewer_popups_conform);
				m_pDefViewer->SetEventHandler(this);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		m_ptrOverAnnot = nullptr;
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				m_ptrDoc = ptrDoc.release();
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	SX::PDF::IMarkupAnnot* CPDFXpansionSDK::AddStickyNote(SX::sx_point& point, SX::PDF::IPage* page)
	{
		//create Sticky Note
		SX::ObjPtr<SX::PDF::IStickyNote> ptrNote = static_cast<SX::PDF::IStickyNote*> (page->get_Annots()->InsertNew(SX::PDF::annot_type_text));
		ptrNote->SetModDate(GetTime());
		ptrNote->set_Description(L"Sticky note body");
		ptrNote->set_Author(L"Author");
		ptrNote->set_Subject(L"Sticky note title");

		ptrNote->set_Icon(SX::PDF::sticky_icon_help);
		ptrNote->SetColor(SX::sx_color_i_rgb(255,201,14));

		// transform annotation region to PDF space
		SX::sx_matrix pageMatrix;
		page->CalcMatrix(pageMatrix);

		SX::Graphics::IMatrix* pMatrix = m_pApp->get_Factory()->Matrix();
		pMatrix->Invert(pageMatrix);
		pMatrix->TransformPoint(pageMatrix,point);
		
		// create Sticky Note rectangle
		SX::sx_rect rect;
		rect.x = point.x-15;
		rect.y = point.y-15;
		rect.height = rect.width = 30;		//will be replaced by note size
		
		// reset appearance of annotation
		ptrNote->SetRectArea(rect);
		ptrNote->ResetAppearance();

		return ptrNote.release();
	}

	void CPDFXpansionSDK::AddPopup(SX::PDF::IMarkupAnnot* pAnnot)
	{
		SX::ObjPtr<SX::PDF::IPopup> ptrPopup = pAnnot->get_Popup();
		SX::sx_rect annotRect;
		pAnnot->GetRectArea(annotRect);
		SX::sx_rect popupRect;
		popupRect.height = 300;									//will be replaced by popup height
		popupRect.width = 400;									//will be replaced by popup width
		popupRect.x = annotRect.x + annotRect.width;
		popupRect.y = annotRect.y + annotRect.height - popupRect.height;		
		ptrPopup->SetRectArea(popupRect);
	}

	SX::PDF::IMarkupAnnot* CPDFXpansionSDK::AddStamp(SX::sx_rect& rect, SX::PDF::IPage* page)
	{
		//Open annotation image
		CString sPath;
		if(m_pUI->OpenFileDialog(L"Image files *.png,*.jpg\0*.png;*.jpg\0\0", sPath))
		{
			SX::ObjPtr<SX::Graphics::IBitmapData> ptrBmp =  m_pApp->get_Factory()->CreateBitmapFromStream(m_pApp->get_Factory()->CreateReadStream(sPath));
			if (!ptrBmp.get())
				return nullptr;

			SX::PDF::IDocResources* pDocResources = m_ptrDoc->get_Resources();
			SX::ObjPtr<SX::PDF::IRasterImage> ptrRasterImage = pDocResources->AddRasterImage(ptrBmp, SX::PDF::Filter_JPEG, 80);
			SX::ObjPtr<SX::PDF::ICompositeImage> ptrCompositImage = pDocResources->AddCompositeImage(ptrRasterImage);

			//create stamp
			SX::ObjPtr<SX::PDF::IStamp> ptrStamp = static_cast<SX::PDF::IStamp*> (page->get_Annots()->InsertNew(SX::PDF::annot_type_stamp));
			ptrStamp->SetModDate(GetTime());
			ptrStamp->set_Description(L"Stamp description");
			ptrStamp->set_Author(L"Author");
			ptrStamp->set_Subject(L"Stamp title");
			ptrStamp->SetAppearance(ptrCompositImage,false);

			// transform annotation region to PDF space
			SX::sx_matrix pageMatrix;
			page->CalcMatrix(pageMatrix);

			SX::Graphics::IMatrix* pMatrix = m_pApp->get_Factory()->Matrix();
			pMatrix->Invert(pageMatrix);
			pMatrix->TransformRect(pageMatrix,rect);
			ptrStamp->SetRectArea(rect);

			return ptrStamp.release();
		}
		return nullptr;
	}

	void CPDFXpansionSDK::Save()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));

				m_ptrDoc->SaveAsFile(sPath);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	SX::sx_bool CPDFXpansionSDK::OnToolEvent(SX::Viewer::IViewerEvent_ToolActivity* pToolsActivity)
	{
		// get current activity tool
		SX::Viewer::IToolState* pToolsState = pToolsActivity->get_State();
		if (pToolsState->get_ToolMode() ==  SX::Viewer::tool_mode_tracker)
		{
			SX::Viewer::IToolTracker* pToolTracker = static_cast<SX::Viewer::IToolTracker*>(pToolsState);
			SX::Viewer::sx_tool_act toolActivity = pToolsState->get_Activity();
			if (toolActivity == SX::Viewer::sx_tool_change)
			{
				switch(pToolTracker->get_Type())
				{
				case SX::Viewer::tracker_type_rect_brush :
					pToolTracker->set_BrushSize(20);
					pToolTracker->SetColor(SX::sx_color_i_rgb(205,205,0));
					break;
				case SX::Viewer::tracker_type_rect:
					pToolTracker->set_Width(2);
					break;
				}
				pToolTracker->set_Type(m_toolTrackerType);
				return true;
			}
			else if (toolActivity == SX::Viewer::sx_tool_apply)
			{
				try
				{
					//get active page
					SX::sx_uint pageNum = pToolTracker->get_Page();
					SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(pageNum);
					SX::ObjPtr<SX::PDF::IMarkupAnnot> ptrAnnot;

					//add new annotation
					switch(pToolTracker->get_Type())
					{
					case SX::Viewer::tracker_type_rect_brush:
						SX::sx_point point;
						pToolTracker->GetEndPoint(point);
						ptrAnnot = AddStickyNote(point, ptrPage);
						break;
					case SX::Viewer::tracker_type_rect:
						SX::sx_point a,b;
						pToolTracker->GetBeginPoint(a);
						pToolTracker->GetEndPoint(b);
						SX::sx_rect rect = PointsToRect(a,b);
						ptrAnnot = AddStamp(rect,ptrPage);
						break;
					}
					//add popup to annotation
					if (ptrAnnot != NULL)
					{
						ptrPage->get_Annots()->ResetPopup(ptrAnnot);
						AddPopup(ptrAnnot);

						IViewerPtr ptrV = m_pDefViewer->get_Viewer();
						ptrV->set_ToolMode(SX::Viewer::tool_mode_none);
						ptrV->Invalidate(true,true);
					}
				}
				catch (SX::IException* e)
				{
					ShowErrorMessage(e->get_ErrorCode());
				}
				return true;
			}
		}
		return false;
	}

	SX::sx_bool CPDFXpansionSDK::OnMouseEvent(SX::Viewer::IViewerEvent_MouseActivity* pMouseActivity)
	{
		switch (pMouseActivity->get_Activity())
		{
			case SX::Viewer::mouse_flags_move:
				if (SX::Viewer::IViewer* pViewer = GetViewer())
				{
					sx_point CanvasPt, PagePt;
					m_ptrOverAnnot = nullptr;
					SX::Viewer::IViewerCanvas* pCanvas = pViewer->get_Canvas();
					// get point at canvas
					CanvasPt.x = pMouseActivity->get_X() + (pCanvas->get_Width()  < pViewer->get_Width()  ? -(sx_double)(pViewer->get_Width() - pCanvas->get_Width())  / 2. : pCanvas->get_HorizontalOffset());
					CanvasPt.y = pMouseActivity->get_Y() + (pCanvas->get_Height() < pViewer->get_Height() ? -(sx_double)(pViewer->get_Height() - pCanvas->get_Height()) / 2. : pCanvas->get_VerticalOffset());
					// find page under pointer
					sx_uint PageIndex = pCanvas->GetPageIndex((sx_uint)CanvasPt.x, (sx_uint)CanvasPt.y);
					if (PageIndex != INVALID_INDEX)
					{
						// canvas to normalized page 
						pCanvas->CanvasToPage(PageIndex, CanvasPt, PagePt);
						// normalized page to PDF page
						SX::Graphics::IMatrix* pMatrix = m_pApp->get_Factory()->Matrix();
						SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(PageIndex);
						SX::sx_matrix mx; ptrPage->CalcMatrix(mx); pMatrix->Invert(mx);
						pMatrix->TransformPoint(mx, PagePt);
						// find an annot under cursor
						m_ptrOverAnnot = ptrPage->get_Annots()->FindItem(PagePt, SX::PDF::annot_markup);
					}
					return true;
				}
				break;
			case SX::Viewer::mouse_flags_cursor:
				if (m_ptrOverAnnot.get())
				{
					m_pUI->SetMouseCursor(LoadCursor(NULL, IDC_HAND));
					pMouseActivity->Cancel();
					return true;
				}
				break;
		}
		return false;
	}

	SX::sx_bool CPDFXpansionSDK::OnEvent(SX::Viewer::IViewerEvent* pEvent)
	{
		if (m_ptrDoc == nullptr)
			return false;

		switch (pEvent->get_Type())
		{
			case SX::Viewer::viewer_event_tool_activity:
				return OnToolEvent((SX::Viewer::IViewerEvent_ToolActivity*)pEvent);
			case SX::Viewer::viewer_event_mouse_activity:
				return OnMouseEvent((SX::Viewer::IViewerEvent_MouseActivity*)pEvent);
		}
		return false;

	}

	SX::sx_rect CPDFXpansionSDK::PointsToRect(SX::sx_point& a, SX::sx_point& b)
	{
		SX::sx_rect rect;
		rect.x = min(a.x,b.x);
		rect.y = min(a.y,b.y);

		rect.width = (a.x>b.x) ? a.x-b.x : b.x-a.x;
		rect.height = (a.y>b.y)? a.y-b.y : b.y-a.y;
		return rect;
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		m_ptrOverAnnot = nullptr;

		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);

				m_ptrDoc = nullptr;
		}catch (SX::IException*) {  }	
	}

	void CPDFXpansionSDK::SetStickyNote()
	{
		IViewerPtr ptrV = m_pDefViewer->get_Viewer();
		m_toolTrackerType = SX::Viewer::tracker_type_rect_brush ;
		ptrV->set_ToolMode(SX::Viewer::tool_mode_tracker);
	}

	void CPDFXpansionSDK::SetStamp()
	{
		IViewerPtr ptrV = m_pDefViewer->get_Viewer();
		m_toolTrackerType = SX::Viewer::tracker_type_rect;
		ptrV->set_ToolMode(SX::Viewer::tool_mode_tracker);
	}

	inline SX::sx_bool CPDFXpansionSDK::IsPDF()
	{
		return m_ptrDoc != nullptr;
	}

	inline SX::sx_bool CPDFXpansionSDK::IsModified()
	{
		if (m_ptrDoc == nullptr)
			return false;

		return m_ptrDoc->get_Properties()->get_Modified();
	}

	SX::sx_time CPDFXpansionSDK::GetTime()
	{
		SYSTEMTIME source = {0};
		GetLocalTime(&source);
		SX::sx_time destination;

		destination.wDay = source.wDay;
		destination.wDayOfWeek = source.wDayOfWeek;
		destination.wHour = source.wHour;
		destination.wMilliseconds = source.wMilliseconds;
		destination.wMinute = source.wMinute;
		destination.wMonth = source.wMonth;
		destination.wSecond = source.wSecond;
		destination.wYear = source.wYear;

		return destination;
	}

} // namespace Sample