#include "stdafx.h"
#include "SDK.h"
#include "PDF Xpansion Key.h"
#include "../PasswordDlg.h"

namespace Sample
{

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::CPDFXpansionSDK() : m_hPDFXpansion(NULL), m_pApp(nullptr), m_pUI(nullptr) { }

	//////////////////////////////////////////////////////////////////////////
	CPDFXpansionSDK::~CPDFXpansionSDK() { m_ptrDoc = nullptr; if(m_hPDFXpansion) ::FreeLibrary(m_hPDFXpansion); }

	//////////////////////////////////////////////////////////////////////////
	bool CPDFXpansionSDK::Init(ISampleAppUI* pUI)
	{
		m_pUI = pUI;
#ifdef _WIN64
		CString sPath(L"pdf-xpansion-windesktop-x64.dll");
#else
		CString sPath(L"pdf-xpansion-windesktop-x86.dll");
#endif
		m_hPDFXpansion = ::LoadLibrary(sPath);
		if(m_hPDFXpansion == NULL)
		{
			m_pUI->MessageBox(CString(L"Can't find PDF Xpansion library: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		// get entry point of library
		if(LibraryEntryPoint entry = reinterpret_cast<LibraryEntryPoint>(GetProcAddress(m_hPDFXpansion, "EntryPoint")))
		{
			try {
				// retrieve root interface of PDF Xpansion	
				m_pApp = entry();
				// authorize the PDF Xpansion
				m_pApp->Authorize(sAuthKey);
			} catch(SX::IException* e) {
				m_pApp = nullptr; CString err;
				switch(e->get_ErrorCode())
				{
				case ErrorAuthFailed   : err = L"The specified license key does not fit used license file"; break;
				case ErrorPathNotFound : err = L"Can't find PDF Xpansion license file: pdf-xpansion.license"; break;
				case ErrorAccessDenied : err = L"Can't find PDF Xpansion resources file: pdf-xpansion.pds"; break;
				case ErrorOutOfRange   : err = L"The trial license is expired, please download actual version of PDF Xpansion."; break;
				case ErrorInvalidUse   : err = "Can't find main PDF Xpansion library file: pdf-xpansion-windesktop-xXX.dll"; break;
				default: err = L"Unknown error, please contact PDF Xpansion team.";
				}
				m_pUI->MessageBox(err, MB_OK | MB_ICONSTOP);
				return false; 
			}
		}
		else
		{
			m_pUI->MessageBox(CString(L"Invalid library file: ") + sPath, MB_OK | MB_ICONSTOP);
			return false;
		}
		return true;
	}

	HWND CPDFXpansionSDK::CreateViewer(HWND hParent, UINT w, UINT h)
	{
		if(m_pApp) 
		{
			sx_rect rc = {0., 0., w, h};
			m_pDefViewer = m_pApp->get_Factory()->CreateWinViewer(rc, hParent, SX::Viewer::viewer_flags_scrollbar_always);
			if(m_pDefViewer)
			{
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				Viewer::IViewerConfig* pConf = ptrV->get_Config();
				// configurate the viewer here
				pConf->set_UseCache(false);
				pConf->set_PageCacheSize(50);
				pConf->set_PageCacheInvalidation(300);
				
				// set viewer tool mode
				ptrV->set_ToolMode(SX::Viewer::tool_mode_none);
				pConf->get_PDF()->set_Popups(SX::Viewer::viewer_popups_single);
				// set pdf settings
				SX::Viewer::IViewerConfigPDF* pPdfConf = pConf->get_PDF(); 
				pPdfConf->set_AnnotsVisible(pPdfConf->get_AnnotsVisible() | SX::PDF::annot_markup);
				pPdfConf->set_AnnotsSensitive(pPdfConf->get_AnnotsSensitive() | SX::PDF::annot_markup);
				pPdfConf->set_AnnotsFocusable(pPdfConf->get_AnnotsFocusable() | SX::PDF::annot_markup);
				m_pDefViewer->SetEventHandler(this);
				pPdfConf->set_Popups(SX::Viewer::viewer_popups_conform);
				return m_pDefViewer->get_Window();
			}
		}
		return NULL;
	}

	SX::Viewer::IViewer* CPDFXpansionSDK::GetViewer()
	{
		return m_pDefViewer ? m_pDefViewer->get_Viewer() : nullptr;
	}

	void CPDFXpansionSDK::OpenPDF()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				IPDFDocPtr ptrDoc = m_pApp->get_Factory()->CreatePDFDocument();	
				ptrDoc->OpenFile(sPath);

				//Check document security
				SX::PDF::IDocSecurity* pDocSecurity = ptrDoc->get_Properties()->get_Security();
				if (!CheckSecurity(pDocSecurity))
					return;

				IVDocPtr ptrVDoc = ptrDoc->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				m_ptrDoc = ptrDoc.release();
				ptrV->set_ToolMode(SX::Viewer::tool_mode_free_text);
				
		}catch (SX::IException*) {  }	
	}

	bool CPDFXpansionSDK::CheckSecurity(SX::PDF::IDocSecurity* pDocSecurity)
	{
		SX::PDF::doc_security dSec = pDocSecurity->get_Security();

		//When document has no security
		if (dSec == SX::PDF::doc_security_none)
			return true;

		//When document has password security
		if (dSec == SX::PDF::doc_security_password)
		{
			SX::PDF::IPasswordSecurity* pPassSec = static_cast<SX::PDF::IPasswordSecurity*>(pDocSecurity->get_StandardSecurity());
			//Try to open document with empty password
			if (pPassSec->SetUserPassword(L""))
				return true;

			CString sPasswrd;
			SX::sx_bool bInvalidPassword = true;
			do 
			{
				//Try to use password entered by user
				CCPasswordDlg passwDialog;
				if (passwDialog.DoModal(m_pDefViewer->get_Window()) != IDOK)
					return false;

				sPasswrd = passwDialog.GetPasswordString();
				bInvalidPassword = !(pPassSec->SetOwnerPassword(sPasswrd) || pPassSec->SetUserPassword(sPasswrd));
				if (bInvalidPassword)
					m_pUI->MessageBox(L"Entered password is invalid", MB_OK | MB_ICONSTOP);
				
			} while (bInvalidPassword);

			return true;

		}

		m_pUI->MessageBox(L"This type of encryption is not supported", MB_OK | MB_ICONSTOP);
		return false;
	}

	void CPDFXpansionSDK::OpenXPS()
	{
		CString sPath;
		if(m_pUI->OpenFileDialog(L"XPS Files *.xps\0*.xps\0OXPS Files *.oxps\0*.oxps\0\0", sPath))
			try {
				IXPSPackPtr ptrPack = m_pApp->get_Factory()->CreateXPSPackage();			
				ptrPack->OpenFile(sPath);
				IVDocPtr ptrVDoc = ptrPack->GetViewableDocument();
				IViewerPtr ptrV = m_pDefViewer->get_Viewer();
				ptrV->set_Document(ptrVDoc);
				ptrV->set_ToolMode(SX::Viewer::tool_mode_none);
				m_ptrDoc = nullptr;
		}catch (SX::IException*) {  }	
	}

	SX::sx_bool CPDFXpansionSDK::OnEvent(SX::Viewer::IViewerEvent* pEvent)
	{
		if (m_ptrDoc == nullptr)
			return false;

		if (pEvent->get_Type() != SX::Viewer::viewer_event_tool_activity)
			return false;

		// get current activity tool
		SX::Viewer::IViewerEvent_ToolActivity* pToolsActivity = static_cast<SX::Viewer::IViewerEvent_ToolActivity*>(pEvent);
		SX::Viewer::IToolState* pToolsState = pToolsActivity->get_State();
		SX::Graphics::IMatrix* pMatrix = m_pApp->get_Factory()->Matrix(); 


		if (pToolsState->get_ToolMode() ==  SX::Viewer::tool_mode_free_text)
		{
			SX::Viewer::IToolFreeText* pToolText = static_cast<SX::Viewer::IToolFreeText*>(pToolsState);
			SX::Viewer::sx_tool_act toolActivity = pToolsState->get_Activity();

			switch(toolActivity)
			{
			case SX::Viewer::sx_tool_start:
				{
					// get current page annotations
					SX::sx_uint pageNum = pToolText->get_Page();
					if (pageNum == INVALID_INDEX)
						break;

					SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(pageNum);
					SX::PDF::IAnnots* pAnnots = ptrPage->get_Annots();

					// search annotation on page
					SX::sx_point pt;
					pToolText->GetToolPoint(pt);
					SX::sx_matrix mx;
					ptrPage->CalcMatrix(mx);
					pMatrix->Invert(mx);
					pMatrix->TransformPoint(mx,pt);
					SX::ObjPtr<SX::PDF::IFreeText> ptrFreeTextAnnot = static_cast<SX::PDF::IFreeText*>(pAnnots->FindItem(pt,SX::PDF::annot_type_freetext));
					// set selected annotation
					pToolText->set_Page(pageNum);
					pToolText->set_FreeText(ptrFreeTextAnnot);
				}
				break;
			case SX::Viewer::sx_tool_free_text_create:
				{
					// get current page annotations
					SX::sx_uint pageNum = pToolText->get_Page();
					if (pageNum == INVALID_INDEX)
						break;
					SX::ObjPtr<SX::PDF::IPage> ptrPage = m_ptrDoc->get_Pages()->Item(pageNum);
					SX::PDF::IAnnots* pAnnots = ptrPage->get_Annots();

					// add new text annotation
					SX::ObjPtr<SX::PDF::IFreeText> ptrTextAnnot = static_cast<SX::PDF::IFreeText*>(pAnnots->InsertNew(SX::PDF::annot_type_freetext));
					
					ptrTextAnnot->SetColor(SX::sx_color_i_rgb(0,200,231));
					ptrTextAnnot->set_Opacity(0.5);
					ptrTextAnnot->SetTextColor(SX::sx_color_i_rgb(0,0,0));
					// set annotation font
					SX::ObjPtr<SX::PDF::IFont> ptrFont = m_ptrDoc->get_Resources()->AddFont(L"Arial");
					ptrTextAnnot->set_TextFont(ptrFont);
					// set free text view
					ptrTextAnnot->set_BorderStyle(SX::PDF::border_style_solid);
					ptrTextAnnot->set_BorderWidth(1);
					ptrTextAnnot->set_EndStyle(SX::PDF::line_ending_open_arrow);	//for textboxes with callout only
					ptrTextAnnot->set_Intent(L"FreeTextCallout");					//for textboxes with callout only
					// set active free text
					pToolText->set_FreeText(ptrTextAnnot);
				}
				break;
			case SX::Viewer::sx_tool_change:
				{
					SX::Viewer::IToolFreeText* pToolFreeText = static_cast<SX::Viewer::IToolFreeText*>(pToolsState);
					SX::Viewer::ITrackerStyle* pTrackerStyle = pToolFreeText->get_TrackerStyle();
					pTrackerStyle->SetFrameColor(SX::sx_color_i_rgb(255,0,0));
					pTrackerStyle->SetFrameStyle(SX::Viewer::line_style_dash);
					pTrackerStyle->SetFrameWidth(2);
				} 
				break;
			}
		}
	}

	inline SX::sx_bool CPDFXpansionSDK::IsModified()
	{
		if (m_ptrDoc == nullptr)
			return false;

		return m_ptrDoc->get_Properties()->get_Modified();
	}

	void CPDFXpansionSDK::ShowErrorMessage(SX::ErrorCode err)
	{
		CString mess = L"";
		switch(err)
		{
		case ErrorOutOfMemory: mess = L"not enough memory"; break;
		case ErrorOutOfSpace: mess = L"not enough disk space"; break;
		case ErrorOutOfRange: mess = L"index out of range"; break;
		case ErrorInvalidFormat: mess = L"invalid data format"; break;
		case ErrorInvalidUse: mess = L"invalid using"; break;
		case ErrorInvalidParam: mess = "invalid argument"; break;
		default: mess = L"Unknown error, please contact PDF Xpansion team.";
		}
		m_pUI->MessageBox(mess, MB_ICONSTOP | MB_OK);
	}

	void CPDFXpansionSDK::SavePDF()
	{
		CString sPath;
		if (m_pUI->SaveFileDialog(L"PDF Files *.pdf\0*.pdf\0\0", sPath))
			try {
				int len = sPath.GetLength();
				if (sPath.Right(4) != _T(".pdf"))
					sPath.Append(_T(".pdf"));

				m_ptrDoc->SaveAsFile(sPath);
		}
		catch (SX::IException* e) { ShowErrorMessage(e->get_ErrorCode()); }
	}
} // namespace Sample