unit MainFrm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Menus, Vcl.OleCtrls,
  PDF_Xpansion_ActiveX_13_OCX, PDF_Xpansion_Wrapper_13_TLB;

type
  TMainFrame = class(TForm)
    PDFXpansionControl: TPDFXpansionControl;
    MainMenu: TMainMenu;
    FileMenuItem: TMenuItem;
    OpenPDFMenuItem: TMenuItem;
    OpenXPSMenuItem: TMenuItem;
    N1: TMenuItem;
    CloseMenuItem: TMenuItem;
    ViewerMenuItem: TMenuItem;
    ModeMenuItem: TMenuItem;
    ModeNormalMenuItem: TMenuItem;
    ModeFlipMenuItem: TMenuItem;
    N2: TMenuItem;
    LayoutMenuItem: TMenuItem;
    LayoutVerticalMenuItem: TMenuItem;
    N3: TMenuItem;
    Layout1ColRowMenuItem: TMenuItem;
    Layout2ColRowMenuItem: TMenuItem;
    Layout4ColRowMenuItem: TMenuItem;
    N4: TMenuItem;
    LayoutCoverPageMenuItem: TMenuItem;
    N5: TMenuItem;
    ZoomFitMenuItem: TMenuItem;
    Zoom50MenuItem: TMenuItem;
    Zoom100MenuItem: TMenuItem;
    Zoom200MenuItem: TMenuItem;
    Zoom500MenuItem: TMenuItem;
    N6: TMenuItem;
    FitWidthMenuItem: TMenuItem;
    FitHeightMenuItem: TMenuItem;
    FitPageMenuItem: TMenuItem;
    N7: TMenuItem;
    RotateMenuItem: TMenuItem;
    RotateCWMenuItem: TMenuItem;
    RotateCCWMenuItem: TMenuItem;

    OpenPDFDialog: TOpenDialog;
    OpenXPSDialog: TOpenDialog;

    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OnOpenPDF(Sender: TObject);
    procedure OnOpenXPS(Sender: TObject);
    procedure OnClose(Sender: TObject);

    procedure OnModeNormal(Sender: TObject);
    procedure OnModeFlip(Sender: TObject);

    procedure OnLayoutVertical(Sender: TObject);
    procedure OnLayout1ColRow(Sender: TObject);
    procedure OnLayout2ColRow(Sender: TObject);
    procedure OnLayout4ColRow(Sender: TObject);
    procedure OnLayoutCoverPage(Sender: TObject);

    procedure OnZoom50(Sender: TObject);
    procedure OnZoom100(Sender: TObject);
    procedure OnZoom200(Sender: TObject);
    procedure OnZoom500(Sender: TObject);

    procedure OnFitWidth(Sender: TObject);
    procedure OnFitHeight(Sender: TObject);
    procedure OnFitPage(Sender: TObject);

    procedure OnRotateCW(Sender: TObject);
    procedure OnRotateCCW(Sender: TObject);

  private
    function GetXpansionApp(): IApplication;

  public
  end;

  PDF_XPANSION_SDK_ENTRY_POINT = function: Pointer; cdecl;

var
  MainFrame: TMainFrame;
  hXpansionModule: HMODULE;
  ptrXpansionApp: IApplication;

implementation

{$R *.dfm}

function TMainFrame.GetXpansionApp(): IApplication;
var
	entryPoint: PDF_XPANSION_SDK_ENTRY_POINT;
begin
	hXpansionModule := LoadLibrary('pdf-xpansion-com-x86.dll');
	if hXpansionModule <> 0 then
        begin
		@entryPoint := GetProcAddress(hXpansionModule, 'EntryPoint');

		if @entryPoint <> nil then
		begin
			 Pointer(ptrXpansionApp) := entryPoint();
		end;
	end;
	Result := ptrXpansionApp;
end;

procedure TMainFrame.FormShow(Sender: TObject);
begin

   hXpansionModule := 0;
   ptrXpansionApp := PDFXpansionControl.Application;
   { get app object directly}
   {ptrXpansionApp := GetXpansionApp();}

   if not ptrXpansionApp.Authorized then
   begin
     ptrXpansionApp.Authorize('D502CB5B11E5394F932A84A46735583E3C84D62DEB804942BA50257E5C311EA2D03356126CC6CE4B89F7A0A91969660611C6404EA90C4546AB7BE23328EBF5C7', '');
   end;

   ptrXpansionApp := nil;
end;

procedure TMainFrame.FormDestroy(Sender: TObject);
begin
	if ptrXpansionApp <> nil then
	begin
		ptrXpansionApp := nil;
	end;

	if hXpansionModule <> 0 then
        begin
		FreeLibrary(hXpansionModule);
	end;
end;

procedure TMainFrame.OnOpenPDF(Sender: TObject);
var pDoc: PDF_IDocument;
begin
  if OpenPDFDialog.Execute then
  begin
    pDoc := PDFXpansionControl.Application.Factory.CreatePDFDocument;
    pDoc.OpenFile(OpenPDFDialog.FileName, 0);
    PDFXpansionControl.Viewer.Document := pDoc.GetViewableDocument();
  end;
end;

procedure TMainFrame.OnOpenXPS(Sender: TObject);
var pDoc: XPS_IPackage;
begin
  if OpenXPSDialog.Execute then
  begin
    pDoc := PDFXpansionControl.Application.Factory.CreateXPSPackage;
    pDoc.OpenFile(OpenXPSDialog.FileName);
    PDFXpansionControl.Viewer.Document := pDoc.GetViewableDocument();
  end;
end;

procedure TMainFrame.OnClose(Sender: TObject);
begin
	Close;
end;

procedure TMainFrame.OnModeNormal(Sender: TObject);
begin
  {PDFXpansionControl.Viewer.Canvas.Mode := viewer_mode_normal;
  PDFXpansionControl.Viewer.Config.Update;}
end;

procedure TMainFrame.OnModeFlip(Sender: TObject);
begin
  {PDFXpansionControl.Viewer.Canvas.Mode := viewer_mode_flip;
  PDFXpansionControl.Viewer.Config.Update;}
end;

procedure TMainFrame.OnLayoutVertical(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.VerticalLayout := not PDFXpansionControl.Viewer.Canvas.VerticalLayout;
end;

procedure TMainFrame.OnLayout1ColRow(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.LayoutMultiplicity := 1;
end;

procedure TMainFrame.OnLayout2ColRow(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.LayoutMultiplicity := 2;
end;

procedure TMainFrame.OnLayout4ColRow(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.LayoutMultiplicity := 4;
end;

procedure TMainFrame.OnLayoutCoverPage(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.SeparateCover := not PDFXpansionControl.Viewer.Canvas.SeparateCover;
end;

procedure TMainFrame.OnZoom50(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Zoom := 0.5;
end;

procedure TMainFrame.OnZoom100(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Zoom := 1.0;
end;

procedure TMainFrame.OnZoom200(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Zoom := 2.0;
end;

procedure TMainFrame.OnZoom500(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Zoom := 5.0;
end;

procedure TMainFrame.OnFitWidth(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Fit := viewer_fit_page_width;
end;

procedure TMainFrame.OnFitHeight(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Fit := viewer_fit_page_height;
end;

procedure TMainFrame.OnFitPage(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Fit := viewer_fit_page;
end;

procedure TMainFrame.OnRotateCW(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Rotation := (PDFXpansionControl.Viewer.Canvas.Rotation + 90) mod 360;
end;

procedure TMainFrame.OnRotateCCW(Sender: TObject);
begin
  PDFXpansionControl.Viewer.Canvas.Rotation := (PDFXpansionControl.Viewer.Canvas.Rotation + 270) mod 360;
end;

end.
