﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form, SX.NET.ZUGFeRD.IValidationStatus
    {
        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SX.NET.ZUGFeRD.IInvoiceDocument doc;
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            try
            {
                if (dlg.ShowDialog() == DialogResult.OK)
                {
                    doc = sxFormsControl.Application.Factory.CreateInvoiceDocument();
                    SX.NET.ZUGFeRD.zf_status docStatus = doc.OpenFile(dlg.FileName);
                    if (docStatus == SX.NET.ZUGFeRD.zf_status.zf_status_valid)
                    {
                        // process invoice
                        SX.NET.ZUGFeRD.IInvoice invoice = doc.Invoice;
                        if (invoice.Validate((uint)(SX.NET.ZUGFeRD.zf_val_option.zf_val_option_schema | SX.NET.ZUGFeRD.zf_val_option.zf_val_option_rules | SX.NET.ZUGFeRD.zf_val_option.zf_val_option_codes), this))
                        {
                            // show document in viewer
                            sxFormsControl.Document = doc.GetViewableDocument();
                            // process invoice
                            ProcessInvoice(invoice); 
                        }
                        else
                        {
                            MessageBox.Show("Invoice validation failed!", "", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                        }
                    }
                    else
                    {
                        string mess = string.Empty;
                        switch (docStatus)
                        {
                            case SX.NET.ZUGFeRD.zf_status.zf_status_not_pdf: mess = "ZUGFeRD file must be a PDF file."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_enc_pdf: mess = "ZUGFeRD file must be a PDF file without encryption."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_no_meta: mess = "ZUGFeRD file must be a PDF file with metadata."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_err_meta: mess = "Metadata of this file isn't conform to ZUGFeRD format."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_not_pdfa: mess = "ZUGFeRD file isn't conform to PDF/A-3 format."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_err_emb: mess = "ZUGFeRD file hasn't embedded invoice or embedding is incorrect."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_not_zf: mess = "ZUGFeRD file hasn't embedded invoice or embedding is incorrect."; break;
                            case SX.NET.ZUGFeRD.zf_status.zf_status_dif_prof: mess = "The profile specified in metadata and invoice profile are different."; break;
                        }
                        MessageBox.Show(mess, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                        return;
                    }
                }
            }
            catch (SX.NET.Exception ex1)
            {
                ShowErrorMessage(ex1.get_ErrorCode());
            }
            catch (System.Exception ex2)
            {
                MessageBox.Show(ex2.Message);
            }
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        //////////////////////////////////////////////////////////////////////////
	    const string Note1 = "Rechnung gemäß Vertrag/Auftrag  03-07/14v, ZV1";
	    const string Note2 = "soft Xpansion GmbH & Co. KG\nKönigsallee 45\n44789 Bochum\nDeutschland\nGeschäftsführer: Dr. Juri Stern\nHandelsregister HRA 5869 (AG Bochum)";
	    const string Note3 = "Zahlungsziel: 14 Tage Netto ohne Abzug";

        private void CreateInvoice_Click(object sender, EventArgs e)
        {
            try
            {
                SX.NET.ZUGFeRD.IInvoiceDocument doc = sxFormsControl.Application.Factory.CreateInvoiceDocument();
                // load printable invoice
                string documentPath = System.IO.Directory.GetParent(System.IO.Directory.GetParent(Application.StartupPath).FullName) + "\\inv.pdf";
                doc.CreateFromPDF(sxFormsControl.Application.Factory.CreateReadStream(documentPath));
                SX.NET.ZUGFeRD.IInvoice invoice = doc.Invoice;
                // set invoice header
                SX.NET.ZUGFeRD.IHeader header = invoice.GetHeader(true);
                header.GetID(true).ID = "123456";
                header.GetName(true).Text = "RECHUNG";
                header.Type = SX.NET.ZUGFeRD.doc_code.doc_code_380;
                header.GetIssueDate(true).Date = new DateTime(2015, 8, 19);
                header.GetIssueDate(false).format = SX.NET.ZUGFeRD.datetime_code.datetime_code_CCYYMMDD;
                header.GetIncludedNote(0, true).GetContent(true).Text = Note1;
                header.GetIncludedNote(1, true).GetContent(true).Text = Note2;
                header.GetIncludedNote(2, true).GetContent(true).Text = Note3;
                SX.NET.ZUGFeRD.ITradeTransaction tran = invoice.GetTransaction(true);

                // set agreement info
                SX.NET.ZUGFeRD.ITradeAgreement agreement = tran.GetAgreement(true);
                SX.NET.ZUGFeRD.ITradeParty seller = agreement.GetSellerParty(true);
				seller.GetName(true).Text = "soft Xpansion GmbH & Co. KG";
				agreement.GetBuyerParty(true).GetName(true).Text = "Musterfirma GmbH";
				tran.GetLineItem(0, true).GetAssociatedDocument(true);
				tran.GetLineItem(0, true).GetDelivery(true).GetBilledQuantity(true).unit = SX.NET.ZUGFeRD.measure_unit.measure_unit_C62;
				tran.GetLineItem(0, true).GetDelivery(true).GetBilledQuantity(true).Value = 0;
				{
					SX.NET.ZUGFeRD.IAmount am = tran.GetLineItem(0, true).GetSettlement(true).GetMonetarySummation(true).GetLineTotal(true);
							   am.Value = 4125.0; am.currency = CurrCode3.CurrCode3_EUR;
				}
				tran.GetLineItem(0, true).GetProduct(true).GetName(true).Text = "PDF Xpansion SDK";
                SX.NET.ZUGFeRD.ITradeAddress address = seller.GetAddress(true);
                address.Postcode = "44789";
                address.GetLineOne(true).Text = "Königsallee 45";
                address.GetCity(true).Text = "Bochum";
                address.Country = CountryCode2.CountryCode2_DE;

                SX.NET.ZUGFeRD.ITaxID tax = seller.GetTaxRegistration(0, true).GetID(true);
                tax.TaxScheme = SX.NET.ZUGFeRD.tax_code.tax_code_VA;
                tax.ID = "DE814621954";
                tax = seller.GetTaxRegistration(1, true).GetID(true);
                tax.TaxScheme = SX.NET.ZUGFeRD.tax_code.tax_code_FC;
                tax.ID = "306/5826/0758";

                address = agreement.GetBuyerParty(true).GetAddress(true);
                address.Postcode = "65432";
                address.GetLineOne(true).Text = "Hauptstr. 12";
                address.GetCity(true).Text = "Musterstadt";
                address.Country = CountryCode2.CountryCode2_DE;

                // set delivery info
                SX.NET.ZUGFeRD.IDateTime date = tran.GetDelivery(true).GetDeliveryChainEvent(0, true).GetOccurrenceDate(0, true);
                date.Date = new DateTime(2015, 8, 1);

                // set settlement info
                SX.NET.ZUGFeRD.ITradeSettlement settlement = tran.GetSettlement(true);
                settlement.GetPaymentReference(0, true).Text = "2015-123456";
                settlement.InvoiceCurrency = CurrCode3.CurrCode3_EUR;

                SX.NET.ZUGFeRD.IPaymentMeans paymentMeans = settlement.GetPaymentMeans(0, true);
                paymentMeans.GetCreditorAccount(true).GetIBANID(true).ID = "DE51430700240151329000";
                paymentMeans.GetCreditorInstitution(true).GetBICID(true).ID = "DEUTDEDB430";

                SX.NET.ZUGFeRD.ITradeTax tradeTax = settlement.GetApplicableTradeTax(0, true);
                SX.NET.ZUGFeRD.IAmount amount = tradeTax.GetCalculatedAmount(0, true);
                amount.Value = 783.75; amount.currency = CurrCode3.CurrCode3_EUR;
                tradeTax.Type = SX.NET.ZUGFeRD.tax_type_code.tax_type_code_VAT;
                amount = tradeTax.GetBasisAmount(0, true);
                amount.Value = 4125.0; amount.currency = CurrCode3.CurrCode3_EUR;
                tradeTax.GetApplicablePercent(true).Value = 19;

                SX.NET.ZUGFeRD.IMonetarySummation summ = settlement.GetMonetarySummation(true);
                amount = summ.GetLineTotal(true);
                amount.Value = 4125.0; amount.currency = CurrCode3.CurrCode3_EUR;
                amount = summ.GetChargeTotal(true);
                amount.Value = 0.0; amount.currency = CurrCode3.CurrCode3_EUR;
                amount = summ.GetAllowanceTotal(true);
                amount.Value = 0.0; amount.currency = CurrCode3.CurrCode3_EUR;
                amount = summ.GetTaxBasisTotal(true);
                amount.Value = 4125.0; amount.currency = CurrCode3.CurrCode3_EUR;
                amount = summ.GetTaxTotal(true);
                amount.Value = 783.75; amount.currency = CurrCode3.CurrCode3_EUR;
                amount = summ.GetGrandTotal(true);
                amount.Value = 4908.75; amount.currency = CurrCode3.CurrCode3_EUR;
                // check required elements of invoice
                if (!invoice.Validate((uint)(SX.NET.ZUGFeRD.zf_val_option.zf_val_option_schema), this))
			    {
                    MessageBox.Show("Invoice isn't complete yet.", "", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
			    }

                // save invoice
                SaveFileDialog dlg = new SaveFileDialog();
                dlg.Filter = "PDF documents (*.pdf)|*.pdf";
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    doc.SaveAsFile(dlg.FileName);

                // show document in viewer
                sxFormsControl.Document = doc.GetViewableDocument(); 
            }
            catch (SX.NET.Exception ex1)
            {
                ShowErrorMessage(ex1.get_ErrorCode());
            }
            catch (System.Exception ex2)
            {
                MessageBox.Show(ex2.Message);
            }
        }

        public bool OnMessage(SX.NET.ZUGFeRD.IValidationMessage pMess)
        {
            StringBuilder sMess = new StringBuilder((pMess.CriticalError)? "Error: " : "Warning: ");
            switch (pMess.Code)
            {
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_schema: sMess.Append("invalid XML schema"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_attr_notused: sMess.Append("attribute not used in the given context"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_elem_notused: sMess.Append("element not used in the given context"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_attr_req: sMess.Append("attribute required"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_elem_req: sMess.Append("element required"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_elem_one_req: sMess.Append("element must occur exactly 1 times"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_elem_one_opt: sMess.Append("element may occur at maximum 1 times"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_value_req: sMess.Append("element value required"); break;
                case SX.NET.ZUGFeRD.zf_val_code.zf_val_code_out_of_profile: sMess.Append("element or attribute not used in the given profile"); break;
                default: sMess.Append("unknown code"); break;
            }
            sMess.Append("\nAdditional info: ");
            sMess.Append(pMess.ExtendedInfo);
            System.Diagnostics.Debug.WriteLine(sMess.ToString()); 

            return true;
        }

        private void ProcessInvoice(SX.NET.ZUGFeRD.IInvoice invoice)
        {
            StringBuilder sb = new StringBuilder("Invoice information\n\nProfile: ");

            switch (invoice.GetProfile(false).Profile)
            {
                case SX.NET.ZUGFeRD.zf_profile.zf_profile_basic: sb.Append("basic"); break;
                case SX.NET.ZUGFeRD.zf_profile.zf_profile_comfort: sb.Append("comfort"); break;
                case SX.NET.ZUGFeRD.zf_profile.zf_profile_extended: sb.Append("extended"); break;
                case SX.NET.ZUGFeRD.zf_profile.zf_profile_unknown: sb.Append("unknown"); break;
            }
            SX.NET.ZUGFeRD.ITradeTransaction tran = invoice.GetTransaction(false);
            if (tran != null)
            {
                SX.NET.ZUGFeRD.ITradeAgreement agreement = tran.GetAgreement(false);
                if (agreement != null)
                {
                    SX.NET.ZUGFeRD.ITradeParty buyerParty = agreement.GetBuyerParty(false);
                    if (buyerParty != null)
                    {
                        SX.NET.ZUGFeRD.ITextElement buyerText = buyerParty.GetName(false);
                        if (buyerText != null) sb.Append(Environment.NewLine + "Buyer: " + buyerText.Text);
                    }

                    SX.NET.ZUGFeRD.ITradeParty sallerParty = agreement.GetSellerParty(false);
                    if (sallerParty != null)
                    {
                        SX.NET.ZUGFeRD.ITextElement sellerText = sallerParty.GetName(false);
                        if (sellerText != null) sb.Append(Environment.NewLine + "Buyer: " + sellerText.Text);
                    }
                }

                SX.NET.ZUGFeRD.ITradeSettlement settlement = tran.GetSettlement(false);
                if (settlement != null)
                {
                    SX.NET.ZUGFeRD.IAmount amount = settlement.GetMonetarySummation(false).GetGrandTotal(false);
                    if (amount != null)
                    {
                        sb.Append(Environment.NewLine + "Total: ");
                        sb.Append(amount.Value);
                        switch (amount.currency)
                        {
                            case CurrCode3.CurrCode3_EUR: sb.Append(" EUR"); break;
                            case CurrCode3.CurrCode3_USD: sb.Append(" USD"); break;
                        }
                    }
                }
                MessageBox.Show(sb.ToString());
            }
        }

        private void OpenZUGFeRDInvoiceXML(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xml";
            dlg.Filter = "XML documents |*.xml";
            dlg.RestoreDirectory = true;

            try
            {
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    SX.NET.ZUGFeRD.IInvoiceStream invoiceStream = sxFormsControl.Application.Factory.CreateInvoiceStream();
                    SX.NET.ISequentialStream seqStream = sxFormsControl.Application.Factory.CreateReadStream(dlg.FileName);
                    invoiceStream.Open(seqStream);

                    SX.NET.ZUGFeRD.IInvoice invoice = invoiceStream.Invoice;
                    if (invoice != null)
                    {
                        if (invoice.Validate((uint)(SX.NET.ZUGFeRD.zf_val_option.zf_val_option_schema | SX.NET.ZUGFeRD.zf_val_option.zf_val_option_rules | SX.NET.ZUGFeRD.zf_val_option.zf_val_option_codes), this))
                        {
                            // process invoice
                            ProcessInvoice(invoice);
                        }
                        else
                        {
                            MessageBox.Show("Invoice validation failed!", "", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                        }
                    }
                    else
                    {
                        MessageBox.Show("Invoice isn't well-formed XML or ZUGFeRD.", "", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    }
                }
            }
            catch (SX.NET.Exception ex1)
            {
                ShowErrorMessage(ex1.get_ErrorCode());
            }
            catch (System.Exception ex2)
            {
                MessageBox.Show(ex2.Message);
            }
        }
    }
}
