﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        SX.NET.PDF.IDocument mainDoc;
        const double PAGE_WIDTH = 595;
        const double PAGE_HEIGHT = 842;

        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
        }

        #region Open_documents_region
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                mainDoc = doc;
                mnuCombineWith.Enabled = mnuSavePDF.Enabled = true;

                sxFormsControl.Document = doc.GetViewableDocument();               
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();

                mainDoc = null;
            }
        }
        #endregion

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory:      mess = "not enough memory"; break;
		        case SX.NET.ErrorCode.ErrorOutOfSpace:       mess = "not enough disk space"; break;
		        case SX.NET.ErrorCode.ErrorOutOfRange:       mess = "index out of range"; break;
		        case SX.NET.ErrorCode.ErrorInvalidFormat:    mess = "invalid data format"; break;
		        case SX.NET.ErrorCode.ErrorInvalidUse:       mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam:     mess = "invalid argument"; break;
		        default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void CombineDocuments(IBaseDocument doc, uint combineOptions)
        {
            try
            {
                uint pageCount = mainDoc.Pages.Count;
                mainDoc.StartCombine(doc, combineOptions);
                mainDoc.CombineAllPages(pageCount, combineOptions);
                mainDoc.EndCombine();
                sxFormsControl.Relayout(sxFormsControl.Canvas.ActivePage);
            }
            catch (SX.NET.Exception ex)
            {
                ShowErrorMessage(ex.get_ErrorCode());
            }
        }

        private void mnuCPDF_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "pdf", Filter = "PDF documents | *.pdf" };
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                using (SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument())
                {
                    try
                    {
                        doc.OpenFile(dlg.FileName, 0);

                        if (!CheckSecurity(doc.Properties.Security))
                            return;
                    }
                    catch (SX.NET.Exception ex)
                    {
                        ShowErrorMessage(ex.get_ErrorCode());
                        return;
                    }

                    uint combineFlags = (uint)(combine.combine_named_dests | combine.combine_outlines | combine.combine_form_fields | combine.combine_layers | combine.combine_links);
                    CombineDocuments(doc, combineFlags);
                }
            }
        }

        private void mnuCXPS_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "xps", Filter = "XPS documents | *.xps" };
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                using (SX.NET.XPS.IPackage pack = sxFormsControl.Application.Factory.CreateXPSPackage())
                {
                    try
                    {
                        pack.OpenFile(dlg.FileName);
                    }
                    catch (SX.NET.Exception ex)
                    {
                        ShowErrorMessage(ex.get_ErrorCode());
                        return;
                    }

                    uint combineFlags = (uint)(combine.combine_named_dests | combine.combine_outlines | combine.combine_links | combine.combine_emb_unk_fonts);
                    CombineDocuments(pack.Documents.Item(0), combineFlags);
                }
            }
        }

        private void mnuCSVG_Click(object sender, EventArgs e)
        {
            //OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "svg", Filter = "SVG documents | *.svg" };
            //if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            //{
            //    try
            //    {
            //        //svgDoc.OpenFile(dlg.FileName);
            //        using (SX.NET.SVG.IDocument svgDoc = sxFormsControl.Application.Factory.CreateSVGDocument())
            //        {
            //            svgDoc.OpenFile(dlg.FileName);
            //            SX.NET.PDF.ICompositeImage svgImage = mainDoc.Resources.AddSvgFragment(svgDoc.Root,0,0,0,0);
            //            SX.NET.PDF.IPage newPage = mainDoc.Pages.Add();
            //            SX.NET.sx_matrix matr = new sx_matrix();
            //            sxFormsControl.Application.Factory.Matrix().Identity(ref matr);
            //            newPage.InsertImage(svgImage, matr, false);
            //            sxFormsControl.Relayout(sxFormsControl.Canvas.ActivePage);
            //        }
            //    }
            //    catch (SX.NET.Exception ex)
            //    {
            //        ShowErrorMessage(ex.get_ErrorCode());
            //    }
            //}
        }

        private void mnuCEMF_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "emf", Filter = "EMF documents | *.emf" };
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                using (SX.NET.ISequentialStream stream = sxFormsControl.Application.Factory.CreateReadStream(dlg.FileName))
                {
                    try
                    {
                        SX.NET.PDF.ICompositeImage emfImage = mainDoc.Resources.AddGdiMetafile(stream, 0, 0);
                        SX.NET.PDF.IPage newPage = mainDoc.Pages.Add();
                        SX.NET.sx_matrix matr = new sx_matrix();
                        sxFormsControl.Application.Factory.Matrix().Identity(ref matr);

                        newPage.InsertImage(emfImage, matr, false);
                        sxFormsControl.Relayout(sxFormsControl.Canvas.ActivePage);
                    }
                    catch (SX.NET.Exception ex)
                    {
                        ShowErrorMessage(ex.get_ErrorCode());
                    }
                    catch (System.Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
            }
        }

        private void mnuCText_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "txt", Filter = "TXT documents | *.txt" };
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                using (SX.NET.ITextDocument txtDoc = sxFormsControl.Application.Factory.CreateTextDocument())
                {
                    try
                    {
                        SX.NET.page_props props = new page_props()
                        {
                            height = PAGE_HEIGHT,               //in points
                            width = PAGE_WIDTH,                 //in points
                            bottom = 15, top = 15, left = 5, right = 5    //in points
                        };

                        using (SX.NET.ISequentialStream stream = sxFormsControl.Application.Factory.CreateReadStream(dlg.FileName))
                        {
                            txtDoc.SetText(stream, props, false, 0);
                            txtDoc.SetTextColor(new SX.NET.sx_color_i_rgb(0, 0, 0));
                            txtDoc.SetTextFont("Time New Roman", 14, 0, 0, 0);
                        }

                        mainDoc.StartCombine(txtDoc, 0);
                        mainDoc.CombineAllPages(mainDoc.Pages.Count, 0);
                        mainDoc.EndCombine();

                        sxFormsControl.Relayout(sxFormsControl.Canvas.ActivePage);
                    }
                    catch (SX.NET.Exception ex)
                    {
                        ShowErrorMessage(ex.get_ErrorCode());
                    }
                    catch(System.Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                }
            }
        }

        private void mnuCImage_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog() { DefaultExt = "png", Filter = "PNG images | *.png|JPG images | *.jpg" };
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    using (SX.NET.IImageDocument imgDoc = sxFormsControl.Application.Factory.CreateImageDocument())
                    {
                        SX.NET.ISequentialStream stream = sxFormsControl.Application.Factory.CreateReadStream(dlg.FileName);
                        SX.NET.Graphics.IBitmapData bmp = sxFormsControl.Application.Factory.CreateBitmapFromStream(stream);
                        SX.NET.PDF.IRasterImage rasterImage = mainDoc.Resources.AddRasterImage(bmp, Filter.Filter_JPEG, 80);
                        SX.NET.PDF.IPage imgPage = mainDoc.Pages.Add();
                        SX.NET.sx_matrix matr = new sx_matrix();
                        sxFormsControl.Application.Factory.Matrix().Identity(ref matr);
                        imgPage.InsertImage(rasterImage, matr, false);

                        sxFormsControl.Relayout(sxFormsControl.Canvas.ActivePage);
                    }
                }
                catch (SX.NET.Exception ex)
                {
                    ShowErrorMessage(ex.get_ErrorCode());
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuSavePDF_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF documents (*.pdf)|*.pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    mainDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex)
                {
                    ShowErrorMessage(ex.get_ErrorCode());
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void splitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            uint combineFlags = (uint)(combine.combine_named_dests | combine.combine_outlines | combine.combine_form_fields | combine.combine_layers | combine.combine_links);
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.AddExtension = true;
            dlg.Filter = "PDF files|*.pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                for (uint index = 0; index < mainDoc.Pages.Count; index++)
                {
                    using (SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument())
                    {
                        try
                        {
                            doc.StartCombine(mainDoc, combineFlags);
                            doc.CombinePage(index, 0, combineFlags);
                            doc.EndCombine();
                            string fileName = string.Format("{0}\\{1}{2}.pdf", System.IO.Path.GetDirectoryName(dlg.FileName), System.IO.Path.GetFileNameWithoutExtension(dlg.FileName), index+1);
                            doc.SaveAsFile(fileName);
                        }
                        catch (SX.NET.Exception ex)
                        {
                            ShowErrorMessage(ex.get_ErrorCode());
                        }
                        catch (System.Exception ex)
                        {
                            MessageBox.Show(ex.Message);
                        }
                    }
                }
            }
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            if (mainDoc == null)
            {
                mnuSavePDF.Enabled = splitToolStripMenuItem.Enabled = mnuCombineWith.Enabled = false;
                exportToXPSToolStripMenuItem.Enabled = false;
            }
            else
            {
                mnuSavePDF.Enabled = mainDoc.Properties.Modified;
                exportToXPSToolStripMenuItem.Enabled = true;
                splitToolStripMenuItem.Enabled = mnuCombineWith.Enabled = true;
            }
        }

        private void exportToXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                // create new XPS document
                SX.NET.XPS.IPackage pack = sxFormsControl.Application.Factory.CreateXPSPackage();
                SX.NET.XPS.IDocument doc = pack.Documents.Add();

                // combine XPS with current PDF
                uint combineFlags = (uint)(combine.combine_named_dests | combine.combine_outlines | combine.combine_links | combine.combine_emb_unk_fonts);
                doc.StartCombine(mainDoc, combineFlags);
                doc.CombineAllPages(0, combineFlags);
                doc.EndCombine();

                // save XPS package
                SaveFileDialog dlg = new SaveFileDialog();
                dlg.Filter = "XPS документы|*.xps";
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    pack.SaveAsFile(dlg.FileName);
                }
            }
            catch (SX.NET.Exception ex)
            {
                ShowErrorMessage(ex.get_ErrorCode());
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
    }
}
