﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF Files *.pdf|*.pdf";
            dlg.RestoreDirectory = true;
            try
            {
                if (dlg.ShowDialog() == DialogResult.OK)
                {
		    SX.NET.ISequentialStream stream = sxFormsControl.Application.Factory.CreateReadStream(dlg.FileName);
		    SX.NET.EInvoice.IInvoiceDocument invDoc = sxFormsControl.Application.Factory.CreateEInvoiceDocument();
		    SX.NET.EInvoice.einv_status status = invDoc.Open(stream, (uint)SX.NET.EInvoice.einv_options.einv_options_check_xml);

                    if (status == SX.NET.EInvoice.einv_status.einv_status_valid)
                    {
                        // optionally display PDF in viewer
                        sxFormsControl.Document = invDoc.GetViewableDocument();

			// extract XML invoice to file
                        switch (invDoc.Standard)
			{
				case SX.NET.EInvoice.einv_standard.einv_standard_zugferd	: MessageBox.Show("Invoice is conform to ZUGFeRD 1.0", "", MessageBoxButtons.OK); break;
				case SX.NET.EInvoice.einv_standard.einv_standard_zugferd20	: MessageBox.Show("Invoice is conform to ZUGFeRD 2.0", "", MessageBoxButtons.OK); break;
				case SX.NET.EInvoice.einv_standard.einv_standard_facturx	: MessageBox.Show("Invoice is conform to FACTUR-X", "", MessageBoxButtons.OK); break;
			}
	 	        SaveFileDialog saveDlg = new SaveFileDialog();
                        saveDlg.DefaultExt = ".xml";
                        saveDlg.Filter = "XML Files *.xml|*.xml";
                        if (saveDlg.ShowDialog() == DialogResult.OK)
                        {
			    SX.NET.ISequentialStream xmlStream = sxFormsControl.Application.Factory.CreateWriteStream(saveDlg.FileName);
                	    invDoc.GetInvoice(xmlStream);
                        }
                    }
                    else
                    {
                        string mess = string.Empty;
                        switch (status)
                        {
                            case SX.NET.EInvoice.einv_status.einv_status_not_pdf	 : mess = "It should be valid PDF file"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_enc_pdf	 : mess = "It should be non-encrypted PDF"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_not_pdfa	 : mess = "It is non-conform to PDF/A-3"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_no_meta	 : mess = "No metadata was found in this file"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_err_meta	 : mess = "Metadata is non-conform to PDF/A-3"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_no_emb	 : mess = "No embedded invoice was found in this file"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_err_emb	 : mess = "Invoice is not embedded correctly"; break;
                            case SX.NET.EInvoice.einv_status.einv_status_nonconform : mess = "Invoice XML is non-conform or declares other profile"; break;
                        }
                        MessageBox.Show(mess, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                        return;
                    }
                }
            }
            catch (SX.NET.Exception ex1)
            {
                ShowErrorMessage(ex1.get_ErrorCode());
            }
            catch (System.Exception ex2)
            {
                MessageBox.Show(ex2.Message);
            }
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        //////////////////////////////////////////////////////////////////////////

        private void createFromPDFToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openXMLDlg = new OpenFileDialog();
            openXMLDlg.DefaultExt = ".xml";
            openXMLDlg.Filter = "XML Files *.xml|*.xml";
            openXMLDlg.RestoreDirectory = true;
            if (openXMLDlg.ShowDialog() == DialogResult.OK)
            {
                OpenFileDialog openPDFDlg = new OpenFileDialog();
                openPDFDlg.DefaultExt = ".pdf";
                openPDFDlg.Filter = "PDF Files *.pdf|*.pdf";
                openPDFDlg.RestoreDirectory = true;
                if (openPDFDlg.ShowDialog() == DialogResult.OK)
                {
                    try
                    {
		        SX.NET.EInvoice.IInvoiceDocument invDoc = sxFormsControl.Application.Factory.CreateEInvoiceDocument();
    		        SX.NET.ISequentialStream pdfStream = sxFormsControl.Application.Factory.CreateReadStream(openPDFDlg.FileName);
		        SX.NET.ISequentialStream xmlStream = sxFormsControl.Application.Factory.CreateReadStream(openXMLDlg.FileName);
			invDoc.CreateFromPDF(xmlStream, pdfStream, (uint)SX.NET.EInvoice.einv_options.einv_options_zugferd20); // use 0 for ZUGFeRD 1.0 and einv_options_facturx for FacturX

                        SaveFileDialog savePDFDlg = new SaveFileDialog();
                        savePDFDlg.DefaultExt = ".pdf";
                        savePDFDlg.Filter = "PDF Files *.pdf|*.pdf";
                        if (savePDFDlg.ShowDialog() == DialogResult.OK)
                        {
			    SX.NET.ISequentialStream outStream = sxFormsControl.Application.Factory.CreateWriteStream(savePDFDlg.FileName);
                	    invDoc.Save(outStream);
                        }
                    }
                    catch (SX.NET.Exception ex1)
                    {
                        ShowErrorMessage(ex1.get_ErrorCode());
                    }
                    catch (System.Exception ex2)
                    {
                        MessageBox.Show(ex2.Message);
                    }
                }
            }
        }

        private void createFromTextToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openXMLDlg = new OpenFileDialog();
            openXMLDlg.DefaultExt = ".xml";
            openXMLDlg.Filter = "XML Files *.xml|*.xml";
            openXMLDlg.RestoreDirectory = true;
            if (openXMLDlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    SX.NET.EInvoice.IInvoiceDocument invDoc = sxFormsControl.Application.Factory.CreateEInvoiceDocument();
		    SX.NET.ISequentialStream xmlStream = sxFormsControl.Application.Factory.CreateReadStream(openXMLDlg.FileName);
		    invDoc.Create(xmlStream, (uint)SX.NET.EInvoice.einv_options.einv_options_zugferd20); // use 0 for ZUGFeRD 1.0 and einv_options_facturx for FacturX
                    BuildPage(invDoc);

                    SaveFileDialog savePDFDlg = new SaveFileDialog();
                    savePDFDlg.DefaultExt = ".pdf";
                    savePDFDlg.Filter = "PDF Files *.pdf|*.pdf";
                    if (savePDFDlg.ShowDialog() == DialogResult.OK)
                    {
		        SX.NET.ISequentialStream outStream = sxFormsControl.Application.Factory.CreateWriteStream(savePDFDlg.FileName);
                        invDoc.Save(outStream);
                    }
                }
                catch (SX.NET.Exception ex1)
                {
                    ShowErrorMessage(ex1.get_ErrorCode());
                }
                catch (System.Exception ex2)
                {
                    MessageBox.Show(ex2.Message);
                }
            }
        }

	private void BuildPage(SX.NET.EInvoice.IInvoiceDocument pPDFInv)
	{
		SX.NET.EInvoice.IPage ptrPage = pPDFInv.CreatePage(595.3, 841.9);
		SX.NET.ISequentialStream ptrFile = sxFormsControl.Application.Factory.CreateReadStream("logo.png");
		if(null != ptrFile)
		{
			SX.NET.EInvoice.IRasterImage ptrLogo = ptrPage.LoadImageW(ptrFile);
			if(null != ptrLogo) ptrPage.PlaceImage(515, 30, 50, 50, ptrLogo);
		}
		SX.NET.EInvoice.ILineStyle ptrNL = ptrPage.CreateLineStyle(0.5);
		SX.NET.EInvoice.ILineStyle ptrBL = ptrPage.CreateLineStyle(1.5);
		SX.NET.EInvoice.ITextStyle ptrST = ptrPage.CreateTextStyle("Calibri", 8);
		SX.NET.EInvoice.ITextStyle ptrTT = ptrPage.CreateTextStyle("Calibri", 8, 700);
		SX.NET.EInvoice.ITextStyle ptrNT = ptrPage.CreateTextStyle("Calibri", 12);
		SX.NET.EInvoice.ITextStyle ptrBT = ptrPage.CreateTextStyle("Calibri", 12, 700);

		ptrPage.PlaceText(50, 70, "PDF Xpansion GmbH · Königsallee 45 · D-44789 Bochum", ptrST);
		ptrPage.PlaceLine(50, 82, 565, 82, ptrNL);

		ptrPage.PlaceText(50, 120, "Fachhochschule Dortmund\nSonnenstraße 96\n44139 Dortmund", ptrBT);

		ptrPage.PlaceText(50, 190, "Rechnung Nr. 1234567", ptrBT);

		ptrPage.PlaceLine(405, 185, 565, 185, ptrBL); ptrPage.PlaceLine(565, 185, 565, 240, ptrBL);
		ptrPage.PlaceLine(565, 240, 405, 240, ptrBL); ptrPage.PlaceLine(405, 240, 405, 185, ptrBL);
		ptrPage.PlaceText(410, 190, "Datum", ptrBT); ptrPage.PlaceText(560, 190, "15.04.2019", ptrNT, 2);
		ptrPage.PlaceText(410, 205, "Vertrag", ptrBT); ptrPage.PlaceText(560, 205, "V12-2345-678", ptrNT, 2);
		ptrPage.PlaceText(410, 220, "Lieferung", ptrBT); ptrPage.PlaceText(560, 220, "April 2019", ptrNT, 2);

		ptrPage.PlaceText(50, 270, "Wir bedanken uns für Ihre Treue und erlauben uns, Ihnen folgenden Betrag in Rechnung zu stellen:", ptrNT);

		ptrPage.PlaceLine(50, 305, 565, 305, ptrBL);
		ptrPage.PlaceText(50, 305, "Nr.    Produkt, Leistung", ptrBT);
		ptrPage.PlaceText(330, 305, "Einzelpreis, €                  Menge             Summe, €", ptrBT);
		ptrPage.PlaceLine(50, 322, 565, 322, ptrBL);
		ptrPage.PlaceText(50, 335, "         PDF Xpansion SDK", ptrBT);
		ptrPage.PlaceText(50, 350, "1       PDF-Datei lesen", ptrNT);
		ptrPage.PlaceText(330, 350, "1.000                                 1                               1.000", ptrNT);
		ptrPage.PlaceText(50, 365, "2       PDF-Datei speichern", ptrNT);
		ptrPage.PlaceText(330, 365, "1.000                                 1                               1.000", ptrNT);
		ptrPage.PlaceText(50, 380, "3       PDF-Dokumente trennen / zusammenfügen", ptrNT);
		ptrPage.PlaceText(330, 380, "500                                    1                               500", ptrNT);
		ptrPage.PlaceLine(50, 405, 565, 405, ptrBL);
		ptrPage.PlaceText(50, 405, "G e s a m t", ptrBT); ptrPage.PlaceText(565, 405, "2.500", ptrBT, 2);
		ptrPage.PlaceLine(50, 422, 565, 422, ptrBL);

		ptrPage.PlaceText(565, 430, "Nettobetrag                                  2.500 €", ptrNT, 2);
		ptrPage.PlaceText(565, 445, "MwSt, 19%                                       475 €", ptrNT, 2);
		ptrPage.PlaceText(565, 460, "Rechnungsbetrag                        2.975 €", ptrBT, 2);
		ptrPage.PlaceText(50, 490, "Zahlungsziel: 14 Tage Netto ohne Abzug", ptrNT);

		ptrPage.PlaceLine(50, 750, 565, 750, ptrNL);
		ptrPage.PlaceText(50, 755, "PDF Xpansion GmbH", ptrTT);
		ptrPage.PlaceText(250, 755, "Bankverbindung", ptrTT);
		ptrPage.PlaceText(450, 755, "Handelsregister", ptrTT);
		ptrPage.PlaceText(50, 770, "Königsallee 45, D-44789 Bochum\nTel. +49 (0)234-123 45 67\nFax +49 (0)234-123 45 89\nE-Mail: info@pdf-xpansion.com\nhttp://www.pdf-xpansion.com", ptrST);
		ptrPage.PlaceText(250, 770, "Deutsche Bank PGK AG, Bochum\nBLZ 430 700 24, Konto 1234567\nIBAN DE51 4307 0024 01234567 00\nUSt.-IdNr.: DE12345678\nSteuer Nr.: 123/4567/1234", ptrST);
		ptrPage.PlaceText(450, 770, "HRA 1234 (AG Bochum)\nGeschäftsführer: Frank Müller", ptrST);
	}
    }
}
