﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form, SX.NET.PDF.IStatusPDFA
    {
        SX.NET.PDF.IDocument mainDoc = null;
        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                mainDoc = doc;
                sxFormsControl.Document = doc.GetViewableDocument();               
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                mainDoc = null;
                sxFormsControl.Document = xpsPack.GetViewableDocument();
            }
        }

        private void mnuExportAsText_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "Text tiles (*.txt)|*.txt";
            StringBuilder sb = new StringBuilder();
            try
            {
                if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    uint pageCount = mainDoc.Pages.Count;
                    for (uint pageIndex = 0; pageIndex < pageCount; pageIndex++)
                    {
                        SX.NET.PDF.IPage page = mainDoc.Pages.Item(pageIndex);
                        sb.Append(page.GetPlainText((uint)(plain_text.plain_text_simulate_spaces | plain_text.plain_text_simulate_lines | plain_text.plain_text_skip_overlapped_chars | plain_text.plain_text_consider_ligatures)));
                        sb.Append(Environment.NewLine);
                    }
                }
                // save text to file
                System.IO.File.WriteAllText(dlg.FileName, sb.ToString());
            }
            catch (SX.NET.Exception ex)
            {
                ShowErrorMessage(ex.get_ErrorCode());
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }
            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void mnuExportAsImage_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF tiles (*.png)|*.png";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    uint pagesCount = mainDoc.Pages.Count;
                    for (uint pageIndex = 0; pageIndex < pagesCount; pageIndex++)
                    {
                        // get current page
                        SX.NET.PDF.IPage page = mainDoc.Pages.Item(pageIndex);
                        // transform Bitmap to PDF space
                        SX.NET.sx_matrix pageMatr = new sx_matrix();
                        page.CalcMatrix(ref pageMatr, 1, sx_rotation_angle.sx_rotation_angle_0, 0, 0, 72);
                        SX.NET.Graphics.IBitmapData ptrBitmap = page.CopyToBitmap(pageMatr, 0,0);
                        // save image
                        string safeName = string.Format("{0}\\{1}{2}.png",System.IO.Path.GetDirectoryName(dlg.FileName),System.IO.Path.GetFileNameWithoutExtension(dlg.FileName),pageIndex+1);
                        using(SX.NET.ISequentialStream saveStream = sxFormsControl.Application.Factory.CreateWriteStream(safeName))
                            ptrBitmap.SaveToStream(saveStream, SX.NET.Graphics.ImageFormat.ImageFormat_PNG, 100);
                    }
                }
                catch (SX.NET.Exception ex)
                {
                    ShowErrorMessage(ex.get_ErrorCode());
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            mnuConvertToPDFA.Enabled = mnuExportAsImage.Enabled = mnuExportAsText.Enabled = mainDoc != null;
        }

        private void convertToPDFAToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                mainDoc.ConformToPDFA(pdfa.pdfa_3a, 0, this);
                SaveFileDialog sfd = new SaveFileDialog();
                sfd.Filter = "PDFA Files *.pdf|*.pdf";
                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    mainDoc.SaveAsFile(sfd.FileName);
                }
            }
            catch(SX.NET.Exception ex)
            {
                if (ex.get_ErrorCode() != SX.NET.ErrorCode.ErrorNonConform)
                    ShowErrorMessage(ex.get_ErrorCode());
            }
        }

        public bool OnMessage(IMessagePDFA pMess)
        {
            if (((pMess.Type & (uint)SX.NET.PDF.pdfa_mess.pdfa_mess_cat)) == (uint)SX.NET.PDF.pdfa_mess.pdfa_mess_error)
            {
                uint messType = pMess.Type & (uint)SX.NET.PDF.pdfa_mess.pdfa_mess_type;
                MessageBox.Show("Document can’t converted, error " + messType.ToString(), "SxError", MessageBoxButtons.OK);
            }
            return true;
        }
    }
}
