﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        SX.NET.PDF.IDocument mainDoc;

        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
            sxFormsControl.Config.PDF.EnableFormFilling = true;
            sxFormsControl.Config.DrawOptions = (uint)(DrawOptEnum.draw_opt_highlight_fields | DrawOptEnum.draw_opt_highlight_req_fields | DrawOptEnum.draw_opt_empty_sign_icon | DrawOptEnum.draw_opt_sign_status_icon); 
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                CheckSignatures(doc.Properties.GetSignatures());

                mainDoc = doc;
                sxFormsControl.Document = doc.GetViewableDocument();               
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();
            }

            mainDoc = null;
        }

        private void CheckSignatures(SX.NET.PDF.ISignatures signatures)
        {
            SX.NET.PDF.IFormField formField = signatures.FirstSignature();
            while (formField != null)
            {
                ShowSignatureInfo(formField);
                formField = signatures.NextSignature(formField);
            }
        }

        private void ShowSignatureInfo(SX.NET.PDF.IFormField formField)
        {
            if (formField == null)
                return;

            SX.NET.PDF.IFieldSignature fieldSignature = (IFieldSignature)formField;
            SX.NET.PDF.ISignature signature = fieldSignature.GetSignature(false);

            if (signature != null && !string.IsNullOrEmpty(signature.Signer))
            {
                StringBuilder sb = new StringBuilder(signature.Signer);
                sb.Append(Environment.NewLine);
                sb.Append(" Status: ");
                switch (signature.Status)
                {
                    case SX.NET.PDF.signature_status.signature_status_invalid: sb.Append("invalid"); break;
                    case SX.NET.PDF.signature_status.signature_status_unknown: sb.Append("unknown"); break;
                    case SX.NET.PDF.signature_status.signature_status_valid: sb.Append("valid"); break;
                    case SX.NET.PDF.signature_status.signature_status_valid_for_revision: sb.Append("valid, but document was updated"); break;
                }
                MessageBox.Show(sb.ToString());
            }
        }

        private void signToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainDoc.Pages.Count == 0)
                return;

            SX.NET.PDF.IPage lastPage = mainDoc.Pages.Item(mainDoc.Pages.Count-1);
            
            //Create new Certificate
            try
            {
                SX.NET.ICryptoCert CryptoCert = sxFormsControl.Application.Factory.CreateCryptoCert();
                DateTime from = DateTime.Now;
                DateTime to = DateTime.Now.AddYears(1);
                CryptoCert.Create("Signature", from, to, (uint)sx_cert_ku.sx_cert_ku_dig_sign, false, null);

                //Add new signature
                SX.NET.PDF.ISignatures signatures = mainDoc.Properties.GetSignatures();
                SX.NET.PDF.IFormField formField = signatures.AddSignature("New signature");

                //Create widget for signature
                SX.NET.PDF.IWidget widget = lastPage.Annots.InsertNewWidget(formField);
                RectangleF rc = new RectangleF();
                lastPage.GetMediaBox(ref rc);
                rc.X += rc.Width * 3 / 4;   //right quarter
                rc.Y += rc.Height / 4;      //width quarter
                rc.Width = rc.Height = 250; //will be reaplaced by image size
                widget.SetRectArea(rc);

                //Get image path
                string imagePath = System.IO.Directory.GetParent(System.IO.Directory.GetParent(Application.StartupPath).FullName).FullName + "\\sign.png";

                //Set widget image
                SX.NET.Graphics.IBitmapData bmp = sxFormsControl.Application.Factory.CreateBitmapFromStream(sxFormsControl.Application.Factory.CreateReadStream(imagePath));
                SX.NET.PDF.IRasterImage rasterImage = mainDoc.Resources.AddRasterImage(bmp, Filter.Filter_JPEG, 80);
                SX.NET.PDF.ICompositeImage compositImage = mainDoc.Resources.AddCompositeImage(rasterImage);
                widget.SetAppearance(compositImage, false);

                SavePDF();
                sxFormsControl.Invalidate(false, true);
            }
            catch (SX.NET.Exception ex)
            {
                ShowErrorMessage(ex.get_ErrorCode());
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void SavePDF()
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF Documents (*.pdf)|*.pdf";
            dlg.DefaultExt = "pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    mainDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex)
                {
                    ShowErrorMessage(ex.get_ErrorCode());
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SavePDF();
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private bool IsPDF()
        {
            return mainDoc != null;
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            if (mainDoc != null)
            {
                signToolStripMenuItem.Enabled = true;
                saveToolStripMenuItem.Enabled = mainDoc.Properties.Modified;
            }
            else
            {
                signToolStripMenuItem.Enabled = saveToolStripMenuItem.Enabled = false;
            }
        }
    }
}
