﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        IApplication m_appl = PDFXpansionSDK.EntryPoint();
        SX.NET.JS.IJSApp m_jsApp;
        SX.NET.JS.IJSDoc m_jsDoc;
        SX.NET.PDF.IDocument pdfDoc;

        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
            sxFormsControl.Config.PDF.EnableFormFilling = true;
			// *********** JScript support ***********			
            try
            {
                m_jsApp = PDFXpansionSDK.JSEntryPoint().CreateApp(m_appl, LangCode2.LangCode2_en, "PDF Xpansion Reader", string.Empty, 12);
                try
                {
					SX.NET.PDF.IDocument pDoc = sxFormsControl.Application.Factory.CreatePDFDocument();
                    pDoc.OpenFile("forms.pdf", 0);
                    m_jsApp.AttachScript(pDoc, null);
                    m_jsApp.OnEvent += new SX.NET.JS.JSEvents(JS_OnEvent);

                    //sxFormsControl.OnEvent += new SX.NET.IViewerEvents(sxFormsControl_OnEvent);
		    sxFormsControl.Events.OnPDFAnnotEvent += new SX.NET.Viewer.EPDFAnnotEvent(sxFormsControl_OnAnnotEvent);
		    sxFormsControl.Events.OnDocActivity += new SX.NET.Viewer.EDocActivity(sxFormsControl_OnDocActivityEvent);

 		    sxFormsControl.Config.RelayDocEvents = true;
                }
                catch { m_jsApp = null; MessageBox.Show("Can't find forms.pdf", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error); }
                
            }
            catch { MessageBox.Show("Can't find pdf-xpansion-js-xXX.dll", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error); }
			// *********** JScript support ***********		
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

		pdfDoc = doc;

                if (!CheckSecurity(doc.Properties.Security))
                    return;
				// *********** JScript support ***********			
                if (m_jsApp != null)
                {
					doc.FormFields.RaiseEvents = true;
					doc.FormFields.AutoCalculate = true;
                    if (m_jsDoc != null)
                        m_jsApp.ReleaseDoc(m_jsDoc);
                    m_jsDoc = m_jsApp.CreateDoc(doc);
                    m_jsApp.AttachScript(doc, m_jsDoc);
                }
				// *********** JScript support ***********			
				sxFormsControl.Document = doc.GetViewableDocument();               
            }
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
       	    {
		pdfDoc = null;

                IPackage xpsPack = m_appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF documents (*.pdf)|*.pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    pdfDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex)
                {
                    ShowErrorMessage(ex.get_ErrorCode());
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
	}

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /*void sxFormsControl_OnEvent(IViewerEvent ve)
        {
		if (ve != null) switch(ve.Type)
		{
			// *********** JScript support ***********			
			case viewer_event.viewer_event_pdf:
			{
				SX.NET.Viewer.IViewerEvent_PDF pEvPDF = ve as SX.NET.Viewer.IViewerEvent_PDF;
				switch (pEvPDF.PDFType)
				{
					case SX.NET.PDF.pdf_event.pdf_event_annot_lost_focus:
					case SX.NET.PDF.pdf_event.pdf_event_annot_mouse_up:
					case SX.NET.PDF.pdf_event.pdf_event_annot_mouse_down:
					case SX.NET.PDF.pdf_event.pdf_event_annot_mouse_enter:
					case SX.NET.PDF.pdf_event.pdf_event_annot_mouse_exit:
					case SX.NET.PDF.pdf_event.pdf_event_annot_get_focus:
					{
						SX.NET.Viewer.IViewerEvent_PDF_Annot pEvAnnot = pEvPDF as SX.NET.Viewer.IViewerEvent_PDF_Annot;
						OnAnnotEvent(pEvAnnot);
					}
					break;
				}
			}
			break;
			case viewer_event.viewer_event_doc_activity:
			{
				SX.NET.Viewer.IViewerEvent_DocActivity pEvDoc = ve as SX.NET.Viewer.IViewerEvent_DocActivity;
				if (pEvDoc.DocEvent != null && pEvDoc.DocEvent.Type == doc_event.doc_event_form_field)
					OnFieldEvent((SX.NET.IDocFieldEvent)pEvDoc.DocEvent);
			}
			break;
			// *********** JScript support ***********			
		}
        }*/

		// *********** JScript support ***********			
        void sxFormsControl_OnAnnotEvent(IViewerEvent_PDF_Annot pEvAnnot)
        {
		SX.NET.PDF.IPDFActions actions = pEvAnnot.Annot.GetEventActions(pEvAnnot.PDFType);
		if (actions != null)
		{
			for (uint i = 0; i < actions.Count; i++)
			{
				SX.NET.PDF.IPDFAction action = actions.Item(i);
				if (action != null)
				{
					switch (action.Type)
					{
					case pdf_action.pdf_action_exec_jscript:
						{
							string script = ((SX.NET.PDF.IActionJScript)action).JScript;
							m_jsApp.AttachScript(script, m_jsDoc, pEvAnnot.PDFEvent);
							if(!pEvAnnot.Handled) pEvAnnot.Handled = true;
						}
						break;
					case pdf_action.pdf_action_reset_form:
						{
							pdfDoc.FormFields.ResetValues();
							pdfDoc.FormFields.ResetAppearances(false);
				                	sxFormsControl.Invalidate(true, true);
						}

						break;
					}
				}       
			}
		}
        }

        void sxFormsControl_OnDocActivityEvent(IViewerEvent_DocActivity pEvDocActivity)
        {
		if (pEvDocActivity.DocEvent != null && pEvDocActivity.DocEvent.Type == doc_event.doc_event_form_field)
		{
			OnFieldEvent((SX.NET.IDocFieldEvent)pEvDocActivity.DocEvent);
		}
	}

        private void OnFieldEvent(IDocFieldEvent pEvField)
        {
            SX.NET.PDF.IPDFActions actions = pEvField.Field.GetEventActions(pEvField.PDFEvent.Type, false);
            if (actions != null)
            {
                for (uint i = 0; i < actions.Count; i++)
                {
                    SX.NET.PDF.IPDFAction action = actions.Item(i);
                    if (action != null && action.Type == pdf_action.pdf_action_exec_jscript)
                    {
                        string script = ((SX.NET.PDF.IActionJScript)action).JScript;
                        m_jsApp.AttachScript(script, m_jsDoc, pEvField.PDFEvent);
			if(!pEvField.Handled) pEvField.Handled = true;
		    }
                }
            }
        }

        bool JS_OnEvent(SX.NET.JS.IJSEvent e)
        {
		switch (e.Type)
		{
		case SX.NET.JS.js_event.js_event_invalidate:
	                sxFormsControl.Invalidate(true, true);
			break;
		case SX.NET.JS.js_event.js_event_app_escript:
			switch ((e as SX.NET.JS.IJSEvEScript).IDS)
			{
			case SX.NET.JS.js_escript_str.js_escript_str_ge:
				(e as SX.NET.JS.IJSEvEScript).SetString("Ungültiger Wert: nur Werte größer oder gleich %s sind gültig.");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_gt_lt:
				(e as SX.NET.JS.IJSEvEScript).SetString("Ungültiger Wert: nur Werte größer oder gleich %s und kleiner oder gleich %s sind gültig.");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_le:
				(e as SX.NET.JS.IJSEvEScript).SetString("Ungültiger Wert: nur Werte kleiner oder gleich %s sind gültig.");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_inv_mnt:
				(e as SX.NET.JS.IJSEvEScript).SetString("** Ungültig **");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_inv_date:
				(e as SX.NET.JS.IJSEvEScript).SetString("Datum oder Zeit ungültig. Bitte prüfen Sie Ihre Angaben. Feld");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_inv_date2:
				(e as SX.NET.JS.IJSEvEScript).SetString("sollte folgendes Format haben");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_inv_value:
				(e as SX.NET.JS.IJSEvEScript).SetString("Das Format des eingegebenen Werts passt nicht zum Feldformat.");
				break;
			case SX.NET.JS.js_escript_str.js_escript_str_month_info:
				(e as SX.NET.JS.IJSEvEScript).SetString("Januar[1]Februar[2]März[3]April[4]Mai[5]Juni[6]Juli[7]August[8]September[9]Oktober[10]November[11]Dezember[12]Sept[9]Jan[1]Feb[2]Mrz[3]Apr[4]Jun[6]Jul[7]Aug[8]Sep[9]Okt[10]Nov[11]Dez[12]");
				break;
			}
			break;			
		case SX.NET.JS.js_event.js_event_wsh_error:
			{
				SX.NET.JS.IJSEvError pJSEvError = e as SX.NET.JS.IJSEvError;
				string desc = pJSEvError.Desc;
				string src = pJSEvError.Source;
			        MessageBox.Show(this, desc, "Line: " + pJSEvError.Line.ToString() + ", Char: " + pJSEvError.Char.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error);			
			}
			break;
		case SX.NET.JS.js_event.js_event_app_alert:
			{
				string msg = (e as SX.NET.JS.IJSEvAlert).Msg;
				string title = (e as SX.NET.JS.IJSEvAlert).Title;
			        MessageBox.Show(this, msg, title, MessageBoxButtons.OK, MessageBoxIcon.None);
			}
			break;
		}
		return true;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
		m_jsApp.ReleaseApp();
        }
		// *********** JScript support ***********			
    }
}
