﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace PDF_Xpansion_Reader
{
    public partial class frmOutlinesDialog : Form
    {
        SX.NET.Viewer.IViewerCanvas m_pCanvas;
        SX.NET.PDF.IDocument m_pDoc;
        List<SX.NET.PDF.IOutlineItem> m_Outlines;

        public frmOutlinesDialog(SX.NET.PDF.IDocument pDoc, SX.NET.Viewer.IViewerCanvas pCanvas)
        {
            InitializeComponent();
            m_pDoc = pDoc;
            m_pCanvas = pCanvas;
        }

        private void frmOutlinesDialog_Load(object sender, EventArgs e)
        {
            btnAddNew.Enabled = btnGoTo.Enabled = false;
            LoadDocumentOutlines();
        }

        private void btnAddNew_Click(object sender, EventArgs e)
        {
            frmAddNewOutline dlgAdd = new frmAddNewOutline(m_pDoc.Pages.Count);
            if (dlgAdd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                SX.NET.PDF.IOutlineItem CurrOutline = tvOutline.SelectedNode.Tag as SX.NET.PDF.IOutlineItem;
                if (CurrOutline != null)
                {
                    SX.NET.PDF.IOutlineItem ChildOutline = CurrOutline.InsertChild(dlgAdd.Title, null);
                    SX.NET.PDF.IDest pDest = ChildOutline.ResetDest();
                    pDest.Page = m_pDoc.Pages.Item(dlgAdd.PageNumber);

                    TreeNode node = AddTreeItem(dlgAdd.Title, ChildOutline, tvOutline.SelectedNode);
                    node.Expand();
                }
            }
        }

        private void btnGoTo_Click(object sender, EventArgs e)
        {
            SX.NET.PDF.IOutlineItem CurrOutline = tvOutline.SelectedNode.Tag as SX.NET.PDF.IOutlineItem;
            if (CurrOutline != null)
                   m_pCanvas.Navigate(CurrOutline); 
            this.Close();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void LoadDocumentOutlines()
        {
            SX.NET.PDF.IOutlineItem pRootOutline = m_pDoc.Outline;
            AddTreeItem("Root", pRootOutline);
            AddChilds(pRootOutline, tvOutline.Nodes[0]);
            tvOutline.ExpandAll();
        }

        TreeNode AddTreeItem(string text, SX.NET.PDF.IOutlineItem pOutline, TreeNode parentNode = null)
        {
            TreeNode node = new TreeNode(text);
            node.Tag = pOutline;
            if (parentNode != null)
                parentNode.Nodes.Add(node);
            else
                tvOutline.Nodes.Add(node);
            return node;
        }

        private void AddChilds(SX.NET.PDF.IOutlineItem pParentOutline, TreeNode pParentNode)
        {
            SX.NET.PDF.IOutlineItem CurOutline = pParentOutline.FirstChild();
            while (CurOutline != null)
            {
                TreeNode node = AddTreeItem(CurOutline.Title, CurOutline, pParentNode);
                AddChilds(CurOutline, node);
                CurOutline = pParentOutline.NextChild(CurOutline);
            }
        }

        private void tvOutline_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (tvOutline.SelectedNode == null)
            {
                btnAddNew.Enabled = btnGoTo.Enabled = false;
            }
            else
            {
                btnAddNew.Enabled = true;
                btnGoTo.Enabled = tvOutline.Nodes[0] != tvOutline.SelectedNode && HasDestination(tvOutline.SelectedNode.Tag as SX.NET.PDF.IOutlineItem);
            }
        }

        private bool HasDestination(SX.NET.PDF.IOutlineItem pOutline)
        {
            if (pOutline.Dest != null)
                return true;

            if (m_pDoc.NamedDests.Get(pOutline.NamedDest) != null)
                return true;

            SX.NET.PDF.IPDFActions pActions = pOutline.GetActions();
            for (uint i = 0; i < pActions.Count; i++)
            {
                SX.NET.PDF.IPDFAction pAction = pActions.Item(i);
                if (pAction.Type == SX.NET.PDF.pdf_action.pdf_action_goto)
                {
                    SX.NET.PDF.IActionGoTo pActionGoTo = (SX.NET.PDF.IActionGoTo)pAction;
                    if (pActionGoTo.Dest != null)
                        return true;

                    if (m_pDoc.NamedDests.Get(pActionGoTo.NamedDest) != null)
                        return true;
                }
            }
            return false;
        }
    }
}
