﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;
using System.Runtime.InteropServices;

static class NativeMethods
{
    private const string Kernel32 = "kernel32.dll";

    [DllImport(Kernel32, SetLastError = true, ExactSpelling = true, CharSet = CharSet.Auto)]
    public static extern IntPtr GlobalLock(IntPtr handle);

    [DllImport(Kernel32, SetLastError = true, ExactSpelling = true, CharSet = CharSet.Auto)]
    public static extern bool GlobalUnlock(IntPtr handle);
}
namespace PDF_Xpansion_Reader
{

    public partial class Form1 : Form
    {
        IBaseDocument baseDocument;

        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);
                
                if (!CheckSecurity(doc.Properties.Security))
                    return;

                baseDocument = doc;

                sxFormsControl.Document = doc.GetViewableDocument();               
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                baseDocument = xpsPack;
                sxFormsControl.Document = xpsPack.GetViewableDocument();
            }
        }

        private void Print(SX.NET.IBaseDocument baseDoc)
        {
            PrintDialog pDialog = new PrintDialog() { AllowSomePages = true, AllowCurrentPage = true };
            //Set paper size
            pDialog.Document = new System.Drawing.Printing.PrintDocument();
            pDialog.Document.PrinterSettings.MinimumPage = 1;
            pDialog.Document.PrinterSettings.MaximumPage = int.MaxValue;
            pDialog.Document.PrinterSettings.FromPage = 1;
            pDialog.Document.PrinterSettings.ToPage = (int)sxFormsControl.Document.GetPageCount();
            System.Drawing.Printing.PaperSize ps = new System.Drawing.Printing.PaperSize();
            ps.RawKind = (int)System.Drawing.Printing.PaperKind.Letter;
            pDialog.Document.DefaultPageSettings.PaperSize =  ps;
            
            if (pDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {

                IntPtr hDevModePtr = pDialog.PrinterSettings.GetHdevmode();
                try
                {
                    IntPtr ptrDevMode = NativeMethods.GlobalLock(hDevModePtr);
                    try
                    {
                        IPrintProvider printProvider = baseDoc.GetPrintProvider("Print document", pDialog.PrinterSettings.PrinterName, ptrDevMode);
                        //Set pages range (if required)
                        switch (pDialog.PrinterSettings.PrintRange)
                        {
                            case System.Drawing.Printing.PrintRange.SomePages:
                                printProvider.PageSet = print_page_set.print_page_set_range;
                                if (pDialog.PrinterSettings.FromPage != pDialog.PrinterSettings.ToPage)
                                    printProvider.PageRange = string.Format("{0}-{1}", pDialog.PrinterSettings.FromPage, pDialog.PrinterSettings.ToPage);
                                else
                                    printProvider.PageRange = pDialog.PrinterSettings.FromPage.ToString();
                                break;
                            case System.Drawing.Printing.PrintRange.CurrentPage:
                                printProvider.PageSet = print_page_set.print_page_set_range;
                                printProvider.PageRange = string.Format(sxFormsControl.Canvas.ActivePage.ToString());
                                break;
                            default:
                                printProvider.PageSet = print_page_set.print_page_set_all;
                                break;
                        }

                        printProvider.StartPrint();

                    }
                    finally
                    {
                        NativeMethods.GlobalUnlock(hDevModePtr);
                    }
                }
                finally
                {
                    Marshal.FreeHGlobal(hDevModePtr);
                }
                
                
            }
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (baseDocument == null)
            {
                MessageBox.Show("File not open");
                return;
            }

            Print(baseDocument);
        }

        SX.NET.Layout.IDecoElement CreatePageDecorator(SX.NET.Layout.IDocument layoutDoc)
        {
            SX.NET.Graphics.IMatrix matrix = sxFormsControl.Application.Factory.Matrix();

            // retrieve size of first page
            SX.NET.Viewer.IViewableDocument viewDoc = baseDocument.GetViewableDocument();
            SizeF size = new SizeF();
            viewDoc.GetPageSize(0, ref size);

            // create header decorator
            SX.NET.Layout.IDecoElement deco = layoutDoc.CreateDeco();
            SX.NET.Content.IRichCollection coll = deco.Objects;
            // add autotext to show document title
            SX.NET.Content.IRichText headerText = (SX.NET.Content.IRichText)coll.Insert(coll.Count, SX.NET.Content.rich_type.rich_type_text);
            SX.NET.Content.ITextStyle headerTextStyle = headerText.GetDefaultStyle();
            headerTextStyle.SetFont(new sx_color_i_rgb(0, 0, 0), "Times", 14, 700, false,1); // times new roman, 14, bold
            // E004 - document title
            string sPattern = string.Format("Document title {0}",  (char)0xE004);
            headerText.GetAutoTextPattern().Pattern = sPattern;
            headerText.GetAutoTextPattern().Alignment = SX.NET.Content.text_alignment.text_alignment_center;
            SX.NET.sx_matrix mx = new sx_matrix(); matrix.Identity(ref mx);
            matrix.Translate(ref mx, size.Width / 2, 20, false);
            headerText.ApplyTransformation(mx);

            // add autotext to show page number
            SX.NET.Content.IRichText footerText = (SX.NET.Content.IRichText)coll.Insert(coll.Count, SX.NET.Content.rich_type.rich_type_text);
            SX.NET.Content.ITextStyle footerTextStyle = footerText.GetDefaultStyle();
            footerTextStyle.SetFont(new sx_color_i_rgb(0, 0, 0), "Times", 12, 400, false,1); // times new roman, 12, normal
            // E001 - page number, E003 - page count
            sPattern = string.Format("Page {0} of {1}", (char)0xE001, (char)0xE003);
            footerText.GetAutoTextPattern().Pattern = sPattern;
            footerText.GetAutoTextPattern().Alignment = SX.NET.Content.text_alignment.text_alignment_right;
            matrix.Identity(ref mx);
            matrix.Translate(ref mx, size.Width - 10, size.Height - 12, false);
            footerText.ApplyTransformation(mx);

            SX.NET.Content.IRichVector vect = (SX.NET.Content.IRichVector)coll.Insert(coll.Count, SX.NET.Content.rich_type.rich_type_vector);
            SX.NET.Content.IPath path = deco.CreatePath();
            PointF pt1 = new PointF(0, 30), pt2 = new PointF((float)size.Width, 30);
            path.AddLine(pt1, pt2);
            vect.Path = path;
            SX.NET.Content.ISolidBrush brush = deco.CreateSolidBrush(new sx_color_i_rgb(197, 197, 197)); // gray color
            SX.NET.Content.IPen pen = deco.CreatePen(1);  // pen width = 1 pt
            vect.Pen = pen; vect.StrokeBrush = brush;

            pt1.Y = pt2.Y = (float)(size.Height - 30);
            SX.NET.Content.IRichVector vect2 = (SX.NET.Content.IRichVector)coll.Insert(coll.Count, SX.NET.Content.rich_type.rich_type_vector);
            SX.NET.Content.IPath path2 = deco.CreatePath();
            path2.AddLine(pt1, pt2);
            vect2.Path = path2;
            vect2.Pen = pen;
            vect2.StrokeBrush = brush;

            return deco;
        }

        private void printWithHeaderFooterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (baseDocument == null)
            {
                MessageBox.Show("File not open");
                return;
            }

            // create layout document
            SX.NET.Layout.IDocument layoutDoc = sxFormsControl.Application.Factory.CreateLayoutDocument();
            layoutDoc.SetRefDoc(baseDocument);
            SX.NET.Layout.IBasicProvider provider = (SX.NET.Layout.IBasicProvider)layoutDoc.CreateProvider(SX.NET.Layout.prov_type.prov_type_basic);
            SX.NET.Layout.IDecoElement deco = CreatePageDecorator(layoutDoc);
            provider.SetOverlay(deco);
            provider.Start(0);

            Print(layoutDoc);
        }

    }
}
