﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        SX.NET.PDF.IDocument mainDoc;
        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                sxFormsControl.Document = doc.GetViewableDocument();
                mainDoc = doc;
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();
                mainDoc = null;
            }
        }

        private void mnuChangeSecurity_Click(object sender, EventArgs e)
        {
            SX.NET.PDF.IPasswordSecurity passSec;
            SX.NET.PDF.encryption_algorithm encryptAlg;
            SX.NET.PDF.encryption_key_length keyLength;

            // get current document security
            SX.NET.PDF.IDocSecurity docSecurity = mainDoc.Properties.Security;
            switch (docSecurity.Security)
            {
                case doc_security.doc_security_password:
                    passSec = (SX.NET.PDF.IPasswordSecurity)docSecurity.StandardSecurity;
                    encryptAlg = passSec.EncryptionAlgorithm;
                    keyLength = passSec.EncryptionKeyLength;
                    break;    
                case doc_security.doc_security_none:
                    encryptAlg = encryption_algorithm.encryption_algorithm_none;
                    keyLength = encryption_key_length.encryption_key_length_unknown;
                    break;
                default:
                    MessageBox.Show("This type of encryption is not supported", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
            }

            // set new security params
            SecurityDialog dlg = new SecurityDialog(encryptAlg, keyLength);
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                return;

            // set new security
            try
            {
                docSecurity.ChangeSecurity(SX.NET.PDF.doc_security.doc_security_password);
                passSec = (SX.NET.PDF.IPasswordSecurity)docSecurity.StandardSecurity;

                passSec.EncryptionAlgorithm = dlg.Algorithm;
                passSec.EncryptionKeyLength = dlg.KeyLength;
                passSec.SetOwnerPassword(dlg.OwnderPassword);
                passSec.SetUserPassword(dlg.UserPassword);

                // set document permissions
                passSec.Permissions = (uint)(doc_permissions.doc_permissions_normal_print | doc_permissions.doc_permissions_modify_annots | doc_permissions.doc_permissions_fill_form);
            }
            catch (SX.NET.Exception ex1)
            {
                ShowErrorMessage(ex1.get_ErrorCode());
            }
            catch (System.Exception ex2)
            {
                MessageBox.Show(ex2.Message);
            }
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void mnuSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF Document|*.pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    mainDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex1)
                {
                    ShowErrorMessage(ex1.get_ErrorCode());
                }
                catch (System.Exception ex2)
                {
                    MessageBox.Show(ex2.Message);
                }
            }
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            mnuChangeSecurity.Enabled = mnuSave.Enabled = mainDoc != null;
        }
    }
}
