﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        SX.NET.PDF.IDocument mainDoc;
	SX.NET.PDF.IAnnot    overAnnot;

        // Viewer tools
        const byte sx_tool_change = 1;
		const byte sx_tool_start = 2;
		const byte sx_tool_apply = 3;
        const byte sx_tool_cancel = 4;
		const byte sx_tool_change_page = 5;

        SX.NET.Viewer.tracker_type trackerType = tracker_type.tracker_type_rect;

        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.ToolMode = tool_mode.tool_mode_none;

            // set pdf settings
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_conform;
            sxFormsControl.Config.PDF.AnnotsVisible = sxFormsControl.Config.PDF.AnnotsVisible | (uint)SX.NET.PDF.annot_type.annot_markup;
            sxFormsControl.Config.PDF.AnnotsSensitive = sxFormsControl.Config.PDF.AnnotsSensitive | (uint)SX.NET.PDF.annot_type.annot_markup;
            sxFormsControl.Config.PDF.AnnotsFocusable = sxFormsControl.Config.PDF.AnnotsFocusable | (uint)SX.NET.PDF.annot_type.annot_markup;

            sxFormsControl.Events.OnTool += new SX.NET.Viewer.ETool(sxFormsControl_OnToolEvent);
            sxFormsControl.Events.OnMouse += new SX.NET.Viewer.EMouse(sxFormsControl_OnMouseEvent);
        }

        void sxFormsControl_OnToolEvent(IViewerEvent_ToolActivity pToolActivity)
        {
            if (mainDoc == null)
                return;

            SX.NET.Viewer.IToolState toolState = pToolActivity.State;

            if (toolState.ToolMode == tool_mode.tool_mode_tracker)
            {
                SX.NET.Viewer.IToolTracker toolTracker = (SX.NET.Viewer.IToolTracker)toolState;
                if (toolState.Activity == sx_tool_apply)
                {
                    try
                    {
                        SX.NET.PDF.IPage page = mainDoc.Pages.Item(toolTracker.Page);
                        SX.NET.PDF.IMarkupAnnot annot = null;

                        switch (toolTracker.Type)
                        {
                            //Add Sticky Note annotation 
                            case tracker_type.tracker_type_rect_brush:
                                PointF point = new PointF();
                                toolTracker.GetEndPoint(ref point);
                                annot = AddStickyNote(point, page);
                                break;
                            //Add Stamp annotation
                            case tracker_type.tracker_type_rect:
                                PointF a = new PointF(), b = new PointF();
                                toolTracker.GetBeginPoint(ref a);
                                toolTracker.GetEndPoint(ref b);
                                RectangleF rect = PointsToRect(a, b);
                                annot = AddStamp(rect, page);
                                break;
                        }

                        if (annot != null)
                        {
                            page.Annots.ResetPopup(annot, false);
                            AddPopup(annot);
                            sxFormsControl.ToolMode = tool_mode.tool_mode_none;
                            sxFormsControl.Invalidate(true, true);
                        }

                    }
                    catch (SX.NET.Exception ex1)
                    {
                        ShowErrorMessage(ex1.get_ErrorCode());
                        return;
                    }
                    catch (System.Exception ex2)
                    {
                        MessageBox.Show(ex2.Message);
                        return;
                    }
                }

                if (toolState.Activity == sx_tool_change)
                {
                    switch (toolTracker.Type)
                    {
                        case SX.NET.Viewer.tracker_type.tracker_type_rect_brush:
                            toolTracker.BrushSize = 1;
                            toolTracker.SetColor(new SX.NET.sx_color_i_rgb(205, 205, 0));
                            break;
                        case SX.NET.Viewer.tracker_type.tracker_type_rect:
                            toolTracker.Width = 2;
                            break;
                    }
                    toolTracker.Type = trackerType;
                }
            }
        }

        void sxFormsControl_OnMouseEvent(IViewerEvent_MouseActivity pMouseActivity)
        {
            if (mainDoc == null)
                return;

            switch (pMouseActivity.Activity)
            {
                case mouse_flags.mouse_flags_move:
                    PointF CanvasPt = new PointF();
                    PointF PagePt = new PointF();

                    overAnnot = null;
                    IViewerCanvas pCanvas = sxFormsControl.Canvas;
                    // get point at canvas
                    CanvasPt.X = pMouseActivity.X;
                    CanvasPt.Y = pMouseActivity.Y;

                    if (pCanvas.Width < sxFormsControl.Width)
                        CanvasPt.X -= (sxFormsControl.Width - pCanvas.Width) / 2;
                    else
                        CanvasPt.X += pCanvas.HorizontalOffset;

                    if (pCanvas.Height < sxFormsControl.Height)
                        CanvasPt.Y -= (sxFormsControl.Height - pCanvas.Height) / 2;
                    else
                        CanvasPt.Y += pCanvas.VerticalOffset;


                    // find page under pointer
                    uint PageIndex = pCanvas.GetPageIndex((uint)CanvasPt.X, (uint)CanvasPt.Y);
                    if ((SX.NET.page_index)PageIndex != SX.NET.page_index.INVALID_INDEX)
                    {
                        // canvas to normalized page 
                        pCanvas.CanvasToPage(PageIndex, ref CanvasPt, ref PagePt);
                        // normalized page to PDF page



                        SX.NET.sx_matrix mx = new sx_matrix();
                        SX.NET.Graphics.IMatrix pMatrix = sxFormsControl.Application.Factory.Matrix();




                        SX.NET.PDF.IPage ptrPage = mainDoc.Pages.Item(PageIndex);
                        ptrPage.CalcMatrix(ref mx, 1.0, sx_rotation_angle.sx_rotation_angle_0, 0, 0, 72);
                        pMatrix.Invert(ref mx);
                        pMatrix.TransformPoint(mx, ref PagePt);
                        // find an annot under cursor
                        overAnnot = ptrPage.Annots.FindItem(PagePt, (uint)SX.NET.PDF.annot_type.annot_markup);
                    }
                    break;
                case mouse_flags.mouse_flags_cursor:
                    if (null != overAnnot)
                        this.Cursor = Cursors.Hand;
                    else
                        this.Cursor = Cursors.Default;
                        pMouseActivity.Cancel();
                    break;
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
	    overAnnot = null;
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                sxFormsControl.Document = doc.GetViewableDocument();
                mainDoc = doc;
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();
            }

            mainDoc = null;
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private RectangleF PointsToRect(PointF a, PointF b)
        {
            RectangleF rect = new RectangleF();
            rect.X = Math.Min(a.X, b.X);
            rect.Y = Math.Min(a.Y, b.Y);
            rect.Width = Math.Abs(a.X - b.X);
            rect.Height = Math.Abs(a.Y - b.Y);
            return rect;
        }

        private void AddPopup(SX.NET.PDF.IMarkupAnnot annot)
        {
            SX.NET.PDF.IPopup popup = annot.Popup;
            RectangleF annotRect = new RectangleF(), popupRect = new RectangleF();
            annot.GetRectArea(ref annotRect);
            popupRect.Height = 300;             //will be replaced by popup height
            popupRect.Width = 400;              //will be replaced by popup width
            popupRect.X = annotRect.X + annotRect.Width;
            popupRect.Y = annotRect.Y + annotRect.Height - popupRect.Height;
            popup.SetRectArea(popupRect);
        }

        private SX.NET.PDF.IMarkupAnnot AddStickyNote(PointF point, SX.NET.PDF.IPage page)
        {
            SX.NET.PDF.IStickyNote note = (SX.NET.PDF.IStickyNote)(page.Annots.InsertNew(SX.NET.PDF.annot_type.annot_type_text));
            note.Description = "Sticky note body";
            note.Author = "Author";
            note.Subject = "Sticky note title";
            note.Icon = sticky_icon.sticky_icon_help;
            note.SetColor(new sx_color_i_rgb(255, 201, 14));

            //transform annotation region to PDF space
            SX.NET.sx_matrix matr = new sx_matrix();
            page.CalcMatrix(ref matr, 1, sx_rotation_angle.sx_rotation_angle_0, 0, 0, 72);

            SX.NET.Graphics.IMatrix pMatrix = sxFormsControl.Application.Factory.Matrix();
            pMatrix.Invert(ref matr);
            pMatrix.TransformPoint(matr, ref point);

            RectangleF rect = new RectangleF();
            rect.Height = rect.Width = 30;      //will be replaced by note size
            rect.X = point.X - rect.Width /2;
            rect.Y = point.Y - rect.Height /2;

            // reset appearance of annotation
            note.SetRectArea(rect);
            note.ResetAppearance();
            return note;
        }

        private SX.NET.PDF.IMarkupAnnot AddStamp(RectangleF rect, SX.NET.PDF.IPage page)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Image files |*.png";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                //Add image to resources
                SX.NET.Graphics.IBitmapData bmp = sxFormsControl.Application.Factory.CreateBitmapFromStream(sxFormsControl.Application.Factory.CreateReadStream(dlg.FileName));
                SX.NET.PDF.IRasterImage rasterImage = mainDoc.Resources.AddRasterImage(bmp, Filter.Filter_JPEG, 80);
                SX.NET.PDF.ICompositeImage compositImage = mainDoc.Resources.AddCompositeImage(rasterImage);

                //Create stamp
                SX.NET.PDF.IStamp stamp = (SX.NET.PDF.IStamp)(page.Annots.InsertNew(SX.NET.PDF.annot_type.annot_type_stamp));
                stamp.SetModDate(DateTime.Now);
                stamp.Description = "Stamp description";
                stamp.Author = "Author";
                stamp.Subject = "Stamp title";
                stamp.SetAppearance(compositImage, false);

                //Transform annotation region to PDF space
                SX.NET.sx_matrix pageMAtrix = new sx_matrix();
                page.CalcMatrix(ref pageMAtrix,1, sx_rotation_angle.sx_rotation_angle_0, 0,0,72);

                SX.NET.Graphics.IMatrix pMatrix = sxFormsControl.Application.Factory.Matrix();
                pMatrix.Invert(ref pageMAtrix);
                pMatrix.TransformRect(pageMAtrix, ref rect);
                
                stamp.SetRectArea(rect);
                return stamp;
            }

            return null;
        }

        private void addStickyNoteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            trackerType = tracker_type.tracker_type_rect_brush;
            sxFormsControl.ToolMode = tool_mode.tool_mode_tracker;
        }

        private void addStampToolStripMenuItem_Click(object sender, EventArgs e)
        {
            trackerType = tracker_type.tracker_type_rect;
            sxFormsControl.ToolMode = tool_mode.tool_mode_tracker;
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF Document|*.pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    mainDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex1)
                {
                    ShowErrorMessage(ex1.get_ErrorCode());
                }
                catch (System.Exception ex2)
                {
                    MessageBox.Show(ex2.Message);
                }
            }
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            saveToolStripMenuItem.Enabled = (mainDoc != null && mainDoc.Properties.Modified);
            addStickyNoteToolStripMenuItem.Enabled = addStampToolStripMenuItem.Enabled = (mainDoc != null);
        }


    }
}
