﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        SX.NET.PDF.IDocument mainDoc;

        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_single;
            sxFormsControl.ToolMode = tool_mode.tool_mode_select_text;
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                sxFormsControl.Document = doc.GetViewableDocument();
                mainDoc = doc;
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();
            }

            mainDoc = null;
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Clipboard.Clear();
                Clipboard.SetText(sxFormsControl.Canvas.MarkedText);
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void AddTextMarkup(SX.NET.PDF.annot_type annotType)
        {
            //Set annotation color
            SX.NET.sx_color_i_rgb cFill = new sx_color_i_rgb();
            switch (annotType)
            {
                case SX.NET.PDF.annot_type.annot_type_highlight: cFill = new sx_color_i_rgb(255, 255, 0); break;
                case SX.NET.PDF.annot_type.annot_type_underline: cFill = new sx_color_i_rgb(0, 0, 255); break;
                case SX.NET.PDF.annot_type.annot_type_strikeout: cFill = new sx_color_i_rgb(255, 0, 0); break;
                default: cFill = new sx_color_i_rgb(0, 0, 0); break;
            }

            try
            {
                // get marked page
                uint activePage = sxFormsControl.Canvas.ActivePage;
                SX.NET.PDF.IPage page = mainDoc.Pages.Item(activePage);

                // create markup annotation
                SX.NET.PDF.ITextMarkup annot = (SX.NET.PDF.ITextMarkup)page.Annots.InsertNew(annotType);

                // transform marked region to PDF space
                SX.NET.sx_matrix mx = new sx_matrix();
                page.CalcMatrix(ref mx, 1, sx_rotation_angle.sx_rotation_angle_0, 0, 0, 72);
                sxFormsControl.Application.Factory.Matrix().Invert(ref mx);

                SX.NET.Graphics.IRegion region = sxFormsControl.Canvas.MarkingRegion;
                region = region.Copy(mx, true);

                // reset appearance of annotation
                annot.SetColor(cFill);
                annot.SetRegion(region);

                // refresh viewer
                sxFormsControl.Canvas.ResetMarking(null, uint.MaxValue);
                sxFormsControl.Invalidate(true, true);
            }
            catch (SX.NET.Exception ex)
            {
                ShowErrorMessage(ex.get_ErrorCode());
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF Documents (*.pdf)|*.pdf";
            dlg.DefaultExt = "pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    mainDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex)
                {
                    ShowErrorMessage(ex.get_ErrorCode());
                }
                catch (System.Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void MarkupText(object sender, EventArgs e)
        {
            string mnuName = ((ToolStripMenuItem)sender).Name;
            switch (mnuName)
            {
                case "mnuHighlight":
                    AddTextMarkup(SX.NET.PDF.annot_type.annot_type_highlight);
                    break;
                case "mnuUnderline":
                    AddTextMarkup(SX.NET.PDF.annot_type.annot_type_underline);
                    break;
                case "mnuStrikeout":
                    AddTextMarkup(SX.NET.PDF.annot_type.annot_type_strikeout);
                    break;
            }
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            saveToolStripMenuItem.Enabled = mainDoc != null && mainDoc.Properties.Modified;
            mnuHighlight.Enabled = mnuStrikeout.Enabled = mnuUnderline.Enabled = (mainDoc != null && sxFormsControl.Canvas.MarkingType == marking_type.marking_type_text);
            copyToolStripMenuItem.Enabled = sxFormsControl.Canvas.MarkingType == marking_type.marking_type_text ;
        }
    }
}
