﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SX.NET;
using SX.NET.PDF;
using SX.NET.Viewer;
//using SX.NET.XPS;

namespace PDF_Xpansion_Reader
{
    public partial class Form1 : Form
    {
        const uint INVALID_INDEX = 0xFFFFFFFF;
        SX.NET.PDF.IDocument mainDoc;
        public Form1()
        {
            InitializeComponent();           
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // configurate the viewer here
            sxFormsControl.Config.UseCache = false;
            sxFormsControl.Config.PageCacheSize = 50;
            sxFormsControl.Config.PageCacheInvalidation = 300;
            sxFormsControl.ToolMode = tool_mode.tool_mode_none;

            // set pdf settings
            sxFormsControl.Config.PDF.Popups = viewer_popups.viewer_popups_conform;
            sxFormsControl.Config.PDF.AnnotsVisible = sxFormsControl.Config.PDF.AnnotsVisible | (uint)annot_type.annot_type_text | (uint)annot_type.annot_type_stamp;
            sxFormsControl.Config.PDF.AnnotsSensitive = sxFormsControl.Config.PDF.AnnotsSensitive | (uint)annot_type.annot_type_stamp;
            sxFormsControl.Config.PDF.AnnotsFocusable = sxFormsControl.Config.PDF.AnnotsFocusable | (uint)annot_type.annot_type_stamp;

            sxFormsControl.Events.OnTool += new SX.NET.Viewer.ETool(sxFormsControl_OnToolEvent);
        }

        void sxFormsControl_OnToolEvent(IViewerEvent_ToolActivity veToolActivity)
        {
            if (mainDoc == null)
                return;

            // get current activity tool
            SX.NET.Viewer.IToolState toolState = veToolActivity.State;
            if (toolState.ToolMode == tool_mode.tool_mode_free_text)
            {
                SX.NET.Viewer.IToolFreeText pToolText = (SX.NET.Viewer.IToolFreeText)toolState;
                switch (toolState.Activity)
                {
                    //sx_tool_change	= 1
                    case 1:
                        {
                            SX.NET.Viewer.IToolFreeText toolFreeText = (SX.NET.Viewer.IToolFreeText)toolState;
                            toolFreeText.TrackerStyle.SetFrameColor(new sx_color_i_rgb(255, 0, 0));
                            toolFreeText.TrackerStyle.SetFrameStyle(line_style.line_style_dash);
                            toolFreeText.TrackerStyle.SetFrameWidth(2);
                        }
                        break;
                    //sx_tool_start = 2
                    case 2:         
                        {
                            // get current page
                            uint pageNum = pToolText.Page;
                            if (pageNum == INVALID_INDEX)
                                break;
                            SX.NET.PDF.IPage page = mainDoc.Pages.Item(pageNum);

                            // search annotation on page
                            PointF pt = new PointF();
                            pToolText.GetToolPoint(ref pt);
                            SX.NET.sx_matrix mx = new sx_matrix();
                            page.CalcMatrix(ref mx, 1, sx_rotation_angle.sx_rotation_angle_0, 0, 0, 72);
                            SX.NET.Graphics.IMatrix matrix = sxFormsControl.Application.Factory.Matrix();
                            matrix.Invert(ref mx);
                            matrix.TransformPoint(mx, ref pt);
                            SX.NET.PDF.IFreeText freeTextAnnot = (SX.NET.PDF.IFreeText)page.Annots.FindItem(pt, 4); //annot_type_freetext		= 0x00000004
                            // set selected annotation
                            pToolText.Page = pageNum;
                            pToolText.FreeText = freeTextAnnot;
                        }
                        break;
                    // sx_tool_free_text_create = 42
                    case 42:
                        {
                            // get current page
                            uint pageNum = pToolText.Page;
                            if (pageNum == INVALID_INDEX)
                                break;
                            SX.NET.PDF.IPage page = mainDoc.Pages.Item(pageNum);

                            SX.NET.PDF.IFreeText freeTextAnnot = (SX.NET.PDF.IFreeText)page.Annots.InsertNew(annot_type.annot_type_freetext);
                            freeTextAnnot.SetColor(new sx_color_i_rgb(0, 200, 231));
                            freeTextAnnot.Opacity = 0.5;
                            freeTextAnnot.SetTextColor(new sx_color_i_rgb(0, 0, 0));
                            // set annotation font
                            freeTextAnnot.TextFont = mainDoc.Resources.AddFont("Arial", 400, 0, 5, true);
                            //set free text view
                            freeTextAnnot.BorderStyle = border_style.border_style_solid;
                            freeTextAnnot.BorderWidth = 1;
                            freeTextAnnot.EndStyle = line_ending.line_ending_open_arrow;    //for textboxes with callout only
                            freeTextAnnot.Intent = "FreeTextCallout";                       //for textboxes with callout only
                            // set active free text
                            pToolText.FreeText = freeTextAnnot;
                        }
                        break;
                }
            }
            
        }

        private void ShowErrorMessage(SX.NET.ErrorCode err)
        {
            string mess = string.Empty;
            switch (err)
            {
                case SX.NET.ErrorCode.ErrorOutOfMemory: mess = "not enough memory"; break;
                case SX.NET.ErrorCode.ErrorOutOfSpace: mess = "not enough disk space"; break;
                case SX.NET.ErrorCode.ErrorOutOfRange: mess = "index out of range"; break;
                case SX.NET.ErrorCode.ErrorInvalidFormat: mess = "invalid data format"; break;
                case SX.NET.ErrorCode.ErrorInvalidUse: mess = "invalid using"; break;
                case SX.NET.ErrorCode.ErrorInvalidParam: mess = "invalid argument"; break;
                default: mess = "Unknown error, please contact PDF Xpansion team."; break;
            }

            MessageBox.Show(mess, "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";
            dlg.RestoreDirectory = true;
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                SX.NET.PDF.IDocument doc = sxFormsControl.Application.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (!CheckSecurity(doc.Properties.Security))
                    return;

                sxFormsControl.Document = doc.GetViewableDocument();
                mainDoc = doc;
                sxFormsControl.ToolMode = tool_mode.tool_mode_free_text;
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog() == System.Windows.Forms.DialogResult.Cancel)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButtons.OK,MessageBoxIcon.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }

        private void openXPSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";            

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                SX.NET.XPS.IPackage xpsPack = appl.Factory.CreateXPSPackage();
                xpsPack.OpenFile(dlg.FileName);
                sxFormsControl.Document = xpsPack.GetViewableDocument();
                mainDoc = null;
            }
        }

        private void mnuSavePDF_Click(object sender, EventArgs e)
        {
            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Filter = "PDF Document|*.pdf";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                try
                {
                    mainDoc.SaveAsFile(dlg.FileName);
                }
                catch (SX.NET.Exception ex1)
                {
                    ShowErrorMessage(ex1.get_ErrorCode());
                }
                catch (System.Exception ex2)
                {
                    MessageBox.Show(ex2.Message);
                }
            }
        }

        private void mnuSavePDF_DropDownOpening(object sender, EventArgs e)
        {
            mnuSavePDF.Enabled = (mainDoc != null && mainDoc.Properties.Modified);
        }
    }
}
