﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Microsoft.Win32;
using SX.NET;
using SX.NET.PDF;
using SX.NET.XPS;

namespace WPF_reader
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {            
            InitializeComponent();
        }

        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            // configurate the viewer here
            PDFControl.Config.UseCache = false;
            PDFControl.Config.PageCacheSize = 50;
            PDFControl.Config.PageCacheInvalidation = 300;
            PDFControl.Config.PDF.Popups = SX.NET.Viewer.viewer_popups.viewer_popups_single;

            //PDFControl.Config.AllowLocalLinks = true;
            //PDFControl.Config.PDF.Popups = SX.NET.Viewer.viewer_popups.viewer_popups_multi;
            //PDFControl.Config.PDF.AnnotsVisible |= (uint)SX.NET.PDF.annot_type.annot_type_link | (uint)SX.NET.PDF.annot_type.annot_type_circle | (uint)SX.NET.PDF.annot_type.annot_type_popup;
            //PDFControl.Config.PDF.AnnotsSensitive |= (uint)SX.NET.PDF.annot_type.annot_type_link | (uint)SX.NET.PDF.annot_type.annot_type_circle | (uint)SX.NET.PDF.annot_type.annot_type_popup;
            //PDFControl.Config.PDF.AnnotsFocusable |= (uint)SX.NET.PDF.annot_type.annot_type_link | (uint)SX.NET.PDF.annot_type.annot_type_circle | (uint)SX.NET.PDF.annot_type.annot_type_popup;
            //PDFControl.Config.PDF.EnableFormFilling = true;
        }

        private void onMenuOpenPDFClick(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();

            
            dlg.DefaultExt = ".pdf";
            dlg.Filter = "PDF documents |*.pdf";

          
            bool result = dlg.ShowDialog().Value;

            if (result == true)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                SX.NET.PDF.IDocument doc = appl.Factory.CreatePDFDocument();
                doc.OpenFile(dlg.FileName, 0);

                if (CheckSecurity(doc.Properties.Security))
                {
                    PDFControl.Document = doc.GetViewableDocument();
                }
            }
                
        }

        private void onMenuOpenXPSClick(object sender, RoutedEventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.DefaultExt = ".xps";
            dlg.Filter = "XPS documents |*.xps";

            Nullable<bool> result = dlg.ShowDialog();

            if (result == true)
            {
                IApplication appl = PDFXpansionSDK.EntryPoint();
                IPackage xpsPack = appl.Factory.CreateXPSPackage();               
                xpsPack.OpenFile(dlg.FileName);
                PDFControl.Document = xpsPack.GetViewableDocument();               
            }
        }

        private bool CheckSecurity(IDocSecurity pDocSecurity)
        {
            doc_security dSec = pDocSecurity.Security;
            //When document has no security
            if (dSec == doc_security.doc_security_none)
                return true;

            //When document has password security
            if (dSec == doc_security.doc_security_password)
            {
                IPasswordSecurity pPassSec = (IPasswordSecurity)pDocSecurity.StandardSecurity;
                //Try to open document with empty password
                if (pPassSec.SetUserPassword(string.Empty))
                    return true;

                bool bInvalidPassword = true;
                do
                {
                    //Try to use password entered by user
                    PasswordDlg passwordDialog = new PasswordDlg();
                    if (passwordDialog.ShowDialog().Value == false)
                        return false;

                    string sPasswrd = passwordDialog.PasswordString;
                    bInvalidPassword = !(pPassSec.SetUserPassword(sPasswrd) || pPassSec.SetOwnerPassword(sPasswrd));

                    if (bInvalidPassword)
                        MessageBox.Show("Entered password is invalid", "", MessageBoxButton.OK , MessageBoxImage.Stop);

                } while (bInvalidPassword);
            }
            return true;
        }
    }
}
